% Matrix Canonical Structure (MCS) Toolbox
% Version 0.7
% 
% The following canonical structures are available
%     mstruct   - for matrices under similarity,
%     cmstruct  - for matrices under congruence,
%     scmstruct - for matrices under *congruence,
%     pstruct   - for matrix pencils under equivalence,
%     spstruct  - for symmetric matrix pencils,
%     sspstruct - for skew-symmetric matrix pencils,
%     mpstruct  - for matrix polynomials,
%     ssstruct  - for state-space system pencils.
%
%   A canonical structure object is created by calling the constructor of
%   the canonical structure class.
%
%   Common methods for canonical structure objects:
%     codim   - Compute the codimension of a canonical structure object.
%     size    - Total size of the represented structure.
%     numblk  - Number of canonical block objects.
%     isempty - True for an empty canonical structure object.
%     char    - Convert a structure object to a string.
%     exist   - Check if a canonical block of a specified type 
%               already exist in the canonical structure object.
%     copy    - Return a copy of the canonical structure object.
%     compare - Compare two canonical structure objects.
%     set     - Set the canonical structure information (not implemented).
%     get     - Get the canonical structure information.
%     getvalidblocks - Return a list of valid canonical blocks.
%
%   Available canonical blocks are:
%     fjblock - Jordan block associated with a finite eigenvalue.
%     zjblock - Jordan block associated with the zero eigenvalue.
%     ijblock - Jordan block associated with the infinite eigenvalue.
%     rsblock - Right singular block.
%     lsblock - Left singular block.
%     gblock  - Gamma block.
%     sgblock - *Gamma block associated with a complex parameter.
%     wblock  - W block associated with a specified eigenvalue.
%     swblock - *W block associated with a specified eigenvalue.
%     mblock  - Singular symmetric M block.
%     smblock - Singular skew-symmetric SM block.
%     hblock  - Symmetric H block associated with a finite or 
%               unspecified eigenvalue.
%     shblock - Skew-symmetric SH block associated with a finite or 
%               unspecified eigenvalue.
%     kblock  - Symmetric K block associated with the infinite eigenvalue.
%     skblock - Skew-symmetric K block associated with the infinite 
%               eigenvalue.
%     
%   The two tables below show which blocks are available for each canonical
%   structure.
%
%            mstruct cmstruct scmstruct 
%    fjblock    x                      
%    zjblock    x        x        x     
%     gblock             x
%    sgblock                      x
%     wblock             x
%    swblock                      x
%
%            pstruct spstruct sspstruct mpstruct ssstruct
%    fjblock    x                          x        x
%    zjblock    x                          x        x
%    ijblock    x                          x        x
%    rsblock    x                          x        x
%    lsblock    x                          x        x
%     mblock            x
%    smblock                       x
%     hblock            x
%    shblock                       x
%     kblock            x
%    skblock                       x
%
%   Available canonical forms are:
%     jcf/jnf - Jordan Canonical Form
%     ccf     - Congruent Canonical Form
%     kcf     - Kronecker Canonical Form
%     bcf     - (generalized) Brunovsky Canonical Form
%
%     The table below shows which form(s) is(are) available for each
%     canonical structure.
%
%               jcf   ccf   kcf   bcf
%     mstruct    x
%     cmstruct         x
%     scmstruct        x
%     pstruct                x
%     spstruct               x
%     sspstruct              x
%     mpstruct
%     ssstruct               x     x
%
% Computational functions
%   pguptri   - Compute the canonical structure information of a matrix pencil.
%   pcluster  - Compute and cluster generalized eigenvalues of a matrix pencil.
%   pggallery - Test matrix pencils for staircase computation and bounds.
%
%   [x] below can be one of the prefixes:
%     m   - matrices under similarity,
%     cm  - matrices under congruence,
%     scm - matrices under *congruence,
%     p   - matrix pencils under equivalence,
%     sp  - symmetric matrix pencils,
%     ssp - skew-symmetric matrix pencils,
%     s2  - controllability or observability system pencils.
%
%   [x]codim    - Computes the codimension.
%   [x]tanspace - Computes the tangent space.
%
% Notation conversion
%   segre2weyr  - Weyr from Segre characteristics.
%   weyr2segre  - Segre from weyr characteristics. 
%   weyr2sizes  - Block sizes from weyr characteristics.
%   sizes2weyr  - Weyr characteristics from block sizes.
%   sizes2segre - Segre characteristics from block sizes.
%
% Auxiliary functions
%   mcstolerance - Set tolerance parameters used by MCS Toolbox.
%   mcsformat    - Set format to display structure and block objects.
%   mcsgetvalidblocks - Return a list of available canonical blocks.
%   mcsevclmth   - Set default method used to cluster eigenvalues in MCS Toolbox.
%