% *** DELETE THIS ***
% Template class for a canonical block.
% + Change class name xblock, where x is an appropriate abreviation for the
% block type.
% + Update constructor, set.blockv(), initblockprint(), and updateSize().
% + Add methods for valid canonical forms and list them under available
% methods below. 
% + Make appropriate changes in tojava() if block is present in
% StratiGraph. Otherwise delete all code in tojava.
% *** END DELETE THIS ***

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

%XBLOCK Create a X block object.
%   XBLOCK creates a canonical block object representing XXX block
%   structure.
%
%   Each n-by-n X block is defined as:
%                  |     |
%          X_n :=  |     |
%                  |     |
%
%   BlockObj = XBLOCK(XBlocks) returns a new canonical block object
%   BlockObj representing a structure of X blocks. The vector XBlocks =
%   [s_n, ..., s_1] is the structure integer partition representing the
%   (unordered) sizes (s_k)-by-(s_k) of the blocks.
%
%   BlockObj = XBLOCK(XBlocks, Notation) also specifies the notation used
%   for XBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = XBLOCK returns an empty X block object.
%
%   The class XBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   XBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     JCF        - Jordan canonical form of the block object.
%     CCF        - Congruence canonical form of the block object.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   XBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also .

classdef xblock < mcsblock
  
  properties (Access = protected)
    m = 0;
    n = 0;
    blockv = [];
  end
  
  properties (Constant, Access = protected)
    type = 'singular'; % Can be 'singular' or 'regular'
  end
  
  methods
    % Constructor
    function obj = xblock(blk,not)

      if nargin == 0
        % *** Alternative 1 ***
        % Return an empty object...
        return
        
        % *** Alternative 2 ***
        % See also below
        blk = [];
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % *** Alternative 1 ***
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);

      % *** Alternative 2 ***
      m = sum(blk);
      n = sum(blk);
      % Initialize superclass
      obj = obj@mcsblock(blk,m,n,not);
      % *** End Alternatives ***

    end % End constructor
    %-------------------------------
    
    function obj = set.blockv(obj,blk)
      % Check that block sizes are valid.
      
      if isempty(blk)
        obj.blockv = [];
        return
      end
      
      if ~isnumeric(blk) || ~isvector(blk) 
        error('The structure integer partition must be a vector.');
      elseif size(blk,1) > 1
        blk = blk';
      end
      
      % CHANGE TO APPROPRIATE VALUE!
      if min(blk) < 1
        error('Block sizes must be at least one.');
        %error('Block sizes must be positive.');
      end
      obj.blockv = blk;
      obj = obj.updateSize;
    end
    %-------------------------------

    % Following methods are provided by the superclass mcsblock, but may be
    % overloaded in the subclass. See, e.g., the classes jblock and
    % mcsblock for examples.
    %
    % function varargout = size(obj,dim)
    % function b = eq(obj1,obj2)
    % function b = neq(obj1,obj2)
    % function varargout = get(obj,varargin)
    % function obj = set(obj,blk,varargin)
    
    
  end % End public methods

%% Protected methods
  methods (Access = protected)
    % INITBLOCKPRINT Returns the label of the block, optional parameter,
    % and offset. Offset is the smallest possible value of m or n.
    % Typically, 1 for regular blocks and 0 for singular blocks.
    function [label,param,offset] = initblockprint(obj)
      label = 'X'; % Block label that will be used when display is called.
      param = '';  % Optional parameter, e.g., '(e)' for a Jordan block 
                   % where e is the eigenvalue. 
      offset = 0;  % Set offset to the smallest possible value of m or n.
                   % (1 for regular blocks and 0 for singular blocks)
    end
    %-------------------------------
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = sum(obj.blockv); % Change to appropriate value
      obj.n = sum(obj.blockv);
    end
   
  end % End protected methods 

%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct})
    function robj = tojava(obj)
      %TOJAVA Converts a right singular block object to a Java representation.
      %   RObj = TOJAVA(BlockObj) returns a Java object RObj representing
      %   the same structure as the canonical block object BlockObj.

      try
        ct = obj.weyr();
        robj = se.umu.stratigraph.core.structure.SingularIndices(...
          se.umu.stratigraph.core.structure.BlockType.R,ct);
      catch e
        error('MCS:xblock:BlockNotFound',...
          'StratiGraph is not present or the requested block is not found in StratiGraph.')
      end

    end
    %-------------------------------
  end  
end % End classdef
