
%FJBLOCK Create a Jordan block object.
%   FJBLOCK creates a canonical block object representing a Jordan block
%   structure associated with a finite or unspecified eigenvalue.
%
%   Each n-by-n Jordan block associated with a finite eigenvalue mu is
%   defined as:
%                           |mu  1    0|
%                J_n(mu) := |   mu .   |
%                           |      .  1|
%                           |0       mu|
%   for matrices and as J_n(mu) - s*I_n for matrix pencils.
%
%   BlockObj = FJBLOCK(JordanBlocks, Ev) returns a new canonical block
%   object BlockObj representing a structure of Jordan blocks. The Jordan
%   blocks are defined by the exponents (h_n, ...., h_1) of the finite
%   elementary divisors. The vector JordanBlocks = [h_n, ..., h_1] is the
%   structure integer partition representing the (unordered) sizes
%   (h_k)-by-(h_k) of the blocks and Ev is the associated eigenvalue. Ev
%   can be a complex number or NaN (unspecified eigenvalue). If Ev is
%   omitted, the eigenvalue is assumed to be unspecified.
%
%   BlockObj = FJBLOCK(JordanBlocks, Ev, Notation) also specifies the
%   notation used for JordanBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = FJBLOCK returns an empty Jordan block object.
%
%
%   The class FJBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   FJBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     JCF        - Jordan canonical form of the block object.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   FJBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also IJBLOCK, ZJBLOCK, MSTRUCT, PSTRUCT, SSSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef fjblock < jblock
  
  properties (Access = protected)
    ev = NaN;
  end

  methods
    % Constructor
    function obj = fjblock(blk,ev,not)
     
      if nargin == 0
        blk = [];
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@jblock(blk,not);
      
      % Set eigenvalue
      if nargin >= 2
        obj.ev = ev;
      end

    end % End constructor
    %-------------------------------

    function J = jcf(obj,not)
      %JCF Jordan canonical form of the block object.
      %   J = JCF(BlockObj) returns the matrix J in Jordan Canonical Form (JCF)
      %   specified by the Jordan block object BlockObj.
      %
      %   ... = JCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also KCF, MSTRUCT/JCF.
      
      if nargin == 1
        not = 'sizes';
      end
      J = kcf(obj,not);
    end
    %-------------------------------

    function varargout = bcf(obj,not)
      %BCF Brunovsky canonical form of the block object.
      %   A = BCF(BlockObj) returns the matrix A of the system pencil S-sT = [A
      %   B; C D] - s[I 0; 0 0] in Brunovsky Canonical Form (BCF) specified by
      %   the Jordan block object BlockObj (the matrix A is in Jordan canonical
      %   form). Consequently, the pencil S-sT consists only of Jordan blocks
      %   associated with a finite eigenvalue (J blocks).
      %
      %   [A,B,C,D] = BCF(BlockObj) also returns the empty matrices B, C and D of
      %   the system pencil S-sT.
      %
      %   ... = BCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also KCF, JCF.
      
      if nargin == 1
        not = 'sizes';
      end
      A = kcf(obj,not);

      if nargout < 2
        varargout{1} = A;
      elseif nargout == 4 
        varargout{1} = A;
        varargout{2} = zeros(obj.m,0);
        varargout{3} = zeros(0,obj.m);
        varargout{4} = [];
      else
        error('Invalid number of output arguments.')
      end
    end
    %-------------------------------

    function [G,H] = kcf(obj,not)
      %KCF Kronecker canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the square matrix pencil G-sH in
      %   Kronecker Canonical Form (KCF) specified by the Jordan block object
      %   BlockObj. The matrix H is an identity matrix and G is block-diagonal
      %   where each block is in Jordan Canonical Form (JCF).
      %
      %   J = KCF(BlockObj) returns the matrix J in JCF.
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also JCF, IJBLOCK/KCF, PSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = eye(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        % By using diag, kcf can also handle unspecified eigenvalues (NaN)
        G(p:p+r-1,p:p+r-1) = diag(ones(r,1)*obj.ev) + diag(ones(r-1,1),1);
        p = p + r;
      end
    end
    %-------------------------------
    
    function obj = set.ev(obj,value)
      % Check that the eigenvalue is valid.
      
      if isa(obj,'zjblock')
        if value == 0
          obj.ev = 0;
        else
          % If the eigenvalue is changed, return an error.
          error('MCS:zjblock:Eigenvalue','Not allowed to change the eigenvalue of a ''zjblock'' object.')
        end
        return
      end
      
      if isempty(value)
        obj.ev = NaN;
        return
      end
      
      % Check that the eigenvalue is valid.
      if ~isnumeric(value) || ~isscalar(value) || isinf(value)
        error('MCS:fjblock:Eigenvalue','The eigenvalue must be a finite scalar or NaN.');
      end
      obj.ev = value;
    end

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block, its eigenvalue, and
      % offset, which is the smallest possible value of m or n. For regular
      % blocks offset is 1.
      
      label = 'J';
      param = obj.ev;
      offset = 1;
    end
  end % End protected methods 

%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct})
    function jblocktype = getSGBlock(~)
      %GETSGBLOCK Get the StratiGraph block type.
      jblocktype = se.umu.stratigraph.core.structure.BlockType.J;
    end
    
    function jobj = tojava(obj)
      %TOJAVA Convert a Jordan block object to a Java representation.
      %   JObj = TOJAVA(BlockObj) returns a Java object JObj representing
      %   the same structure as the canonical block object BlockObj.

      try
        ct = obj.weyr();
        if ~isempty(ct)
          if ~isnan(obj.ev)
            jobj = se.umu.stratigraph.core.structure.BlockParameter( ...
              se.umu.stratigraph.core.structure.BlockType.J, ct, ...
              se.umu.stratigraph.core.util.Eigenvalue(real(obj.ev),imag(obj.ev)));
          else % unspecified eigenvalue
            jobj = se.umu.stratigraph.core.structure.BlockParameter( ...
              se.umu.stratigraph.core.structure.BlockType.J, ct);
          end
        else
          jobj = se.umu.stratigraph.core.structure.BlockParameter( ...
              se.umu.stratigraph.core.structure.BlockType.J);
        end
      catch e
        error('MCS:fjblock:BlockNotFound',...
          'StratiGraph is not present or the requested block is not found in StratiGraph.')
      end
              
    end
    %-------------------------------
  end  
end

