%GBLOCK Create a Gamma block object.
%   GBLOCK creates a canonical block object representing a Gamma block
%   structure.
%
%   Each n-by-n Gamma block is defined as:
%                      |0         ...|
%                      |        ...  |
%           Gamma_n := |      1  1   |
%                      |  -1 -1      |
%                      |1  1        0|
%   
%   BlockObj = GBLOCK(GBlocks) returns a new canonical block object
%   BlockObj representing a structure of Gamma blocks. The vector GBlocks =
%   [epsilon_n, ..., epsilon_1] is the structure integer partition
%   representing the (unordered) sizes (epsilon_k)-by-(epsilon_k) of the
%   blocks.
%
%   BlockObj = GBLOCK(GBlocks, Notation) also specifies the notation used
%   for GBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = GBLOCK returns an empty Gamma block object.
%
%
%   The class GBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   GBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     CCF        - Congruence canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   GBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also SGBLOCK, CMSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef gblock < pblock
 
  properties (Access = protected)
    ev = NaN;
  end
  
  methods
    % Constructor
    function obj = gblock(blk,not)

      if nargin == 0
        % Return an empty object...
        return
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);
    end % End constructor
    %-------------------------------
    
    function [H] = ccf(obj,not)
      %CCF Congruence canonical form of the block object.
      %   H = CCF(BlockObj) returns the matrix H in Congruence Canonical Form
      %   (CCF) specified by the Gamma block object BlockObj. The matrix H only
      %   consists of Gamma blocks.
      %
      %   ... = CCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also CMSTRUCT/CCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      H = zeros(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
         r = blocks(i);
         Z = zeros(r,r);
         for j = 1:r
           Z(r-j+1,j)=(-1)^(j-1);
         end
         H(p:p+r-1,p:p+r-1) = Z*(diag(ones(r,1)) + diag(ones(r-1,1),1));
         p = p + r;
      end
    end
    %-------------------------------

    function obj = set.ev(obj,value)
      % If the parameter is changed, return an error.
      error('MCS:gblock:Parameter','Not allowed to change the parameter of a ''gblock'' object.')
    end
    %-------------------------------
    
    function obj = get.ev(obj)
      % When asking for the parameter, return an error.
      error('MCS:gblock:Parameter','The ''gblock'' object does not have a parameter.')
    end

    
  end % End public methods

%% Protected methods
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block and offset, which is the
      % smallest possible value of m or n. Typically, 1 for regular blocks
      % and 0 for singular blocks.

      label = 'G'; % Label that will be used when display is called.
      param = [];
      offset = 0; % Set offset to the smallest possible value of m or n.
                  % (1 for regular blocks and 0 for singular blocks)
    end
    %-------------------------------
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = sum(obj.blockv);
      obj.n = sum(obj.blockv);
    end
  end % End protected methods 
  
end % End classdef
