%HBLOCK Create a H block object.
%   HBLOCK creates a canonical block object representing a symmetric H
%   block structure associated with a finite or unspecified eigenvalue.
%
%   Each n-by-n symmetric H block associated with an eigenvalue mu is
%   defined as:
%                      | 0     mu|     |0     1|
%           H_n(mu) := |    mu 1 |  - s|    1  |
%                      |  . .    |     |  .    |
%                      |mu 1   0 |     |1     0|
%
%   BlockObj = HBLOCK(HBlocks, Ev) returns a new canonical block
%   object BlockObj representing a structure of symmetric H blocks
%   ("anti-diagonal" Jordan blocks) associated with a finite or
%   unspecified eigenvalue. The vector HBlocks = [h_n, ..., h_1] is
%   the structure integer partition representing the (unordered)
%   sizes (h_k)-by-(h_k) of the blocks and Ev is the associated
%   eigenvalue. Ev can be a complex number or NaN (unspecified
%   eigenvalue). If Ev is omitted, the eigenvalue is assumed to be
%   unspecified.
%
%   BlockObj = HBLOCK(HBlocks, Ev, Notation) also specifies the
%   notation used for HBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = HBLOCK returns an empty H block object.
%
%
%   The class HBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   HBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker-like canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   HBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also SHBLOCK, SPSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef hblock < pblock
  
  properties (Access = protected)
    ev = NaN;
  end
  
  methods
    % Constructor
    function obj = hblock(blk,ev,not)
      
      if nargin == 0
        blk = [];
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@pblock(blk,not);
      
      % Set eigenvalue
      if nargin >= 2
        obj.ev = ev;
      end

    end % End constructor
    %-------------------------------

    function [G,H] = kcf(obj,not)
      %KCF Kronecker-like canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the square symmetric matrix pencil G-sH
      %   in Kronecker-like canonical form specified by the H block object
      %   BlockObj.
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SPSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = eye(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        Z = zeros(r,r);
        for j = 1:r
          Z(r-j+1,j)=1;
        end
        % By using diag, kcf can also handle unspecified eigenvalues (NaN)
        G(p:p+r-1,p:p+r-1) =Z*(diag(ones(r,1)*obj.ev) + diag(ones(r-1,1),1));
        H(p:p+r-1,p:p+r-1) = Z;
        p = p + r;
      end
    end
    %-------------------------------
    
    function obj = set.ev(obj,value)
      % Check that the eigenvalue is valid.
      
      if isempty(value)
        obj.ev = NaN;
        return
      end
      
      % Check that the eigenvalue is valid.
      if ~isnumeric(value) || ~isscalar(value) || isinf(value)
        error('MCS:hblock:Eigenvalue','The eigenvalue must be a finite scalar or NaN.');
      end
      obj.ev = value;
    end

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block, its eigenvalue, and
      % offset. For regular blocks offset is 1.
      
      label = 'H';
      param = obj.ev;
      offset = 1;
    end
  end % End protected methods 

end

