%IJBLOCK Create a Jordan block object associated with the infinite eigenvalue.
%   IJBLOCK creates a canonical block object representing a Jordan block
%   structure associated with the infinite eigenvalue.
%
%   Each n-by-n Jordan block associated with the infinite eigenvalue is
%   defined as:
%                      |1 0    0|     |0 1    0|
%               N_n := |  1 .   |  - s|  0 .   |
%                      |    .  0|     |    .  1|
%                      |0      1|     |0      0|
%
%   BlockObj = IJBLOCK(IJordanBlocks) returns a new canonical block object
%   BlockObj representing a structure of Jordan blocks associated with the
%   infinite eigenvalue. The Jordan blocks are defined by the exponents
%   (s_n, ...., s_1) of the infinite elementary divisors. The vector
%   IJordanBlocks = [s_n, ..., s_1] is the structure integer partition
%   representing the (unordered) sizes (s_k)-by-(s_k) of the blocks.
%
%   BlockObj = IJBLOCK(IJordanBlocks, Notation) also specifies the notation
%   used for IJordanBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = IJBLOCK returns an empty Jordan block object.
%
%
%   The class IJBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   IJBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   IJBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also FJBLOCK, ZJBLOCK, MSTRUCT, PSTRUCT, SSSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef ijblock < jblock
  
  properties (Access = protected)
    ev = inf;
  end
  
  methods
    % Constructor
    function obj = ijblock(blk,not)

      if nargin == 0
        blk = [];
      end

      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Initialize superclass
      obj = obj@jblock(blk,not);
    end % End constructor
    %-------------------------------
    
    function varargout = bcf(obj,not)
      %BCF Brunovsky canonical form of the block object.
      %   [A,B,C,D] = BCF(BlockObj) returns the matrix quadruple (A,B,C,D) of the
      %   system pencil S-sT = [A B; C D] - s[I 0; 0 0] in Brunovsky Canonical
      %   Form (BCF) specified by the Jordan block object BlockObj associated
      %   with the infinite eigenvalue. Consequently, the pencil S-sT consists
      %   only of Jordan blocks associated with a infinite eigenvalue (N blocks).
      %
      %   ... = BCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also KCF, JCF.

      narginchk(1,2)
      nargoutchk(4,4)
      
      % Zeros at infinity (= sizes - 1)
      if nargin == 2 && strcmp(not,'segre')
        infZeros = sort(obj.blockv(obj.blockv>1),'descend')-1;
      else
        infZeros = obj.blockv(obj.blockv>1)-1;
      end        
      
      ijLength = length(obj.blockv);
      dLength = length(infZeros);
      
      n = sum(infZeros);
      A = zeros(n,n);
      B = zeros(n,ijLength);
      C = zeros(ijLength,n);
      p = 1;
      for i = 1:dLength
        r = infZeros(i);
        A(p:p+r-1,p:p+r-1) = diag(ones(r-1,1),-1);
        B(p,i) = 1;
        C(i,p+r-1) = 1;
        p = p+r;
      end
      
      if ijLength == dLength
        D = zeros(ijLength,ijLength);
      else
        D = blkdiag(zeros(dLength,dLength), eye(ijLength - dLength));
      end

      if nargout == 4
        varargout{1} = A;
        varargout{2} = B;
        varargout{3} = C;
        varargout{4} = D;
      end
    end
    %-------------------------------
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the matrix pencil G-sH in Kronecker
      %   Canonical Form (KCF) specified by the Jordan block object BlockObj
      %   associated with the infinite eigenvalue. The matrix G is an identity
      %   matrix and H is in Jordan canonical form (JCF).
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also FJBLOCK/KCF, PSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = eye(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1; q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        H(p:p+r-2,q+1:q+r-1) = eye(r-1);
        p = p + r;
        q = q + r;
      end
    end
    %-------------------------------

    function obj = set.ev(obj,value)
      % If the eigenvalue is changed, return an error.
      error('MCS:ijblock:Eigenvalue','Not allowed to change the eigenvalue of an ''ijblock'' object.')
    end
    %-------------------------------

  end % End public methods

  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block, its eigenvalue, and
      % offset, which is the smallest possible value of m or n. For regular
      % blocks offset is 1.

      label = 'N';
      param = [];
      offset = 1;
    end
  end % End protected methods 
  
%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct})
    function jblocktype = getSGBlock(~)
      %GETSGBLOCK Get the StratiGraph block type.
      jblocktype = se.umu.stratigraph.core.structure.BlockType.N;
    end
    
    function ijobj = tojava(obj)
      %TOJAVA Convert a Jordan block object to a Java representation.
      %   IJObj = TOJAVA(BlockObj) returns a Java object IJObj representing
      %   the same structure as the canonical block object BlockObj.

      try
        ijobj = se.umu.stratigraph.core.structure.BlockParameter( ...
              se.umu.stratigraph.core.structure.BlockType.N, obj.weyr());
      catch e
        error('MCS:ijblock:BlockNotFound',...
          'StratiGraph is not present or the requested block is not found in StratiGraph.')
      end

    end    
  end  
end % End classdef
