%KBLOCK Create a K block object associated with the infinite eigenvalue.
%   KBLOCK creates a canonical block object representing a symmetric K
%   block structure associated with the infinite eigenvalue.
%
%   Each n-by-n symmetric K block is defined as:
%               |0     1|     |0     0|
%        K_n := |    1  |  - s|    0 1|
%               |  .    |     |  . .  |
%               |1     0|     |0 1   0|
%
%   BlockObj = KBLOCK(KBlocks) returns a new canonical block object
%   BlockObj representing a structure of symmetric K blocks ("anti-
%   diagonal" Jordan blocks) associated with the infinite eigenvalue. The
%   vector KBlocks = [s_n, ..., s_1] is the structure integer partition
%   representing the (unordered) sizes (s_k)-by-(s_k) of the blocks.
%
%   BlockObj = KBLOCK(SKBlocks, Notation) also specifies the notation used
%   for KBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = KBLOCK returns an empty K block object.
%
%
%   The class KBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   KBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker-like canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   KBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also SKBLOCK, SPSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef kblock < pblock
  
  properties (Access = protected)
    ev = inf;
  end
  
  methods
    % Constructor
    function obj = kblock(blk,not)

      if nargin == 0
        blk = [];
      end

      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Initialize superclass
      obj = obj@pblock(blk,not);
    end % End constructor
    %-------------------------------
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker-like canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the square symmetric matrix pencil G-sH
      %   in Kronecker-like canonical form specified by the K block object
      %   BlockObj associated with the infinite eigenvalue.
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SPSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        Z = zeros(r,r);
        for j = 1:r
          Z(r-j+1,j)=1;
        end
        % By using diag, kcf can also handle unspecified eigenvalues (NaN)
        H(p:p+r-1,p:p+r-1) =Z*diag(ones(r-1,1),1);
        G(p:p+r-1,p:p+r-1) = Z;
        p = p + r;
      end
    end
    %-------------------------------

    function obj = set.ev(obj,value)
      % If the eigenvalue is changed, return an error.
      error('MCS:kblock:Eigenvalue','Not allowed to change the eigenvalue of an ''kblock'' object.')
    end
    %-------------------------------

  end % End public methods

  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block, its eigenvalue, and offset,
      % which is the smallest possible value of m or n. For regular blocks offset
      % is 1.

      label = 'K';
      param = [];
      offset = 1;
    end
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = sum(obj.blockv);
      obj.n = obj.m;
    end

  end % End protected methods 
   
end % End classdef
