%LSBLOCK Create a left singular block object.
%   LSBLOCK creates a canonical block object representing a left singular
%   block structure.
%
%   Each (n+1)-by-n left singular block is defined as:
%                      |0     0|     |1     0|
%             L_n^T := |1  .   |  - s|0  .   |
%                      |   .  0|     |   .  1|
%                      |0     1|     |0     0|
%
%   BlockObj = LSBLOCK(LSBlocks) returns a new canonical block object
%   BlockObj representing a structure of left singular blocks (L^T blocks).
%   The left singular blocks are defined by the row (left) minimal indices
%   (eta_n, ..., eta_1), and the vector LSBlocks = [eta_n, ..., eta_1] is
%   the structure integer partition representing the (unordered) sizes
%   (eta_k+1)-by-(eta_k) of the blocks.
%
%   BlockObj = LSBLOCK(LSBlocks, Notation) also specifies the notation used
%   for LSBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = LSBLOCK returns an empty left singular block object.
%
%
%   The class LSBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   LSBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   LSBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also RSBLOCK, PSTRUCT, SSSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef lsblock < sblock
  
  methods
    % Constructor
    function obj = lsblock(blk,not)

      if nargin == 0
        % Return an empty object...
        return
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);

    end % End constructor
    %-------------------------------
    
    function varargout = bcf(obj,not)
      %BCF Brunovsky canonical form of the block object.
      %   [A,C] = BCF(BlockObj) returns the pair of matrices (A,C) of the system
      %   pencil S-sT = [A; C] - s[I; 0] in Brunovsky Canonical Form (BCF)
      %   specified by the left singular block object BlockObj. Consequently, the
      %   pencil S-sT consists only of left singular blocks (L^T blocks).
      %
      %   [A,B,C,D] = BCF(BlockObj) also returns the empty matrices B and D of
      %   the system pencil S-sT = [A B; C D] - s[I 0; 0 0].
      %
      %   ... = BCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also KCF, SSSTRUCT/BCF.

      if nargin == 2 && strcmp(not,'segre')
        rowInd = sort(obj.blockv(obj.blockv>0),'descend');
      else
        rowInd = obj.blockv(obj.blockv>0);
      end
      
      riLength = length(rowInd);
      lsLength = length(obj.blockv);
      
      A = zeros(obj.n,obj.n);
      C = zeros(lsLength,obj.n);
      p = 1;
      for i = 1:riLength
        % Matrix A
        r = rowInd(i);
        A(p:p+r-1,p:p+r-1) = diag(ones(r-1,1),-1);
        % Matrix C
        C(i,p+r-1) = 1;
        p = p + r;
      end
      
      if nargout == 2
        varargout{1} = A;
        varargout{2} = C;
      elseif nargout == 4
        varargout{1} = A;
        varargout{2} = zeros(obj.n,0);
        varargout{3} = C;
        varargout{4} = zeros(lsLength,0);
      else
        error('Invalid number of output arguments.')
      end        
    end
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the matrix pencil G-sH in Kronecker
      %   Canonical Form (KCF) specified by the left singular block object
      %   BlockObj. The pencil G-sH only consists of left singular blocks (L^T
      %   blocks).
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also PSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1; q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        G(p+1:p+r,q:q+r-1) = eye(r);
        H(p:p+r-1,q:q+r-1) = eye(r);
        p = p + r+1;
        q = q + r;
      end
    end
    %-------------------------------
    
  end % End public methods

  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block and offset, which is the
      % smallest possible value of m or n. Typically, 1 for regular blocks and 0
      % for singular blocks.

      label = 'L'; % Label that will be used when display is called.
      param = [];
      offset = 0; % Set offset to the smallest possible value of m or n.
                  % (1 for regular blocks and 0 for singular blocks)
    end
    %-------------------------------
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = sum(obj.blockv) + length(obj.blockv);
      obj.n = sum(obj.blockv);
    end
  end % End protected methods 
  
%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct})
    function jblocktype = getSGBlock(~)
      %GETSGBLOCK Get the StratiGraph block type.
      jblocktype = se.umu.stratigraph.core.structure.BlockType.L;
    end
    
    function lobj = tojava(obj)
      %TOJAVA Convert a left singular block object to a Java representation.
      %   LObj = TOJAVA(BlockObj) returns a Java object LObj representing
      %   the same structure as the canonical block object BlockObj.

      try
        ct = obj.weyr();
        lobj = se.umu.stratigraph.core.structure.BlockParameter( ...
          se.umu.stratigraph.core.structure.BlockType.L, ct);
      catch e
        error('MCS:lsblock:BlockNotFound',...
          'StratiGraph is not present or the requested block is not found in StratiGraph.')
      end

    end

  end  
end % End classdef
