%MBLOCK Create a singular M block object.
%   MBLOCK creates a canonical block object representing a symmetric
%   singular M block structure.
%
%   Each (2n+1)-by-(2n+1) symmetric M block is defined as:
%               | 0   G_n^T|     | 0   F_n^T|
%        M_n := |          |  - s|          |,
%               |G_n    0  |     |F_n    0  |
%   where
%               |0 1   0|             |1 0   0|
%        G_n := |  . .  |, and F_n := |  . .  |
%               |0   0 1|             |0   1 0|
%
%   NOTE: G_n - sF_n forms a right singular block L_n, and
%         G_n^T - sF_n^T a left singular block L_n^T.
%
%   BlockObj = MBLOCK(MBlocks) returns a new canonical block object
%   BlockObj representing a structure of symmetric M blocks (pairs of
%   singular blocks). The vector MBlocks = [epsilon_n, ..., epsilon_1] is
%   the structure integer partition representing the (unordered) sizes
%   (2*epsilon_k+1)-by-(2*epsilon_k+1) of the blocks.
%
%   BlockObj = MBLOCK(MBlocks, Notation) also specifies the notation used
%   for MBlocks.  Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   BlockObj = MBLOCK returns an empty M block object.
%
%
%   The class MBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   MBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker-like canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   MBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also SMBLOCK, SPSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef mblock < sblock
  
  methods
    % Constructor
    function obj = mblock(blk,not)
      
      if nargin == 0
        % Return an empty object...
        return
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);
      
    end % End constructor
    %-------------------------------
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker-like canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the symmetric matrix pencil G-sH in the
      %   Kronecker-like canonical form specified by the singular M block object
      %   BlockObj. The pencil G-sH only consists of singular blocks (SM blocks).
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SPSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1; q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        G(p+1:p+r,r+q+1:r+q+r) = eye(r);
        H(p:p+r-1,r+q+1:r+q+r) = eye(r);
        G(r+p+1:r+p+r,q+1:q+r) = eye(r);
        H(r+p+1:r+p+r,q:q+r-1) = eye(r);
        p = p + 2*r+1;
        q = q + 2*r+1;
      end
    end
    %-------------------------------
    
  end % End public methods
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block and offset, which is the
      % smallest possible value of m or n. Typically, 1 for regular blocks and 0
      % for singular blocks.

      label = 'M'; % Label that will be used when display is called.
      param = [];
      offset = 0; % Set offset to the smallest possible value of m or n.
      % (1 for regular blocks and 0 for singular blocks)
    end
    %-------------------------------
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.
      
      obj.m = 2*sum(obj.blockv) + length(obj.blockv);
      obj.n = 2*sum(obj.blockv) + length(obj.blockv);
    end
  end % End protected methods
  
end % End classdef
