%MCSBLOCK Abstract class for generic canonical blocks.
%   MCSBLOCK provides a template and generic methods for canonical block
%   objects.
%
%   MCSBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   MCSBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also MCSSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef mcsblock < matlab.mixin.Copyable
  % matlab.mixin.Copyable class is an abstract handle class that provides
  % the copy method for copying handle objects.
  
  properties (Abstract, Access=protected)
    m
    n
    blockv % Structure integer partition stored in 'sizes' notation (may be unordered).
  end
  
  properties (Abstract, Constant, Access=protected)
    type   % Declared as 'singular' or 'regular' in subclass.
  end
  
  %% Public methods
  methods
    function b = eq(obj1,obj2)
      %EQ (==) Equal relation between two block objects.
      %   Block1 == Block2 checks if the two canonical block objects are equal.
      %
      %   See also NE, COMPARE.

      if ~strcmp(class(obj1),class(obj2))
        b = false; 
        return
      end

      lblocks1 = obj1.segre;
      lblocks2 = obj2.segre;
      b = length(lblocks1) == length(lblocks2);
      if b && ~isempty(lblocks1)
        b = all(lblocks1==lblocks2);
      end
    end
    
    function b = ne(obj1,obj2)
      %NE (~=) Not equal relation between two block objects.
      %   Block1 ~= Block2 checks if the two canonical block objects are not
      %   equal.
      %
      %   See also EQ, COMPARE.
      
      b = ~eq(obj1,obj2);
    end
    %-------------------------------
    
    function varargout = size(obj,dim)
      %SIZE Total size of the represented canonical blocks.
      %   D = SIZE(BlockObj) returns a two-element row vector D = [M, N]
      %   containing the number of rows and columns of the canonical block object
      %   BlockObj, i.e., the sum of the sizes of the represented canonical
      %   blocks.
      %
      %   [M,N] = SIZE(BlockObj) returns the number of rows and columns in
      %   separate output variables.
      %
      %   SIZE(BlockObj,Dim) returns the length of the dimension specified by the
      %   scalar Dim. For example, SIZE(X,1) returns the number of rows.
      
      if nargin == 1
        nargoutchk(0,2);
        
        if nargout < 2
          varargout{1} = [obj.m,obj.n];
        else
          varargout{1} = obj.m;
          varargout{2} = obj.n;
        end
      else
        nargoutchk(0,1);
        
        switch dim
          case 1
            varargout{1} = obj.m;
          case 2
            varargout{1} = obj.n;
          otherwise
            varargout{1} = 1;
        end
      end
    end 
    %-------------------------------

    function n = numblk(obj)
      %NUMBLK Number of canonical blocks.
      %   N = NUMBLK(BlockObj) returns the number of canonical blocks in the
      %   block object BlockObj.
      %
      %   See also SIZE, MCSSTRUCT/NUMBLK.
      
      n = length(obj.blockv);
    end
    %----------------------
    
    function b = isempty(obj)
      %ISEMPTY True for empty canonical block object.
      %   ISEMPTY(BlockObj) returns logical 1 (true) if BlockObj is an empty
      %   canonical block object and logical 0 (false) otherwise. An empty block
      %   object has an empty structure integer partition.
      
      b = isempty(obj.blockv);
    end
    %-------------------------------

    function b = issingular(obj)
      %ISSINGULAR True for singular canonical block object.
      %   ISSINGULAR(BlockObj) returns logical 1 (true) if BlockObj represents a
      %   singular canonical block and logical 0 (false) otherwise.
      
      b = strcmp(obj.type,'singular');
    end
    
    function b = isregular(obj)
      %ISREGULAR True for regular canonical block object.
      %   ISREGULAR(BlockObj) returns logical 1 (true) if BlockObj represents a
      %   regular canonical block and logical 0 (false) otherwise.
      
      b = strcmp(obj.type,'regular');
    end
    %-------------------------------

    function [c,d] = compare(obj1,obj2,varargin)
      %COMPARE Compare two block objects.
      %   C = COMPARE(B1,B2) compares the two block objects B1 and B2 of the same
      %   class, and returns 0 if equal, 1 if B1 > B2, and -1 if B1 < B2. The
      %   comparison is done by comparing the sizes of the canonical blocks.
      %   First the largest block from each object are compared then the second
      %   largest until one is larger than the other or no more blocks exist.
      %   Any parameters are ignored.
      %
      %   C = COMPARE(B1,B2,'weyr') uses the Weyr characteristics resulting in
      %   that the comparison is done by comparing the number of blocks. First
      %   the number of all blocks are compared then the number of second
      %   smallest and larger blocks until one quantity is larger than the other.
      %
      %   [C2, D] = COMPARE(B1,B2,Tol) or 
      %   [C2, D] = COMPARE(B1,B2,Tol,'weyr') returns the row vector C2 = [C,P],
      %   where C is the result from above and P is the result from comparing the
      %   parameters (eigenvalues). It uses the tolerance Tol for comparing the
      %   parameters. Parameter p1 from B1 is assumed to be equal to p2 from B2
      %   if abs(p1-p2) <= Tol, larger if p1 > p2, otherwise smaller. For blocks
      %   with no parameter P is always 0. The optional D returns the difference
      %   abs(p1-p2) between the two parameters, or NaN if the blocks have no
      %   parameter.
      %
      %   C = COMPARE(B1,B2,'size') only compares the total sizes of the block
      %   objects. Returns 0 if equal, 1 if S1 > S2, and -1 if S1 < S2.
      %
      %   See also EQ.
      
      narginchk(2,3);
      % Note, the parameter case is handled in the pblock class
      
      if ~strcmp(class(obj1),class(obj2))
        error('MCS:mcsblock:compare:Uncomparable',...
          'The two block objects must be of the same class.');
      end
      
      c = 0;
      if nargin > 2 && ischar(varargin{1})
        switch lower(varargin{1})
          case 'weyr'
            b1 = obj1.weyr;
            b2 = obj2.weyr;
          case 'size'
            % Only need to compare one dimension
            if obj1.size(1) > obj2.size(1)
              c = 1;
            elseif obj1.size(1) < obj2.size(1)
              c = -1;
            end
            return
          otherwise
            error('Additional argument must be ''weyr'' or ''size''.');
        end
      else
        b1 = obj1.segre;
        b2 = obj2.segre;
      end
      
      l1 = length(b1);
      l2 = length(b2);
      i = 1;
      while ~c && i <= l1
        if i > l2
          c = 1;
          break
        else
          if b1(i) > b2(i)
            c = 1;
            break
          elseif b1(i) < b2(i) 
            c = -1;
            break
          end
        end
        i = i+1;
      end
      if ~c && i <= l2
        c = -1;
      end
      d = NaN;
    end
    %-------------------------------
    
    function s = char(obj)
      %CHAR Convert a block object to a string.
      %   S = CHAR(BlockObj) returns the canonical structure information of the
      %   block object BlockObj as a string in canonical block notation.
      %
      %   See also MCSSTRUCT/CHAR.
      
      blk = obj.block2cell('block');

      if length(blk) > 1
        s = [blk{1} sprintf(' + %s',blk{2:length(blk)})];
      else
        s = blk{1};
      end
      
    end
    %-------------------------------

    function disp(obj)
      %DISP Display a canonical block object.
      %   DISP(BlockObj) is called for the canonical block object BlockObj when
      %   the semicolon is not used to terminate a statement. Returns the
      %   canonical form represented as a string.
      
      cmp = isequal(get(0,'FormatSpacing'),'compact');

      % If empty block object
      if isempty(obj)
        fprintf('     Empty %s object\n',class(obj))
        if ~cmp, disp(' '); end
        return
      end

      % Get the default format ('segre', 'segrec', 'weyr', 'sizes', or 'block')
      not = mcsformat;
      
      out = obj.block2cell(not);
      
      switch not
        case {'segre','segrec','weyr','sizes'}
          fmt = ['     %',num2str(length(out{1})),'s %-',...
            num2str(length(out{2}+2)),'s %-',num2str(length(out{3})),'s\n'];
          
          fprintf(fmt,out{1},out{2},out{3});
        case 'block'
          rc = 6;
          fprintf('     ');
          if ~isempty(out)
            fprintf('%s',out{1});
          end
          for i = 2:length(out)
            if rc + length(out{i}) > 65
              rc = 5;
              fprintf(1,'\n     ');
            end
            rc = rc + length(out)+3;
            fprintf(1,' + %s',out{i});
          end
          fprintf(1,'\n');
      end
      
      if ~cmp, disp(' '), end
    end 
    %-------------------------------
    
    function out = block2cell(obj,not)
      %BLOCK2CELL Convert a block object to a cell array of strings.
      %   Out = BLOCK2CELL(BlockObj,Format) returns the canonical structure
      %   information as strings in the cell array Out where Format can be
      %   'block', 'segre', 'segrec', 'weyr', or 'sizes'. If Format is 'block'
      %   (for canonical block), Out is N-by-1, where N is the number of
      %   canonical blocks and Out{k} returns the canonical block structure, k =
      %   1,...,N. Otherwise, Out is 1-by-3, where Out{1,1} returns the label,
      %   Out{1,2} the structure integer partition (as Segre-type, Weyr-type, or
      %   as unordered sizes), and Out{1,3} the total dimension.
      %
      %   If Format is 'segrec' a compact variant of 'segre' is used where
      %   multiple blocks of the same size is written as n*..., where n is the
      %   number of blocks.
      %
      %   See also CHAR.

      [label,p,offset] = blockinfo(obj);
      
      % Create parameter string formated as set by Matlab's format
      if ~isempty(p)
        param = ['(',strtrim(evalc('disp(p)')),')'];
      else
        param = '';
      end
        
      if isempty(obj.blockv)
        if strcmp(not,'block')
          out = {['Empty ',label,'-block']};
        else
          out = {[label,param,' ='], '( Empty )', '(0x0)'};
        end
        return
      end

      switch not
        case 'segrec'
          out = cell(1,3);
          % Block label
          out{1,1} = [label,param,' ='];

          blk = obj.segre;
          bcell = cell(1,max(blk));
          ki = 0;
          for cblk = max(blk):-1:0+offset % Loop through all possible sizes
            nblk = sum(blk==cblk);
            if nblk > 0
              ki = ki+1;
              if nblk > 1
                bcell{ki} = [num2str(nblk) '*' num2str(cblk)];
              else
                bcell{ki} = num2str(cblk);
              end
            end
          end
          out{1,2} = ['(',strjoin(bcell(1:ki),'  '),')'];

          % Total size
          out{1,3} = ['(',num2str(obj.m),'x',num2str(obj.n),')'];
        case {'segre','weyr','sizes'}
          out = cell(1,3);
          % Block label
          out{1,1} = [label,param,' ='];

          % Structure integer partition
          switch not
            case 'segre'
              out{1,2} = ['(',num2str(obj.segre),')'];
            case 'weyr'
              out{1,2} = ['(',num2str(obj.weyr),')'];
            otherwise
              out{1,2} = ['(',num2str(obj.blockv),')'];
          end

          % Total size
          out{1,3} = ['(',num2str(obj.m),'x',num2str(obj.n),')'];
        case 'block'
          out = cell(length(obj.blockv),1);
          blk = obj.segre;
          ki = 0;
          for cblk = max(blk):-1:0+offset % Loop through all possible sizes
            nblk = sum(blk==cblk);
            if nblk > 0
              if nblk > 1
                no = [num2str(nblk) '*'];
              else
                no = '';
              end
              ki = ki+1;
              out{ki} = [no,label,num2str(cblk),param];
            end
          end
          out = out(1:ki);
        otherwise
          error('Unknown notation.');
      end % End case
    end 
    %-------------------------------

    function obj = set(obj,varargin)
      %SET Set the canonical structure information.
      %   SET(BlockObj,'StructInt',Blockv) or
      %   SET(BlockObj,'StructInt',Blockv,Notation) sets the structure integer
      %   partition to Blockv. Notation specify in which notation the structure
      %   integer partition Blockv is represented in. Valid values of Notation
      %   are 'sizes' (default), 'segre', or 'weyr'.
      %
      %   SET(BlockObj,Blockv,Notation) is a compact form which sets the
      %   canonical block structure to the structure integer partition Blockv in
      %   notation Notation, where Notation can be omitted.
      %
      %   See also GET.

      % Call builtin set if OBJ isn't a block object.
      if ~isa(obj, 'mcsblock')
        try
          builtin('set', obj, varargin{:});
        catch aException
          rethrow(aException);
        end
        return;
      end

      narginchk(2,4)
      
      % In which form is set called...
      if ischar(varargin{1})
        if mod(nargin-1,2)
          error('PropertyName and PropertyValue must come in pairs.')
        end
        not = 'sizes';
        blk = [];
        for k = 1:2:(nargin-1)
          prop = varargin{k};
          value = varargin{k+1};
          switch lower(prop)
            case 'structint'
              blk =  value;
            case 'notation'
              not = value;
            otherwise
              error('Invalid property name.')
          end
        end
        if ~isempty(blk)
          obj.blockv = obj.toSizes(blk,not);
        end
      else % Compact form...
        if nargin == 2
          not = 'sizes';
        else
          not = varargin{2};
        end
        obj.blockv = obj.toSizes(varargin{1},not);
      end
      
    end 
    %-------------------------------

    function varargout = get(obj,varargin)
      %GET Get the canonical structure information.
      %   Blockv = GET(BlockObj,'StructInt',Notation) or 
      %   Blockv = GET(BlockObj,Notation) returns the structure integer partition
      %   Blockv. The optional Notation specify in which notation the structure
      %   integer partition Blockv is represented in. Valid values of Notation
      %   are 'sizes' (default), 'segre', or 'weyr'.
      %
      %   See also SET.
      
      % Call builtin get if OBJ isn't a block object.
      if ~isa(obj, 'mcsblock')
        try
          builtin('get', obj, varargin{:});
        catch aException
          rethrow(aException);
        end
        return;
      end

      narginchk(1,3)
      nargoutchk(0,1)
      
      if nargin == 1
        varargout{1} = obj.blockv;
        return
      elseif nargin == 3
        % If second argument is, e.g., 'Parameter' or 'Eigenvalue'
        % return an empty matrix.
        if ~strcmp(varargin{1},'StructInt')
          varargout{1} = [];
          return
        end
      
        switch varargin{2}
          case 'sizes'
            varargout{1} = obj.blockv;
          case 'segre'
            varargout{1} = obj.segre();
          case 'weyr'
            varargout{1} = obj.weyr();
          otherwise
            error('MCS:mcsblock:get:InvalidArguments',['Notation must be either '...
              '''sizes'', ''segre'', or ''weyr''']);
        end
      else % Two arguments
        switch lower(varargin{1})
          case {'structint', 'sizes'}
            varargout{1} = obj.blockv;
          case 'segre'
            varargout{1} = obj.segre();
          case 'weyr'
            varargout{1} = obj.weyr();
          case {'parameter', 'eigenvalue'}
            varargout{1} = [];
          otherwise
            error('MCS:mcsblock:get:InvalidArguments',...
              ['Second argument must be either ''StructInt'', '...
              '''sizes'', ''segre'', or ''weyr''']);
        end
      end
        
    end 
    %------------------------------

    function sizes = sizes(obj)
      %SIZES Canonical block sizes.
      %   SIZES(BlockObj) returns the canonical block sizes unordered (in the
      %   same order as created).
      %
      %   See also SEGRE, WEYR.

      sizes = obj.blockv;
    end 
    %------------------------------

    function segre = segre(obj)
      %SEGRE Segre characteristics.
      %   SEGRE(BlockObj) returns the canonical block sizes represented in Segre
      %   characteristics (non-increasing order).
      %
      %   See also SIZES, WEYR.

      segre = sort(obj.blockv,'descend');
    end 
    %------------------------------
    
    function weyr = weyr(obj)
      %WEYR Weyr characteristics.
      %   WEYR(BlockObj) returns the canonical block sizes represented in Weyr
      %   characteristics.
      %
      %   See also SIZES, SEGRE.
      
      narginchk(1,1);
      nargoutchk(0,1);
      
      switch obj.type
        case {'regular'}
          offset = 0;
        case {'singular'}
          offset = 1;
        otherwise
          error('Unknown type');
      end
      
      segre = obj.segre();
      weyr = [];
      if ~isempty(segre)
        r = segre(1) + offset;
        weyr = ones(1,r);
      end
      for i = 2:length(segre)
        r = segre(i) + offset;
        weyr(1:r) = weyr(1:r) + ones(1,r);
      end
      
    end
    %------------------------------

  end % End methods
  
%% Protected methods
  methods (Access = protected)
    function blk = toSizes(obj,blk,not)
      %TOSIZES Convert block notation to sizes.
      %   Auxiliary function for converting an integer partition to
      %   "sizes".
      
      if ~ischar(not)
        error('MCS:mcsblock:BlockNotation','Notation must be either ''segre'', ''weyr'', or ''sizes''.');
      end
      
      switch not
        case 'weyr'
          % If Weyr, convert blk to Segre
          blk = weyr2segre(blk,obj.type);
        case {'segre','sizes'}
          % If segre or sizes, do nothing
        otherwise
          error('MCS:mcsblock:BlockNotation','Notation must be either ''segre'', ''weyr'', or ''sizes''.');
      end
    end
    %-------------------------------
  end
  
%% Abstract methods
  methods (Abstract, Access = protected)
    [label,param,offset] = blockinfo(obj)
    % [Label,Param,Offset] = BLOCKINFO(BlockObj) returns the label of the
    % block, optional parameter, and offset. Parameter should return a string
    % of any optional parameter printed after the label (otherwise return an
    % empty string), e.g., '(e)' for a Jordan block where e is the eigenvalue.
    % Offset is the smallest possible value of m or n. Typically, 1 for Jordan
    % blocks and 0 for singular blocks.

    
  end % End abstract protected methods

end % End classdef
    