%PBLOCK Create an abstract parametric block object.
%   PBLOCK is an abstract canonical block class representing a parametric
%   block structure.
%
%   The class PBLOCK also provides generic methods for extracting
%   information and modifying the canonical block object.
%
%   See also MCSBLOCK.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

%   Inherited by: JBLOCK, KBLOCK, HBLOCK, GBLOCK, SKBLOCK, SHBLOCK,
%   SGBLOCK.

classdef pblock < mcsblock
  
  properties (Access = protected)
    m = 0;
    n = 0;
    blockv = [];
  end
  
  properties (Abstract, Access = protected)
    ev
  end
  
  properties (Constant, Access = protected)
    type = 'regular';
  end
  
  methods
    % Constructor
    %   This constructor can only be called from a subclass to PBLOCK.
    %
    %   BlockObj = PBLOCK(ParamBlocks) creates an abstract object BlockObj of a
    %   canonical structure block with a parameter, where the vector
    %   ParamBlocks is the structure integer partition representing the sizes
    %   (unordered) of the parametric blocks.
    %
    %   BlockObj = PBLOCK(ParamBlocks, Notation) also specifies the notation
    %   used for ParamBlocks.  Valid notations are:
    %      'segre'   Sizes are ordered in a non-increasing order.
    %      'weyr'    Weyr characteristics.
    %      'sizes'   Sizes may be unordered. (default)
    %
    %   BlockObj = PBLOCK creates an empty parametric block object.
    function obj = pblock(blk,not)
      
      if nargin == 0
        % Return an empty object...
        return
      end
      
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);
    end % End constructor
    %-------------------------------
    
    function b = eq(obj1,obj2)
      %EQ (==) Check if two block objects are equal.
      %   Block1 == Block2 checks if the two canonical block objects are equal.
      %   Note that for two blocks objects to be equal, those parameters must
      %   also be exactly equal.
      
      if ~strcmp(class(obj1),class(obj2))
        b = false; 
        return
      end
      
      jb1 = obj1.segre;
      jb2 = obj2.segre;
      ev1 = obj1.ev;
      ev2 = obj2.ev;
      
      if isnan(ev1)
        b = isnan(ev2);
      elseif isinf(ev1)
        b = isinf(ev2);
      else
        b = ev1 == ev2;
      end
      
      b = b && (length(jb1) == length(jb2));
      if b && ~isempty(jb1)
        b = all(jb1==jb2);
      end
    end
    %-------------------------------

    function b = ne(obj1,obj2)
      %NE (~=) Not equal relation between two block objects.
      %   Block1 ~= Block2 checks if the two canonical block objects are not
      %   equal.
      
      b = ~eq(obj1,obj2);
    end
    %-------------------------------
    
    function [c,d] = compare(obj1,obj2,varargin)
      %COMPARE Compare two parameter block objects.
      %   C = COMPARE(B1,B2) compares the two block objects B1 and B2 of the same
      %   class, and returns 0 if equal, 1 if B1 > B2, and -1 if B1 < B2. The
      %   comparison is done by comparing the sizes of the canonical blocks.
      %   First the largest block from each object are compared then the second
      %   largest until one is larger than the other or no more blocks exist.
      %   Any parameters are ignored.
      %
      %   C = COMPARE(B1,B2,'weyr') uses the Weyr characteristics resulting in
      %   that the comparison is done by comparing the number of blocks. First
      %   the number of all blocks are compared then the number of second
      %   smallest and larger blocks until one quantity is larger than the other.
      %
      %   [C2, D] = COMPARE(B1,B2,Tol) or 
      %   [C2, D] = COMPARE(B1,B2,Tol,'weyr') returns the row vector C2 = [C,P],
      %   where C is the result from above and P is the result from comparing the
      %   parameters (eigenvalues). It uses the tolerance Tol for comparing the
      %   parameters. Parameter p1 from B1 is assumed to be equal to p2 from B2
      %   if abs(p1-p2) <= Tol, larger if p1 > p2, otherwise smaller. For blocks
      %   with no parameter P is always 0. The optional D returns the difference
      %   abs(p1-p2) between the two parameters, or NaN if the blocks have no
      %   parameter.
      %
      %   C = COMPARE(B1,B2,'size') only compares the total sizes of the block
      %   objects. Returns 0 if equal, 1 if S1 > S2, and -1 if S1 < S2.
      %
      %   See also EQ.
      
      narginchk(2,4);
      
      if ~strcmp(class(obj1),class(obj2))
        error('MCS:mcsblock:compare:Uncomparable',...
          'The two block objects must be of the same class.');
      end
      
      if nargin == 2
        c = compare@mcsblock(obj1,obj2);
      elseif nargin == 3 && ~isnumeric(varargin{1})
        c = compare@mcsblock(obj1,obj2,varargin{1});
      else
        if isnumeric(varargin{1})
          s = compare@mcsblock(obj1,obj2);
          tol = varargin{1};
        else
          s = compare@mcsblock(obj1,obj2,varargin{1});
          tol = varargin{2};
        end
        if (isnan(obj1.ev) && isnan(obj2.ev)) || (isinf(obj1.ev) && isinf(obj2.ev))
          p = 0;
          d = 0;
        elseif isnan(obj1.ev) || isnan(obj2.ev)
          p = 1;
          d = NaN;
        elseif abs(obj1.ev - obj2.ev) <= tol
          p = 0;
          d = abs(obj1.ev - obj2.ev);
        elseif obj1.ev > obj2.ev
          p = 1;
          d = abs(obj1.ev - obj2.ev);
        else
          p = -1;
          d = abs(obj1.ev - obj2.ev);
        end
        c = [s p];
      end
        
    end
    %-------------------------------
    
    function obj = set(obj,varargin)
      %SET Set the canonical structure information.
      %   SET(BlockObj,'PropertyName',PropertyValue) sets the value of the
      %   specified property for the canonical block object BlockObj.
      %   SET(BlockObj,'PropertyName1',PropertyValue1,'PropertyName2',PropertyValue2,...)
      %   sets multiple property values with a single statement.
      %
      %   PropertyName can be 'StructInt', 'Eigenvalue', or 'Parameter'.
      %   'StructInt' sets the structure integer partition, where PropertyValue
      %   is a vector of the block sizes. Either 'Eigenvalue' or 'Parameter' can
      %   be specified to set the associated parameter (eigenvalue).
      %
      %   SET(BlockObj,'StructInt',PropertyValue,...,Notation) also specifies in
      %   which notation the structure integer partition is represented in. Valid
      %   values of Notation are 'sizes' (default), 'segre', or 'weyr'.
      %   Specifying notation only make sense when 'StructInt' is also set.
      %
      %   SET(BlockObj,Blockv,P,Notation) is a compact form which sets the
      %   canonical block structure to the structure integer partition Blockv in
      %   notation Notation with the associated parameter P. P and Notation can
      %   be omitted.
      %
      %   See also GET.
      
      % Call builtin set if OBJ isn't a Jordan block object.
      if ~isa(obj, 'pblock')
        try
          builtin('set', obj, varargin{:});
        catch aException
          rethrow(aException);
        end
        return;
      end
      
      % In which form is set called...
      if ischar(varargin{1})
        arg = {};
        if mod(nargin-1,2)
          % Notation is specified
          if ~ischar(varargin{end})
            error('MCS:pblock:set:Notation',['Notation must be either '...
              '''sizes'', ''segre'', or ''weyr''']);
          else
            arg(1) = varargin{end};
          end
        end
        for k = 1:2:(nargin-1)
          prop = lower(varargin{k});
          value = varargin{k+1};
          switch prop
            case 'structint'
              arg = [{prop, value} arg];
            case {'eigenvalue', 'parameter'}
              obj.ev = value;
            otherwise
              error('MCS:pblock:set:InvalidProperty','Invalid property name.')
          end
        end
        if length(arg) > 1
          obj = set@mcsblock(obj,arg{:});
        end
      else % Compact form
        if nargin == 4
          obj = set@mcsblock(obj,varargin{1},varargin{3});
        else
          obj = set@mcsblock(obj,varargin{1});
        end
        if nargin > 2 && ~isempty(varargin{2})
          obj.ev = varargin{2};
        end
      end
    end
    %-------------------------------
    
    function varargout = get(obj,varargin)
      %GET Get the canonical structure information.
      %   V = GET(BlockObj,PropertyName) returns the value of the specified
      %   property for the canonical block object BlockObj. PropertyName can be
      %   'StructInt', 'Eigenvalue', or 'Parameter'. 'StructInt' returns the
      %   structure integer partition in Sizes notation, and 'Eigenvalue' or
      %   'Parameter' returns the associated parameter (eigenvalue).
      %
      %   Blockv = GET(BlockObj,'StructInt',Notation) also specifies the notation
      %   of the returned vector Blockv. Valid values of Notation are 'segre',
      %   'weyr', and 'sizes' (default).
      %
      %   [Blockv,P] = GET(BlockObj,Notation) returns the structure integer
      %   partition with notation Notation in the vector Blockv, and the
      %   associated parameter in P. If Notation is omitted, 'sizes' is assumed.
      %
      %   See also SET.
      
      % Call builtin get if OBJ isn't a Jordan block object.
      % Ex. get(s, s);
      if ~isa(obj, 'pblock')
        try
          builtin('get', obj, varargin{:});
        catch aException
          rethrow(aException);
        end
        return;
      end
      
      narginchk(1,3)
      nargoutchk(0,2)
      
      % If no arguments
      if nargin == 1
        varargout{1} = get@mcsblock(obj);
        varargout{2} = obj.ev;
        return
      end
      
      switch lower(varargin{1})
        case 'structint'
          varargout{1} = get@mcsblock(obj,varargin{:});
        case {'eigenvalue', 'parameter'}
          varargout{1} = obj.ev;
        otherwise
          varargout{1} = get@mcsblock(obj,varargin{:});
          varargout{2} = obj.ev;
      end
    end
    %-------------------------------
    
    function obj = set.blockv(obj,blk)
      % Check that block sizes are valid.
      
      if isempty(blk)
        obj.blockv = [];
        return
      end
      
      if ~isnumeric(blk) || ~isvector(blk)
        error('MCS:pblock:StructInt','The structure integer partition must be a vector.');
      elseif size(blk,1) > 1
        blk = blk';
      end
      
      if ~isreal(blk) || any(rem(blk,1) ~= 0) || min(blk) < 1 
        error('MCS:pblock:BlockSize','The block sizes must be integer valued and greater than zero.');
      end
      obj.blockv = blk;
      obj = obj.updateSize;
    end
    
  end % End public methods
  
  methods (Access = protected)
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.
      
      obj.m = sum(obj.blockv);
      obj.n = obj.m;
    end
  end
  
  methods (Abstract, Access = protected)
    [label,param,offset] = blockinfo(obj)
  end % End protected methods
end % End classdef
