%RSBLOCK Create a right singular block object.
%   RSBLOCK creates a canonical block object representing a right singular
%   block structure.
%
%   Each n-by-(n+1) right singular block is defined as:
%                      |0 1   0|     |1 0   0|
%               L_n := |  . .  |  - s|  . .  |
%                      |0   0 1|     |0   1 0|
%
%   BlockObj = RSBLOCK(RSBlocks) returns a new canonical block object
%   BlockObj representing a structure of right singular blocks (L blocks).
%   The right singular blocks are defined by the column (right) minimal
%   indices (epsilon_n, ..., epsilon_1), and the vector RSBlocks =
%   [epsilon_n, ..., epsilon_1] is the structure integer partition
%   representing the (unordered) sizes (epsilon_k)-by-(epsilon_k+1) of the
%   blocks.
%
%   BlockObj = RSBLOCK(RSBlocks, Notation) also specifies the notation used
%   for RSBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = RSBLOCK returns an empty right singular block object.
%
%
%   The class RSBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   RSBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   RSBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also LSBLOCK, PSTRUCT, SSSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef rsblock < sblock
  
  methods
    % Constructor
    function obj = rsblock(blk,not)

      if nargin == 0
        % Return an empty object...
        return
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Check block sizes and compute total size
      obj.blockv = obj.toSizes(blk,not);
    end % End constructor
    %-------------------------------
    
    function varargout = bcf(obj,not)
      %BCF Brunovsky canonical form of the block object.
      %   [A,B] = BCF(BlockObj) returns the matrix pair (A,B) of the system
      %   pencil S-sT = [A B] - s[I 0] in Brunovsky Canonical Form (BCF)
      %   specified by the right singular block object BlockObj. Consequently,
      %   the pencil S-sT consists only of right singular blocks (L blocks).
      %
      %   [A,B,C,D] = BCF(BlockObj) also returns the empty matrices C and D of
      %   the system pencil S-sT = [A B; C D] - s[I 0; 0 0].
      %
      %   ... = BCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also KCF, SSSTRUCT/BCF.

      if nargin == 2 && strcmp(not,'segre')
        colInd = sort(obj.blockv(obj.blockv>0),'descend');
      else
        colInd = obj.blockv(obj.blockv>0);
      end
      
      ciLength = length(colInd);
      rsLength = length(obj.blockv);
      
      A = zeros(obj.m,obj.m);
      B = zeros(obj.m,rsLength);
      p = 1;
      for i = 1:ciLength
        % Matrix A
        r = colInd(i);
        A(p:p+r-1,p:p+r-1) = diag(ones(r-1,1),1);
        % Matrix B
        B(p+r-1,i) = 1;
        p = p + r;
      end
      
      if nargout == 2
        varargout{1} = A;
        varargout{2} = B;
      elseif nargout == 4
        varargout{1} = A;
        varargout{2} = B;
        varargout{3} = zeros(0,obj.m);
        varargout{4} = zeros(0,rsLength);
      else
        error('MCS:rsblock:bcf:InvalidArguments','Invalid number of output arguments.')
      end        
    end
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the matrix pencil G-sH in Kronecker
      %   Canonical Form (KCF) specified by the right singular block object
      %   BlockObj. The pencil G-sH only consists of right singular blocks (L
      %   blocks).
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also PSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1; q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        G(p:p+r-1,q+1:q+r) = eye(r);
        H(p:p+r-1,q:q+r-1) = eye(r);
        p = p + r;
        q = q + r+1;
      end
    end
    %-------------------------------
    
  end % End public methods

%% Protected methods
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block and offset, which is the
      % smallest possible value of m or n. Typically, 1 for regular blocks and 0
      % for singular blocks.

      label = 'R'; % Label that will be used when display is called.
      param = [];
      offset = 0; % Set offset to the smallest possible value of m or n.
                  % (1 for regular blocks and 0 for singular blocks)
    end
    %-------------------------------
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = sum(obj.blockv);
      obj.n = sum(obj.blockv) + length(obj.blockv);
    end
  end % End protected methods 
  
%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct})
    function jblocktype = getSGBlock(~)
      %GETSGBLOCK Get the StratiGraph block type.
      jblocktype = se.umu.stratigraph.core.structure.BlockType.R;
    end
    
    function robj = tojava(obj)
      %TOJAVA Convert a right singular block object to a Java representation.
      %   RObj = TOJAVA(BlockObj) returns a Java object RObj representing
      %   the same structure as the canonical block object BlockObj.

      try
        ct = obj.weyr();
        robj = se.umu.stratigraph.core.structure.BlockParameter(...
          se.umu.stratigraph.core.structure.BlockType.R, ct);
      catch e
        error('MCS:rsblock:BlockNotFound',...
          'StratiGraph is not present or the requested block is not found in StratiGraph.')
      end

    end
    %-------------------------------
  end  
end % End classdef
