%SGBLOCK Create a *Gamma block object.
%   SGBLOCK creates a canonical block object representing a *Gamma block
%   structure.
%
%   Each n-by-n *Gamma block associated with a complex parameter mu is
%   defined as:
%                             |0         ...|
%                             |        ...  |
%           *Gamma_n(mu) := mu|      1  1   |
%                             |  -1 -1      |
%                             |1  1        0|
%   where |mu| = 1.
%
%   BlockObj = SGBLOCK(SGBlocks, P) returns a new canonical block object
%   BlockObj representing a structure of *Gamma blocks. The vector SGBlocks
%   = [h_n, ..., h_1] is the structure integer partition representing the
%   (unordered) sizes (h_k)-by-(h_k) of the blocks and P is the associated
%   complex parameter. The parameter P must be on the complex unit circle,
%   i.e., abs(P) = 1.
%
%   BlockObj = SGBLOCK(SGBlocks, P, Notation) also specifies the notation
%   used for SGBlocks.  Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = SGBLOCK returns an empty *Gamma block object.
%
%
%   The class SGBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   SGBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     CCF        - Congruence canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   SGBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also GBLOCK, SCMSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef sgblock < pblock
  
  properties (Access = protected)
    ev = NaN;
  end
  
  methods
    % Constructor
    function obj = sgblock(blk,ev,not)
      
      if nargin == 0
        blk = [];
      elseif nargin < 2
        error('Must specify the parameter.')
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@pblock(blk,not);
      
      % Set eigenvalue
      if nargin >= 2
        obj.ev = ev;
      end

    end % End constructor
    
    %-------------------------------

    function [H] = ccf(obj,not)
      %CCF Congruence canonical form of the block object.
      %   H = CCF(BlockObj) returns the matrix H in Congruence Canonical Form
      %   (CCF) specified by the *Gamma block object BlockObj. The matrix H only
      %   consists of *Gamma blocks.
      %
      %   ... = CCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SCMSTRUCT/CCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      H = zeros(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
         r = blocks(i);
         Z = zeros(r,r);
         for j = 1:r
           Z(r-j+1,j)=(-1)^(j-1);
         end
         H(p:p+r-1,p:p+r-1) = Z*(diag(ones(r,1)) + diag(ones(r-1,1),1))*obj.ev;
         p = p + r;
      end
      
    end
    %-------------------------------
    
    function obj = set.ev(obj,value)
      % Check that the parameter is valid.
      if ~isnumeric(value) || ~isscalar(value) || isinf(value)
        error('MCS:sgblock:Eigenvalue','The eigenvalue must be a finite scalar.');
      end
      
      if  (abs(value) < 1-2*eps) || (abs(value) > 1+2*eps)     
         error('MCS:sgblock:Parameter','Complex modulus of the parameter must be equal to one.')
      end
      
      obj.ev = value;
    end

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO returns the label of the block, its eigenvalue, and
      % offset. For regular blocks offset is 1.
      
      label = 'SG';
      param = obj.ev;
      offset = 0;
    end
  end % End protected methods 

end

