%SHBLOCK Create an SH block object.
%   SHBLOCK creates a canonical block object representing a skew-symmetric
%   SH block structure associated with a finite or unspecified eigenvalue.
%
%   Each 2n-by-2n skew-symmetric SH block is defined as:
%                |  0        J_n(mu)|     |  0    I_n|
%        SH_n := |                  |  - s|          |,      
%                |-J_n(mu)^T    0   |     |-I_n    0 |
%   where J_n(mu) is an n-by-n Jordan block associated with the eigenvalue
%   mu.
%
%   BlockObj = SHBLOCK(SHBlocks, Ev) returns a new canonical block object
%   BlockObj representing a structure of skew-symmetric SH blocks (pairs of
%   Jordan blocks) associated with a finite or unspecified eigenvalue. The
%   vector SHBlocks = [h_n, ..., h_1] is the structure integer partition
%   representing the (unordered) sizes (2*h_k)-by-(2*h_k) of the blocks and
%   Ev is the associated eigenvalue. Ev can be a complex number or NaN
%   (unspecified eigenvalue). If Ev is omitted, the eigenvalue is assumed
%   to be unspecified.
%
%   BlockObj = SHBLOCK(SHBlocks, Ev, Notation) also specifies the notation
%   used for SHBlocks.  Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   BlockObj = SHBLOCK returns an empty SH block object.
%
%
%   The class SHBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   SHBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker-like canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   SHBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also HBLOCK, SSPSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef shblock < pblock
    
  
  properties (Access = protected)
    ev = NaN;
  end
  
  methods
    % Constructor
    function obj = shblock(blk,ev,not)
      
      if nargin == 0
        blk = [];
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@pblock(blk,not);
      
      % Set eigenvalue
      if nargin >= 2
        obj.ev = ev;
      end

    end % End constructor
    %-------------------------------

    function [G,H] = kcf(obj,not)
      %KCF Kronecker-like canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the square skew-symmetric matrix pencil
      %   G-sH in Kronecker-like canonical form specified by the SH block object
      %   BlockObj.
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SSPSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1;
      q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        % By using diag, kcf can also handle unspecified eigenvalues (NaN)
        W = diag(ones(r,1)*obj.ev) + diag(ones(r-1,1),1);
        G(p:p+r-1,r+q:r+q+r-1) = W;
        G(r+p:r+p+r-1,q:q+r-1) = -W';
        H(p:p+r-1,r+q:r+q+r-1) = eye(r);
        H(r+p:r+p+r-1,q:q+r-1) = -eye(r);
        p = p + 2*r;
        q = q + 2*r;
      end
    end
    %-------------------------------
    
    function obj = set.ev(obj,value)
      % Check that the eigenvalue is valid.
      
      if isempty(value)
        obj.ev = NaN;
        return
      end
      
      % Check that the eigenvalue is valid.
      if ~isnumeric(value) || ~isscalar(value) || isinf(value)
        error('MCS:shblock:Eigenvalue','The eigenvalue must be a finite scalar or NaN.');
      end
      obj.ev = value;
    end

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO Returns the label of the block, its eigenvalue, and
      % offset. For regular blocks offset is 1.
      
      label = 'SH';
      param = obj.ev;
      offset = 1;
    end
       
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = 2*sum(obj.blockv);
      obj.n = obj.m;
    end

  end % End protected methods 

end

