%SKBLOCK Create an SK block object associated with the infinite eigenvalue.
%   SKBLOCK creates a canonical block object representing a skew-symmetric
%   SK block structure associated with the infinite eigenvalue.
%
%   Each 2n-by-2n skew-symmetric SK block is defined as:
%                |  0    I_n|     |  0        J_n(0)|
%        SK_n := |          |  - s|                 |,
%                |-I_n    0 |     |-J_n(0)^T    0   |
%   where J_n(0) is an n-by-n Jordan block associated with the zero
%   eigenvalue.
%
%   BlockObj = SKBLOCK(SKBlocks) returns a new canonical block object
%   BlockObj representing a structure of skew-symmetric SK blocks (pairs of
%   Jordan blocks) associated with the infinite eigenvalue. The vector
%   SKBlocks = [s_n, ..., s_1] is the structure integer partition
%   representing the (unordered) sizes (2*s_k)-by-(2*s_k) of the blocks.
%
%   BlockObj = SKBLOCK(SKBlocks, Notation) also specifies the
%   notation used for SKBlocks.  Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   BlockObj = SKBLOCK returns an empty SK block object.
%
%
%   The class SKBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   SKBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     KCF        - Kronecker-like canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   SKBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also KBLOCK, SSPSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef skblock < pblock
  
  properties (Access = protected)
    ev = inf;
  end
  
  methods
    % Constructor
    function obj = skblock(blk,not)

      if nargin == 0
        blk = [];
      end

      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Initialize superclass
      obj = obj@pblock(blk,not);
    end % End constructor
    %-------------------------------
    
    function [G,H] = kcf(obj,not)
      %KCF Kronecker-like canonical form of the block object.
      %   [G,H] = KCF(BlockObj) returns the matrix pencil G-sH in Kronecker-like
      %   canonical form specified by the SK block object BlockObj associated
      %   with the infinite eigenvalue.
      %
      %   ... = KCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also SSPSTRUCT/KCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      G = zeros(obj.m,obj.n);
      H = zeros(obj.m,obj.n);
      p = 1; q = 1;
      for i = 1:length(blocks)
        r = blocks(i);
        G(p:p+r-1,r+q:r+q+r-1)   = eye(r);
        G(r+p:r+p+r-1,q:q+r-1)   = -eye(r);
        H(p:p+r-2,r+q+1:r+q+r-1) = eye(r-1);
        H(r+p+1:r+p+r-1,q:q+r-2) = -eye(r-1);
        p = p + 2*r;
        q = q + 2*r;
      end
    end
    %-------------------------------

    function obj = set.ev(obj,value)
      % If the eigenvalue is changed, return an error.
      error('MCS:skblock:Eigenvalue','Not allowed to change the eigenvalue of an ''skblock'' object.')
    end
    %-------------------------------

  end % End public methods

  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO Returns the label of the block, its eigenvalue, and offset,
      % which is the smallest possible value of m or n. For regular blocks offset
      % is 1.

      label = 'SK';
      param = [];
      offset = 1;
    end
    
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = 2*sum(obj.blockv);
      obj.n = obj.m;
    end

  end % End protected methods 
  
end % End classdef
