%SWBLOCK Create a *W block object.
%   SWBLOCK creates a canonical block object representing a *W block
%   structure associated with a specified and admissible eigenvalue.
%
%   Each 2n-by-2n *W block associated with a finite eigenvalue mu is
%   defined as:
%                           |  0     I_n|
%               *W_n(mu) := |           |
%                           |J_n(mu)  0 |
%   where |mu| > 1 and J_n(mu) is an n-by-n Jordan block with the
%   associated eigenvalue mu.
%
%   BlockObj = SWBLOCK(SWBlocks, Ev) returns a new canonical block object
%   BlockObj representing a structure of *W blocks associated with finite
%   Jordan blocks. The vector SWBlocks = [h_n, ..., h_1] is the structure
%   integer partition representing the (unordered) sizes (2*h_k)-by-(2*h_k)
%   of the blocks and Ev is the associated eigenvalue. Ev must be a complex
%   scalar with its complex modulus greater than one, i.e., abs(Ev) > 1.
%
%   BlockObj = SWBLOCK(SWBlocks, Ev, Notation) also specifies the notation
%   used for SWBlocks.  Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   BlockObj = SWBLOCK returns an empty *W block object.
%
%
%   The class SWBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   SWBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     CCF        - Congruence canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   SWBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also WBLOCK, SCMSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef swblock < wblock  
  
  methods
    % Constructor
    function obj = swblock(blk,ev,not)
      
      if nargin == 0
        blk = [];
      elseif nargin < 2
        error('Must specify the eigenvalue.')
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@wblock(blk,ev,not);
      
    end % End constructor
    %-------------------------------
    

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO Returns the label of the block, its eigenvalue, and offset,
      % which is the smallest possible value of m or n. For regular blocks offset
      % is 1.
      
      label = 'SW';
      param = obj.ev;
      offset = 1;
    end
  end

end

