%WBLOCK Create a W block object.
%   WBLOCK creates a canonical block object representing a W block
%   structure associated with a specified and admissible eigenvalue.
%
%   Each 2n-by-2n W block associated with a finite eigenvalue mu is defined
%   as:
%                      |  0    I_n|
%               W_n := |          |
%                      |J_n(mu) 0 |
%   where mu ~= {0, (-1)^(n+1)} and J_n(mu) is an n-by-n Jordan block with
%   the associated eigenvalue mu.
%
%   BlockObj = WBLOCK(WBlocks, Ev) returns a new canonical block object
%   BlockObj representing a structure of W blocks associated with finite
%   Jordan blocks. The vector WBlocks = [h_n, ..., h_1] is the structure
%   integer partition representing the (unordered) sizes (2*h_k)-by-(2*h_k)
%   of the blocks and Ev is the associated eigenvalue. Ev must be a
%   non-zero complex scalar and not equal to (-1)^('size of the block'/2 +
%   1) for all blocks.
%
%   BlockObj = WBLOCK(WBlocks, Ev, Notation) also specifies the notation
%   used for WBlocks.  Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   BlockObj = WBLOCK returns an empty W block object.
%
%
%   The class WBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   WBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     CCF        - Congruence canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   WBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also SWBLOCK, CMSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef wblock < pblock
  
  properties (Access = protected)
    ev = NaN;
  end
  
  methods
    % Constructor
    function obj = wblock(blk,ev,not)
      
      if nargin == 0
        blk = [];
      elseif nargin < 2
        error('Must specify the eigenvalue.')
      end
      
      % If notation is not specified asume unordered sizes
      if nargin < 3
        not = 'sizes';
      end

      % Initialize superclass
      obj = obj@pblock(blk,not);
      
      % Set eigenvalue
      if nargin >= 2
        obj.ev = ev;
      end

    end % End constructor

    %-------------------------------

    function [H] = ccf(obj,not)
      %CCF Congruence canonical form of the block object.
      %   H = CCF(BlockObj) returns the square matrix H in Congruence Canonical
      %   Form (CCF) specified by the W block object BlockObj.
      %
      %   ... = CCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also ZJBLOCK/CCF, CMSTRUCT/CCF.
      
      if nargin == 2 && strcmp(not,'segre')
        blocks = sort(obj.blockv,'descend');
      else
        blocks = obj.blockv;
      end
      
      H = zeros(obj.m,obj.n);
      p = 1;
      for i = 1:length(blocks)
        r = blocks(i); 
        H(p:p+r-1,r+p:r+p+r-1) = eye(r);
        H(r+p:r+p+r-1,p:p+r-1) = diag(ones(r,1)*obj.ev) + diag(ones(r-1,1),1);
        p = p + 2*r;
      end
    end
    %-------------------------------
    
    function obj = set.ev(obj,value)
      % Check that the eigenvalue is valid.

      if ~isnumeric(value) || ~isscalar(value) || isinf(value)
        error('MCS:wblock:Parameter','The parameter must be a finite scalar.');
      end
      
      if isa(obj,'swblock') 
        if abs(value) <= 1
          error('MCS:swblock:Parameter',...
            'Complex modulus of the parameter must be greater than one.')
        end
      else
        for i = 1:length(obj.blockv)
          r = obj.blockv(i);
          if ((abs(value) > -2*eps) && (abs(value) < 2*eps)) || ...
              (value > (-1)^(r+1) - 2*eps) && (value < (-1)^(r+1) + 2*eps)
            error('MCS:wblock:Parameter',...
              'The parameter must be non-zero and not equal to (-1)^(''size of the block''/2 + 1) for all blocks.')
          end
        end
      end
      
      obj.ev = value;
    end

  end
  
  methods (Access = protected)
    function [label,param,offset] = blockinfo(obj)
      % BLOCKINFO Returns the label of the block, its eigenvalue, and
      % offset, which is the smallest possible value of m or n. For regular
      % blocks offset is 1.
      
      label = 'W';
      param = obj.ev;
      offset = 1;
    end
       
    function obj = updateSize(obj)
      %UPDATESIZE Update the size of the block object.

      obj.m = 2*sum(obj.blockv);
      obj.n = obj.m;
    end

  end % End protected methods 

end

