%ZJBLOCK Create a Jordan block object associated with the zero eigenvalue.
%   ZJBLOCK creates a canonical block object representing a Jordan block
%   structure associated with the zero eigenvalue.  
%
%   Each n-by-n Jordan block associated with the zero eigenvalue is defined
%   as:
%                          |0 1   0|
%                J_n(0) := |  0 .  |
%                          |    . 1|
%                          |0     0|
%
%   BlockObj = ZJBLOCK(ZJordanBlocks) returns a new canonical block object
%   BlockObj representing a structure of Jordan blocks associated with the
%   zero eigenvalue. The vector ZJordanBlocks = [s_n, ..., s_1] is the
%   structure integer partition representing the (unordered) sizes
%   (s_k)-by-(s_k) of the blocks.
%
%   BlockObj = ZJBLOCK(ZJordanBlocks, Arg) also specifies the notation Arg
%   used for ZJordanBlocks.  Valid notation arguments are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   BlockObj = ZJBLOCK returns an empty Jordan block object.
%
%
%   The class ZJBLOCK provides the following methods for extracting
%   information and modifying the canonical block object.
%
%   ZJBLOCK Methods:
%     SIZE       - Total size of the represented canonical blocks.
%     NUMBLK     - Number of canonical blocks.
%     COPY       - Return a copy of the block object.
%     COMPARE    - Compare two block objects.
%     ISEMPTY    - True for empty canonical block object.
%     ISSINGULAR - True for singular canonical block object.
%     ISREGULAR  - True for regular canonical block object.
%     SET        - Set the canonical structure information.
%     GET        - Get the canonical structure information.
%     SIZES      - Canonical block sizes.
%     SEGRE      - Segre characteristics.
%     WEYR       - Weyr characteristics.
%     JCF        - Jordan canonical form of the block object.
%     CCF        - Congruence canonical form of the block object.
%     KCF        - Kronecker canonical form of the block object.
%     BCF        - Brunovsky canonical form of the block object.
%     CHAR       - Convert a block object to a string.
%     BLOCK2CELL - Convert a block object to a cell array of strings.
%
%   ZJBLOCK Operators:
%     ==, ~= (EQ, NE) - Check if two block objects are equal.
%
%   See also FJBLOCK, IJBLOCK, CMSTRUCT, SCMSTRUCT, MSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef zjblock < fjblock
  
  methods
    % Constructor
    function obj = zjblock(blk,not)

      if nargin == 0
        blk = [];
      end

      % If notation is not specified asume unordered sizes
      if nargin < 2
        not = 'sizes';
      end
      
      % Initialize superclass
      obj = obj@fjblock(blk,0,not);
    end % End constructor
     
    %-------------------------------
    
    function H = ccf(obj,not)
      %CCF Congruence canonical form of the block object.
      %   H = CCF(BlockObj) returns the matrix H in the Congruence Canonical Form
      %   (CCF) specified by the Jordan block object BlockObj associated with the
      %   zero eigenvalue.
      %
      %   ... = CCF(BlockObj,'segre') sorts the blocks in non-increasing block
      %   size order. By default are the blocks presented in the order they were
      %   created.
      %
      %   See also JCF, CMSTRUCT/CCF, SCMSTRUCT/CCF.
      
      if nargin == 1
        not = 'sizes';
      end
      H = kcf(obj,not);
    end
    %-------------------------------

  end % End public methods

end % End classdef
