function [T] = cmtanspace(A)
%CMTANSPACE Tangent space of the congruence orbit of a matrix.
%   T = CMTANSPACE(A) returns the matrix representation 
%
%     T = kron(A.',In) + kron(In, A)*P
%
%   of the tangent space to the congruence orbit(A) at A, where A is an
%   n-by-n matrix, I_n is the n-by-n identity matrix, and P is the
%   (n^2)-by-(n^2) permutation matrix that can "transpose" n-by-n matrices,
%   i.e., vec(X')=P*vec(X) for any n-by-n matrix X. The tangent space is
%   the range of the (n^2)-by-(n^2) matrix T.
%
%   The tangent space consists of the matrices of the form
%      T_A = X'*A + A*X, 
%   where X is an n-by-n matrix. Equivalently, the tangent vectors T_A can
%   be represented as
%     vec(T_A) = (kron(A.',In)+kron(In, A)*P) vec(X).
%
%   See also CMCODIM, MTANSPACE, SCMTANSPACE.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

  narginchk(1,1)
  nargoutchk(0,1)
  [n,m]=size(A);
  if n ~= m 
      error('MCS:cmtanspace','The input matrix must be square.')
  else
      In = eye(n);
      X=zeros((n)^2,(n)^2);
      for t=1:(n)
         for l=1:(n)
            X(l+n*(t-1),n*(l-1)+t)=1;
         end
      end
      T = [kron(A.',In)+kron(In, A)*X];
  end