function varargout = mcsevclmth(mthd,persist)
%MCSEVCLMTH Default method used to cluster eigenvalues in MCS Toolbox.
%   CLMethod = MCSEVCLMTH returns the name of the used method to cluster
%   eigenvalues in the toolbox as a string. If no default method is previously
%   set, the 'var' method is returned (see below). The default method can
%   usually be overriden by a parameter to the individual functions.
%
%   [CLMethod,IsPersistant] = MCSEVCLMTH also returns if the used method is
%   persistent between sessions or not.
%
%   MCSEVCLMTH(Method) sets the default method to cluster eigenvalues.
%   Valid methods are the following:
%     'norm'   - Use an "over-simple" method where the tolerance
%                parameter is used to separate different clusters. 
%                See also PRIVATE/CENORM.
%     'var', 'min', 'max', 'avg', 'mean'
%              - Use a hierarchical clustering algorithm where the
%                clustering is determined by a distance function.
%                See also PRIVATE/CEHIERARCHY.
%     'gersh'  - Use a method based on Gershgorin circles to cluster
%                the eigenvalues. 
%                See also PRIVATE/MCEGERSHGORIN, PRIVATE/PCEGERSHGORIN.
%
%   MCSEVCLMTH(Method,Persistent) also set if the used cluster method shall be
%   persistent between sessions (Persistent = true) or not (Persistent = false).
%   Default is false.
%
%   See also PGUPTRI, MGUPTRI, PCLUSTER, MCLUSTER.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

global MCS_PREF;

if nargin > 0 && (~ischar(mthd) || ...
    ~any(strcmpi(mthd,{'norm','gersh','min','max','avg','mean','var'})))
  error('Method must be one of: ''norm'', ''min'', ''max'', ''avg'', ''mean'', ''var'', or ''gersh''.')
end

if nargin > 0
  mthd = lower(mthd);
  nargoutchk(0,0);
  
  if nargin < 2
    persist = 0;
  end
  
  MCS_PREF.evclmth = mthd;
  
  if persist
    if ispref('mcs','evclmth')
      setpref('mcs','evclmth',mthd);
    else
      addpref('mcs','evclmth',mthd);
    end
  end
else
  nargoutchk(0,2);
  
  if isfield(MCS_PREF,'evclmth')
    mthd = MCS_PREF.evclmth;
  elseif ispref('mcs','evclmth')
    mthd = getpref('mcs','evclmth');
  else
    mthd = 'var';
  end
end

if nargin == 0
  varargout{1} = mthd;
end
if nargout > 1
  if ispref('mcs','evclmth')
    ispersistant = ...
      strcmp(getpref('mcs','evclmth'),mthd);
  else
    ispersistant = false;
  end
  varargout{2} = ispersistant;
end


