function varargout = pcluster(varargin)
%PCLUSTER Compute and cluster generalized eigenvalues of a matrix pencil.
%   EIGS = PCLUSTER(G,H) computes and clusters the generalized eigenvalues of
%   the square matrix pencil G-sH. The avarage of the generalized eigenvalues
%   classified to belong to the same cluster is considered as a (numerical)
%   multiple eigenvalue. Eigs is a two-column matrix, where the first column is
%   the generalized eigenvalues of G-sH and the second column is the
%   corresponding cluster sizes. The number of rows in Eigs is consequently the
%   same as the number of distinct (possible multiple) eigenvalues of G-sH. By
%   default the cluster method returned by MCSEVCLMTH is used. Optional
%   parameters to the default cluster method can be appended to the argument
%   list, see below for available parameters.
%
%   [Eig,S] = PCLUSTER(G,H) instead only computes the cluster size of the
%   largest eigenvalue and returns the eigenvalue and cluster size in two
%   separate variables.
%
%   Eigs = PCLUSTER(Method,G,H, ...) or
%   [Eig,S] = PCLUSTER(Method,G,H, ...) also specifies the method used to
%   cluster the eigenvalues with optional extra parameters. For available
%   methods see below.
%
%   ... = PCLUSTER('norm',G,H [,Tol]) uses an "over-simple" method where the
%   eigenvalues that are within the norm tolerance Tol are classified to belong
%   to the same cluster. By default Tol =
%   sqrt(eps)*max(norm(G,'fro'),norm(H,'fro')). 
%   For details see PRIVATE/CENORM.
%
%   ... = PCLUSTER(Method,G,H [,DeltaMax [,Pmin]]) uses a hierarchical
%   clustering algorithm where the clustering is determined by a distance
%   function D that satisfy D <= DeltaMax. Pmin is minimum number of clusters
%   generated. Available distance functions D(X,Y) (specified with Method) are,
%   where X and Y are two different clusters with r and s eigenvalues,
%   respectively:
%      'min'  -  D(X,Y) = min(abs(X(i) - Y(j))), for all i,j
%      'max'  -  D(X,Y) = max(abs(X(i) - Y(j))), for all i,j
%      'avg'  -  D(X,Y) = 1/(r*s) * (sum^{r,s} abs(X(i) - Y(j)))
%      'mean' -  D(X,Y) = abs(sum(X)/r - sum(Y)/s)
%      'var'  -  D(X,Y) = var(X U Y), (variance of X union Y)
%   Default values are DeltaMax =sqrt(eps)*max(norm(G,'fro'),norm(H,'fro')) and
%   Pmin = 1. If the matrix pencil is known to have k multiple eigenvalues,
%   appropriate parameters are DeltaMax = inf and Pmin = k. 
%   For details see PRIVATE/CEHIERARCHY.
%
%   ... = PCLUSTER('gersh',G,H [,SizeE,SizeF [,Goal]]) or
%   ... = PCLUSTER('gersh',G,H [,SizeEF [,Goal]]) uses a method based on
%   Gershgorin circles to cluster the eigenvalues closest to Goal or the
%   largest eigenvalue (default). The function starts with isolating the
%   cluster closest to the point Goal in the complex plane. If the cluster
%   containing the largest eigenvalue is desired to be first then Goal
%   should be set to Inf or omitted. SizeE and SizeF should be estimates of
%   the 1-norm of the errors known to affect G and H, respectively. The
%   error estimates can also be specified as vector SizeEF = [SizeE SizeF].
%   If not specified or empty, SizeE = sqrt(eps)*norm(G,'fro') and SizeF =
%   sqrt(eps)*norm(H,'fro').
%   For details see PRIVATE/PCEGERSHGORIN.
%
%   [Eigs,E,EvCluster] = PCLUSTER(...) also returns all computed eigenvalues E
%   of G-sH and the clustering of the eigenvalues in EvCluster. The integer
%   value k in EvCluster(i) represents the cluster the eigenvalue E(i) belongs
%   to, where Ev(k) is the new eigenvalue for the cluster. The eigenvalues in E
%   are sorted in non-increasing order.
%
%   See also PRIVATE/CENORM, PRIVATE/CEHIERARCHY, PRIVATE/PCEGERSHGORIN,
%   MCLUSTER, MCSEVCLMTH.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.
  
nargoutchk(0,3);
narginchk(2,6);

if nargin == 2 || isnumeric(varargin{1})
  method = mcsevclmth;
  G = varargin{1};
  H = varargin{2};
  karg = 0;
else
  method = lower(varargin{1});
  G = varargin{2};
  H = varargin{3};
  karg = 1;
end

error(sizechk('square',G));
error(sizechk('ndims',G,2));
error(sizechk('equal',G,H));

onlylargest = nargout == 2;

switch method
  case 'norm'
    if nargin > karg+2 && ~isempty(varargin{karg+3})
      tol = varargin{karg+3};
    else
      tol = sqrt(eps)*max(norm(G,'fro'),norm(H,'fro'));
    end
    [ev,s,e,c] = cenorm(G,H,tol,onlylargest);
  case 'gersh'
    switch nargin
      case karg+2
        sizeE = sqrt(eps)*norm(G,'fro');
        sizeF = sqrt(eps)*norm(H,'fro');
        goal  = Inf;
      case karg+3 
        if isempty(varargin{karg+3})
          sizeE = sqrt(eps)*norm(G,'fro');
          sizeF = sqrt(eps)*norm(H,'fro');
        else
          if length(varargin{karg+3}) ~= 2
            error('Both SizeE and SizeF must be specified as separate arguments or as a vector [SizeE, SizeF] for ''gersh''.')
          end
          sizeEF = varargin{karg+3};
          sizeE = sizeEF(1);
          sizeF = sizeEF(2);
        end
        goal = Inf;
      case karg+4
        sizeE = varargin{karg+3};
        sizeF = varargin{karg+4};
        goal = Inf;
      case karg+5
        sizeE = varargin{karg+3};
        sizeF = varargin{karg+4};
        goal  = varargin{karg+5};
    end
    if isempty(sizeE)
      sizeE = sqrt(eps)*norm(G,'fro');
    end
    if isempty(sizeF)
      sizeF = sqrt(eps)*norm(H,'fro');
    end
    [ev,s,e,c] = pcegershgorin(G,H,goal,sizeE,sizeF,onlylargest);
  case {'min','max','avg','mean','var'}
    if onlylargest && nargin == karg+4
      warning('Ignoring ''Pmin'' since only the largest eigenvalue is clustered.')
      Pmin = ~onlylargest;
    elseif onlylargest
      Pmin = ~onlylargest;
    elseif nargin > karg+3
      Pmin = varargin{karg+4};
    else
      Pmin = 1;
    end
    if nargin > karg+2 && ~isempty(varargin{karg+3})
      tol = varargin{karg+3};
    else
      tol = sqrt(eps)*max(norm(G,'fro'),norm(H,'fro'));
    end
    [ev,s,e,c] = cehierarchy(G,H,tol,Pmin,method);
  otherwise
    error(['Unkown cluster method ''' method '''.']);
end

if ~onlylargest
  varargout{1} = [ev,s];
  if nargout == 3
    varargout{2} = e;
    varargout{3} = c;
  end
elseif ~isempty(ev)
  varargout{1} = ev;
  varargout{2} = s;
else
  varargout{1} = [];
  varargout{2} = 0;
end
