function [G,H,varargout] = pggallery(pencilname,pert,varargin)
%PGGALLERY Test matrix pencils for staircase computation and bounds.
%   [G,H] = PGGALLERY(PencilName) returns a matrix pencil G-sH for testing
%   computation routines of the staircase form (canonical structure) and
%   bounds. The string PencilName specifies the matrix pencil (see below).
%
%   [S,T,Eg,Eh] = PGGALLERY(PencilName,Pert) also adds a random
%   perturbation of size Pert to the pencil, where S = G + Eg and T = H +
%   Eh. If Pert is zero or empty, no perturbation is added.
%
%   [G,H,PStr] = PGGALLERY(...) or
%   [S,T,Eg,Eh,PStr] = PGGALLERY(...) returns the correct canonical form
%   for the matrix pencil G-sH as a PSTRUCT object in PStr, where in the
%   descriptions below the block notation is used:
%     Rn     - is an n-by-(n+1) right singular block,
%     Ln     - is an (n+1)-by-n left singular block,
%     Jn(mu) - is an n-by-n Jordan block with eigenvalue mu, and
%     Nn     - is an n-by-n Jordan block with an infinite eigenvalue.
%   A number before the block denotes the number of blocks of the same
%   size and type.
%
%
%   Available test matrix pencils are (specified with PencilName and with
%   optional parameters added as input arguments after Pert):
%
%   'boley1' - A 7x8 matrix pencil already in staircase form that
%     corresponds to state-space system which is controllable but close to
%     an uncontrollable system, for eps=1 at a distance 6e-4 [Boley90,
%     Example 2, p. 639] (see also [EdelmanMa00]). The default value for
%     the optional parameter eps is 1. The canonical form is R7 and for
%     eps=0, R6 + J1(a).
%     
%           [0 1 0 ... ]               [1 -1 ...  -1 7]
%           [  0 1 0   ]               [0 1 -1 .. -1 6] 
%       G = [   . . .  ]  and H(eps) = [   . . .   : :]
%           [     0 1 0]               [       1  -1 2]
%           [       0 1]               [         eps 1]
%
%   'boley2' - A 3x4 matrix pencil from [Boley90, Example 3, p.639] with
%     ill-conditioned eigenvalues. The canonical form is R3.
%
%   'degreg' - A degenerated regular 8x8 matrix pencil constructed from
%     the Kronecker canonical form 
%       J2(6) + 2J1(6) + J3(2) + J1(2).
%
%   'dk_c1', 'dk_c2', 'dk_c3' - Examples from [DemmelKagstrom88, p. 142].
%     The examples have successively more ill-conditioned eigenvalues. The
%     canonical form is R2 + J1(1) + J1(2) for all three cases.
%
%   'em' - Example from [EdelmanMa00, p. 1018]. The default value for the
%     optional parameter d is 1.5e-8. The canonical form is R1 + J2(0).
%     This pencil is sensitive even for for small perturbations (1e-14).
%
%           [0 0 1 0]           [d 0 0 0]
%       G = [0 0 0 1]  and  H = [0 d 0 0]
%           [0 0 0 0]           [0 0 1 0]
%
%   'lmi' - An example of an 11x9 matrix pencil with the canonical form
%     L0 + L8 + N1 originally comming from preprocessing of LMI:s
%     [Helmerson09].
%
%   'ones' - An example provided by [Mason04]:
%      G = ones(55,35) and H = pi*ones(55,35). The exact canoical form is
%      34R0 + 54L0 + J1(e), where e = (1/1925)/(pi/1925).
%
%   'rand' - A 5x7 random matrix pencil with the canonical form 
%     5L1 + 15L2 (the generic pencil). With the two optional arguments m
%     and n the size of the matrix pencil can be specified. If only one
%     argument is provided the matrix pencil will be square and regular,
%     see also 'square_rand'.
%
%   'rand_large' - A 650x503 random matrix pencil with the canonical
%     form 85L3 + 62L4.
%
%   'rand_medium' - A 55x35 random matrix pencil with the canonical
%     form 5L1 + 15L2.
%
%   'rand_square' - A 5x5 square regular random matrix pencil. With the
%     optional n the size of the matrix pencil can be specified.
%
%   'reference' - A 14x12 matrix pencil with all types of canonical blocks
%     constructed from the Kronecker canonical form
%       R1 + R0 + L2 + 3L0 + 2J1(6) + J1(2+5i) + J1(0) + N2 + N1
%
%   'test1' - A 3x4 matrix pencil which is constructed from the exact
%     Kronecker canonical form 
%       R0 + R1 + L0 + J1(1e-4i) 
%     by an equivalence transformation with two perturbed orthogonal
%     matrices [Karlsson15]. The constructed matrices G and H are
%     ill-conditioned, O(cond(G)) = O(cond(H)) = 1e16. PGUPTRI compute the
%     correct canonical structure if the order of the regular blocks are
%     'ifz', 'izf', or 'fiz' (the two matrices G and H are swaped in the
%     computations), but fails for the other orders.
% 
%   'test2' - A larger variant of 'test2' where the size of the matrix
%     pencil is 33x39 [Karlsson15]. The correct canonical form is
%       2R0 + 2R2 + 2R5 + 2J1(0) + 5J1(0.05) + 4J2(0) + J4(0).
%     The constructed matrix G is ill-conditioned, O(cond(G)) = 1e16.
%     PGUPTRI compute the correct canonical structure if the order of the
%     regular blocks are 'ifz', 'izf', or 'fiz' (the two matrices G and H
%     are swaped in the computations), but fails for the other orders.
%
%
%   References
%
%   [Boley90] D. Boley. Estimating the sensitivity of the algebraic
%     structure of pencils with simple eigenvalue estimates. SIAM J. Matrix
%     Anal. Appl., Vol 11, pp. 632-643, 1990.
%
%   [EdelmanMa00] A. Edelman and Y. Ma. Staircase failures explained by
%     orthogonal versal forms. SIAM J. Matrix Anal. Appl., Vol 21, pp.
%     1004-1025, 2000.
%
%   [DemmelKagstrom88] J. Demmel and B. Kagstrom. Accurate solutions of
%     ill-posed problems in control theory. SIAM J. Matrix Anal. Appl., Vol
%     9, pp. 126-145, 1988.
%
%   [Helmerson09] A. Helmersson, Dept Electrical Engineering, Linkoping
%     University, Sweden. Private communication, 2009.
%
%   [Karlsson15] L. Karlsson, Dept Computing Science, Umea University,
%     Sweden. Private communication, 2015.
%
%   [Mason04] J. Mason, Sandia National Labs, Albuquerque, NM. Private
%     communication, 2004.
%
%   See also MGGALLERY, PGUPTRI.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

nargoutchk(2,5);

if ~ischar(pencilname)
  error('MCS:pggallery:PencilName',...
    'The first argument must be the name of the matrix pencil.');
end

switch lower(pencilname)
  case 'boley1'
    G = [zeros(7,1) eye(7)];
    H = [eye(7)-triu(ones(7),1) (7:-1:1)'];
    if nargin == 3
      H(7,7) = varargin{1};
    end
    pstr = pstruct('rsblock', 7);
    
  case 'boley2'
    G = [zeros(3,1) eye(3)];
    H = [1 -149 537 -27; 1 -50 180 -9; 1 -154 546 -25];
    pstr = pstruct('rsblock',3);
    %pstr = pstruct('rsblock', 0, 'fjblock', {1,1}, 'fjblock', {1,2}, 'fjblock', {1,3});
    
  case 'em'
    if nargin == 3
      d = varargin{1};
    else
      d = 1.5e-8;
    end
    G = [0 0 1 0;0 0 0 1;0 0 0 0];
    H = [d 0 0 0;0 d 0 0;0 0 1 0];
    pstr = pstruct('rsblock', 1, 'fjblock', {2,0});
    
  case 'degreg'
    pstr = pstruct(...
      'fjblock',{[2 1 1],6},...
      'fjblock',{[3 1],2});
    [G,H,~,~] = pstr.kcf;
    
  case 'dk_c1'
    G = [1 -2 0 0 0;1 0 -1 0 0; 0 0 0 1 0; 0 0 0 0 2];
    H = [zeros(4,1) eye(4)];
    pstr = pstruct('rsblock', 2, 'fjblock', {1,1}, 'fjblock', {1,2});
    
  case 'dk_c2'
    G = [1 -2 -10 0 0;1 0 -1 0 0; 0 0 0 1 -7.5; 0 0 0 0 2];
    H = [zeros(4,1) eye(4)];
    pstr = pstruct('rsblock', 2, 'fjblock', {1,1}, 'fjblock', {1,2});
    
  case 'dk_c3'
    G = [1 -2 -100 0 0;1 0 -1 0 0; 0 0 0 1 -75; 0 0 0 0 2];
    H = [zeros(4,1) eye(4)];
    pstr = pstruct('rsblock', 2, 'fjblock', {1,1}, 'fjblock', {1,2});
    
  case 'lmi'
    G = [
      -43.2592e-003   499.3426e-003  -373.3369e-003   499.2867e-003   983.4503e-003     0.0000e+000     6.6639e-003   -39.6239e-003  -396.2117e-006
      -561.3580e-003   324.0847e-006   530.0007e-006   655.9454e-009  -904.4014e-006     0.0000e+000    -6.1283e-006    36.4389e-006   364.3645e-009
      -33.1763e-003  -186.0648e-006    -4.3407e+000   -71.4769e-006   -19.9014e+000     0.0000e+000   667.7825e-006    -3.9707e-003   -39.7040e-006
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000    20.0000e+000   -30.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      61.6694e-003   -76.1272e-006   -91.3258e-003   -10.9818e-006    15.1414e-003     0.0000e+000  -881.9574e-003  -610.0583e-006    -6.1002e-006
      3.9465e-003   190.1705e-006   -26.2048e-003    65.2983e-006   -90.0317e-003     0.0000e+000  -610.0583e-006  -878.4326e-003    -8.7837e-003
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     8.8200e-003  -882.0600e-003
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     1.3282e+000     0.0000e+000     0.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     1.6267e+000   -68.7528e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      -43.0948e-003  -500.6493e-003   940.3970e-003  -500.7112e-003   980.5912e-003     0.0000e+000     6.6445e-003   -39.5087e-003  -395.0598e-006
      ];

    H = [
      704.8118e-003    13.1404e-003   499.2867e-003   451.2575e-006   -49.1725e-003     0.0000e+000    -7.5549e-003    44.9220e-003     0.0000e+000
      -647.2175e-006   638.1853e-006   655.9454e-009   999.9996e-003    45.2201e-006     0.0000e+000     6.9477e-006   -41.3112e-006     0.0000e+000
      70.5258e-003   -69.5416e-003   -71.4769e-006    45.2201e-006   995.0725e-003     0.0000e+000  -757.0715e-006     4.5016e-003     0.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     1.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      10.8357e-003   -10.6845e-003   -10.9818e-006     6.9477e-006  -757.0715e-006     0.0000e+000   999.8837e-003   691.6290e-006     0.0000e+000
      -64.4295e-003    63.5303e-003    65.2983e-006   -41.3112e-006     4.5016e-003     0.0000e+000   691.6290e-006   995.8875e-003     0.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     1.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000     0.0000e+000
      702.7658e-003    15.1579e-003  -500.7112e-003   449.9456e-006   -49.0296e-003     0.0000e+000    -7.5330e-003    44.7914e-003     0.0000e+000
      ];
    pstr = pstruct('lsblock',[8,0],'ijblock',1);
    
  case 'ones'
    G=ones(55,35);
    H=pi*ones(55,35);
    e = (1/1925)/(pi/1925); % exact eigenvalue
    pstr = pstruct('rsblock', zeros(1,34), 'lsblock', zeros(1,54), ...
      'fjblock', {1,e});
    
  case 'rand'
    if nargin == 3
      % Square case
      m = varargin{1};
      n = m;
    elseif nargin == 4
      m = varargin{1};
      n = varargin{2};
    else
      m = 5;
      n = 7;
    end

    G = randn(m,n);
    H = randn(m,n);

    if n>m
      d = n-m;
      a = floor(m/d);
      c = mod(m,d);
      R = [(a+1)*ones(1,c) a*ones(1,d-c)];
      L = [];
      FJ = [];
      e = [];
    elseif m>n
      d = m-n;
      a = floor(n/d);
      c = mod(n,d);
      R = [];
      L = [(a+1)*ones(1,c) a*ones(1,d-c)];
      FJ = [];
      e = [];
    else
      R = [];
      L = [];
      e = eig(G,H);
      FJ = num2cell(ones(1,n));
    end
    pstr = pstruct(R,L,FJ,e);
    
  case 'rand_large'
    G = randn(650,503);
    H = randn(650,503);
    pstr = pstruct('lsblock', [4*ones(1,62) 3*ones(1,85)]);
    
  case 'rand_medium'
    G = randn(55,35);
    H = randn(55,35);
    pstr = pstruct('lsblock', [2*ones(1,15)  ones(1,5)]);
    
  case 'rand_square'
    if nargin == 3
      n = varargin{1};
    else
      n = 5;
    end
    G = randn(n);
    H = randn(n);
    e = eig(G,H);
    FJ = num2cell(ones(1,n));
    pstr = pstruct([],[],FJ,e);
      
  case 'test1'
    pstr = pstruct([0 1], 0, 1, 1e-4i);
    
    [G, H] = pstr.kcf;
    m = 3; n = 4;
    X = orth(rand(m)); Y = orth(rand(n));
    Xn = 1/m*randn(m);
    Yn = 1/n*randn(n);
    X = X+Xn;
    Y = Y+Yn;
    G = X*G*Y;
    H = X*H*Y;
    
  case 'test2'
    R = [0 0 2 2 5 5];
    FJ = {[1 1 2 2 2 2 4] [1 1 1 1 1]};
    pstr = pstruct(R, [], FJ, [0 0.05], []);
    
    [G, H] = pstr.kcf;
    m = 33; n = 39;
    X = orth(rand(m)); Y = orth(rand(n));
    Xn = 1/m*randn(m);
    Yn = 1/n*randn(n);
    X = X+Xn;
    Y = Y+Yn;
    G = X*G*Y;
    H = X*H*Y;
    
  case 'reference'
    pstr = pstruct(...
      'rsblock',[1 0],...
      'lsblock',[2 0 0 0],...
      'fjblock',{[1 1],6},...
      'fjblock',{1,2+5i},...
      'fjblock',{1,0},...
      'ijblock',[2 1]);
    [G,H,~,~] = pstr.kcf;
    
  case 'reference_reg'
    pstr = pstruct('fjblock',{2,2+5i},...
      'fjblock',{[2 1],0},...
      'ijblock',[2 1]);
    [G,H,~,~] = pstr.kcf;
    
  otherwise
    error('Unknown matrix pencil name.')
end

% If pert is present, add random perturbation to the matrices.
[m,n] = size(G);
if nargin >= 2 && ~isempty(pert) && pert ~= 0
  Eg = pert*rand(m,n);
  Eh = pert*rand(m,n);
  G = G + Eg;
  H = H + Eh;
else
  Eg = zeros(m,n);
  Eh = zeros(m,n);
end

if nargout == 3
  varargout{1} = pstr;
elseif nargout == 4
  varargout{1} = Eg;
  varargout{2} = Eh;
elseif nargout == 5
  varargout{1} = Eg;
  varargout{2} = Eh;
  varargout{3} = pstr;
end
