function varargout = pguptri(G,H,varargin)
%PGUPTRI Compute the canonical structure information of a matrix pencil.
%   PStr = PGUPTRI(G,H) returns a pstruct object PStr representing the
%   Kronecker canonical structure of G-sH, i.e., the sizes of existing
%   singular blocks and regular blocks with associated eigenvalues. The
%   Kronecker canonical structure information is computed using the Guptri
%   staircase algorithm.
%
%   PStr = PGUPTRI(G,H,Tol,Gap) determines the structure with respect to
%   the deflation tolerance, Tol, and the required gap, Gap, in the
%   singular values of the orthogonal deflations. Tol and Gap are used to
%   make rank decisions by searching for adjacent singular values whose
%   ratio exceeds Gap and the smaller one is less than Tol. If Tol is a
%   vector [TolG TolH], different tolerances Tol are used for G and H. Tol
%   is the absolute uncertainty in the data and should be at least about
%   eps and nominally between 1e-8 and 1e-12. Gap should be at least 1 and
%   nominally 1000. If Tol and Gap are not provided, default tolerances are
%   used (see MCSTOLERANCE). By setting Tol or Gap to empty matrix the
%   default value is used for that tolerance.
%  
%   PGUPTRI(G,H,Tol,Gap,...) or
%   PGUPTRI(G,H,...) can take the additional arguments in any order (if
%   specified, set Tol and/or Gap to empty matrix to use their default values):
%     'ZFI' | 'ZIF' | 'IFZ' | 'IZF' | 'FZI' | 'FIZ'
%        The string describes in which order the regular subpencils should
%        appear in the Guptri form. The letters F, Z, and I stand for
%        finite, zero, and infinite eigenvalues, respectively. The default
%        order is 'ZFI'.
%     'norm' | 'gersh' | 'min' | 'max' | 'avg' | 'mean' | 'var'
%        Specifies the method used to cluster the eigenvalues with default
%        tolerance. Optional tolerance parameter to the cluster method can
%        be specified in a subsequent argument CTol. For example, the following
%        call set CTol = eps for 'norm'
%          > pstr = pguptri(G,H,'norm',eps)
%        See PCLUSTER for a full desciption of each method. By default, the
%        cluster method returned by MCSEVCLMTH is used.
%     'absolute' | 'relative' | 'relative_strict'
%        If 'absolute', the tolerance Tol is used to determine the
%        nullspace of G and H. 
%        If 'relative', relative tolerances are used for G and H:
%          TolG = norm(G,'fro')*Tol(1) and
%          TolH = norm(H,'fro')*Tol(2),
%        where Tol normally should be the machine precision but this will
%        in some cases be to optimistic.
%        If 'relative_strict', the value of Tol is ignored and (normally)
%        strict relative tolerances are used for G and H:
%          TolG = max(size(G))*eps(norm(G,'fro')) and 
%          TolH = max(size(H))*eps(norm(H,'fro')). 
%        Default is 'absolute'.
%     'zeros'|'nozeros'
%        If 'zeros', the singular values interpreted as zeros (with respect
%        to Tol and Gap) in the deflation process are set to zero.
%        Otherwise ('nozeros'), small singular values are kept and used in
%        the computations. The default is 'zeros'.
%     'forceimpose'
%        If 'forceimpose', also singular values larger than Tol*Gap will be
%        forced to zero in the separation steps. Default is that if a
%        "large" singular value is going to be set to zero the current
%        separation will stop and continue with the next. Instead a warning
%        is issued and the returned S and T (see below) are not in exact
%        Guptri staircase form.
%     'suppresswarnings'
%        Suppress all warnings. This option should be used with care!
%
%   PGUPTRI can also return the following data:
%      [S,T]                = PGUPTRI(...) 
%      [S,T,PStr]           = PGUPTRI(...)
%      [S,T,P,Q]            = PGUPTRI(...)
%      [S,T,P,Q,PStr]       = PGUPTRI(...) 
%      [S,T,P,Q,dG,dH]      = PGUPTRI(...) 
%      [S,T,P,Q,dG,dH,PStr] = PGUPTRI(...) 
%   where P and Q are unitary (orthonormal) and S and T are in generalized
%   upper triangular form (Guptri staircase form), such that P(S-sT)Q' =
%   (G+dG)-s(H+dH). The Frobenius norm of dG and dH is an upper bound on
%   the distance from G-sH to a matrix pencil with the computed Kronecker
%   structure as the exact one. Given the argument 'zeros' (default),
%   norm([dG,dH]) is of size norm([G,H])*Tol. Otherwise, given the argument
%   'nozeros', S-sT is an "exact" orthogonal equivalence transformation of
%   G-sH, and norm([dG,dH]) is of size norm([G,H])*eps, where dG and dH are
%   formed by P*S*Q'-G and P*T*Q'-H, respectively.
%
%   Algorithm:
%    Based on the algorithm described in
%    J. Demmel and B. Kagstrom. The generalized Schur decomposition of an
%    arbitrary pencil A-lambda B: Robust software with error bounds and
%    applications. Part i: Theory and algorithms. ACM Trans. Math.
%    Software. Vol. 19, No. 2, pp. 160-174, 1993.
%    J. Demmel and B. Kagstrom. The generalized Schur decomposition of an
%    arbitrary pencil A-lambda B: Robust software with error bounds and
%    applications. Part ii: Software and applications. ACM Trans. Math.
%    Software. Vol. 19, No. 2, pp. 175-201, 1993.
%
%   See also MGUPTRI, PSTRUCT, PCLUSTER, MCSTOLERANCE, MCSEVCLMTH.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

%----------------------------------------------------------------
% Debug switch
DEBUG = 0;

% Checking required arguments
assert(nargin > 1,'Not enough input arguments.')
nargoutchk(0,7);
error(sizechk('ndims',G,2));
error(sizechk('ndims',H,2));
error(sizechk('equal',G,H));

compst = nargout > 1;
comppq = nargout > 3;
comperr = nargout > 5;
if DEBUG, comppq = true; end

% Defualt values
setzero = 1;           % Force small singular values to zero
regorder = 'zfi';      % Order of the regular pencils
rstrfun = 'rzstr';
lstrfun = 'listr';
ziorder = true;        % Order of zero and infinite eigenvalue.
forceimpose = false;   % If true, set singular values to zero even if large
clmethod = mcsevclmth; % Default eigenvalue clustering method
cltol = [];
suppressWarnings = false;

startarg = 1;
[toleps,gap] = mcstolerance;  % Default tolerance parameters
if nargin > 2 && isnumeric(varargin{1})
  if nargin == 3
    error('Missing argument ''Gap''.');
  elseif nargin > 3
    if ~isempty(varargin{1})
      toleps = varargin{1};
    end
    if ~isempty(varargin{2})
      gap = varargin{2};
    end
  end

  % Absolute tolerance
  % Used when compressing columns/rows of G and H
  if length(toleps) == 1
    tol = [toleps toleps];
  else
    tol = toleps;
  end
  startarg = 3;
else
  tol = [toleps toleps];
end

% Checking additional arguments
for i = startarg:length(varargin)
  switch lower(varargin{i})
    case 'zeros'
      setzero = 1;
    case 'nozeros'
      setzero = 0;
    case 'relative'
      % Relative tolerance
      tol(1) = norm(G,'fro')*tol(1);
      tol(2) = norm(H,'fro')*tol(2);
    case 'relative_strict'
      % Strict relative tolerance
      tol(1) = max(size(G))*eps(norm(G,'fro')); 
      tol(2) = max(size(H))*eps(norm(H,'fro'));
      %  Alternative relative tolerance is:
      %  * TolG = TolH = norm(G+H)*eps^(1/3)
    case 'absolute'
      % Do nothing
    case 'forceimpose'
      forceimpose = true;
    case {'zfi' , 'zif' , 'fzi'}
      regorder = lower(varargin{i});
    case {'ifz' , 'izf' , 'fiz'}
      regorder = lower(varargin{i});
      % Use ristr and lzstr instead
      rstrfun = 'ristr';
      lstrfun = 'lzstr';
      ziorder = false;
    case {'norm','gersh','min','max','avg','mean','var'}
      clmethod = lower(varargin{i});
      if nargin > i+2 && isnumeric(varargin{i+1})
        cltol = varargin{i+1};
      end
    case 'suppresswarnings'
      suppressWarnings = true;
    otherwise
      if ischar(varargin{i})
        error(['Unknown argument ''' varargin{i} '''.']);
      end
  end
end

if suppressWarnings
  warnState = warning('off','all');
end

% Step 1.
%  Find and put the right singular structure and the regular structure of
%  the zero/infinite eigenvalue in the mrz x nrz upper left corner of
%  (G,H).
%
%                 n
%  Srz =      nrz
%           nr   nz
%         +----+----+----+
%       mr| Sr |  * |  * |
%   mrz   |----+----+    |
% m     mz|    | Sz |  * |
%         |----+----+----+
%         |         | S2 |
%         +---------+----+

[m,n] = size(G);

[Srz,Trz,Prz,Qrz,rweyr,zweyr] = feval(rstrfun,G,H,tol,gap,setzero);

% The reduced pencil will be stored in the pair (S,T)
S = Srz;
T = Trz;

if comperr
  dG = zeros(m,n); 
  dH = zeros(m,n);
end
if comppq
  P = Prz; 
  Q = Qrz;
end

[mr,nr] = weyrsize(rweyr,'right');
[mz,nz] = weyrsize(zweyr,'jordan');
mrz = mr+mz;
nrz = nr+nz;

if DEBUG
  fdebug('Step 1:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
    'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H,...
    'mr x nr = ',mr,'x',nr,'\nmz x nz = ',mz,'x',nz,'\n');
end

%----------------------------------------------------------------
% Step 2 and 3.
%  If there are both right singular blocks and Jordan blocks associated
%  with the zero/infinite eigenvalue, reduce again to separate them.

% Step 2.
%  Separate the right and regular structures. Reduce first mrz rows, nrz
%  columns, swapping roles of (G,H). Insist on computing the same right
%  singular structure as in Step 1.

if comperr
  [Tr,Sr,Pr,Qr,dH(1:mrz,1:nrz),dG(1:mrz,1:nrz),r2flag] = ...
    feval(rstrfun,T(1:mrz,1:nrz),S(1:mrz,1:nrz),tol,gap,setzero,rweyr,[],forceimpose);
else
  [Tr,Sr,Pr,Qr,r2flag] = ...
    feval(rstrfun,T(1:mrz,1:nrz),S(1:mrz,1:nrz),tol,gap,setzero,rweyr,[],forceimpose);
end

% Check error
if r2flag == 1 && ziorder
  warning('MCS:pguptri:SeparationFailure',...
    'Failed to separate the right singular and the zero eigenvalue blocks.')
elseif r2flag == 1
  warning('MCS:pguptri:SeparationFailure',...
    'Failed to separate the right singular and the infinite eigenvalue blocks.')
else
  if r2flag == 2 && ziorder
    warning('MCS:guptri:WrongStaircaseForm',...
      ['Warning issued when separating the right singular and the zero eigenvalue blocks.'...
      'The returned staircase form has an error larger than the given tolerance.'])
  elseif r2flag == 2
    warning('MCS:guptri:WrongStaircaseForm',...
      ['Warning issued when separating the right singular and the infinite eigenvalue blocks.'...
      'The returned staircase form has an error larger than the given tolerance.'])
  end
  
  % +--+-+--+
  % |r |#|##|
  % | z|#|##|
  % +--+-+--+
  % |  |f|  |
  % +--+-+--+
  % |  | |i |
  % |  | | l|
  % +--+-+--+
  S(1:mrz,nrz+1:n) = Pr'*S(1:mrz,nrz+1:n);
  T(1:mrz,nrz+1:n) = Pr'*T(1:mrz,nrz+1:n);

  % +--+-+--+
  % |r | |  |
  % | z| |  |
  % +--+-+--+
  % |##|f|  |
  % +--+-+--+
  % |##| |i |
  % |##| | l|
  % +--+-+--+
  S(mrz+1:m,1:nrz) = S(mrz+1:m,1:nrz)*Qr;
  T(mrz+1:m,1:nrz) = T(mrz+1:m,1:nrz)*Qr;

  % +--+-+--+
  % |##| |  +
  % |##| |  +
  % +--+-+--+
  % |  |f|  |
  % +--+-+--+
  % |  | |i |
  % |  | | l|
  % +--+-+--+
  S(1:mrz,1:nrz) = Sr;
  T(1:mrz,1:nrz) = Tr;
  
  if comppq
    P(:,1:mrz)    = P(:,1:mrz)*Pr;
    Q(:,1:nrz)    = Q(:,1:nrz)*Qr;
  end
  
  if DEBUG
    fdebug('Step 2:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
      'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
  end
  
  %----------------------------------------------------------------
  % Step 3.
  %  Recompute the block structure of the zero/infinite eigenvalue. Insist
  %  on computing the same jordan structure as in Step 1.
  
  % Sz = Sr(mr+1:mrz,nr+1:nrz)
  % Tz = Tr(mr+1:mrz,nr+1:nrz)
  if comperr
    [Sz,Tz,Pz,Qz,E,F,r3flag] = ...
      feval(rstrfun,Sr(mr+1:mrz,nr+1:nrz),Tr(mr+1:mrz,nr+1:nrz),tol,gap,setzero,[],zweyr,forceimpose);
    dG(mr+1:mrz,nr+1:nrz) = dG(mr+1:mrz,nr+1:nrz)+E; 
    dH(mr+1:mrz,nr+1:nrz) = dH(mr+1:mrz,nr+1:nrz)+F;
  else
    [Sz,Tz,Pz,Qz,r3flag] = ...
      feval(rstrfun,Sr(mr+1:mrz,nr+1:nrz),Tr(mr+1:mrz,nr+1:nrz),tol,gap,setzero,[],zweyr,forceimpose);
  end
  
  if r3flag == 1 && ziorder
    warning('MCS:pguptri:SeparationFailure',...
      'Failed to impose correct block structure on the zero eigenvalue blocks.')
  elseif r3flag
    warning('MCS:pguptri:SeparationFailure',...
      'Failed to impose correct block structure on the infinite eigenvalue blocks.')
  else
    if r3flag == 2 && ziorder
      warning('MCS:guptri:WrongStaircaseForm',...
        ['Warning issued when imposing structure on the zero eigenvalue blocks.'...
        'The returned staircase form has an error larger than the given tolerance.'])
    elseif r3flag == 2
      warning('MCS:guptri:WrongStaircaseForm',...
        ['Warning issued when imposing structure on the infinite eigenvalue blocks.'...
        'The returned staircase form has an error larger than the given tolerance.'])
    end
    
    % +--+-+--+
    % |r | |  |
    % |#z|#|##|
    % +--+-+--+
    % |  |f|  |
    % +--+-+--+
    % |  | |i |
    % |  | | l|
    % +--+-+--+
    S(mr+1:mrz,[1:mr nrz+1:n]) = Pz'*S(mr+1:mrz,[1:mr nrz+1:n]);
    T(mr+1:mrz,[1:mr nrz+1:n]) = Pz'*T(mr+1:mrz,[1:mr nrz+1:n]);

    % +--+-+--+
    % |r#| |  |
    % | z| |  |
    % +--+-+--+
    % | #|f|  |
    % +--+-+--+
    % | #| |i |
    % | #| | l|
    % +--+-+--+
    S([1:mr mrz+1:m],nr+1:nrz) = S([1:mr mrz+1:m],nr+1:nrz)*Qz;
    T([1:mr mrz+1:m],nr+1:nrz) = T([1:mr mrz+1:m],nr+1:nrz)*Qz;
    
    % +--+-+--+
    % |r | |  |
    % | #| |  |
    % +--+-+--+
    % |  |f|  |
    % +--+-+--+
    % |  | |i |
    % |  | | l|
    % +--+-+--+
    S(mr+1:mrz,nr+1:nrz) = Sz;
    T(mr+1:mrz,nr+1:nrz) = Tz;
    
    if comppq
      P(:,mr+1:mrz) = P(:,mr+1:mrz)*Pz;
      Q(:,nr+1:nrz) = Q(:,nr+1:nrz)*Qz;
    end
  end
  
  if DEBUG
    fdebug('Step 3:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
      'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
  end
end


%----------------------------------------------------------------
% Step 4.
%  Find and put the Jordan structure of the infinite/zero eigenvalue and
%  the left singular structure in the mli x nli lower right corner of
%  (G,H).
%
% Sli = S(mrz+1:m, nrz+1:n)
% Sli = T(mrz+1:m, nrz+1:n)
%                                       n2
% G =                   Sli =             nli
%      nrz   n2                    nf   ni   nl
%    +-----+-----+               +----+---------+
% mrz| Srz |  *  |             mf| Sf |  *    * |
%    +-----+-----+               |----+----+----|
% m2 |     | Sli |      m2     mi|    | Si |  * |
%    +-----+-----+         mli   |    +----+----+
%                              ml|    |    | Sl |
%                                +----+----+----+

[m2,n2] = size(S(mrz+1:m, nrz+1:n));

[Sli,Tli,Pli,Qli,lweyr,iweyr] = ...
  feval(lstrfun,S(mrz+1:m, nrz+1:n),T(mrz+1:m, nrz+1:n),tol,gap,setzero);

% +--+-+--+
% |r | |  |
% | z| |  |
% +--+-+--+
% |##|f|  |
% +--+-+--+
% |##| |i |
% |##| | l|
% +--+-+--+
S(mrz+1:m,1:nrz) = Pli'*S(mrz+1:m,1:nrz);
T(mrz+1:m,1:nrz) = Pli'*T(mrz+1:m,1:nrz);

% +--+-+--+
% |r |#|##|
% | z|#|##|
% +--+-+--+
% |  |f|  |
% +--+-+--+
% |  | |i |
% |  | | l|
% +--+-+--+
S(1:mrz,nrz+1:n) = S(1:mrz,nrz+1:n)*Qli;
T(1:mrz,nrz+1:n) = T(1:mrz,nrz+1:n)*Qli;

% +--+-+--+
% |r | |  |
% | z| |  |
% +--+-+--+
% |  |#|##|
% +--+-+--+
% |  |#|##|
% |  |#|##|
% +--+-+--+
S(mrz+1:m,nrz+1:n) = Sli;
T(mrz+1:m,nrz+1:n) = Tli;

if comppq
  P(:,mrz+1:m) = P(:,mrz+1:m)*Pli;
  Q(:,nrz+1:n) = Q(:,nrz+1:n)*Qli;
end

[ml,nl] = weyrsize(lweyr,'left');
[mi,ni] = weyrsize(iweyr,'jordan');
mli = ml+mi;
nli = nl+ni;

mf = m2-mli;
nf = n2-nli;

if DEBUG
  fdebug('Step 4:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
    'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H,...
    'ml x nl = ',ml,'x',nl,'\nmi x ni = ',mi,'x',ni,'\n');
end

%----------------------------------------------------------------
% Step 5 and 6.
%  If there are both left singular blocks and Jordan blocks associated
%  with the infinite/zero eigenvalue; reduce again to separate them.
%
% Step 5.
%  Separate the left and regular structures. Reduce last mli columns and
%  nli rows, swapping roles of (G,H). Insist on computing the same left
%  singular structure as in Step 4.

% listr(Bli, Ali, ..., lkstr)
if comperr
  [Tl,Sl,Pl,Ql,E,F,l5flag] = feval(lstrfun,T(m-mli+1:m, n-nli+1:n), ...
    S(m-mli+1:m, n-nli+1:n),tol,gap,setzero,lweyr,[],forceimpose);
    dG(m-mli+1:m, n-nli+1:n) = dG(m-mli+1:m, n-nli+1:n)+F; 
    dH(m-mli+1:m, n-nli+1:n) = dH(m-mli+1:m, n-nli+1:n)+E;
else
  [Tl,Sl,Pl,Ql,l5flag] = feval(lstrfun,T(m-mli+1:m, n-nli+1:n), ...
    S(m-mli+1:m, n-nli+1:n),tol,gap,setzero,lweyr,[],forceimpose);
end

if l5flag == 1 && ziorder
  warning('MCS:pguptri:SeparationFailure',...
    'Failed to separate the left singular and the infinite eigenvalue blocks.')
elseif l5flag == 1
  warning('MCS:pguptri:SeparationFailure',...
    'Failed to separate the left singular and the zero eigenvalue blocks.')
else
  if l5flag == 2 && ziorder
    warning('MCS:guptri:WrongStaircaseForm',...
      ['Warning issued when separating the left singular and the infinite eigenvalue blocks.'...
      'The returned staircase form has an error larger than the given tolerance.'])
  elseif l5flag == 2
    warning('MCS:guptri:WrongStaircaseForm',...
      ['Warning issued when separating the left singular and the zero eigenvalue blocks.'...
      'The returned staircase form has an error larger than the given tolerance.'])
  end
  
  % +--+-+--+
  % |r | |  |
  % | z| |  |
  % +--+-+--+
  % |  |f|  |
  % +--+-+--+
  % |##|#|i |
  % |##|#| l|
  % +--+-+--+
  S(mrz+1+mf:m,1:nrz+nf) = Pl'*S(mrz+1+mf:m,1:nrz+nf);
  T(mrz+1+mf:m,1:nrz+nf) = Pl'*T(mrz+1+mf:m,1:nrz+nf);
  
  % +--+-+--+
  % |r | |##|
  % | z| |##|
  % +--+-+--+
  % |  |f|##|
  % +--+-+--+
  % |  | |i |
  % |  | | l|
  % +--+-+--+
  S(1:mrz+mf,nrz+1+nf:n) = S(1:mrz+mf,nrz+1+nf:n)*Ql;
  T(1:mrz+mf,nrz+1+nf:n) = T(1:mrz+mf,nrz+1+nf:n)*Ql;
  
  % +--+-+--+
  % |r | |  |
  % | z| |  |
  % +--+-+--+
  % |  |f|  |
  % +--+-+--+
  % |  | |##|
  % |  | |##|
  % +--+-+--+
  S(mrz+1+mf:m,nrz+1+nf:n) = Sl;
  T(mrz+1+mf:m,nrz+1+nf:n) = Tl;
  
  if comppq
    P(:,mrz+1+mf:m) = P(:,mrz+1+mf:m)*Pl;
    Q(:,nrz+1+nf:n) = Q(:,nrz+1+nf:n)*Ql;
  end
  
  if DEBUG
    fdebug('Step 5:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
      'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
  end
  
  %----------------------------------------------------------------
  % Step 6.
  %  Recompute the block structure of the infinite/zero eigenvalue. Insist
  %  on computing the same Jordan structure as in Step 4.
  
  if comperr
    [Si,Ti,Pi,Qi,E,F,l6flag] = ...
      feval(lstrfun,Sl(1:mi,1:ni),Tl(1:mi,1:ni),tol,gap,setzero,[],iweyr,forceimpose);
    dG(1:mi,1:ni) = dG(1:mi,1:ni)+E; 
    dH(1:mi,1:ni) = dH(1:mi,1:ni)+F;
  else
    [Si,Ti,Pi,Qi,l6flag] = ...
      feval(lstrfun,Sl(1:mi,1:ni),Tl(1:mi,1:ni),tol,gap,setzero,[],iweyr,forceimpose);
  end
  
  if l6flag == 1 && ziorder
    warning('MCS:pguptri:SeparationFailure',...
      'Failed to impose correct block structure on the infinite eigenvalue blocks.')
  elseif l6flag == 1
    warning('MCS:pguptri:SeparationFailure',...
      'Failed to impose correct block structure on the zero eigenvalue blocks.')
  else
    if l6flag == 2 && ziorder
      warning('MCS:guptri:WrongStaircaseForm',...
        ['Warning issued when imposing structure on the infinite eigenvalue blocks.'...
        'The returned staircase form has an error larger than the given tolerance.'])
    elseif l6flag == 2
      warning('MCS:guptri:WrongStaircaseForm',...
        ['Warning issued when imposing structure on the zero eigenvalue blocks.'...
        'The returned staircase form has an error larger than the given tolerance.'])
    end
    
    % +--+-+--+
    % |r | |  |
    % | z| |  |
    % +--+-+--+
    % |  |f|  |
    % +--+-+--+
    % |##|#|i#|
    % |  | | l|
    % +--+-+--+
    S(m-mli+1:m-ml,[1:n-nli n-nl+1:n]) = Pi'*S(m-mli+1:m-ml,[1:n-nli n-nl+1:n]);
    T(m-mli+1:m-ml,[1:n-nli n-nl+1:n]) = Pi'*T(m-mli+1:m-ml,[1:n-nli n-nl+1:n]);
    
    % +--+-+--+
    % |r | |# |
    % | z| |# |
    % +--+-+--+
    % |  |f|# |
    % +--+-+--+
    % |  | |i |
    % |  | |#l|
    % +--+-+--+
    S([1:mrz+mf m-ml+1:m],nrz+nf+1:n-nl) = ...
      S([1:mrz+mf m-ml+1:m],nrz+nf+1:n-nl)*Qi;
    T([1:mrz+mf m-ml+1:m],nrz+nf+1:n-nl) = ...
      T([1:mrz+mf m-ml+1:m],nrz+nf+1:n-nl)*Qi;
    
    % +--+-+--+
    % |r | |  |
    % | z| |  |
    % +--+-+--+
    % |  |f|  |
    % +--+-+--+
    % |  | |# |
    % |  | | l|
    % +--+-+--+
    S(mrz+mf+1:m-ml,nrz+nf+1:n-nl) = Si;
    T(mrz+mf+1:m-ml,nrz+nf+1:n-nl) = Ti;
    
    if comppq
      P(:,mrz+mf+1:m-ml) = P(:,mrz+mf+1:m-ml)*Pi;
      Q(:,nrz+nf+1:n-nl) = Q(:,nrz+nf+1:n-nl)*Qi;
    end
  end
  
  if DEBUG
    fdebug('Step 6:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
      'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
  end
end

%----------------------------------------------------------------
% Step 7.
%  Reduce remaining regular part (corresponding to the nonzero finite
%  eigenvalues) to upper triangular form.
%

nji = 1;
mji = 1;

jblocksv = {};
eigv     = [];

% If the remaining part is not square we have failed to compute the
% staircase form.
if m-mli-mrz ~= n-nli-nrz
  error('MCS:pguptri:StaircaseFailed',...
    'Failed to compute a stable staircase form. Try to change the tolerances.')
end

if DEBUG
  CNR = 0;
  fdebug('Step 7: The eigenvalues computed with eig are:',...
    eig(S(mrz+mji:m-mli,nrz+nji:n-nli),T(mrz+mji:m-mli,nrz+nji:n-nli)),'\n')
end

% Use the RZ-algorithm to compute the Guptri form of the remaining
% regular pencil. Impose the cluster size s obtained from PCLUSTER for
% the eigenvalue e. Any imposing warnings are issued in NULLITYCHECK.
% Alternativly, the QZ algorithm can be used instead.
while n-nli-nrz-nji+1 > 0
  Sj = S(mrz+mji:m-mli,nrz+nji:n-nli);
  Tj = T(mrz+mji:m-mli,nrz+nji:n-nli);
  
  % Find the cluster size s of the largest eigenvalue e
  [e,s] = pcluster(clmethod,Sj,Tj,cltol);
  
  % Impose the cluster size s for the eigenvalue e. Any imposing warnings
  % are issued in NULLITYCHECK.
  if setzero && comperr
    [Sj,Tj,Pj,Qj,Ej,Fj,fweyr] = fstr(Sj,Tj,[e,s],tol,gap,setzero);
    dG(mrz+mji:m-mli,nrz+nji:n-nli) = dG(mrz+mji:m-mli,nrz+nji:n-nli)+Ej;
    dH(mrz+mji:m-mli,nrz+nji:n-nli) = dH(mrz+mji:m-mli,nrz+nji:n-nli)+Fj;
  else
    [Sj,Tj,Pj,Qj,fweyr] = fstr(Sj,Tj,[e,s],tol,gap,setzero);
  end
  
  if DEBUG
    CNR = CNR + 1;
    fdebug('In step 7:','Size of computed cluster ', CNR,' is ', s,'x',s,...
      ' with eigenvalue ',e ,'\n');
    %eig(Sj,Tj)
    if setzero && comperr
      fdebug('In step 7:','Error from fstr: norm(E) = ', norm(Ej), ' norm(F)= ', norm(Fj),'\n');
    end
  end
  
  rsz = weyrsize(fweyr,1,'jordan');
  if rsz == 0
    error('MCS:pguptri:EigClusteringFailure',...
      'Failed to cluster the eigenvalues.');
  elseif rsz ~= s
    warning('MCS:pguptri:DiffSizeJordan',...
      'PCLUSTER and FSTR do not return the same cluster size: %d ~= %d',s,rsz);
  end
  
  S(mrz+mji:m-mli,[1:nrz+nji-1 n-nli+1:n]) = ...
    Pj'*S(mrz+mji:m-mli,[1:nrz+nji-1 n-nli+1:n]);
  T(mrz+mji:m-mli,[1:nrz+nji-1 n-nli+1:n]) = ...
    Pj'*T(mrz+mji:m-mli,[1:nrz+nji-1 n-nli+1:n]);
  S([1:mrz+mji-1 m-mli+1:m],nrz+nji:n-nli) = ...
    S([1:mrz+mji-1 m-mli+1:m],nrz+nji:n-nli)*Qj;
  T([1:mrz+mji-1 m-mli+1:m],nrz+nji:n-nli) = ...
    T([1:mrz+mji-1 m-mli+1:m],nrz+nji:n-nli)*Qj;
  
  S(mrz+mji:m-mli,nrz+nji:n-nli) = Sj;
  T(mrz+mji:m-mli,nrz+nji:n-nli) = Tj;
  
  if comppq
    P(:,mrz+mji:m-mli) = P(:,mrz+mji:m-mli)*Pj;
    Q(:,nrz+nji:n-nli) = Q(:,nrz+nji:n-nli)*Qj;
  end
  
  mji = mji + rsz;
  nji = nji + rsz;
  
  if ~isempty(fweyr)
    jblocksv = cat(2,jblocksv,{weyr2segre(fweyr,'regular')});
    eigv     = [eigv, e];
  end
end

if DEBUG
  fdebug('Step 7:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
    'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
end

%----------------------------------------------------------------
% Post processing

if compst
  % Any warnings?
  if r2flag == 1
    if ziorder
      if l5flag == 1
        warning('MCS:pguptri:SeparationFailure',...
          'Returned order RZ-F-IL, where RZ and IL are not separated.')
      else
        warning('MCS:pguptri:SeparationFailure',...
          'Returned order RZ-F-I-L, where RZ is not separated.')
      end
    else
      if l5flag == 1
        warning('MCS:pguptri:SeparationFailure',...
          'Returned order RI-F-ZL, where RI and ZL are not separated.')
      else
        warning('MCS:pguptri:SeparationFailure',...
          'Returned order RI-F-Z-L, where RI is not separated.')
      end
    end
  elseif l5flag == 1
    if ziorder
      warning('MCS:pguptri:SeparationFailure',...
        'Returned order R-Z-F-IL, where IL is not separated.')
    else
      warning('MCS:pguptri:SeparationFailure',...
        'Returned order R-I-F-ZL, where ZL is not separated.')
    end
  end
  
  % Reorder regular blocks if needed
  % If 'zfi' or 'ifz' already in correct order
  try
    switch regorder
      case {'fzi','fiz'}
        if r2flag ~= 1
          % Swap Z and F blocks... (ZFI -> FZI) or
          % Swap I and F blocks... (IFZ -> FIZ)
          [S,T,Pswap,Qswap] = swapblk(S,T,mrz-mz+1,mrz,nrz-nz+1,nrz,mrz+1,mrz+mf,nrz+1,nrz+nf);
          if comppq
            P = P*Pswap;
            Q = Q*Qswap;
          end
        end
      case {'zif','izf'}
        if l5flag ~= 1
          % Swap F and I blocks... (ZFI -> ZIF) or
          % Swap F and Z blocks... (IFZ -> IZF)
          [S,T,Pswap,Qswap] = swapblk(S,T,mrz+1,mrz+mf,nrz+1,nrz+nf,m-mli+1,m-ml,n-nli+1,n-nl);
          if comppq
            P = P*Pswap;
            Q = Q*Qswap;
          end
        end
    end
    if DEBUG
      fdebug('Reorder:','ErrNorm(G)=',norm(P*S*Q'-G,'fro'),P*S*Q'-G,...
        'ErrNorm(H)=',norm(P*T*Q'-H,'fro'),P*T*Q'-H);
    end

  catch ME
    % Catch errors from slgsylv (called in swapblk)
    switch ME.identifier
      case 'MCS:gsylve:Error'
        if ziorder; temp = 'ZFI'; else, temp = 'IFZ'; end
        warning('MCS:pguptri:SeparationFailure',...
          'The regular blocks could not be reordered. Error in gsylve:\n  ''%s''\nReturned order %s',...
          ME.message, temp)
      otherwise
        rethrow(ME)
    end
  end
  
end


%----------------------------------------------------------------
% Finishing

if suppressWarnings
  warning(warnState); % Restore warning
end

% Construct the matrix pencil object
if ziorder
  % Add the zero eigenvalue blocks as fjblocks to the matrix pencil object
  if nz > 0 && mz > 0
    jblocksv = cat(2,jblocksv,{weyr2segre(zweyr,'regular')});
    eigv     = cat(2,eigv,0);
  end
  
  pstr = pstruct(weyr2segre(rweyr,'singular'),...
    weyr2segre(lweyr,'singular'),...
    jblocksv,eigv,...
    weyr2segre(iweyr,'regular'));
else
  % Add the zero eigenvalue blocks as fjblocks to the matrix pencil object
  if ni > 0 && mi > 0
    jblocksv = cat(2,jblocksv,{weyr2segre(iweyr,'regular')});
    eigv     = cat(2,eigv,0);
  end
  
  pstr = pstruct(weyr2segre(rweyr,'singular'),...
    weyr2segre(lweyr,'singular'),...
    jblocksv,eigv,...
    weyr2segre(zweyr,'regular'));
end

% Create output
if nargout < 2 
  varargout{1} = pstr;
elseif nargout > 1
  varargout{1} = S;
  varargout{2} = T;
  if nargout == 3 
    varargout{3} = pstr;
  elseif nargout > 3
    varargout{3} = P;
    varargout{4} = Q;
    if nargout == 5 
      varargout{5} = pstr;
    elseif nargout > 5
      varargout{5} = dG;
      varargout{6} = dH;
      if nargout == 7 
        varargout{7} = pstr;
      end
    end
  end
end

