function [e,s,D0,D1,ev,C] = cehierarchy(A,varargin)
%CEHIERARCHY Compute and cluster eigenvalues of a matrix or matrix pencil.
%   [Ev,Sizes] = CEHIERARCHY(A,DeltaMax,Pmin,Method) or
%   [Ev,Sizes,Delta0,Delta1] = CEHIERARCHY(A,DeltaMax,Pmin,Method) clusters
%   the eigenvalues corresponding to a numerically multiple eigenvalue of
%   the matrix A. The vectors Ev and Sizes return the mean values of the
%   eigenvalues in each isolated cluster and the cluster sizes,
%   respectively. The clustering is based on a hierarchical clustering
%   algorithm where the clustering is determined by a distance function D
%   (specified with Method) that satisfy D <= DeltaMax. The maximum
%   distance Delta0 between the eigenvalues within each cluster and the
%   minimum distance Delta1 to remaining clusters fulfill Delta0 <=
%   DeltaMax < Delta1. It follows that if Delta0 is close to Delta1 the
%   clusters are not well separated. The delta distances are returned in
%   the vectors Delta0 and Delta1. The number of clusters P generated
%   satisfies P >= Pmin. If Pmin = 0 only the cluster containing the
%   largest eigenvalue is returned in Ev and Sizes. Otherwise, Pmin will be
%   automatically adjusted to satisfy 1 <= Pmin <= n, where n is the size
%   of the matrix A. Appropriate initial values of the parameters are
%   DeltaMax = norm(A)*sqrt(eps) and Pmin = 1. If the matrix A is known to
%   have k multiple eigenvalues, appropriate parameters are DeltaMax = inf
%   and Pmin = k.
%
%   [Ev,Sizes] = CEHIERARCHY(G,H,DeltaMax,Pmin,Method) or
%   [Ev,Sizes,Delta0,Delta1] = CEHIERARCHY(G,H,DeltaMax,Pmin,Method)
%   clusters the generalized eigenvalues of the square matrix pencil G-sH.
%   The same method as for matrices is used.
%
%   The string Method specifies the distance function D(X,Y) to be used,
%   where X and Y are two different clusters with r and s eigenvalues,
%   respectively:
%      'min'  -  D(X,Y) = min(abs(X(i) - Y(j))), for all i,j
%      'max'  -  D(X,Y) = max(abs(X(i) - Y(j))), for all i,j
%      'avg'  -  D(X,Y) = 1/(r*s) * (sum^{r,s} abs(X(i) - Y(j)))
%      'mean' -  D(X,Y) = abs(sum(X)/r - sum(Y)/s)
%      'var'  -  D(X,Y) = var(X U Y), (variance of X union Y)
%
%   [Ev,Sizes,Delta0,Delta1,E,EvCluster] = CEHIERARCHY(...) also returns
%   all computed eigenvalues E of A or G-sH and the clustering of the
%   eigenvalues in EvCluster. The integer value k in EvCluster(i)
%   represents the cluster the eigenvalue E(i) belongs to, where Ev(k) is
%   the new eigenvalue for the cluster. The eigenvalues in E are sorted in
%   non-increasing order.
%
%   Algorithm:
%    The function uses the hierarchical clustering algorithm described in
%    U. Kintzel, Preprint 33-2003, Institute of Mathematics, Technische
%    Universitt Berlin, 2003.
%    and
%    M.R. Anderberg. Cluster Analysis for Applications. Chapter 6, Academic
%    Press, 1973.
%
%   See also MCLUSTER, PCLUSTER.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.


narginchk(4,5)

if nargin == 5
  ev = sort(eig(A,varargin{1}), 'descend');
  karg = 1;
else
  ev = sort(eig(A),'descend');
  karg = 0;
end
deltaMax = varargin{karg+1};
pmin = varargin{karg+2};
method = varargin{karg+3};

n = length(ev);

if pmin == 0
  % Cluster only the largest eigenvalue
  pmin = 1;
  onlylargest = true;
else
  % else correct it if necessary
  pmin = min(max(1,pmin),n);
  onlylargest = false;
end

if ~any(strcmpi(method,{'min','max','avg','mean','var'}))
  error('Unknown distance function.')
end

% Distance function
dfun = strcat('ec',lower(method));

C = [(1:n)' zeros(n,1)]; % Initialize clusters
p = n;
go = true;
% Cluster the two closest eigenvalues (clusters of eigenvalues) until
% D > DeltaMax or p == Pmin is reached.
while go && p > pmin
  delta = inf;
  for k = 1:p-1
    for l = k+1:p
      % Call distance function
      d = feval(dfun,ev,C(:,1),k,l);
      if d < delta 
        delta = d;
        r = k; s = l;
      end
    end
  end
  if  delta <= deltaMax
    % Update clusters
    C(C(:,1)==s,1) = r;
    C(C(:,1)>s,1) = C(C(:,1)>s,1)-1;
    p = p-1;
    if C(C(:,1)==r,2) < delta
      C(C(:,1)==r,2) = delta; % Set max distance within cluster
    end
    %D0(s) = inf; % Set distance to inf for removed cluster
  else
    % No more clusters found fullfilling D <= DeltaMax
    go = false;
  end
end

% Compute Delta1 if needed
if nargout > 3
  D1 = ones(p,1)*inf;
  for k = 1:p-1
    for l = k+1:p
      % Call distance function
      d = feval(dfun,ev,C(:,1),k,l);
      if d < D1(k)
        D1(k) = d;
      end
      if d < D1(l)
        D1(l) = d;
      end
    end
    if onlylargest
      D1 = D1(1);
      break;
    end
  end
end


if onlylargest
  k = C(:,1)==C(1,1);
  e = mean(ev(k));
  s = sum(k);
  D0 = C(1,2);
else
  e = zeros(p,1);
  s = zeros(p,1);
  D0 = zeros(p,1);
  for k = 1:p
    cluster = C(:,1)==k;
    e(k) = mean(ev(cluster));
    s(k) = sum(cluster);
    d = C(cluster,2);
    D0(k) = d(1);
  end
end

%--------------------------------------------------------------
function dmin = ecmin(e,C,k,l)
%ECMIN Compute the minimum distance between two clusters.

dmin = inf;

%Ck = C==k;
%Cl = C==l;

ek = e(C==k);
el = e(C==l);

for s = 1:length(ek)
  delta = abs(el-ek(s));
  if isinf(dmin) || any(delta < dmin)
    dmin = min(delta);
  end
end

%--------------------------------------------------------------
function dmax = ecmax(e,C,k,l)
%ECMAX Compute the maximum distance between two clusters.

dmax = -1;

%Ck = C==k;
%Cl = C==l;

ek = e(C==k);
el = e(C==l);

for s = 1:length(ek)
  delta = abs(el-ek(s));
  if any(delta > dmax)
    dmax = max(delta);
  end
end
if dmax == -1
  dmax = inf;
end

%--------------------------------------------------------------
function davg = ecavg(e,C,k,l)
%ECAVG Compute the average distance between two clusters.

davg = 0;

%Ck = C==k;
%Cl = C==l;

ek = e(C==k);
el = e(C==l);

for s = 1:length(ek)
  davg = davg + sum(abs(el-ek(s)));
end
if davg > 0
  davg = davg/(length(ek)*length(el));
else
  davg = inf;
end

%--------------------------------------------------------------
function dmean = ecmean(e,C,k,l)
%ECMEAN Compute the mean distance between two clusters.

dmean = inf;

%Ck = C==k;
%Cl = C==l;

ek = e(C==k);
el = e(C==l);

if ~isempty(ek) && ~isempty(el)
  dmean = abs(mean(ek) - mean(el));
end

%--------------------------------------------------------------
function dvar = ecvar(e,C,k,l)
%ECVAR Compute the distance between two clusters using the variance.

dvar = inf;

ek = e(C==k);
el = e(C==l);
if ~isempty(ek) && ~isempty(el)
  dvar = var([ek; el]);
end


