function varargout = cenorm(A,varargin)
%CENORM Compute and cluster eigenvalues of a matrix or matrix pencil. 
%   [Ev,Sizes] = CENORM(A,Tol,OnlyLargest) clusters the eigenvalues
%   corresponding to a numerically multiple eigenvalue of the matrix A. Two
%   eigenvalues e1 and e2 that are within abs(e1-e2) <= Tol are classified
%   to belong to the same cluster. The vectors Ev and Sizes return the mean
%   values of the eigenvalues in each isolated cluster and the cluster
%   sizes, respectively. Set the OnlyLargest to true if only the cluster
%   containing the largest eigenvalue should be determined, otherwise
%   false.
%
%   [Ev,Sizes] = CENORM(G,H,Tol,OnlyLargest) clusters the generalized
%   eigenvalues of the square matrix pencil G-sH. The same method as for
%   matrices is used.
%
%   [Ev,Sizes,E,EvCluster] = CENORM(...) also returns all computed
%   eigenvalues E of A or G-sH and the clustering of the eigenvalues in
%   EvCluster. The integer value k in EvCluster(i) represents the cluster
%   the eigenvalue E(i) belongs to, where Ev(k) is the new eigenvalue for
%   the cluster. The eigenvalues in E are sorted in non-increasing order.
%   If OnlyLargest is true, only the clustering for the first (largest)
%   eigenvalue is correct.
%
%   See also MCLUSTER, PCLUSTER.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(2,4)

if nargin == 4
  eigv = sort(eig(A,varargin{1}), 'descend');
  karg = 1;
else
  eigv = sort(eig(A),'descend');
  karg = 0;
end
tol = varargin{karg+1};
onlylargest = varargin{karg+2};

n = length(eigv);
lambda = zeros(n,1);
neigs = zeros(n,1);
cluster = zeros(n,1);
i = 1;
j = 0;
while (i <= length(eigv) && ~onlylargest ) || (onlylargest && i == 1)
  e = eigv(i);
  s = 1;
  while i+s <= length(eigv) && (abs(eigv(i+s)-eigv(i+s-1)) <= tol)
    e = e + eigv(i+s);
    s = s+1;
  end
  j = j+1;
  lambda(j) = e/s;
  neigs(j) = s;
  cluster(i:i+s-1) = j;
  i = i+s;
end

if nargout < 2
  varargout{1} = lambda(1:j);
elseif nargout == 2
  varargout{1} = lambda(1:j);
  varargout{2} = neigs(1:j);
elseif nargout == 4
  varargout{1} = lambda(1:j);
  varargout{2} = neigs(1:j);
  varargout{3} = eigv;
  varargout{4} = cluster;
end