function varargout = fstr(G,H,e,tol,gap,setzero,fweyr,forceimpose)
%FSTR F-staircase form of the finite eigenvalue blocks.
%   [FWeyr,Flag] = FSTR(G,H,Ev,Tol,Gap,SetZero) finds in the square matrix
%   pencil G-sH the regular blocks associated with the finite eigenvalue Ev
%   with respect to Tol and Gap. If Ev is given as a two-element vector Ev
%   = [Eig,Size] the second element Size is used to impose the total size
%   of the blocks associated with the given eigenvalue Eig. In this case,
%   Flag indicates if any warnings are issued when imposing the size.
%   Possible values of Flag are (or a sum of):
%      0 - No warnings.
%      1 - The Tol tolerance is increased.
%      2 - The Gap tolerance is decreased.
%      4 - Failed to compute a stable Gap tolerance.
%      8 - The matrix H does not have full rank.
%   Tol is a vector [TolG,TolH], where, together with Gap, TolG is used for
%   the column compression of G and TolH for the column compression of H.
%   The Weyr characteristics of the blocks are returned in FWeyr. If the
%   parameter SetZero is true, the lower parts of the transformed G and H
%   are forced to zero, otherwise not.
%
%   [S,T,FWeyr,Flag] = FSTR(G,H,Ev,Tol,Gap,SetZero) also returns the
%   deflated S and T in the Guptri staircase form.
%
%   [S,T,P,Q,FWeyr,Flag] = FSTR(G,H,Ev,Tol,Gap,SetZero) or
%   [S,T,P,Q,E,F,FWeyr,Flag] = FSTR(G,H,Ev,Tol,Gap,SetZero) also returns
%   the transformation matrices P and Q, such that P(S-sT)Q = G-sH. If
%   SetZero is true, the imposed perturbations are returned in E and F such
%   that P(S-sT)Q = (G+E)-s(H+F), otherwise E and F are zero matrices.
%
%   [S,T,IFlag] = FSTR(G,H,Ev,Tol,Gap,SetZero,FWeyr,ForceImpose) returns
%   the deflated matrices S and T, with the imposed structure given by
%   FWeyr with the associated eigenvalue Ev. Passing Ev as a two-element
%   vector has no meaning when imposing structures. If imposed separation
%   succeeds, IFlag is 0. Otherwise, IFlag is
%      1 - ForceImpose = false and the largest singular value imposed to
%          zero is greater than Tol*Gap. G and H are returned in S and T,
%          respectively.
%      2 - ForceImpose = true and the largest singular value(s) is/are
%          forced to zero even if they are larger than Tol*Gap. In addition
%          a warning is issued, which can be suppressed with
%             warning('off','MCS:nullityimpose:ImposedZeroSingularValue')
%
%   [S,T,P,Q,IFlag] = FSTR(G,H,Ev,Tol,Gap,SetZero,FWeyr,ForceImpose) also
%   returns the transformation matrices P and Q, used to form S and T.
%
%   [S,T,P,Q,E,F,IFlag] = FSTR(G,H,Ev,Tol,Gap,SetZero,FWeyr,ForceImpose)
%   also returns the matrices E and F with the imposed perturbations, such
%   that P(S-sT)Q = (G+E)-s(H+F).
%
%   See also PGUPTRI, RZSTR, RISTR, LISTR, LZSTR.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

% !Argument check disabled!
% % Argument check
% narginchk(6,8);
% nargoutchk(1,7);
% 
% % Checking required arguments
% error(sizechk('square',G));
% error(sizechk('square',H));
% error(sizechk('equal',G,H));

if size(e,1) ~= 1 || size(e,2) > 2 || size(e,2) < 1
  error('The parameter ''Ev'' must be of length one or two.');
end

% Set options
impose   = nargin == 8;
forcesz = length(e) == 2;
compst  = impose || nargout > 2;
compef  = nargout > 6 || (nargout > 5 && impose);
comppq  = (nargout > 3 && impose) || (nargout > 3);

gn = size(G,1); % Initial G, gn-by-gn

% If 'forsesz' is true the total size of the blocks ('forcen') to the
% specified eigenvalue is given as input. FSTR will try to impose this size
% and if needed adjusting the tolerance parameters.
if forcesz
  forcen = e(2);
end

% If 'compst' is true the staircase matrices S and T are explicitly
% computed.
if compst
  S = zeros(gn,gn); % Template for S, gn-by-gn
  T = zeros(gn,gn); % Template for T, hn-by-hn
end
% If 'compef' is true the error matrices E and F are explicitly computed.
if compef
  E = zeros(gn,gn); % Template for E, gn-by-gn
  F = zeros(gn,gn); % Template for F, hn-by-hn
end
% If 'comppq' is true the tranformation matrices P and Q are explicitly
% computed.
if comppq
  P = eye(gn,gn);   % Template for P, gn-by-gn
  Q = eye(gn,gn);   % Template for Q, gn-by-gn
end

% Intitial indices
% If 'impose' is true the nullspace dimensions are given as input
% and imposed. The Weyr indices are stored in 'nidx'.
if impose
  nidx = [fweyr 0];
  % Store original matrices if separation fails
  Gin = G;
  Hin = H;
else
  nidx = [];
end

iflag = 0;
nflag = 0;
nflagH = 0;
revP = false;

if isinf(e(1))
  warning('MCS:fstr:InfiniteEigenvalue',...
    ['E is the infinite eigenvalue. ', ...
    'The staircase form is computed on the reversed pencil H - mu*G. ',... 
    'Better to use listr or ristr instead.']);
  tmp = G;
  H   = G;
  G   = tmp;
  e(1) = 0;
  revP = true;
end

if size(H,1) > rank(H)
  % H should be of full rank
  if impose
    [Uh,Sh,Vh] = svd(H); % svd in decreasing order
    [Sht,iflag] = nullityimpose(Sh,tol(2),gap,'column',0,1);
    H = H + Uh*Sht*Vh';
    F = F + Uh*Sht*Vh';
  else
    nflagH = 8;
    warning('MCS:fstr:SingularH','H does not have full rank.');
  end
end

%----------------------------------------------------------------
% Compute the RZ-staircase form of the shifted pencil (G-Eig*H) - s*H.
%

% Shift with the eigenvalue Ev
G = G - e(1)*H;

%----------------------------------------------------------------
% Begin iteration
%
%            gn
%              n
%       gi-1 ni
%        +--------+
%    gi-1|  |     |
%        |--+--+--+
% gn  ni |  |  |  |
%    n   |  +--+--+
%        |  |  |  |
%        +--+--+--+

i = 1;              % i  = number of iterations
gi = 1;             % gi = global row index
ni = gn;            % ni ~= 0 (just to get into the loop)
n = size(G,1);
while (ni > 0 || (forcesz && forcen > 0)) && n > 0
  
  %--------------------------------------------------------------
  % Step 1. Compress the columns in Gi to give ni
  [Ug,Sg,Vg] = isvd(G);
  if impose
    % We impose a given nullspace dimension.
    ni = nidx(i);
    
    [Sgt,iflag] = nullityimpose(Sg,tol(1),gap,'column',ni,forceimpose);
    if iflag && ~forceimpose
      break;
    end
    Sg  = Sg+Sgt;
    
    % Update E
    if compef
      E = E + P(:,gi:end)*Ug*Sgt*Vg'*Q(:,gi:end)';
    end
  elseif forcesz
    tolG = tol(1);
    % The total block sizes are given as input and stored in the 'forcen'
    % variable. In the end this must be satisfied.
    
    % Notice that 'forcen' is the total block size and 'ni' is a Weyr
    % index. However, the sum of all 'ni' should still be 'forcen'.
    [ni,tolG,gap]  = nullitycheck(Sg,tolG,gap,'column');
    while ni > 0 && forcen - ni < 0
      % The size of the remaining nullspace is too large with the given
      % tolerance parameters. Tries to adjust the tolerances tolG and gap
      % and thereby satisfying the 'forcen' condition.
      [ni,tolG,gap,flag] = nullitycheck(Sg,tolG,gap,'column',forcen);
      if flag
        nflag = flag;
      end
    end
    if ni == 0 && forcen > 0
      % No remaing nullspace is found with the given tolerance parameters.
      % Tries to adjust the parameters and thereby satisfying the 'forcen'
      % condition.
      [ni,~,~,flag] = nullitycheck(Sg,tolG,gap,'column',forcen);
      if flag
        nflag = flag;
      end
    end
    forcen = forcen - ni;
  else
    % No restrictions on the size of the nullspace is given as input.
    ni = nullitycheck(Sg,tol(1),gap,'column');
  end
  
  % Update G and H
  G = Ug*Sg;  % == G*Vg
  H = H*Vg;
  % Update S and T
  if compst
    S(1:gi-1,gi:end) = S(1:gi-1,gi:end)*Vg;
    T(1:gi-1,gi:end) = T(1:gi-1,gi:end)*Vg;
  end
  % Update P and Q
  if comppq
    Q(:,gi:end)      = Q(:,gi:end)*Vg;
  end

  %--------------------------------------------------------------
  % NOTE! Step 2 in the RZ algorithm is skiped since H has full rank
  
  %--------------------------------------------------------------
  % Step 3. Triangularize [H0 H2] to give Pi
  
  [Pi, Ri] = qr(H); % qr([H0 H2])
  
  % Update Hi
  H = Ri;
  
  % Update Gi
  G(:,ni+1:end) = Pi'*G(:,ni+1:end);
  
  % Update S and T
  if compst
    S(gi:end,1:gi-1)  = Pi'*S(gi:end,1:gi-1);
    T(gi:end,1:gi-1)  = Pi'*T(gi:end,1:gi-1);
  end
  % Update P and Q
  if comppq
    P(:,gi:end) = P(:,gi:end)*Pi;
  end
  
  %--------------------------------------------------------------
  % Preparing next iteration step
  
  % Set zeros on deflated part
  if setzero
    % Update E and F
    if compef
      E(:,1:ni) = E(:,1:ni) + G(:,1:ni);
      F(ni+1:end,1:ni) = F(ni+1:end,1:ni) + H(ni+1:end,1:ni);
    end
    G(:,1:ni)        = 0;
    H(ni+1:end,1:ni) = 0;
  end
  
  % Update S and T
  if compst
    S(gi:end,gi:end) = G;
    T(gi:end,gi:end) = H;
  end
  
  % New G
  G = G(ni+1:end,ni+1:end);
  
  % New H
  H = H(ni+1:end,ni+1:end);
  
  % Store indices
  if ~impose
    nidx(i) = ni;
  end
  
  % Update iterators
  i = i+1;
  gi = gi + ni;
  n = size(G,1);
end


%--------------------------------------------------------------
% Create output

if impose && iflag == 1
  % If separation failed
  S = Gin;
  T = Hin;
  if comppq
    P = eye(gn,gn);
    Q = eye(gn,gn);
  end
  if compef
    E = zeros(gn,gn);
    F = zeros(gn,gn);
  end
end

if compst
  if revP
    tmp = S;
    S   = T;
    T   = tmp;
  end
  
  % Shift back the output with the eigenvalue Ev
  S = S+e(1)*T;
end

if impose
  varargout{1} = S;
  varargout{2} = T;
  if nargout == 3
    varargout{3} = iflag;
  else
    varargout{3} = P;
    varargout{4} = Q;
    if nargout == 5
      varargout{5} = iflag;
    else
      varargout{5} = E;
      varargout{6} = F;
      if nargout == 7
        varargout{7} = iflag;
      end
    end
  end
else
  if nargout < 3
    varargout{1} = nidx;
    if nargout == 2
      varargout{2} = nflag + nflagH;
    end
  else
    varargout{1} = S;
    varargout{2} = T;
    if nargout < 5
      varargout{3} = nidx;
      if nargout == 4
        varargout{4} = nflag + nflagH;
      end
    elseif nargout < 7
      varargout{3} = P;
      varargout{4} = Q;
      varargout{5} = nidx;
      if nargout == 6
        varargout{6} = nflag + nflagH;
      end
    else
      varargout{3} = P;
      varargout{4} = Q;
      varargout{5} = E;
      varargout{6} = F;
      varargout{7} = nidx;
      if nargout == 8
        varargout{8} = nflag + nflagH;
      end
    end
  end
end