/* GSYLVE mex function for solving the generalized Sylvester equation.
 *
 * Matlab call:
 *   [X,Y] = GSYLVE(A,B,C,D,E,F) solves the generalized Sylvester linear
 *   matrix equation
 *
 *         A*X - Y*B = Scale*C,
 *   and
 *         D*X - Y*E = Scale*F,
 *
 *   where A and D are M-by-M matrices, B and E are N-by-N matrices, and
 *   C, F, X and Y are M-by-N matrices. The matrix pairs (A,D) and (B,E) 
 *   are asumed to be in generalized Schur form.
 *
 *   The scaling factor Scale, 0 <= Scale <= 1, is chosen to avoid
 *   overflow. Normally, Scale = 1. If Scale is not equal to 1 a warning
 *   is issued. If 0 < Scale < 1, X and Y are the solutions to a system
 *   with slightly perturbed C and F. If Scale = 0, X and Y is the
 *   solutions to the homogeneous system with C = F = 0.
 *
 *   [X,Y,Dif] = GSYLVE(A,B,C,D,E,F) also returns a Frobenius norm-based 
 *   estimate Dif, which is an upper bound of Dif[(A,D),(B,E)] and measures 
 *   the separation of the spectrum of the matrix pair (A,D) from the spectrum 
 *   of the matrix pair (B,E). 
 *
 *   [X,Y,Dif,Scale] = GSYLVE(A,B,C,D,E,F) also returns the scaling factor. 
 *   In this case, no warning is issued.
 *
 *   The mex-function calls the Lapacke routine ZTGSYL to do the
 *   computations.
 *
 *   This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
 *   MCS Toolbox is a freely-available software package licensed under the modified
 *   BSD license.
 */

#define LAPACK_COMPLEX_CPP
#define HAVE_LAPACK_CONFIG_H

#include "mex.hpp"
#include "mexAdapter.hpp"
#include "lapacke.h"

#ifndef MAX
#define MAX(x,y) (((x) > (y)) ? (x) : (y))
#endif

using namespace matlab::data;
using matlab::mex::ArgumentList;
using matlab::engine::convertUTF8StringToUTF16String;

class MexFunction : public matlab::mex::Function {
public:
  void operator()(ArgumentList outputs, ArgumentList inputs) {
    std::ostringstream stream;

    checkArguments(outputs, inputs);

    lapack_int m = inputs[0].getDimensions().at(0);
    lapack_int n = inputs[1].getDimensions().at(0);
    ArrayDimensions outDimensions = inputs[2].getDimensions();
    
    if (m == 0 || n == 0) {
      outputs[0] = factory.createArray<lapack_complex_double>(outDimensions);
      outputs[1] = factory.createArray<lapack_complex_double>(outDimensions);
      if (outputs.size() > 2)
        outputs[2] = factory.createScalar<double>(0.0);
      if (outputs.size() > 3)
        outputs[3] = factory.createScalar<double>(1.0);
      return; 
    }
    
    lapack_int ijob = 0;
    if (outputs.size() > 2 )  {
      // Look ahead strategy is used to estimate Dif[(A,D), (B,E)]
      ijob = 1;
    }

    buffer_ptr_t<lapack_complex_double> A = inputArrayToComplex(inputs[0]);
    buffer_ptr_t<lapack_complex_double> B = inputArrayToComplex(inputs[1]);
    buffer_ptr_t<lapack_complex_double> C = inputArrayToComplex(inputs[2]);
    buffer_ptr_t<lapack_complex_double> D = inputArrayToComplex(inputs[3]);
    buffer_ptr_t<lapack_complex_double> E = inputArrayToComplex(inputs[4]);
    buffer_ptr_t<lapack_complex_double> F = inputArrayToComplex(inputs[5]);

    const lapack_complex_double *Aptr = A.get();
    const lapack_complex_double *Bptr = B.get();
    lapack_complex_double *Cptr = C.release();
    const lapack_complex_double *Dptr = D.get();
    const lapack_complex_double *Eptr = E.get();
    lapack_complex_double *Fptr = F.release();
    
    lapack_int LDA = MAX(1,m);
    lapack_int LDB = MAX(1,n);
    lapack_int LDC = LDA;
    lapack_int LDD = LDA;
    lapack_int LDE = LDB;
    lapack_int LDF = LDA;
    
    double scale, dif;
    lapack_int info;
    char trans = 'N';
    
    // Solve the generalized Sylvester equation
    info = LAPACKE_ztgsyl(LAPACK_COL_MAJOR, trans, ijob, m, n, Aptr, LDA, Bptr, 
            LDB, Cptr, LDC, Dptr, LDD, Eptr, LDE, Fptr, LDF, &scale, &dif);
    
    if (info < 0) {
      stream << "Argumnet " << -info << " to ztgsyl has an illegal value.";
      returnMatlabError(stream);
    }
    
    TypedArray<lapack_complex_double> X = factory.createArray<lapack_complex_double>(outDimensions, Cptr, Cptr+(m+n));
    TypedArray<lapack_complex_double> Y = factory.createArray<lapack_complex_double>(outDimensions, Fptr, Fptr+(m+n));
    outputs[0] = X;
    outputs[1] = Y;
    if (outputs.size() > 2)
      outputs[2] = factory.createScalar(dif);
    if (outputs.size() > 3)
      outputs[3] = factory.createScalar(scale);

    if (info > 0) {
      stream << "The pencils (A,D) and (B,E) have common or close eigenvalues.";
      returnMatlabWarning("MCS:gsylve:Info", stream);
    }
    
    if (scale != 1.0 && outputs.size() < 4) {
      stream << "The right hand side were scaled by " << scale << " to avoid overflow.";
      returnMatlabWarning("MCS:gsylve:Scale", stream);
    }
  }

private:
  std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
  ArrayFactory factory;

  
  // *************************************
  buffer_ptr_t<lapack_complex_double> inputArrayToComplex(Array inArray) {
    if (inArray.getType() == ArrayType::COMPLEX_DOUBLE) {
      TypedArray<lapack_complex_double> outArray = std::move(inArray);
      return outArray.release();
    } else {
      // Convert from double to complex storage
      ArrayDimensions dim = inArray.getDimensions();
      TypedArray<lapack_complex_double> outArray = factory.createArray<lapack_complex_double>(dim);
      for (int n = 0; n < dim.at(1); ++n) {
        for(int m = 0; m < dim.at(0); ++m) {
          outArray[m][n] = std::complex<double>(inArray[m][n], 0.0);
        }
      }
      
      return outArray.release();
    }
  }
  
  // *************************************
  void checkArguments(ArgumentList outputs, ArgumentList inputs) {
    
    ArrayDimensions dim;
    std::ostringstream stream;
    
    // Check number of inputs
    if (inputs.size() != 6) {
      stream << "GSYLVE require 6 input arguments.";
      returnMatlabError(stream);
    }
    
    // Check number of outputs
    if (outputs.size() < 2 || outputs.size() > 4) {
      stream << "GSYLVE require at least 2 and at most 4 output arguments.";
      returnMatlabError(stream);
    }
    
    // Check dimensions of input parameters
    //
    //   A(MxM), D(MxM), B(NxN), E(NxN), C(MxN), and F(MxN)
    
    // Check all data matrices
    for (int k = 0; k < 6; ++k) {
      if ((inputs[k].getType() != ArrayType::DOUBLE &&
              inputs[k].getType() != ArrayType::COMPLEX_DOUBLE) ||
              inputs[k].isEmpty())
      {
        stream << "Input " << k << " must be a non-empty real or complex array.";
        returnMatlabError(stream);
      }
    }
    
    size_t m = inputs[0].getDimensions().at(0);
    size_t n = inputs[1].getDimensions().at(0);
    
    dim = inputs[0].getDimensions();
    if (dim.size() != 2 || dim[0] != dim[1]) {
      stream << "Matrix A must be an MxM array.";
      returnMatlabError(stream);
    }
    
    dim = inputs[1].getDimensions();
    if (dim.size() != 2 || dim[0] != dim[1]) {
      stream << "Matrix B must be an NxN array.";
      returnMatlabError(stream);
    }
    
    dim = inputs[2].getDimensions();
    if (dim.size() != 2 || dim[0] != m || dim[1] != n) {
      stream << "Matrix C must be an MxN array.";
      returnMatlabError(stream);
    }
    
    dim = inputs[3].getDimensions();
    if (dim.size() != 2 || dim[0] != m || dim[0] != dim[1]) {
      stream << "Matrix D must be an MxM array.";
      returnMatlabError(stream);
    }
    
    dim = inputs[4].getDimensions();
    if (dim.size() != 2 || dim[0] != n || dim[0] != dim[1]) {
      stream << "Matrix E must be an NxN array.";
      returnMatlabError(stream);
    }
    
    dim = inputs[5].getDimensions();
    if (dim.size() != 2 || dim[0] != m || dim[1] != n) {
      stream << "Matrix F must be an MxN array.";
      returnMatlabError(stream);
    }
    
  }

  // *************************************
  void dispMatlab(std::ostringstream& stream)
  {
    dispMatlab(stream.str());
    stream.str("");
  }
  
  // *************************************
  void dispMatlab(std::string str)
  {
    matlabPtr->feval(u"disp", 0,
            std::vector<Array>({ factory.createScalar(str) }));
  }

  // *************************************
  void returnMatlabError(std::ostringstream& stream)
  {
    returnMatlabError("MCS:gsylve:Error", stream);
  }

  // *************************************
  void returnMatlabError(std::string id, std::ostringstream& stream)
  {
    matlabPtr->feval(u"error", 0,
            std::vector<Array>({ factory.createScalar(id), 
                    factory.createScalar(stream.str()) }));
    stream.str("");
  }

  // *************************************
  void returnMatlabWarning(std::string id, std::ostringstream& stream)
  {
    matlabPtr->feval(u"warning", 0,
            std::vector<Array>({ factory.createScalar(id), 
                    factory.createScalar(stream.str()) }));
    stream.str("");
  }
};
