function varargout = jstr(A,e,tol,gap,setzero,reltol,jweyr)
%JSTR Staircase form of the finite eigenvalue blocks.
%   [JWeyr,Flag] = JSTR(A,Ev,Tol,Gap,SetZero) finds in the matrix A the
%   Jordan blocks associated with the eigenvalue Ev with respect to Tol and
%   Gap. If Ev is given as a two-element vector E = [Eig,Size] the second
%   element Size is used to impose the total size of the blocks associated
%   with the given eigenvalue Eig.  In this case, Flag indicates if any
%   warnings are issued when imposing the size. Possible values of Flag are
%   (or a sum of):
%      0 - No warnings.
%      1 - The Tol tolerance is increased.
%      2 - The Gap tolerance is decreased.
%      4 - Failed to compute a stable Gap tolerance.
%   Tol and Gap are used for the column compression of A. The Weyr
%   characteristics of the blocks are returned in JWeyr. If the parameter
%   SetZero is true, the lower parts of the transformed A are forced to
%   zero, otherwise not.
%
%   [J,JWeyr,Flag] = JSTR(A,Ev,Tol,Gap,SetZero) also returns the deflated
%   matrix J in Jordan-Schur staircase form.
%
%   [J,Z,JWeyr,Flag] = JSTR(A,Ev,Tol,Gap,SetZero) also returns the
%   transformation matrix Z, such that Z*J*Z' = A.
%
%   ... = JSTR(A,Ev,Tol,Gap,SetZero,Reltol) scales the tolerance Tol with
%   largest eigenvalue if Reltol is true.
%
%   [J,IFlag] = JSTR(A,Ev,Tol,Gap,SetZero,Reltol,JWeyr,ForceImpose) returns
%   the deflated matrix J, with the imposed structure given in JWeyr and
%   the associated eigenvalue Ev. Passing Ev as a two-element vector has no
%   meaning when imposing structures. If imposed separation succeeds, IFlag
%   is 0. If the largest singular value imposed to zero is greater than
%   Tol*Gap and ForceImpose is false, IFlag returns 1 and A is returned in
%   J. Otherwise, if ForceImpose is true, IFlag returns 2 and the singular
%   values are forced to zero even if they are larger than Tol*Gap.
%
%   [J,Z,IFlag] = JSTR(A,Tol,Gap,SetZero,Reltol,JWeyr,ForceImpose) also
%   returns the transformation matrix Z used to form J.
%
%   [J,Z,E,IFlag] = JSTR(A,Tol,Gap,SetZero,Reltol,JWeyr,ForceImpose) also
%   returns the matrix E with the imposed perturbations, such that Z*J*Z' =
%   (A+E).
%
%   See also MGUPTRI, FSTR.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

% Argument check
narginchk(5,7);
nargoutchk(0,4);

% Checking required arguments
error(sizechk('square',A));
if size(e,1) ~= 1 || size(e,2) > 2 || size(e,2) < 1
  error('The parameter ''Ev'' must be of length one or two.');
end

% Set options
impose   = nargin == 7;
forcesz = length(e) == 2;
compe  = nargout == 3 && impose;
compz  = (nargout > 1 && impose) || (nargout > 2 && ~impose);
if nargin < 6, reltol = false; end

n = size(A,1);   % Initial A, n-by-n

% If 'impose' is true the nullspace dimensions are given as input
% and imposed. The Weyr indices are stored in 'nidx'.
if impose
  nidx = [jweyr(:); 0];
  % Store original matrix if separation fails
  Ain = A;
else
  nidx = [];
end

% If 'forsesz' is true the total size of the blocks ('forcen') to the
% specified eigenvalue is given as input. JSTR will try to impose this size
% and if needed adjusting the tolerance parameters.
if forcesz
  forcen = e(2);
end

% If 'compe' is true the error matrix E is explicitly computed.
if compe
  E = zeros(n); % Template for E, an-by-an
end
% If 'compp' is true the transformation matrix P is explicitly computed.
if compz
  Z = eye(n);   % Template for Z, am-by-am
end

iflag = 0;
nflag = 0;

% Shift A with the eigenvalue e
A  = A - e(1)*eye(n);

%----------------------------------------------------------------
% Begin iteration
%
%            gn
%              n
%       gi-1 ni
%        +--------+
%    gi-1|  |     |
%        |--+--+--+
% gn  ni |  |  |  |
%    n   |  +--+--+
%        |  |  |  |
%        +--+--+--+

i = 1; % number of iterations
k = 1;
r = n;
ctol = tol; % Tolerance used in finding null spaces. This is
            % adjusted if a relative tolerance is used.
while ((r > 0) || (forcesz && forcen > 0)) && k <= n
  % Calculate the singular values in increasing order.
  [U,S,V] = isvd(A(k:n,k:n));
  
  if reltol
    % Tolerance scaled with the largest eigenvalue.
    ctol = S(n-k+1,n-k+1)*tol;
  end
  
  if impose
    % We impose a given nullspace dimension
    
    r = nidx(i); % Nullspace
    
    % The following nullspace
    if length(nidx) < i+1
      r2 = 0;           
    else
      r2 = nidx(i+1);
    end
    
    %% TODO Revise!
    rc = nullitycheck(S,ctol,gap,'column');
    if rc < r
      % To small nullspace, i.e., singular values must be removed.
      [St,iflag] = nullityimpose(S,ctol,gap,'column',r,forceimpose);
      if iflag && ~forceimpose
        break;
      end
      S = S + St;

      % Update E
      if compe
        E = E + Z(:,k:n)*U*St*V'*Z(:,k:n)';
      end
    end
    
    if r2 > 0
      % Forse A(k:k+r-1,k+r:k+r+r2-1) to have full column rank. svd in
      % decreasing order
      [SeU,SeS,SeV] = svd(A(k:k+r-1,k+r:k+r+r2-1));
      Se = SeU*(nullityimpose(SeS,ctol,gap,'column',0))*SeV;
      A(k:k+r-1,k+r:k+r+r2-1) = A(k:k+r-1,k+r:k+r+r2-1) + Se;
      if compe
        Ea = zeros(n-k+1);
        Ea(1:r,1+r:r+r2) = Se;
        E = E + Z(:,k:n)*V*Ea*V'*Z(:,k:n)';
      end
    end
    % End revise
  elseif forcesz
    % The total block sizes are given as input and stored in the 'forcen'
    % variable. In the end this must be satisfied.
    
    % Notice that 'forcen' is the total block size and 'r' is a Weyr index.
    % However, the sum of all 'r' should still be 'forcen'.
    [r,ctol,gap]  = nullitycheck(S,ctol,gap,'column');
    while r > 0 && forcen < r 
      % The size of the remaing nullspace is too large with the given
      % tolerance parameters. Tries to adjust the parameters and thereby
      % satisfying the 'forcen' condition.
      [r,ctol,gap,flag] = nullitycheck(S,ctol,gap,'column',forcen);
      if flag
        nflag = flag;
      end
    end
    if r == 0 && forcen > 0
      % No remaing nullspace is found with the given tolerance parameters.
      % Tries to adjust the parameters and thereby satisfying the 'forcen'
      % condition.
      [r,~,~,flag] = nullitycheck(S,ctol,gap,'column',forcen);
      if flag
        nflag = flag;
      end
    end
    forcen = forcen - r;
  else
    % No restrictions on the size of the nullspace is given as input.
    r = nullitycheck(S,ctol,gap,'column');
  end

  %Update A
  A(1:k-1,k:n) = A(1:k-1,k:n)*V;
  A(k:n,k:n)   = V'*U*S; % == V'*A*V
  
  % Update P
  if compz
    Z(:,k:n) = Z(:,k:n)*V;
  end
  
  % Set zeros on deflated part
  if setzero
    % Update E
    if compe
      E(k+r:n,k:k+r-1) = E(k+r:n,k:k+r-1) + A(k+r:n,k:k+r-1);
    end
    A(k+r:n,k:k+r-1) = 0;
  end
  
  % Store indices
  if ~impose
    nidx(i) = r;
  end
  
  % Update iterators
  i = i+1;
  k = k+r;
end

% Shift back the output with the eigenvalue Ev
A = A+e(1)*eye(n);

%--------------------------------------------------------------
% Create output

if ~impose
   if nidx(i-1) ~= 0
     % Add a terminating zero
     nidx(i) = 0;
   end
   l = length(nidx);
   fweyr = zeros(1,l);
   if l > 0
     fweyr(l) = nidx(l);
     for i = l-1:-1:1
       fweyr(i) = max([0 fweyr(i+1:l)]) + nidx(i) - nidx(i+1);
     end
   end
elseif iflag == 1
  % If separation failed
  A = Ain;
  if comppq
    Z = eye(n);
  end
  if compef
    E = zeros(n);
  end
end

if impose
  varargout{1} = A;
  switch nargout
    case 2
      varargout{2} = iflag;
    case 3
      varargout{2} = Z;
      varargout{3} = iflag;
    case 4
      varargout{2} = Z;
      varargout{3} = E;
      varargout{4} = iflag;
  end
else
  switch nargout
    case {0,1}
      varargout{1} = fweyr;
    case 2
      varargout{1} = fweyr;
      varargout{2} = nflag;
    case 3
      varargout{1} = A;
      varargout{2} = fweyr;
      varargout{3} = nflag;
    case 4
      varargout{1} = A;
      varargout{2} = Z;
      varargout{3} = fweyr;
      varargout{4} = nflag;
  end
end
