function varargout = listr(G,H,tol,gap,setzero,lweyr,iweyr,forceimpose)
%LISTR LI-staircase form of the left singular and infinite eigenvalue blocks.
%   [LWeyr,IWeyr] = LISTR(G,H,Tol,Gap,SetZero) finds the left singular
%   blocks and the Jordan blocks with associated infinite eigenvalue in
%   G-sH with respect to Tol and Gap. Tol is a vector [TolG TolH], where
%   TolG is used for the row compression of G and TolH for the row
%   compression of H. The Weyr characteristics of the blocks are returned
%   in LWeyr and IWeyr, respectively. If the parameter SetZero is true, the
%   lower parts of the transformed G and H are forced to zero, otherwise
%   not.
%
%   [S,T,LWeyr,IWeyr] = LISTR(G,H,Tol,Gap,SetZero) also returns the
%   deflated S and T in the GUPTRI staircase form.
%
%   [S,T,P,Q,LWeyr,IWeyr] = LISTR(G,H,Tol,Gap,SetZero) also returns the
%   transformation matrices P and Q, such that P(S-sT)Q' = G-sH.
%
%   [S,T,Flag] = LISTR(G,H,Tol,Gap,SetZero,LWeyr,IWeyr,ForceImpose) returns
%   the deflated matrices S and T, with the imposed structure given by
%   LWeyr and IWeyr. Flag is non-zero in the following cases. If the
%   largest singular value imposed to zero is greater than Tol*Gap and
%   ForceImpose is false, Flag returns 1 and G and H are returned in S and
%   T, respectively. Otherwise, if ForceImpose is true, Flag returns 2 and
%   the singular values are forced to zero even if they are larger than
%   Tol*Gap.
%
%   [S,T,P,Q,Flag] = LISTR(G,H,Tol,Gap,SetZero,LWeyr,IWeyr,ForceImpose)
%   also returns the transformation matrices P and Q, used to form S and T.
%
%   [S,T,P,Q,E,F,Flag] = LISTR(G,H,Tol,Gap,SetZero,LWeyr,IWeyr,ForceImpose)
%   also returns the matrices E and F with the imposed permutations, such
%   that P(S-sT)Q' = (G+E)-s(H+F).
%
%   See also LZSTR, RZSTR, RISTR, FSTR.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

% !Argument check disabled!
% % Argument check
% narginchk(5,8);
% nargoutchk(2,7);
% error(sizechk('ndims',G,2));
% error(sizechk('ndims',H,2));
% error(sizechk('equal',G,H));
% 
% if nargin ~= 5 && nargin ~= 8
%   error('Invalid number of input arguments');
% end

%----------------------------------------------------------------
% Debug switch
DEBUG = 0;

impose  = nargin == 8;           % Impose the given structure
compst = impose || nargout > 2;  % Compute S and T?
compef = nargout > 5 && impose;  % Compute E and F?
comppq = (nargout > 3 && impose) || (nargout > 4);  % Compute P and Q?

if DEBUG
  Gin = G;
  Hin = H;
  compst = true;
  comppq = true;
end

[gm, gn] = size(G); % Initial G, gm-by-gn

if compst
  S = zeros(gm,gn); % Template for S, gm-by-gn
  T = zeros(gm,gn); % Template for T, hm-by-hn
end
if comppq
  P = eye(gm,gm);   % Template for P, gm-by-gm
  Q = eye(gn,gn);   % Template for Q, gn-by-gn
end
if compef
  E = zeros(gm,gn); % Template for E, gm-by-gn
  F = zeros(gm,gn); % Template for F, hm-by-hn
end

% Intitial indices
if impose
  nidx = zeros(1,max(length(lweyr),length(iweyr))+1);
  nidx(1:length(lweyr)) = lweyr(1:end);
  nidx(1:length(iweyr)) = nidx(1:length(iweyr)) + iweyr(1:end);
  ridx = zeros(1,max(length(lweyr),length(iweyr))+1);
  ridx(1:length(lweyr)-1) = lweyr(2:end);
  ridx(1:length(iweyr))   = ridx(1:length(iweyr)) + iweyr(1:end);
  % Store original matrices if separation fails
  Gin = G;
  Hin = H;
else
  nidx = zeros(1,max(gn,gm));
  ridx = zeros(1,max(gn,gm));
end
iflag = 0;

%--------------------------------------------------------------
% Begin iteration
%                                                                                                    <                                                                                                        -
%            gn                                                                                      <                                                                                                        -
%           n                                                                                        <                                                                                                        -
%            ni                                                                                      <                                                                                                        -
%        +--------+                                                                                  <                                                                                                        -
%        |  |  |  |                                                                                  <                                                                                                        -
%    m   |--+--+  +                                                                                  <                                                                                                        -
% gm  ri |  |  |  |                                                                                  <                                                                                                        -
%        |--+--+--+                                                                                  <                                                                                                        -
%        |     |  |                                                                                  <                                                                                                        -
%        +--+--+--+     

i = 1;              % i  = number of iterations
ni = gn;            % ni ~= 0 (just to get into the loop)
ri = gm;            % ri ~= 0
while (ni >= 0) && (ri > 0)
  [m, n] = size(G);
  
  %--------------------------------------------------------------
  % Step 1. Compress the rows in Hi to give ni
  if n == 0 % m-by-0
    % Update indices
    if ~impose
      nidx(i) = m;
      ridx(i) = 0;
    end
    i = i+1;
    break % If n == 0 we are done
  elseif m == 0 % 0-by-n
    ni = 0;
  else          % m-by-n
    [Uh,Sh,Vh] = svd(H); % svd in decreasing order
    if impose
      ni  = nidx(i);
      
      [Sht,iflag] = nullityimpose(Sh,tol(2),gap,'row',ni,forceimpose);
      if iflag && ~forceimpose
        break;
      end
      Sh = Sh+Sht; % Impose zero singular value
    else
      ni = nullitycheck(Sh,tol(2),gap,'row');
    end
  end
  
  if ni > 0 && n > 0 % Notice: ni > 0 => m > 0
    % Update G and H
    H = Sh*Vh'; % == Uh'*H
    G = Uh'*G;
    % Update E and F
    if compef
      F = F + P(:,1:m)*(Uh*Sht*Vh')*Q(:,1:n)';
    end
    % Update S and T
    if compst
      S(1:m,n+1:gn) = Uh'*S(1:m,n+1:gn);
      T(1:m,n+1:gn) = Uh'*T(1:m,n+1:gn);
    end
    % Update P and Q
    if comppq
      P(:,1:m) = P(:,1:m)*Uh;
    end
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('LISTR Step 1:','Error = ', norm(G-Gtemp(1:m,1:n)) + ...
      norm(H-Htemp(1:m,1:n)),'\n', 'ni = ', ni,'\n');
  end
  
  %--------------------------------------------------------------
  % Step 2. Compress the rows in Gi to give ri
  
  if ni > 0 && n > 0
    [Ug,Sg,Vg] = svd(G(m-ni+1:m,:)); % svd in decreasing order
    if impose
      ri  = ridx(i);
      
      [Sgt,iflag] = nullityimpose(Sg,tol(1),gap,'row',ni-ri,forceimpose);
      if iflag && ~forceimpose
        break;
      end
      Sg = Sg+Sgt; % Impose zero singular value

      % Update E and F
      if compef
        E = E + P(:,m-ni+1:m)*Ug*Sgt*Vg'*Q(:,1:n)';
      end
    else
      ri = ni - nullitycheck(Sg,tol(1),gap,'row');
    end
    
    % Update G and H
    G(m-ni+1:m,:) = Sg*Vg'; % == Ug'*G(m-ni+1:m,:)
    H(m-ni+1:m,:) = Ug'*H(m-ni+1:m,:);

    % Update S and T
    if compst
      S(m-ni+1:m,n+1:gn) = Ug'*S(m-ni+1:m,n+1:gn);
      T(m-ni+1:m,n+1:gn) = Ug'*T(m-ni+1:m,n+1:gn);
    end
    % Update P and Q
    if comppq
      P(:,1:m) = P(:,1:m)*[ eye(m-ni) zeros(m-ni,ni)
        zeros(ni,m-ni) Ug];
    end
  else
    ri = 0;
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('LISTR Step 2:','Error = ', norm(G-Gtemp(1:m,1:n)) + ...
      norm(H-Htemp(1:m,1:n)),'\n', 'ri = ', ri,'\n');
  end
  
  %--------------------------------------------------------------
  % Step 3. Triangularize [G0; G2] to give Qi
  
  if ni > 0 && n > 0 % Notice: ni > 0 => m > 0
    [Ri,Qi] = rq(G(1:m-(ni-ri),:)); % qr([G0 G2])
    
    % Update Gi
    G(1:m-(ni-ri),:) = Ri;
    
    % Update Hi
    H = H*Qi';
    
    if setzero
      % Update E and F
      if compef
        E(m-(ni-ri)+1:m,1:n) = E(m-(ni-ri)+1:m,1:n) + G(m-(ni-ri)+1:end,:);
      end
      G(m-(ni-ri)+1:end,:) = 0;
    else
      G(m-(ni-ri)+1:end,:) = G(m-(ni-ri)+1:end,:)*Qi';
    end
    
    % Update S and T
    if compst
      S(m+1:end,1:n) = S(m+1:end,1:n)*Qi';
      T(m+1:end,1:n) = T(m+1:end,1:n)*Qi';
    end
    % Update P and Q
    if comppq
      Q(:,1:n) = Q(:,1:n)*Qi';
    end
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('LISTR Step 3:','Rel. error (G,H) = ', ...
      norm(G-Gtemp(1:m,1:n))/norm(Gtemp(1:m,1:n)) + ...
      norm(H-Htemp(1:m,1:n))/norm(Htemp(1:m,1:n)),'\n');
  end
  
  %--------------------------------------------------------------
  % Preparing next iteration step.
  
  % Set zeros on deflated part
  if setzero
    % Update E and F
    if compef
      E(m-ni+1:m,1:n-ri) = E(m-ni+1:m,1:n-ri) + G(m-ni+1:end,1:end-ri);
      F(m-ni+1:m,1:n) = F(m-ni+1:m,1:n) + H(m-ni+1:end,:);
    end
    G(m-ni+1:end,1:end-ri) = 0;
    H(m-ni+1:end,:) = 0;
  end
  
  % Update S and T
  if compst
    S(1:m,1:n) = G;
    T(1:m,1:n) = H;
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('LISTR Prep step:','Error = ', norm(S-Gtemp) + ...
      norm(T-Htemp),'\n');
  end
  
  % New H
  H = H(1:(m-ni),1:(n-ri));        % Warning: ?-by-0 if ni=n
  
  % New G
  G = G(1:(m-ni),1:(n-ri));
  
  % Update indices
  if ~impose
    nidx(i) = ni;
    ridx(i) = ri;
  end
  
  % Update iterators
  i = i+1;
end

%--------------------------------------------------------------
% Create output

% #L^T_(k-1) blocks = ni(k) - ri(k)
% #N_k blocks       = ri(k) - ni(k+1)

if ~impose
  l = i-1; % = length(nidx)
  lweyr = zeros(1,l);
  iweyr = zeros(1,l);
  if l > 0
    lweyr(l) = nidx(l)-ridx(l);
    iweyr(l) = ridx(l);
    for i = l-1:-1:1
      lweyr(i) = max([0 lweyr(i+1:l)]) + nidx(i)-ridx(i);
      iweyr(i) = max([0 iweyr(i+1:l)]) + ridx(i)-nidx(i+1);
    end
  end
elseif iflag == 1
  % If separation failed
  S = Gin;
  T = Hin;
  if comppq
    P = eye(gm,gm);
    Q = eye(gn,gn);
  end
  if compef
    E = zeros(gm,gn);
    F = zeros(gm,gn);
  end
end

if DEBUG && ~impose
  fdebug('Computed structure:', 'L weyr = ', lweyr, 'I weyr = ', iweyr)
  S,T
end

if nargout == 2 && ~impose
  varargout{1} = lweyr;
  varargout{2} = iweyr;
elseif compst
  varargout{1} = S;
  varargout{2} = T;
  varargout{3} = iflag;
end
if nargout == 4 && ~impose
  varargout{3} = lweyr;
  varargout{4} = iweyr;
elseif comppq
  varargout{3} = P;
  varargout{4} = Q;
  varargout{5} = iflag;
end
if nargout == 6 && ~impose
  varargout{5} = lweyr;
  varargout{6} = iweyr;
elseif compef
  varargout{5} = E;
  varargout{6} = F;
  varargout{7} = iflag;
end
