function varargout = mcegershgorin(T,goal,sizeE,varargin)
%MCEGERSHGORIN Compute and cluster eigenvalues of a matrix.
%   Ev = MCEGERSHGORIN(A,Goal,SizeE) or
%   [Ev,Sizes] = MCEGERSHGORIN(A,Goal,SizeE) clusters the eigenvalues
%   corresponding to a numerically multiple eigenvalue of the matrix A. The
%   vectors Ev and Sizes contain the mean values of the eigenvalues in each
%   isolated cluster and the cluster sizes, respectively. The heuristic is based
%   on Gershgorin circles without diagonal scaling. The function starts with
%   isolating the cluster closest to the point Goal in the complex plane. If the
%   cluster containing the largest eigenvalue is desired to be first then Goal
%   should be set to Inf. SizeE should be an estimate of the 1-norm of the
%   errors known to affect A.
%
%   [U,T,Ind] = MCEGERSHGORIN(A,Goal,SizeE) or
%   [Ev,Sizes,U,T,Ind] = MCEGERSHGORIN(A,Goal,SizeE) also returns the matrix
%   T=U'*A*U in complex Schur form, where U is unitary. The set Ind provides the
%   clustering of the eigenvalues in T, where Ind{k} is the indices for
%   eigenvalue Ev(k). For example, if Ind{1} = 1:5 and Ind{2} = 6:7 then A has
%   one Jordan block of size 5 corresponding to T(Ind{1},Ind{1}). The second
%   Jordan block is of size 2 and corresponds to T(Ind{2},Ind{2}).
%
%   [Ev,Sizes,E,EvCluster] = MCEGERSHGORIN(A,Goal,SizeE) returns all computed
%   eigenvalues E of A and the clustering of the eigenvalues in EvCluster. The
%   integer value k in EvCluster(i) represents the cluster the eigenvalue E(i)
%   belongs to, where Ev(k) is the new eigenvalue for the cluster. The
%   eigenvalues in E are sorted in non-increasing order from Goal. If
%   OnlyLargest is true (see below), only the clustering for the first (largest)
%   eigenvalue is correct.
%
%   ... = MCEGERSHGORIN(A,Goal,SizeE,OnlyLargest) set OnlyLargest to true if
%   only the cluster closest to Goal should be determined, otherwise false
%   (default).
%
%   ... = MCEGERSHGORIN(T,Goal,SizeE,OnlyLargest,U) asumes that T is
%   upper-triangular and U is the unitary matrix such that U*T*U' = A. If T is
%   not upper-triangular, U is discarded and T is assumed to be a general
%   matrix.
%
%   Algorithm:
%    The function uses the clustering algorithm described in 
%    B. Kagstrom and P. Wiberg. Extracting partial canonical structures for
%    large scale eigenvalue problems. Num. Alg., Vol. 24, pp. 195-237,
%    2000.
%
%   See also MCLUSTER.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(3,5)
nargoutchk(0,5)

if nargin > 3
  onlylargest = varargin{1};
end

nT = size(T,1);
if nargin < 5
  [U,T] = schur(T,'complex');
elseif any(any(tril(T,-1)))
  warning('MCS:mcegershgorin:Schur',...
    'T is not upper-triangular. Compex Schur form is recomputed and U discarded.');
  [U,T] = schur(T,'complex');
else
  U = varargin{2};
end

if isinf(goal)
  goal = max(diag(T)); % The maximum complex modulus (magnitude)
end

% Reorder the Schur triangle form such that the eigenvalues are ordered
% with increasing distance to goal
[ev,ii] = sort(abs(diag(T)-goal));
index(ii) = length(ii):-1:1;
[U,T] = ordschur(U,T,index);

lambda = zeros(nT,1);
ind = cell(nT,1);
neigs = zeros(nT,1);
cluster = zeros(nT,1);
blk = 0; p = 1;
% Cluster the matrix...
while p <= nT
  dT = diag(T(p:end,p:end));
  n = length(dT);
  halfDist = 0.5*abs(dT*ones(1,n)-ones(n,1)*dT.');

  D = zeros(n,1);
  s = 0; 
  while D(n)<1 && (s+1)<n
    s = s+1;
    v = halfDist;
    maxr = inf(n,1);

    i = 0; go = 1;
    while go && i<n
      i = i+1;
      [minjk,j,k] = minv(v(1:s,s+1:n),maxr);
      v(j,k) = minjk;
      maxr(j) = minjk;
      v(j,:) = 2*v(j,:)-maxr(j);
      v(:,j) = 2*v(j,:)-maxr(j);
      v(i,i) = 0;

      factor = sizeE + sum( abs(T(p:(p+i-2),(p+i-1))./D(1:i-1)) );
      D(i) = maxr(i)/factor;
      go = D(i)>=1;
    end;
  end
  if s == 0 % One last block of size 1x1.
    s = 1;
  elseif s == n-1 && D(i) < 1
    s = n; 
  end
  blk = blk+1;
  lambda(blk) = mean(dT(1:s));
  ind{blk} = p:(p+s-1);
  neigs(blk) = s;
  cluster(p:p+s-1) = blk;
  p = p+s;

  % Break if only the cluster closest to goal is wanted
  if onlylargest
    break
  end
end

switch nargout
  case {0,1}
    varargout{1} = lambda(1:blk);
  case 2
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
  case 3
    varargout{1} = U;
    varargout{2} = T;
    varargout{3} = ind(1:blk);
  case 4
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
    varargout{3} = ev;
    varargout{4} = cluster;
  case 5
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
    varargout{3} = U;
    varargout{4} = T;
    varargout{5} = ind(1:blk);
end


% ==========================================================
function [minjk,j,k] = minv(v,maxr)
% Find the smallest index and value for v that has not been assigned yet.

minjk = Inf; k = numel(v);
for a=1:numel(v)
  if v(a) < minjk && isinf(maxr(a))
    k = a;
    minjk = v(a);
  end
end
j = rem(k-1, size(v,1)) + 1; % Convert from linear index to row in v.
