function [ndim,tol,mgap,flag] = nullitycheck(S,tol,gap,dim,ndim)
%NULLITYCHECK Determine the dimension of the nullspace of a matrix.
%   NDim = NULLITYCHECK(S,Tol,Gap,Dim) returns the dimension NDim of the
%   row or column nullspace of a diagonal matrix S with respect to Tol and
%   Gap. The matrix S should consist of the singular values (given by SVD)
%   of a matrix A for which the nullspace should be determined. The
%   parameter Dim can be either:
%      'column' for the column nullspace,
%      'row'    for the row nullspace.
%   A singular value s(i) is determined to be zero if s(i) < Tol and
%   s(i-1)/s(i) >= Gap, where s is in decreasing order and i <= 'number of
%   singular values'. If the function fails to estimate the dimension of
%   the nullspace with the given tolerance parameters, Tol is increased. If
%   any of the tolerances is changed it will result in a warning. The
%   warning is suppressed with
%      warning('off','MCS:nullitycheck:Tolerance')
%
%   [NDim,Tol,Gap] = NULLITYCHECK(S,Tol,Gap,Dim) or
%   [NDim,Tol,Gap,Flag] = NULLITYCHECK(S,Tol,Gap,Dim) also returns the
%   (adjusted) tolerances Gap and Tol. Possible values of Flag are (or a
%   sum of):
%      0 - No warnings.
%      1 - The Tol tolerance is increased.
%   and if N is given as input (see below):
%      2 - The Gap tolerance is decreased.
%      4 - Failed to compute a stable Gap tolerance.
%
%   [NDim,Tol,Gap] = NULLITYCHECK(S,Tol,Gap,Dim,N) or
%   [NDim,Tol,Gap,Flag] = NULLITYCHECK(S,Tol,Gap,Dim,N) tries to adjust the
%   tolerance parameters such that S will have an estimated N-dimensional
%   nullspace. If the tolerance is changed it will result in a
%   'MCS:nullitycheck:Tolerance' warning.
%
%   See also NULLITYIMPOSE.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.
  

% ARGUMENT CHECK
narginchk(4,5);

% If the NDIM parameter is given, the tolerance parameters will be imposed.
impose = nargin == 5;

if nargout == 2
  error('Not enough output arguments.');
end
if impose
  nargoutchk(3,4);
else
  nargoutchk(0,4);
  ndim = 0;
end

% Extract the singular values of S
switch dim
  case 'row'
    %row = true;
    sb = sum(S,2);
  case 'column'
    %row = false;
    sb = sum(S,1);
  otherwise
    error(['Unknown argument ''' dim '''.']);
end

% Initialize output
mgap = gap;  % Minimium stable gap.
flag = 0;

n = length(sb);
if n == 0
  ndim = 0;
  return;
elseif n == 1
  % If only one singular value

  if ~impose && sb <= tol
    ndim = 1;
  elseif impose && 1 > n-ndim
    if sb >= tol
      % If we impose the tolerance parameters and the minimum stable gap is
      % decreased then this must be updated.
      mgap = min(gap,sb/eps);
    end
  end
else
  
  % If increasing order, reverse
  if sb(1) < max(sb)
    % sb may have trailing zeros, flip only the singular values.
    [~,k] = max(sb(end:-1:1));
    sb(1:(n-k+1)) = sb((n-k+1):-1:1);
  end
  
  % Estimate NDim, or Tol and Gap if imposed nullspace
  
  % When the tolerance parameters are imposed, the minimum stable gap is
  % the minimum value of the gap parameter that still will satisfy the
  % condition  sb(i)/sb(i+1) < Gap (notice that i decreases in the
  % iteration).
  
  t1   = eps;  % sb(i+1)
  i    = n;    % i decreases in the iteration
  while ( impose && i > n-ndim ) ...
      || (~impose && i >= 1 && ( sb(i) < tol || sb(i)/t1 < gap ) )
    
    if ~impose
      % Increase size of nullspace
      ndim = ndim + 1;
    elseif sb(i) >= tol
      % If we impose the tolerance parameters and the minimum stable gap is
      % decreased then this must be updated.
      mgap = min(mgap,sb(i)/t1);
    end
    
    % If no gap is found and sb(i) > Tol, the Tol parameter needs to be
    % increased.
    if sb(i) > tol
      tol = sb(i);
      flag = 1;
      warning('MCS:nullitycheck:Tolerance','Tolerance increased to %.2e',tol);
    end
    
    if sb(i) < eps
      t1 = eps;
    else
      t1 = sb(i);
    end
    i  = i - 1;
  end
end

% If we try to impose the tolerance parameters we must finally check if the
% gap parameter is valid. Given NDim, if the found minimum stable gap is
% smaller than the largest previously found minimum stable gap, the
% estimate of the Gap parameter fails.
if impose && (n-ndim > 1) && (sb(n-ndim-1)/sb(n-ndim) < gap)
  gap = ceil(sb(n-ndim-1)/sb(n-ndim));
  
  if gap < mgap
    flag = flag + 4;
    warning('MCS:nullitycheck:Tolerance',...
      'Failed to compute a stable gap tolerance.');
  else
    flag = flag + 2;
    warning('MCS:nullitycheck:Tolerance','Gap tolerance decreased to %.2e',gap);
  end
end

