function [ST,flag] = nullityimpose(S,tol,gap,dim,nk,force)
%NULLITYIMPOSE Impose a nullspace of the diagonal matrix S from ISVD.
%   [ST,Flag] = NULLITYIMPOSE(S,Tol,Gap,Dim,N,Force) returns a matrix ST
%   such that S+ST has an N-dimensional row or column nullspace with
%   respect to the tolerance Tol. The parameter Dim can be either:
%      'column' - for the column nullspace,
%      'row'    - for the row nullspace.
%   If rank(S,Tol) < (size(S,k) - N), where k is 1 if Dim='column' and 2 if
%   Dim='row', then ST is perturbed with max(Tol, eps*Gap) such that S+ST
%   has an N-dimensional row or column nullspace, and a warning is issued.
%   The warning is suppressed with
%      warning('off','MCS:nullityimpose:IncreasedRank')
%
%   Possible values of Flag are:
%      0 - No warnings.
%      1 - Force = false and the largest singular value imposed to zero is
%          greater than Tol*Gap. ST returns the zero matrix. 
%      2 - Force = true and the largest singular value(s) is/are forced to zero,
%          In addition a warning is issued. The warning is suppressed with
%             warning('off','MCS:nullityimpose:ImposedZeroSingularValue')
%
%   See also NULLITYCHECK.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.
  
% Argument Check
narginchk(6,6);
nargoutchk(0,2);

flag = 0;
ST = zeros(size(S));

if nk < 0
  error('The dimension N must be greater than zero.')
end

sDiff = diff(size(S)); % If m>=n then sDiff<=0, else sDiff>0

% Extract singular values of S
switch lower(dim)
  case 'row'
    sb = sum(S,2);
    if sDiff < 0 && nk < -sDiff 
      error('N must be greater or equal to -diff(size(S)) when m > n.')
    end
    rows = true;
  case 'column'
    sb = sum(S,1);
    if sDiff > 0 && nk < sDiff 
      error('N must be greater or equal to diff(size(S)) when m < n.')
    end
    rows = false;
  otherwise
    error(['Unknown argument ''' dim '''.']);
end

svinc = sb(1) <= sb(end);
% Sort sb in increasing order
[sb,I] = sort(sb);

%  Check that rank(S,tol) >= size(S,k) - nk
if sum(sb < tol) <= nk
  if nk == 0
    return
  end
  
  if sb(nk) >= tol*gap
    if force
      warning('MCS:nullityimpose:ImposedZeroSingularValue',...
        'The largest singular value (%e) imposed to zero is greater than tol*gap = %1.2e.',...
        sb(nk), tol*gap)
      flag = 2;
    else
      flag = 1;
      return
    end
  end
else
  % Need to increase the rank 
  
  % If the largest sing. value that should be zero is approximated by eps
  % then to fullfill the tol and gap constraints the smallest non-zero
  % sing. value must be max(eps*gap,tol).
  t = max(eps*gap,tol);
  
  % sb(nk+1) is the smallest sing. value that should be non-zero
  sk = (sb(nk+1:end) < t);

  if sDiff <= 0
    % Tall S
    if svinc
      % Sing. values in lower part
      if rows
        ST(I(nk+1:end),I(nk+1:end)+sDiff) = diag(t*sk);
      else
        ST(I(nk+1:end)-sDiff,I(nk+1:end)) = diag(t*sk);
      end
    else
      % Upper part
      ST(I(nk+1:end),I(nk+1:end)) = diag(t*sk);
    end
  else
    % Wide S
    if svinc
      % Right part
      if rows
        ST(I(nk+1:end),I(nk+1:end)+sDiff) = diag(t*sk);
      else
        ST(I(nk+1:end)-sDiff,I(nk+1:end)) = diag(t*sk);
      end
    else
      % Left part
      ST(I(nk+1:end),I(nk+1:end)) = diag(t*sk);
    end
  end
  
  warning('MCS:nullityimpose:IncreasedRank',...
    'The rank is increased by %d with a total perturbation of %1.3e.',...
    sum(sk),sum(t*sk))
end

% Construct matrix ST such that S+ST has an N-dimensional column/row
% nullspace
if rows
  ST(I(1:nk),:) = -S(I(1:nk),:);
else
  ST(:,I(1:nk)) = -S(:,I(1:nk));
end

