function varargout = pcegershgorin(G,H,goal,sizeE,sizeF,varargin)
%PCEGERSHGORIN Compute and cluster the generalized eigenvalues of a matrix pencil.
%   Ev = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF) or
%   [Ev,Sizes] = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF) clusters the generalized
%   eigenvalues corresponding to a numerically multiple eigenvalue of the square
%   matrix pencil G-sH. The vectors Ev and Sizes contain the mean values of the
%   eigenvalues in each isolated cluster and the cluster sizes, respectively.
%   The heuristic is based on Gershgorin circles without diagonal scaling. The
%   function starts with isolating the cluster closest to the point Goal in the
%   complex plane. If the cluster containing the largest eigenvalue is desired
%   to be first then Goal should be set to Inf. SizeE and SizeF should be
%   estimates of the 1-norm of the errors known to affect G and H, respectively.
%
%   [Ev,Sizes,S,T,Q,Z,Ind] = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF) or
%   [S,T,Q,Z,Ind] = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF) also returns the complex
%   QZ factorization of G-sH in the matrices S = Q*G*Z and T = Q*H*Z, where S
%   and T are triangular, and Q and Z are unitary matrices. The set Ind provides
%   the clustering of the eigenvalues in S-sT, where Ind{k} is the indices for
%   eigenvalue Ev(k). For example, if Ind{1} = 1:5 and Ind{2} = 6:7 then G-sH
%   has one regular block of size 5 corresponding to
%   S(Ind{1},Ind{1})-sT(Ind{1},Ind{1}). The second regular block is of size 2
%   and corresponds to S(Ind{2},Ind{2})-sT(Ind{2},Ind{2}).
%   
%   [Ev,Sizes,E,EvCluster] = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF) returns all
%   computed generalized eigenvalues E of G-sH and the clustering of the
%   eigenvalues in EvCluster. The integer value k in EvCluster(i) represents the
%   cluster the eigenvalue E(i) belongs to, where Ev(k) is the new eigenvalue
%   for the cluster. The eigenvalues in E are sorted in non-increasing order
%   from Goal. If OnlyLargest is true (see below), only the clustering for the
%   first (largest) eigenvalue is correct.
%
%   ... = PCEGERSHGORIN(G,H,Goal,SizeE,SizeF,OnlyLargest) set OnlyLargest to
%   true if only the cluster closest to Goal should be determined, otherwise
%   false (default).
%
%   ... = PCEGERSHGORIN(S,T,Goal,SizeE,SizeF,OnlyLargest,Q,Z) asumes that S-sT
%   is upper triangular such that Q*(S-sT)*Z = G-sH. If S or T is not upper
%   triangular, Q and Z are discarded and S-sT is assumed be a general matrix
%   pencil.
%
%   Algorithm:
%    The function uses the clustering algorithm described in 
%    B. Kagstrom and P. Wiberg. Extracting partial canonical structures for
%    large scale eigenvalue problems. Num. Alg., Vol. 24, pp. 195-237,
%    2000.
%
%   See also PCLUSTER.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(5,8)
nargoutchk(0,7)

if nargout == 3
  error('Wrong number of output arguments.')
end

onlylargest = 0;
if nargin == 6
  onlylargest = varargin{1};
elseif nargin == 7
  error('Wrong number of input arguments.')
elseif nargin == 8
  onlylargest = varargin{1};
  Q = varargin{2};
  Z = varargin{3};
end

nG = size(G,1);
if nargin < 7
  [G,H,Q,Z] = qz(G,H,'complex'); 
elseif any(any(tril(G,-1))) || any(any(tril(H,-1)))
  warning('MCS:pcegershgorin:Schur',...
    'S or T is not upper-triangular. QZ factorization is recomputed and Q and Z discarded.');
  [G,H,Q,Z] = qz(G,H,'complex'); 
end
if isinf(goal)
  goal = max(diag(G)./diag(H));
end
sizeEF = sizeE^2+sizeF^2;

% Reorder the generalized Schur form such that the eigenvalues are ordered
% with increasing distance to goal
cdist = zeros(nG,1);
for i=1:nG
  cdist(i) = chordal(goal,1,G(i,i),H(i,i));
end
[~,ii] = sort(cdist);
select(ii) = nG:-1:1;
[G,H,Q,Z] = ordqz(G,H,Q,Z,select);

lambda = zeros(nG,1);
ind = cell(nG,1);
neigs = zeros(nG,1);
cluster = zeros(nG,1);
halfDist = zeros(nG,nG);
blk = 0; p = 1;
% Cluster the pencil...
while p <= nG
  dG = diag(G(p:end,p:end));
  dH = diag(H(p:end,p:end));
  n = length(dG);
  % Compute chordal metric between all pairs of eigenvalues
  for i=1:n
    for j=1:n
      halfDist(i,j) = 0.5*chordal(dG(i),dH(i),dG(j),dH(j));
    end
  end
  
  D = zeros(n,1);
  s = 0; 
  while D(n)<1 && (s+1)<n
    s = s+1;
    v = halfDist;
    maxr = inf(n,1);
    
    i = 0; go = 1;
    while go && i<n
      i = i+1;
      [minjk,j,k] = minv(v(1:s,s+1:n),maxr);
      v(j,k) = minjk;
      maxr(j) = minjk;
      v(j,:) = 2*v(j,:)-maxr(j);
      v(:,j) = 2*v(j,:)-maxr(j);
      v(j,j) = 0;

      factor = sizeEF + sum(abs(G(p:p+i-2,p+i-1)).^2)...
              + sum(abs(H(p:p+i-2,p+i-1)).^2);
      if abs(dG(i))^2 + abs(dH(i))^2 <= eps
        D(i) = Inf;
      else
        factor = sqrt( factor / (abs(dG(i))^2+abs(dH(i))^2));
        D(i) = maxr(i)/factor;
      end
      go = D(i)>=1;
    end
  end
  if s == 0 % One last block of size 1x1.
    s = 1;
  elseif s == n-1 && D(i) < 1
    s = n; 
  end
  blk = blk+1;
  lambda(blk) = mean(dG(1:s)./dH(1:s));
  ind{blk} = p:(p+s-1);
  neigs(blk) = s;
  cluster(p:p+s-1) = blk;
  p = p+s;

  % Break if only the cluster closest to goal is wanted
  if onlylargest
    break
  end
end

switch nargout
  case {0,1}
    varargout{1} = lambda(1:blk);
  case 2
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
  case 4
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
    varargout{3} = diag(G)./diag(H);
    varargout{4} = cluster;
  case 5
    varargout{1} = G;
    varargout{2} = H;
    varargout{3} = Q;
    varargout{4} = Z;
    varargout{5} = ind(1:blk);
  case {6,7}
    varargout{1} = lambda(1:blk);
    varargout{2} = neigs(1:blk);
    varargout{3} = G;
    varargout{4} = H;
    varargout{5} = Q;
    varargout{6} = Z;
    if nargout == 7
      varargout{7} = ind(1:blk);
    end
end

return

% ==========================================================
function [minjk,j,k] = minv(v,maxr)
%  Find the smallest index and value for v that has not been assigned yet.

[s,t] = size(v);
minjk = inf;
for a=1:s
  for b=1:t
    if v(a,b)<minjk
      if isinf(maxr(a))
        j = a;
        k = b+s;
        minjk = v(a,b);
      elseif isinf(maxr(b+s))
        j = b+s;
        k = a;
        minjk = v(a,b);
      end
    end
  end
end

% ==========================================================
function dist = chordal(alpha,beta,gamma,delta)
%  The chordal distance between (alpha,beta) and (gamma,delta).

dist = abs(alpha*delta-beta*gamma) / ...
  ( sqrt(abs(alpha)^2+abs(beta)^2)*sqrt(abs(delta)^2+abs(gamma)^2) );
