function varargout = rzstr(G,H,tol,gap,setzero,rweyr,zweyr,forceimpose)
%RZSTR RZ-staircase form of the right singular and zero eigenvalue blocks.
%   [RWeyr, ZWeyr] = RZSTR(G,H,Tol,Gap,SetZero) finds the right singular
%   blocks and the Jordan blocks with associated zero eigenvalue in G-sH
%   with respect to Tol and Gap. Tol is a vector [TolG TolH], where TolG is
%   used for the column compression of G and TolH for the column
%   compression of H. The Weyr characteristics of the blocks are returned
%   in RWeyr and ZWeyr, respectively. If the parameter SetZero is true, the
%   lower parts of the transformed G and H are forced to zero, otherwise
%   not.
%
%   [S,T,RWeyr,ZWeyr] = RZSTR(G,H,Tol,Gap,SetZero) also returns the
%   deflated S and T in the GUPTRI staircase form.
%
%   [S,T,P,Q,RWeyr,ZWeyr] = RZSTR(G,H,Tol,Gap,SetZero) also returns the
%   transformation matrices P and Q, such that P(S-sT)Q' = G-sH.
%
%   [S,T,Flag] = RZSTR(G,H,Tol,Gap,SetZero,RWeyr,ZWeyr,ForceImpose) returns
%   the deflated matrices S and T, with the imposed structure given by
%   RWeyr and ZWeyr. Flag is non-zero in the following cases. If the
%   largest singular value imposed to zero is greater than Tol*Gap and
%   ForceImpose is false, Flag returns 1 and G and H are returned in S and
%   T, respectively. Otherwise, if ForceImpose is true, Flag returns 2 and
%   the singular values are forced to zero even if they are larger than
%   Tol*Gap.
%
%   [S,T,P,Q,Flag] = RZSTR(G,H,Tol,Gap,SetZero,RWeyr,ZWeyr,ForceImpose)
%   also returns the transformation matrices P and Q, used to form S and T.
%
%   [S,T,P,Q,E,F,Flag] = RZSTR(G,H,Tol,Gap,SetZero,RWeyr,ZWeyr,ForceImpose)
%   also returns the matrices E and F with the imposed permutations, such
%   that P(S-sT)Q' = (G+E)-s(H+F).
%
%   See also PGUPTRI, RISTR, LISTR, LZSTR, FSTR.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

% !Argument check disabled!
% % Argument check
% narginchk(5,8);
% nargoutchk(2,7);
% error(sizechk('ndims',G,2));
% error(sizechk('ndims',H,2));
% error(sizechk('equal',G,H));
% 
% if nargin ~= 5 && nargin ~= 8
%   error('Invalid number of input arguments');
% end


%----------------------------------------------------------------
% Debug switch
DEBUG = 0;

impose  = nargin == 8;           % Impose the given structure
compst = impose || nargout > 2;  % Compute S and T?
compef = nargout > 5 && impose;  % Compute E and F?
comppq = (nargout > 3 && impose) || (nargout > 4);  % Compute P and Q?

if DEBUG
  Gin = G;
  Hin = H;
  compst = true;
  comppq = true;
end

[gm, gn] = size(G); % Initial G, gm-by-gn

if compst
  S = zeros(gm,gn); % Template for S, gm-by-gn
  T = zeros(gm,gn); % Template for T, hm-by-hn
end
if comppq
  P = eye(gm,gm);   % Template for P, gm-by-gm
  Q = eye(gn,gn);   % Template for Q, gn-by-gn
end
if compef
  E = zeros(gm,gn); % Template for E, gm-by-gn
  F = zeros(gm,gn); % Template for F, hm-by-hn
end

% Intitial indices
if impose
  nidx = zeros(1,max(length(rweyr),length(zweyr))+1);
  nidx(1:length(rweyr)) = rweyr(1:end);
  nidx(1:length(zweyr)) = nidx(1:length(zweyr)) + zweyr(1:end);
  ridx = zeros(1,max(length(rweyr),length(zweyr))+1);
  ridx(1:length(rweyr)-1) = rweyr(2:end);
  ridx(1:length(zweyr))   = ridx(1:length(zweyr)) + zweyr(1:end);
  % Store original matrices if separation fails
  Gin = G;
  Hin = H;
else
  nidx = zeros(1,max(gn,gm));
  ridx = zeros(1,max(gn,gm));
end
iflag = 0;


if DEBUG
  if impose
    fdebug('Enter RZSTR...','Impose structures:\n',...
      'nidx = ',nidx,'ridx = ',ridx,...
      'Tol = ',tol, 'Gap =', gap, '\n')
  else
    fdebug('Enter RZSTR...','Tol = ',tol, 'Gap = ', gap, '\n')
  end
  G,H
  %Fig = plotnz(S,T,tol(1),{},{});
end

%----------------------------------------------------------------
% Begin iteration
%
%            gn
%              n
%       gi-1 ni
%        +--------+
%    gj-1|  |     |
%        |--+--+--+
% gm  ri |  |  |  |
%    m   |  +--+--+
%        |  |  |  |
%        +--+--+--+

i  = 1;             % i  = number of iterations
gi = 1;             % gi = global row index
gj = 1;             % gj = global col index
ni = gn;            % ni ~= 0 (just to get into the loop)
ri = gm;            % ri ~= 0
while (ni > 0) && (ri >= 0)
  [m, n] = size(G);
  
  %--------------------------------------------------------------
  % Step 1. Compress the columns in Gi to give ni
  if m == 0 % 0-by-n
    % Update indices
    if ~impose
      nidx(i) = n;
      ridx(i) = 0;
    end
    i = i+1;
    break % If m == 0 we are done
  elseif n == 0 % m-by-0
    ni = 0;
  else          % m-by-n
    [Ug,Sg,Vg] = isvd(G); % svd in increasing order
    if impose
      ni = nidx(i);
      
      [Sgt,iflag] = nullityimpose(Sg,tol(1),gap,'column',ni,forceimpose);
      if iflag && ~forceimpose
        break;
      end
      Sg  = Sg+Sgt;
    else
      % Column nullity
      ni = nullitycheck(Sg,tol(1),gap,'column');
    end
  end
  
  if ni > 0 && m > 0 % Notice: ni > 0 => n > 0
    % Update G and H
% CHECK: When impose, Ug*(Sg + Sgt) ~= G*Vg!
% Do we need to update Ug and Vg in nullityimpose()?
    G = Ug*Sg;  % := [~0 G1] (Ug*Sg == G*Vg)
    H = H*Vg;   % := [H1 H2]
    
    % Update E and F
    if compef
      E = E + P(:,gi:end)*Ug*Sgt*Vg'*Q(:,gj:end)';
    end
    % Update S and T
    if compst
      S(1:gi-1,gj:end) = S(1:gi-1,gj:end)*Vg;
      T(1:gi-1,gj:end) = T(1:gi-1,gj:end)*Vg;
    end
    % Update P and Q
    if comppq
      Q(:,gj:end) = Q(:,gj:end)*Vg;
    end
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('RZSTR Step 1:','Rel error (G,H) = ', ...
      norm(G-Gtemp(gi:end,gj:end))/norm(Gtemp(gi:end,gj:end)), ' , ', ...
      norm(H-Htemp(gi:end,gj:end))/norm(Htemp(gi:end,gj:end)),'\n',...
      'ni = ', ni,'\n');
    G,H

  end
  
  %--------------------------------------------------------------
  % Step 2. Compress the columns in Hi to give ni - ri
  
  if ni > 0 && m > 0 % Notice: ni > 0 => n > 0
    % Notice: ni = 0 => H1 is m-by-0 => ri = 0
    [Uh,Sh,Vh] = isvd(H(:,1:ni)); % svd in increasing order

    if impose
      ri = ridx(i);
      
      [Sht,iflag] = nullityimpose(Sh,tol(2),gap,'column',ni-ri,forceimpose);
      if iflag && ~forceimpose
        break;
      end
      Sh = Sh+Sht;

      % Update E and F
      if compef
        F = F + P(:,gi:end)*Uh*Sht*Vh'*Q(:,gj:gj+ni-1)';
      end
    else
      ri = ni - nullitycheck(Sh,tol(2),gap,'column');
    end
    
    % Update G and H
    G(:,1:ni) = G(:,1:ni)*Vh;
    H(:,1:ni) = Uh*Sh;        % = [~0 H0] (Uh*Sh == H(:,1:ni)*Vh)
    
    % Update S and T
    if compst
      S(1:gi-1,gj:gj+ni-1) = S(1:gi-1,gj:gj+ni-1)*Vh;
      T(1:gi-1,gj:gj+ni-1) = T(1:gi-1,gj:gj+ni-1)*Vh;
    end
    % Update P and Q
    if comppq
      Q(:,gj:gj+ni-1)      = Q(:,gj:gj+ni-1)*Vh;
    end
  else
    ri = 0;
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('RZSTR Step 2:','Rel error (G,H) = ', ...
      norm(G-Gtemp(gi:end,gj:end))/norm(Gtemp(gi:end,gj:end)), ' , ', ...
      norm(H-Htemp(gi:end,gj:end))/norm(Htemp(gi:end,gj:end)),'\n',...
      'ri = ', ri,'\n');
    G,H
    
  end
  
  %--------------------------------------------------------------
  % Step 3. Triangularize [H0 H2] to give Pi
  
  if ni > 0 && m > 0 % Notice: ni > 0 => n > 0
    % H0 = H1(:,ni-ri+1:ni);   % Warning: m-by-0  if ri=0
    [Pi, Ri] = qr(H(:,ni-ri+1:end)); % qr([H0 H2])
    
    % Update Hi
    H(:,ni-ri+1:end) = Ri;
    
    % Update Gi
    G(:,ni+1:end) = Pi'*G(:,ni+1:end); % = P*G1

    if setzero
      % Update E and F
      if compef
        F(gi:end,gj:gj+ni-ri-1) = F(gi:end,gj:gj+ni-ri-1) + Pi'*H(:,1:ni-ri);
      end
      H(:,1:ni-ri) = 0;
    else
      G(:,1:ni) = Pi'*G(:,1:ni);
      H(:,1:ni-ri) = Pi'*H(:,1:ni-ri);
    end
    
    % Update S and T
    if compst
      S(gi:end,1:gj-1)  = Pi'*S(gi:end,1:gj-1);
      T(gi:end,1:gj-1)  = Pi'*T(gi:end,1:gj-1);
    end
    % Update P and Q
    if comppq
      P(:,gi:end) = P(:,gi:end)*Pi; % P(gi:gm,:)' == Pi'*P(gi:gm,:)'
    end
  end

  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('RZSTR Step 3:','Rel. error (G,H) = ', ...
      norm(G-Gtemp(gi:end,gj:end))/norm(Gtemp(gi:end,gj:end)), ' , ', ...
      norm(H-Htemp(gi:end,gj:end))/norm(Htemp(gi:end,gj:end)),'\n');
    G,H
    
  end

  %--------------------------------------------------------------
  % Preparing next iteration step.
  
  % Set zeros on deflated part
  if setzero
    % Update E and F
    if compef
      E(gi:end,gj:gj+ni-1) = E(gi:end,gj:gj+ni-1) + G(:,1:ni);
      F(gi+ri:end,gj:gj+ni-1) = F(gi+ri:end,gj:gj+ni-1) + H(ri+1:end,1:ni);
    end 
    G(:,1:ni)        = 0;
    H(ri+1:end,1:ni) = 0;
  end
  
  % Update S and T
  if compst
    S(gi:end,gj:end) = G;
    T(gi:end,gj:end) = H;
  end
  
  if DEBUG
    Gtemp = P'*Gin*Q; Htemp = P'*Hin*Q;
    fdebug('RZSTR Prep step:','Rel. error (G,H) = ', ...
      norm(S-Gtemp)/norm(Gtemp), ' , ', ...
      norm(T-Htemp)/norm(Htemp),'\n');
    ni,ri
    G,H
    %plotnz(S,T,tol(1),Fig,{gi:gm,gj:gn},{gi:gm,gj:gn});
    %pause
  end
  
  % New G
  G = G(ri+1:end,ni+1:end);
  
  % New H
  H = H(ri+1:end,ni+1:end);
  
  % Update indices
  if ~impose
    nidx(i) = ni;
    ridx(i) = ri;
  end
  i = i+1;
  
  % Update iterators
  gi = gi + ri;
  gj = gj + ni;
end % End while


%--------------------------------------------------------------
% Create output

% #L_(k-1) blocks = ni(k) - ri(k)
% #J(0)_k blocks  = ri(k) - ni(k+1)

if ~impose
  l = i-1; % = max length of nidx and ridx
  rweyr = zeros(1,l);
  zweyr = zeros(1,l);
  if l > 0
    rweyr(l) = nidx(l) - ridx(l);
    zweyr(l) = ridx(l);
    for i = l-1:-1:1
      rweyr(i) = max([0 rweyr(i+1:l)]) + nidx(i)-ridx(i);
      zweyr(i) = max([0 zweyr(i+1:l)]) + ridx(i)-nidx(i+1);
    end
  end
else
  % If separation failed
  if iflag == 1
    S = Gin;
    T = Hin;
    if comppq
      P = eye(gm,gm);
      Q = eye(gn,gn);
    end
    if compef
      E = zeros(gm,gn);
      F = zeros(gm,gn);
    end
  end
end

if DEBUG && ~impose
  fdebug('Computed structure:', 'R weyr = ', rweyr, 'Z weyr = ', zweyr)
  S,T
end

if nargout == 2 && ~impose
  varargout{1} = rweyr;
  varargout{2} = zweyr;
elseif compst
  varargout{1} = S;
  varargout{2} = T;
  varargout{3} = iflag;
end
if nargout == 4 && ~impose
  varargout{3} = rweyr;
  varargout{4} = zweyr;
elseif comppq
  varargout{3} = P;
  varargout{4} = Q;
  varargout{5} = iflag;
end
if nargout == 6 && ~impose
  varargout{5} = rweyr;
  varargout{6} = zweyr;
elseif compef
  varargout{5} = E;
  varargout{6} = F;
  varargout{7} = iflag;
end