function msg = sizechk(method,A,varargin)
%SIZECHK Validate sizes of input arguments.
%   Msg = SIZECHK(method,...,[MSG]) returns an appropriate error message
%   Msg if the argument(s) does not fulfill the validation.
%
%   SIZECHK('ndims',A,N,[MSG]) validates that A is an N-dimensional
%   array.  The number of dimensions in an array is always greater
%   than or equal to 2.
%
%   SIZECHK('square',A,[MSG]) validates that A is a square matrix.
%
%   SIZECHK('size',A,N,[MSG]) validates that A is an N-by-N matrix.
%
%   SIZECHK('size',A,M,N,[MSG]) validates that A is an M-by-N
%   matrix. 
%
%   SIZECHK('equal',A,B,[MSG]) validates that A has the same size 
%   as B.
%
%   SIZECHK('equal',A,B,DIM,[MSG]) validates that A and B has the
%   same size in dimension DIM.
%
%   SIZECHK('vector',V,[MSG]) validates that V is a vector.
%
%   SIZECHK('vector',V,DIM,[MSG]) validates that V is a vector in
%   dimension DIM, e.g, DIM = 2, indicates a row-vector.
%
%   SIZECHK('cell-vector',V,[MSG]) validates that V is a
%   cell-vector . 
%
%   SIZECHK('cell-vector',V,DIM,[MSG]) validates that V is a
%   cell-vector in dimension DIM, e.g, DIM = 2, indicates a row-vector.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(2,5);

if ~ischar(method)
  error('Unknown method');
end

msgs.error = 0;
if nargin > 2 && ischar(varargin{nargin-2})
  msgs.msg = varargin{nargin-2};
  msgs.nargin = nargin-1;
else
  msgs.msg = [];
  msgs.nargin = nargin;
end

[m,n] = size(A);
switch method
  case 'ndims'
    narginchk(3,3);
    if ndims(A) ~= varargin{1}
      msgs = setmsg(msgs,sprintf('%s must be %d-dimensional', ...
        namereveal(inputname(2)), ...
        varargin{1}));
    end
  case 'square'
    narginchk(2,2);
    if m ~= n
      msgs = setmsg(msgs,sprintf('%s must be square', ...
        namereveal(inputname(2))));
    end
  case 'size'
    narginchk(3,4);
    if m ~= varargin{1}
      msgs = setmsg(msgs,sprintf('%s must have %d rows', ...
        namereveal(inputname(2)), varargin{1}));
    elseif msgs.nargin > 3 && n ~= varargin{2}
      msgs = setmsg(msgs,sprintf('%s must have %d columns', ...
        namereveal(inputname(2)), varargin{2}));
    elseif msgs.nargin == 3 && n ~= varargin{1}
      msgs = setmsg(msgs,sprintf('%s must have %d columns', ...
        namereveal(inputname(2)), varargin{1}));
    end
  case 'equal'
    narginchk(3,4);
    [m2,n2] = size(varargin{1});
    if msgs.nargin == 3
      if ndims(A) ~= ndims(varargin{1})
        msgs = setmsg(msgs,sprintf('%s must be %d-dimensional', ...
          namereveal(inputname(3)), ...
          ndims(A)));
      end
      if n ~= n2 || m ~= m2
        msgs = setmsg(msgs,sprintf('%s must have same size', ...
          namereveal(inputname(2), inputname(3))));
      end
    else
      switch varargin{2}
        case 1
          if m ~= m2
            msgs = setmsg(msgs,sprintf('%s must have same number of rows', ...
              namereveal(inputname(2), inputname(3))));
          end
        case 2
          if n ~= n2
            msgs = setmsg(msgs,sprintf('%s must have same number of columns', ...
              namereveal(inputname(2), inputname(3))));
          end
        otherwise
          error('Dimension is too big');
      end
    end
  case 'vector'
    narginchk(2,3);
    if msgs.nargin == 3
      switch varargin{1}
        case 2
        case 1
          if n > 1 || (m > 0 && n < 1)
            msgs = setmsg(msgs,sprintf('%s must be a column vector', ...
              namereveal(inputname(2))));
          end
          if m > 1 || (n > 0 && m < 1)
            msgs = setmsg(msgs,sprintf('%s must be a row vector', ...
              namereveal(inputname(2))));
          end
        otherwise
          error('Dimension is too big');
      end
    elseif m > 1 && n > 1
      msgs = setmsg(msgs,sprintf('%s must be a vector', ...
        namereveal(inputname(2))));
    end
  case 'cell-vector'
    narginchk(2,3);
    if ~iscell(A)
      msgs = setmsg(msgs,sprintf('%s must be a cell array', ...
        namereveal(inputname(2))));
    end
    if msgs.nargin == 3
      switch varargin{1}
        case 1
          if m > 1 || (n > 0 && m < 1)
            msgs = setmsg(msgs,sprintf('Argument %s must be a row vector', ...
              namereveal(inputname(2))));
          end
        case 2
          if n > 1 || (m > 0 && n < 1)
            msgs = setmsg(msgs,sprintf('Argument %s must be a column vector', ...
              namereveal(inputname(2))));
          end
        otherwise
          error('Dimension is too big');
      end
    elseif m > 1 && n > 1
      msgs = setmsg(msgs,sprintf('Argument %s must be a vector', ...
        namereveal(inputname(2))));
    end
  otherwise
    error('Unknown method');
end

if msgs.error
  msg = msgs.msg;
else
  msg = [];
end

function msgs = setmsg(msgs, msg)
if isempty(msgs.msg)
  msgs.msg   = msg;
end
msgs.error = 1;

function str = namereveal(a,b)
if nargin == 2 && ~isempty(a) && ~isempty(b)
  str = sprintf('Arguments %s and %s',a,b);
elseif nargin == 2
  str = 'Arguments';
elseif ~isempty(a)
  str = sprintf('Argument %s',a);
else
  str = 'Argument';
end
