function [G,H,P,Q,Dif] = swapblk(G,H,m1b,m1e,n1b,n1e,m2b,m2e,n2b,n2e)
%SWAPBLK Swap order of two adjacent subpencils in a matrix pencil.
%   [Gp,Hp,U,V] = SWAPBLK(G,H,m1b,m1e,n1b,n1e,m2b,m2e,n2b,n2e) change order
%   of the two adjacent square regular subpencils 
%
%        G1-sH1 := G(m1b:m1e,n1b:n1e)-sH(m1b:m1e,n1b:n1e) 
%   and 
%        G2-sH2 := G(m2b:m2e,n2b:n2e)-sH(m2b:m2e,n2b:n2e) 
%
%   in the matrix pencil G-sH, such that Gp - sHp = U'*(G - sH)*V. The
%   matrix pairs (G1,H1) and (G2,H2) must be in generalized Schur form and
%   G1-sH1 must be to the top-left of G2-sH2 in the pencil G-sH. The matrix
%   pencil G-sH may have complex entries.
%
%   [Gp,Hp,U,V,Dif] = SWAPBLK(G,H,m1b,m1e,n1b,n1e,m2b,m2e,n2b,n2e)  also
%   returns the Frobenius norm-based estimate Dif, which is an upper bound
%   of Dif[(G1,H1),(G2,H2)] and measures the separation of the spectrum of the
%   matrix pair (G1,H1) from the spectrum of the matrix pair (G2,H2).

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

[m,n] = size(G);

m1 = m1b:m1e;
n1 = n1b:n1e;
m2 = m2b:m2e;
n2 = n2b:n2e;

% If any of the indices are empty we have nothing to swap...
if isempty(m1) || isempty(n1) || isempty(m2) || isempty(n2)
  P = eye(m);
  Q = eye(n);
  return
end

if m1e >= m2b || n1e >= n2b
  error('MCS:swapblk:BlockOrder','The first subpencil must be to the top-left of the second.')
end

if length(m1) ~= length(n1) || length(m2) ~= length(n2)
  error('MCS:swapblk:NonSquare', 'The subpencils to swap must be square.')
end

% Solve the generalized Sylvester equation
%   AX + YB = -C 
%   DX + YE = -F
% The Lapacke routine ZTGSYL is used.
[X,Y,Dif] = gsylve(G(m1,n1),-G(m2,n2),-G(m1,n2),H(m1,n1),-H(m2,n2),-H(m1,n2));

if Dif < eps
  P = eye(m);
  Q = eye(n);
  warning('MCS:swapblk:CloseSpectrum',['The two subpencils are not swaped.' %
    'The separation between the spectrums of (G1,H1) and (G2,H2) is to small: Gsylve returns dif = %0.3e'],Dif);
  return
end

[mX,nX] = size(X);
[Xorth,~] = qr([eye(nX) zeros(nX,mX); X eye(mX)]);
[~,Yorth] = rq([eye(nX) zeros(nX,mX); Y eye(mX)]);

%                             
%   Prow*( G([m1 m2],[n1 n2]) - sH([m1 m2],[n1 n2]) )*Pcol = 
%
%     [ G(m2,n2)-sH(m2,n2)          0          ]
%     [ G(m1,n2)-sH(m1,n2)  G(m1,n1)-sH(m1,n1) ]
%
% where Prow and Pcol are row and column permutations, respectively. Then
% from the solution (X,Y) of the generalized Sylvester equation: 
%
%   Yorth*Prow*(G([m1 m2],[n1 n2]) - sH([m1 m2],[n1 n2]) )*Pcol*Xorth =
%
%     [ S(m3,n3)-sT(m3,n3)          0          ]
%     [         ~0          S(m4,n4)-sT(m4,n4) ]
%
% where S(m3,n3)-sT(m3,n3) and G(m2,n2)-sH(m2,n2) have the same
% eigenvalues, and S(m4,n4)-sT(m4,n4) and G(m1,n1)-sH(m1,n1) have the same 
% eigenvalues.
%

mblk = m1b:m2e;
nblk = n1b:n2e;

P = eye(m);
Q = eye(n);
P(mblk,mblk) = Yorth*P([m2 m1],mblk);
Q(nblk,nblk) = Q(nblk,[n2 n1])*Xorth;

G = P*G*Q;
H = P*H*Q;
P = P';




