function varargout = weyrsize(weyr,varargin)
%WEYRSIZE Total size of blocks given in Weyr characteristics.
%   D = WEYRSIZE(Weyr,Type) returns the two-element row vector D = [M,N]
%   containing the matrix size of the blocks given in Weyr characteristics.
%   Parameter Type is the type of blocks and can be:
%     'jordan' for Jordan blocks,
%     'left'   for left singular blocks, and
%     'right'  for right singular blocks.
%
%   [M,N] = WEYRSIZE(Weyr,Type) returns the number of rows and columns in M
%   and N, respectively.
%
%   N = WEYRSIZE(Weyr,Dim,Type) returns the size of dimension Dim in N.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.
    
narginchk(2,3);

m = 0;
n = 0;

if nargin == 2
  nargoutchk(0,2);
  type = varargin{1};
else
  nargoutchk(0,1);
  dim  = varargin{1};
  type =  varargin{2};
end

if ~isempty(weyr)
  sizes = [weyr(1:length(weyr)-1) - weyr(2:length(weyr)) ...
    weyr(length(weyr))];
  
  switch type
    case 'jordan'
      n = sum(sizes .* (1:length(weyr)));
      m = n;
    case 'left'
      n = sum(sizes .* (0:length(weyr)-1));
      m = n + weyr(1);
    case 'right'
      m = sum(sizes .* (0:length(weyr)-1));
      n = m + weyr(1);
    otherwise
      error('Unknown type of block.');
  end
end

if nargout < 2 && nargin <= 2
  varargout{1} = [m,n];
elseif nargin > 2 && dim == 1
  varargout{1} = m;
elseif nargin > 2 && dim == 2
  varargout{1} = n;
elseif nargin > 2 && dim > 2
  varargout{1} = 1;
else
  varargout{1} = m;
  varargout{2} = n;
end
