function varargout = mcsformat(fmt,persist)
%MCSFORMAT Set format to display structure and block objects.
%   MCSFORMAT with no argument sets the output format to display canonical
%   structure and block objects in MCS Toolbox to the default BLOCK format.
%
%   The output format can be changed to different formats as follows:
%     MCSFORMAT BLOCK    Canonical block format.
%     MCSFORMAT SEGRE    Segre-type characteristics, where the sizes of the
%                        canonical blocks are in non-increasing order.
%     MCSFORMAT SEGREC   A compact variant of SEGRE where multiple blocks
%                        of the same size is written as n*..., where n is
%                        the number of blocks.
%     MCSFORMAT WEYR     Weyr-type characteristics representation of the
%                        canonical blocks, where the integer in position k
%                        is the number of blocks greater than or equal to
%                        k for regular blocks or k-1 for singular blocks.
%     MCSFORMAT SIZES    Same as SEGRE but where the sizes of the canonical
%                        blocks may be unordered (displyed in the order
%                        they where created).
%
%   MCSFORMAT can also be called as a function:
%     MCSFORMAT(Fmt) 
%   where Fmt is the output format as a string. For example
%     mcsformat('segre')
%
%   MCSFORMAT(Fmt, Persistent) also set if the used format shall be
%   persistent between Matlab sessions (Persistent = true) or not
%   (Persistent = false). Default is false.
%
%   Fmt = MCSFORMAT returns the current output format as a string.
%   [Fmt,IsPersistent] = MCSFORMAT also returns if the used format is
%   persistent between Matlab sessions or not.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.


narginchk(0,2);
nargoutchk(0,2);

global MCS_PREF;

if nargin > 0 && (~ischar(fmt) || ~any(strcmpi(fmt,{'weyr','segre','segrec','sizes','block'})))
  error('Unknown format.')
end

if nargin > 0
  fmt = lower(fmt);

  MCS_PREF.format = fmt;
  
  if nargin > 1 && persist
    if ispref('mcs','format')
      setpref('mcs','format',fmt);
    else
      addpref('mcs','format',fmt);
    end
  end
else
  if nargout > 0
    if  isfield(MCS_PREF,'format')
      fmt = MCS_PREF.format;
    elseif ispref('mcs','format')
      fmt = getpref('mcs','format');
    else
      fmt = 'block';
      MCS_PREF.format = fmt;
    end
  else
    fmt = 'block';
    MCS_PREF.format = fmt;
  end
end

if nargout == 2
  ispersistent = false;
  if ispref('mcs','format')
    ispersistent = strcmp(getpref('mcs','format'), ...
      MCS_PREF.format);
  end
end

if nargin == 0 && nargout > 0
  varargout{1} = fmt;
  if nargout > 1
    varargout{2} = ispersistent;
  end
end
