function varargout = mcstolerance(tol,gap,persist)
%MCSTOLERANCE Default tolerance parameters used by MCS Toolbox. 
%   T = MCSTOLERANCE returns the two-element row vector T = [Tol, Gap]
%   containing the default tolerance parameters used for the functions in
%   the toolbox. The default values are Tol = 1e-12 and Gap = 1000, if they
%   have not been changed (see below). The tolerances can also usually be
%   overridden by giving them as parameters to the individual routines.
%
%   [Tol,Gap] = MCSTOLERANCE returns the tolerance parameters as separate
%   output variables.
%
%   [Tol,Gap,IsPersistent] = MCSTOLERANCE also returns if the used
%   tolerance parameters are persistent between sessions or not.
%
%   MCSTOLERANCE(Tol,Gap) The positive parameters Tol and Gap is set as
%   default tolerance parameters in all toolbox routines. Tol should
%   reflect the relative uncertainty in the data and should be at least
%   about eps (nominally between 1e-8 and 1e-12). Gap determines the ratio
%   between the smallest preserved singular value and the largest singular
%   value imposed to zero. Gap should be at least 1 (nominally 1000).
%
%   MCSTOLERANCE(Tol,Gap,Persistent) The values of Tol and Gap can be made
%   persistent between Matlab sessions by setting the parameter Persistent
%   to non-zero (true). Default is false.
%
%   If the value of Gap is set to zero or if the value of Tol is less then
%   eps a warning is issued. This warning message can be suppressed by
%   typing
%     warning('off','MCS:mcstolerance:ZeroTolerance')
%
%   See also MCSEVCLMTH.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

global MCS_PREF;

if nargin > 0
  narginchk(2,3);
  nargoutchk(0,0);
  
  if tol < 0 || gap < 0
    error('Both toleranace and gap must be larger or equal to zero.');
  end
  if nargin < 3
    persist = 0;
  end
  
  MCS_PREF.tol = tol;
  MCS_PREF.gap = gap;
  
  if persist
    if ispref('mcs','tol')
      setpref('mcs','tol',tol);
    else
      addpref('mcs','tol',tol);
    end
    if ispref('mcs','gap')
      setpref('mcs','gap',gap);
    else
      addpref('mcs','gap',gap);
    end
  end
  
  try
    sgsettolerance(tol,gap);
  catch 
    % SG is probably not present
  end
else
  nargoutchk(0,3);
  
  if isfield(MCS_PREF,'tol')
    tol = MCS_PREF.tol;
  elseif ispref('mcs','tol')
    tol = getpref('mcs','tol');
  else
    tol = 1e-12;
  end
  
  if isfield(MCS_PREF,'gap')
    gap = MCS_PREF.gap;
  elseif ispref('mcs','gap')
    gap = getpref('mcs','gap');
  else
    gap = 1000;
  end
end

if gap == 0
  warning('MCS:mcstolerance:ZeroTolerance', ...
    'Tolerance parameter Gap is zero');
end
if tol < eps
  warning('MCS:mcstolerance:ZeroTolerance', ...
    'Tolerance parameter Tol is less then eps.');
end

switch nargout
  case {0,1}
    varargout{1} = [tol,gap];
  case 2
    varargout{1} = tol;
    varargout{2} = gap;
  case 3
    ispersistant = false;
    if ispref('mcs','tol') && ...
        ispref('mcs','gap')
      ispersistant = ...
        getpref('mcs','tol') == tol & ...
        getpref('mcs','gap') == gap;
    end
    varargout{3} = ispersistant;
end
