function [T] = ptanspace(G, H)
%PTANSPACE Tangent space of the orbit of a matrix pencil.
%   T = PTANSPACE(G,H) returns the matrix representation 
%
%          | kron(G.',I_m) - kron(I_n,G) |
%      T = |                             |
%          | kron(H.',I_m) - kron(I_n,H) |
%
%   of the tangent space to the strict equivalence orbit(G-sH) at G-sH,
%   where G-sH is a m-by-n matrix pencil and I_x is the x-by-x identity
%   matrix. The tangent space is the range of the (2mn)-by-(m^2+n^2) matrix
%   T.
%
%   The tangent space consists of the matrix pencils of the form
%      T_G - sT_H = X(G-sH) - (G-sH)Y,
%   where X is an m-by-m matrix and Y is an n-by-n matrix. Equivalently,
%   the tangent vectors T_G - sT_H can be represented as 
%
%      |vec(T_G)|   |kron(G.',I_m)|          |kron(I_n,G)|
%      |        | = |             | vec(X) - |           | vec(Y).
%      |vec(T_H)|   |kron(H.',I_m)|          |kron(I_n,H)|
%
%   See also PCODIM, MTANSPACE.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(2,2)
nargoutchk(0,1)
if size(G) ~= size(H)
  error('MCS:ptanspace','The matrices must have the same size.')
end

Im = eye(size(G,1));
In = eye(size(G,2));

T = [kron(G.',Im) -kron(In, G); kron(H.',Im) -kron(In,H)];
