function [c] = s2codim(varargin)
%S2CODIM Codimension of a matrix pair.
%   S2CODIM([Type],A,B) or
%   S2CODIM([Type],A,C) computes the codimension of the tangent space of
%   the feedback equivalence orbit of a system pencil [A B]-s[I 0] (or
%   [A;C]-s[I;0]). Default tolerance parameter of RANK is used.
%
%   The optional argument Type is a string that for a controllability pair
%   (A,B) should be 'ab' and for an observability pair (A,C) should be
%   'ac'. If the argument is absent, the sizes of A and B (or C) will
%   determine the type. If B is square, 'ab' is assumed.
%
%   S2CODIM([Type],A,C,Tol) or
%   S2CODIM([Type],A,B,Tol) uses the specified tolerance Tol.
%
%   S2CODIM(SSstr) determines the codimension of the orbit of state-space
%   structure object SSstr. The codimension is determined with respect to
%   the represented canonical structure not the tangent space. For further
%   information, see SSSTRUCT/CODIM.
%
%   See also S2TANSPACE, SSSTRUCT/CODIM, PCODIM.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.
  
nargoutchk(0,1);
narginchk(1,5);

if isa(varargin{1},'ssstruct')
  narginchk(1,2);
  c = codim(varargin{:});
else
  [type,A,X,n,x,narg] = pairchk(varargin{:});
  
  if nargin > narg
    tol = varargin{narg+1};
  else
    switch type
      case 'ab'
        tol = (n+x) * norm([A X]) * eps;
      case 'ac'
        tol = (n+x) * norm([A;X]) * eps;
    end
  end
  
  T = s2tanspace(A,X);
  c = size(T, 1) - rank(T, tol);
end
