function T = s2tanspace(varargin)
%S2TANSPACE Tangent space of the orbit of a matrix pair.
%   S2TANSPACE([Type],A,B) returns the matrix representation
%
%               | kron(A.',I_n)-kron(I_n,A)  kron(I_n,B)        0      |
%      T(A,B) = |                                                      |
%               |      kron(B.',I_n)             0        kron(I_m,B)  |
%
%   of the tangent space to the feedback equivalence orbit([A B]-s[I 0]) at
%   (A,B). 
%   S2TANSPACE([Type],A,C) returns the matrix representation
%
%               | kron(A.',I_n)-kron(I_n,A)  kron(C.',I_n)        0      |
%      T(A,C) = |                                                        |
%               |     -kron(I_n,C)               0         kron(C.',I_p) |
%
%   of the tangent space to the feedback equivalence orbit([A;C]-s[I;0]) at
%   (A,C).
%
%   The optional parameter, Type, is a string argument that for a
%   controllability pair (A,B) should be 'ab' and for an observability
%   pair (A,C) should be 'ac'. If the argument is absent, the sizes of A
%   and B (or C) will determine the type. If B is square, 'ab' is assumed.
%
%   The tangent space for (A,B) and (A,C) consist of the matrices of the
%   forms
%      |T_A  T_B| = X |A  B| + |A  B| |-X 0; V W|
%   and
%      | T_A |   |X  Y| | A |   | A |
%      |     | = |    | |   | - |   | X,
%      | T_C |   |0  Z| | C |   | C |
%   respectively, where X, Y, Z, V, and W are matrices of conforming sizes.
%   Equivalently, the tangent vectors T_A and T_B at (A,B) can be
%   represented as
%
%      |vec(T_A)|   |kron(A.',I_n)-kron(I_n,A)|
%      |        | = |                         | vec(X) + 
%      |vec(T_B)|   |      kron(B.',I_n)      |
%
%                    |kron(I_n,B)|          |     0     |
%                    |           | vec(V) + |           | vec(W),
%                    |     0     |          |kron(I_m,B)|
%
%   and the tangent vectors T_A and T_C at (A,C) as
%
%      |vec(T_A)|   |kron(A.',I_n)-kron(I_n,A)|
%      |        | = |                         | vec(X) + 
%      |vec(T_C)|   |      -kron(I_n,C)       |
%
%                    |kron(C.',I_n)|          |      0      |
%                    |             | vec(Y) + |             | vec(Z).
%                    |      0      |          |kron(C.',I_p)|
%
%   See also S2CODIM, SSSTRUCT/CODIM.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

narginchk(2,3)
nargoutchk(0,1)
  
[type,A,X,n,x] = pairchk(varargin{:});

In = eye(n);
Ix = eye(x);
switch(type)
  case 'ab'
    %T(A,B) = [A^T * I_n - I_n * A   I_n * B       0  ]
    %         [     B^T * I_n           0 ,    I_m * B]
    T = [kron(A.',In)-kron(In, A), kron(In,X),     zeros(n^2,x^2);...
      kron(X.',In),              zeros(n*x,n*x), kron(Ix,X)];
  case 'ac'
    %T(A,C) = [A^T * I_n - I_n * A   C^T * I_n      0    ]
    %         [     -I_n * C             0      C^T * I_p]
    T = [kron(A.',In)-kron(In, A), kron(X.',In),    zeros(n^2,x^2);...
      -kron(In,X),               zeros(n*x,n*x),  kron(X.',Ix)];
end

  
  

  

