function [T] = scmtanspace(A)
%SCMTANSPACE Tangent space of the *congruence orbit of a matrix.
%   T = SCMTANSPACE(A) returns the matrix representation 
%
%       |kron(re(A).',In)+kron(In,re(A))*P  -kron(im(A).',In)+kron(In,im(A))*P|
%   T = |                                                                     |
%       |kron(im(A).',In)+kron(In,im(A))*P   kron(re(A).',In)-kron(In,re(A))*P|
%
%   of the tangent space to the *congruence orbit(A) at A over the field of
%   real numbers, where A is an n-by-n matrix, re(A) and im(A) are the real and
%   imaginary parts of A, I_n is the n-by-n identity matrix, and P is the
%   (n^2)-by-(n^2) permutation matrix that can "transpose" n-by-n matrices,
%   i.e., vec(X')=P*vec(X) for any n-by-n matrix X. The tangent space is
%   the range of the (2*n^2)-by-(2*n^2) matrix T.
%
%   The tangent space consists of the matrix of the form
%      T_A = X.'*A + A*X,
%   where X is an n-by-n matrix. Equivalently, the tangent vectors 
%   T_A can be represented as 
%
%      |vec(re(T_A))|   |kron(re(A).',In)+kron(In, re(A))*P|             
%      |            | = |                                  | vec(re(X)) + 
%      |vec(im(T_A))|   |kron(im(A).',In)+kron(In, im(A))*P|
%
%                        |-kron(im(A).',In)+kron(In, im(A))*P|
%                        |                                   | vec(im(X))
%                        | kron(re(A).',In)-kron(In, re(A))*P|
%
%   See also SCMCODIM, CMTANSPACE.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

  narginchk(1,1)
  nargoutchk(0,1)
  [n,m]=size(A);
  if n ~= m 
      error('MCS:scmtanspace','The input matrix must be square.')
  else
      In = eye(n);
      X=zeros((n)^2,(n)^2);
      for t=1:(n)
         for l=1:(n)
            X(l+n*(t-1),n*(l-1)+t)=1;
         end
      end
      i=complex(0,1);
      rA=(A+conj(A))/2;
      iA=(A-conj(A))/(i*2);
      T = [kron(rA.',In)+kron(In, rA)*X  -kron(iA.',In)+kron(In, iA)*X;...
        kron(iA.',In)+kron(In, iA)*X  kron(rA.',In)-kron(In, rA)*X];
  end