function [T] = sptanspace(G, H)
%SPTANSPACE Tangent space to the congruence orbit of a symmetric matrix pencil.
%   T = SPTANSPACE(G,H) returns the matrix representation 
%
%          | kron(G.',In) + kron(In,G)*P | 
%      T = |                             |
%          | kron(H.',In) + kron(In,H)*P |
%
%   of the tangent space to the congruence orbit(G-sH) at G-sH, where G-sH
%   is a n-by-n symmetric matrix pencil and I_n is the n-by-n identity
%   matrix, and P is the (n^2)-by-(n^2) permutation matrix that can
%   "transpose" n-by-n matrices, i.e., vec(X')=P*vec(X) for any n-by-n
%   matrix X. The tangent space is the range of the (2n^2)-by-(n^2) matrix
%   T.
%
%   The tangent space consists of the matrix pencils of the form
%      T_G - sT_H = X'(G-sH) + (G-sH)X,
%   where X is an n-by-n matrix. Equivalently, the tangent vectors 
%   T_G - sT_H can be represented as 
%
%      |vec(T_G)|   |kron(G.',I_n)|          |kron(I_n,G)|
%      |        | = |             | vec(X) - |           | P*vec(X)
%      |vec(T_H)|   |kron(H.',I_n)|          |kron(I_n,H)|
%
%   See also SPCODIM, PTANSPACE, SSPTANSPACE.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

  narginchk(2,2)
  nargoutchk(0,1)
  if size(G) ~= size(H)
    error('MCS:sptanspace','The matrices must have the same size.')
  end

  
  [n,m]=size(G);
  if n ~= m 
      error('MCS:sptanspace','The input matrices must be square.')
  else
      In = eye(n);
      X=zeros((n)^2,(n)^2);
      for t=1:(n)
         for l=1:(n)
            X(l+n*(t-1),n*(l-1)+t)=1;
         end
      end
      T = [kron(G.',In)+kron(In, G)*X; kron(H.',In)+kron(In,H)*X];
  end

