classdef (Abstract) canonicalforms
  %CANONICALFORMS Methods for computing the canonical form.
  %   CANONICALFORMS returns a matrix or matrix pencil in the canonical
  %   form of the specified canonical structure object.
  %
  %   Can only be accessed through a canonical structure object (a subclass
  %   of mcsstruct).

  % *** NOTE! *** 
  %
  % Set access to valid canonical structures in the call to the method
  % validateStructure. Corresponding canonical form method must be
  % implemented in all associated canonical block classes.
  %
  %*** NOTE! ***
  
  methods
    %% *** Matrices ***
    function varargout = jnf(obj)
      %JNF Matrix in the Jordan normal form.
      %   J = JNF(StructObj) returns a matrix J in Jordan Normal/Canonical
      %   Form of a matrix structure object StructObj. This methods calls
      %   JCF.
      %
      %   See also JCF.
      [varargout{1:nargout}] = jcf(obj);
    end
    %------------------------------
    
    function varargout = jcf(obj,not)
      %JCF Matrix in the Jordan canonical form.
      %   J = JCF(StructObj) takes a matrix structure object StructObj and
      %   returns a matrix J in Jordan Canonical Form (JCF) corresponding
      %   to the canonical structure information.
      %
      %   [X,Z] = JCF(StructObj) returns the two matrices Z and X such
      %   that ZJZ' = X, where Z is a random orthonormal matrix.
      %
      %   [X,Z,J] = JCF(StructObj) also returns J in JCF.
      %
      %   ... = JCF(StructObj,'segre') sorts the blocks in each canonical
      %   block object in non-increasing block size order. By default are
      %   the blocks presented in the order they were created.
      %
      %   Valid canonical structure objects are: mstruct.
      %
      %   Example:
      %      >> J = jcf(mstruct([3 1],4)); returns a matrix J with one
      %      Jordan block of size 3-by-3 and one of size 1-by-1, both with
      %      eigenvalue 4.
      %
      %   See also MSTRUCT, KCF.

      % Validate that the canonical form exists for the structure object
      validateStruct(obj,{'mstruct'});

      narginchk(1,2);
      nargoutchk(0,3);
      
      if nargin == 1
        not = 'sizes';
      else
        if ~strcmp(not,{'sizes','segre'})
          error('MCS:canonicalforms:Notation','Block ordering must be either ''segre'' or ''sizes''.');
        end
      end
      
      [m,n] = size(obj);
      J = zeros(m,n);
      p = 1;

      for k = 1:length(obj.cstructure)
        [s,~] = size(obj.cstructure{k});
        J(p:p+s-1,p:p+s-1) = jcf(obj.cstructure{k},not);
        p = p + s;
      end
      
      switch nargout
        case {0,1}
          varargout{1} = J;
        case {2,3}
          Z = orth(rand(m,n));
          varargout{1} = Z*J*Z';
          varargout{2} = Z;
          if nargout == 3
            varargout{3} = J;
          end
      end
    end
    %------------------------------
    
    function varargout = ccf(obj,not)
      %CCF Matrix in the congruent canonical form.
      %   C = CCF(StructObj) takes a matrix structure object StructObj and
      %   returns a matrix C in Congruent Canonical Form (CCF)
      %   corresponding to the canonical structure information.
      %
      %   [X,Z] = CCF(StructObj) returns the two matrices Z and X such
      %   that Z*C*Z' = X, where Z is a random orthonormal matrix.
      %
      %   [X,Z,C] = CCF(StructObj) also returns C in CCF.
      %
      %   ... = CCF(StructObj,'segre') sorts the blocks in each canonical
      %   block object in non-increasing block size order. By default are
      %   the blocks presented in the order they were created.
      %
      %   Valid canonical structure objects are: cmstruct, scmstruct.
      %
      %   Example:
      %      >> C = ccf(cmstruct([2],[2],4));
      %      returns a 2-by-2 Gamma block and one W block of
      %      size 4-by-4 with the associated eigenvalue 4:
      %
      %            |0 -1  0  0  0  0|
      %            |1  1  0  0  0  0|
      %            |0  0  0  0  1  0|
      %        C = |0  0  0  0  0  1|
      %            |0  0  4  1  0  0|
      %            |0  0  0  4  0  0|
      %
      %   See also CMSTRUCT, SCMSTRUCT.
      
      % Validate that the canonical form exists for the structure object
      validateStruct(obj,{'cmstruct','scmstruct'});

      narginchk(1,2);
      nargoutchk(0,3);
      
      if nargin == 1
        not = 'sizes';
      else
        if ~strcmp(not,{'sizes','segre'})
          error('MCS:canonicalforms:Notation','Block ordering must be either ''segre'' or ''sizes''.');
        end
      end
      
      [m,n] = size(obj);
      C = zeros(m,n);
      p = 1;
      for k = 1:length(obj.cstructure)
        [s,~] = size(obj.cstructure{k});
        C(p:p+s-1,p:p+s-1) = ccf(obj.cstructure{k},not);
        p = p + s;
      end
      
      switch nargout
        case {0,1}
          varargout{1} = C;
        case {2,3}
          Z = orth(rand(m,n));
          varargout{1} = Z*C*Z';
          varargout{2} = Z;
          if nargout == 3
            varargout{3} = C;
          end
      end
    end
    %------------------------------
    
    %% *** Matrix pencils ***
    function varargout = kcf(obj,not)
      %KCF Matrix pencil in the Kronecker canonical form.
      %   [G,H] = KCF(StructObj) takes a pencil structure object
      %   StructObj and returns a matrix pencil G-sH in Kronecker Canonical
      %   Form (KCF) corresponding to the structure. (See below for the
      %   corresponding syntax for matrices.)
      %
      %   [A,B,P,Q] = KCF(StructObj) returns the matrices A, B, P, and Q
      %   such that P(G-sH)Q' = A-sB where P and Q are random orthonormal
      %   matrices.
      %
      %   [A,B,P,Q,G,H] = KCF(StructObj) also returns G and H in KCF.
      %
      %   ... = KCF(StructObj,'segre') sorts the blocks in each canonical
      %   block object in non-increasing block size order. By default are
      %   the blocks presented in the order they were created.
      %
      %   Valid canonical structure objects are: pstruct, spstruct,
      %   sspstruct, ssstruct.
      %
      %   Examples:
      %      >> pstr = pstruct([], [], {[2] [1]}, [1 3], [2]);
      %      >> [G,H] = pstr.kcf;
      %      returns a matrix pencil of the following form:
      %                         | J2(1)-sI    0       0   |
      %                  G-sH = |   0      J1(3)-sI   0   |
      %                         |   0         0      N2   |
      %
      %   See also PSTRUCT, SPSTRUCT, SSPSTRUCT, SSSTRUCT.

      % Validate that the canonical form exists for the structure object
      validateStruct(obj,{'pstruct','ssstruct',...
          'sspstruct','spstruct'});
      
      if nargin == 1
        not = 'sizes';
      else
        if ~strcmp(not,{'sizes','segre'})
          error('MCS:canonicalforms:Notation','Block ordering must be either ''segre'' or ''sizes''.');
        end
      end
      
      narginchk(1,2);
      nargoutchk(2,6);
      
      if nargout == 3 || nargout == 5
        error('Invalid number of output arguments.');
      end
  
      [m,n] = size(obj);
      G = zeros(m,n);
      H = zeros(m,n);
      p = 1;
      q = 1;

      for k = 1:length(obj.cstructure)
        [s,t] = size(obj.cstructure{k});
        [G(p:p+s-1,q:q+t-1),H(p:p+s-1,q:q+t-1)] = kcf(obj.cstructure{k},not);
        p = p + s;
        q = q + t;
      end
      
      switch nargout
        case 2
          varargout{1} = G;
          varargout{2} = H;
        case {4,6}
          P = orth(rand(m,m));
          Q = orth(rand(n,n));
          varargout{1} = P*G*Q';
          varargout{2} = P*H*Q';
          varargout{3} = P;
          varargout{4} = Q;
          if nargout == 6
            varargout{5} = G;
            varargout{6} = H;
          end
      end
    end % End KCF
    %------------------------------
    
    %% *** System pencils (stat-space, descriptor, etc.)***
    function varargout = bcf(obj,varargin)
      %BCF System pencil in the Brunovsky canonical form.
      %   [A,B,C,D] = BCF(StructObj) takes a state-space system structure
      %   object StructObj and returns the corresponding system pencil
      %       S-sT = [A B; C D] -s[I 0; 0 0],
      %   in a generalized Brunovsky Canonical Form (BCF).
      %
      %   [S,T] = BCF(StructObj) returns the system pencil S-sT above.
      %
      %   [A,B] = BCF(StructObj,'ab') returns the matrix pair (A,B) of the
      %   sub-system pencil [A B]-s[I 0] in controllability BCF.
      %
      %   [A,C] = BCF(StructObj,'ac') returns the matrix pair (A,C) of the
      %   sub-system pencil [A; C]-s[I; 0] in observability BCF.
      %
      %   [A2,B2,C2,D2,P,S,T,R,Q] = BCF(StructObj) returns a transformed
      %   system pencil and the transformation matrices such that
      %       |P S|(|A B| _ |sI 0|)|P' 0 |
      %       |0 T|(|C D|   | 0 0|)|R  Q'|
      %      =
      %       |A2 B2| _ |sI 0|
      %       |C2 D2|   | 0 0|
      %   where P, T, and Q are random orthonormal matrices and S and R are
      %   random matrices of conforming sizes. 
      %
      %   [A2,B2,P,R,Q] = BCF(StructObj,'ab') or
      %   [A2,C2,P,S,T] = BCF(StructObj,'ac') returns the transformed
      %   system pencil and the transformation matrices of the sub-system
      %   pencils [A B]-s[I 0] or [A; C]-s[I; 0], respectively.
      %
      %   [G,H,P,Q,S,T] = BCF(StructObj) returns the matrices G, H, P, and
      %   Q such that P(S-sT)Q' = G-sH where P and Q are random orthonormal
      %   matrices. 
      %
      %   ... = BCF(...,'segre') sorts the blocks in each canonical
      %   block object in non-increasing block size order. By default are
      %   the blocks presented in the order they were created.
      %
      %   Valid canonical structure objects are: ssstruct
      %
      %   See also SSSTRUCT, KCF.
      
      % Validate that the canonical form exists for the structure object
      validateStruct(obj,{'ssstruct'});

      narginchk(1,3)
      
      not = 'sizes';
      type = '';
      if nargin == 2
        switch varargin{1}
          case 'ab'
            type = 'ab';
          case 'ac'
            type = 'ac';
          case 'segre'
            not = 'segre';
          otherwise
            error('MCS:canonicalforms:UnknownArgument','Unknown argument.')
        end
      elseif nargin == 3
        type = varargin{1};
        not = varargin{2};
      end
      
      if ~isempty(type)
        if nargout == 2 || nargout == 5
          switch type
            case 'ab'
              strobj = obj.ctrbpair();
            case 'ac'
              strobj = obj.obsvpair();
            otherwise
              error('MCS:canonicalforms:UnknownArgument','Unknown argument.')
          end
        else
          error('In this form BCF must return 2 or 5 outputs.')
        end
      else
        strobj = obj;
      end
      
      A = []; B = []; C  = []; D = [];
      for k = 1:length(strobj.cstructure)
        [Ab,Bb,Cb,Db] = bcf(strobj.cstructure{k},not);
        A = blkdiag(A,Ab);
        B = blkdiag(B,Bb);
        C = blkdiag(C,Cb);
        D = blkdiag(D,Db);
      end
      
      if ~isempty(type)
        if nargout == 2
          varargout{1} = A;
          if strcmp(type,'ab')
            varargout{2} = B;
          else
            varargout{2} = C;
          end
        else
          P = orth(rand(size(A,1)));
          varargout{3} = P;
          if strcmp(type,'ab')
            Q = orth(rand(size(B,2)));
            R = rand(size(B'));
            varargout{1} = P*A*P' + P*B*R;
            varargout{2} = P*B*Q';
            varargout{4} = R;
            varargout{5} = Q;
          else
            T = orth(rand(size(C,1)));
            S = rand(size(C'));
            varargout{1} = P*A*P' + S*C*P';
            varargout{2} = T*C*P';
            varargout{4} = S;
            varargout{5} = T;
          end
        end
      else % Return full system pencil
      
        if nargout < 2
          varargout{1} = [A B ; C D];
        elseif nargout == 2
          varargout{1} = [A B; C D];
          varargout{2} = [eye(size(A)) zeros(size(B)); zeros(size([C D]))];
        elseif nargout == 4
          varargout{1} = A;
          varargout{2} = B;
          varargout{3} = C;
          varargout{4} = D;
        elseif nargout == 6
          S = [A B; C D];
          T = [eye(size(A)) zeros(size(B)); zeros(size([C D]))];
          P = orth(rand(size(S,1)));
          Q = orth(rand(size(S,2)));
          varargout{1} = P*S*Q';
          varargout{2} = P*T*Q';
          varargout{3} = P;
          varargout{4} = Q;
          varargout{5} = S;
          varargout{6} = T;
        elseif nargout == 9
          P = orth(rand(size(A,1)));
          T = orth(rand(size(C,1)));
          Q = orth(rand(size(B,2)));
          S = rand(size(C'));
          R = rand(size(B'));
          varargout{1} = P*A*P' + S*C*P' + P*B*R + S*D*R;
          varargout{2} = P*B*Q' + S*D*Q';
          varargout{3} = T*C*P' + T*D*R;
          varargout{4} = T*D*Q';
          varargout{5} = P;
          varargout{6} = S;
          varargout{7} = T;
          varargout{8} = R;
          varargout{9} = Q;
        else
          error('Invalid number of output arguments.')
        end
      end
    end
    %------------------------------

  end % End methods
  
  methods (Access = private)
    function validateStruct(obj,validStructs)
      %VALIDATESTRUCT Validate the canonical from.
      % Validate that the canonical form exists for the canonical structure
      % object.
      
      if ~any(cellfun(@(x)isa(obj,x),validStructs))
        % To get information of the calling function use dbstack.
        error('MCS:canonicalforms:notValidStruct',...
          'The canonical form is not supported for this canonical structure object.')
      end
    end
  end % End private methods
end % End CLASSDEF

