%CMSTRUCT Create a congruence matrix structure object.
%   CMSTRUCT creates a canonical structure object representing a matrix
%   A in its canonical form under congruence.
%
%   A CMSTRUCT object can consist of the following three canonical blocks:
%     gblock - Gammma block defined as:
%                          |0         . .|
%                          |        . .  |
%               Gamma_n := |      1  1   |         n-by-n
%                          |  -1 -1      |
%                          |1  1        0|
%
%     wblock - W block associated with a specified and admissible
%              eigenvalue mu, defined as:
%                         |  0    I_n|
%                  W_n := |          |            2n-by-2n
%                         |J_n(mu) 0 |
%              where mu ~= {0, (-1)^(n+1)} and J_n(mu) is an n-by-n Jordan
%              block associated with the eigenvalue mu.
%
%     zjblock - Jordan block associated with the zero eigenvalue,
%               defined as:
%                          |0 1   0|
%                J_n(0) := |  0 .  |               n-by-n
%                          |    . 1|
%                          |0     0|
%
%
%   StructObj = CMSTRUCT() returns an empty canonical structure object
%   StructObj representing a matrix A under congruence. The canonical
%   structure information can be specified using one of the three forms
%   'Size vector form', Property-value form', or 'Object form', which are
%   explained below.
%
%   Size vector form 
%   ---------------- 
%   StructObj = CMSTRUCT(GBlocks,WBlocksv,Eigv) returns a new canonical
%   structure object StructObj representing a matrix A under congruence.
%   The parameter GBlocks defines the Gamma blocks of the structure and is
%   given as row-vectors with the sizes of the blocks. The parameter
%   WBlocksv defines the W blocks associated with the finite Jordan blocks
%   of the structure and must be a cell-array of row-vectors where each
%   vector contains the sizes of blocks associated with the same
%   eigenvalue. Eigv is a row-vector with the values of each associated
%   eigenvalue corresponding to the row-vectors in WBlocksv, where the
%   eigenvalues must be non-zero complex scalars not equal to (-1)^('size
%   of the block'/2 + 1) for all blocks associated with the same
%   eigenvalue.
%
%   StructObj = CMSTRUCT(GBlocks,WBlocksv,Eigv,ZJBlocks) also sets the
%   sizes of the Jordan blocks with an associated zero eigenvalue.
%
%   Property-value form
%   -------------------
%   StructObj =
%   CMSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...) specifies
%   the block types BlockName and the sizes StructInt in property-value
%   form. BlockName is a string specifying the canonical block to be
%   created, and can be one of:
%      'gblock'  - Gamma blocks
%      'wblock' - W blocks associated with finite eigenvalues
%      'zjblock' - Jordan blocks associated with the zero eigenvalue.
%
%   W blocks associated with a specified finite eigenvalue are given as a
%   cell-array tuple in the form {StructInt Eigenvalue}. 
%
%   Example:
%      >> cmstr = cmstruct('wblock',{[3 2],2},'wblock',{[1], 4})
%      creates a structure object with one W block of size 6x6 and
%      one of size 4x4 both with the associated eigenvalue 2, and one
%      W block of size 2x2 with associated the eigenvalue 4.
%
%   CMSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------
%   StructObj = CMSTRUCT(BlockObj1,BlockObj2,...) creates a structure
%   object from the listed canonical block objects. The block objects must
%   be valid blocks for the structure.
%
%   Examples:
%      >> cmstr = cmstruct([],[3 1],4,[1])
%      returns a matrix structure with one W block of size 6x6 and
%      one of size 2x2 both with the associated eigenvalue 4, and one
%      Jordan block of size 1x1 with the zero eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> cmstr = cmstruct('zjblock',1,'wblock',{[3 1],4})
%
%      >> cmstr = cmstruct([], {[2] [1]}, [1 3], [2])
%      returns a matrix of the following form:
%                      | W2(1)    0     0   |
%                      |   0    W1(3)   0   |
%                      |   0      0   J2(0) |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class CMSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   CMSTRUCT Methods:
%     CODIM    - Compute the codimension of a matrix under congruence.
%     CCF      - Return the matrix in the congruence canonical form.
%     SIZE     - Total size of the represented structure.
%     NUMBLK   - Number of canonical block objects.
%     ISEMPTY  - True for empty canonical structure.
%     CHAR     - Convert a structure object to a string.
%     EXIST    - Check if a canonical block of a specified type 
%                already exist.
%     COPY     - Return a copy of the structure object.
%     COMPARE  - Compare two canonical structure objects.
%     SET      - Set the canonical structure information (not implemented).
%     GET      - Get the canonical structure information.
%     GETVALIDBLOCKS - Return a list of available canonical blocks.
%
%   CMSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also GBLOCK, WBLOCK, ZJBLOCK, SCMSTRUCT, MSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef cmstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {'gblock','wblock','zjblock'};
    
    % StratiGraph Object name of the canonical structure
    SGObjectName = '';
    
    % StratiGraph setup name
    SGSetupName = '';

    % StratiGraph expander class name
    SGExpanderName = '';
  end
  
  methods
    % Constructor
    function obj = cmstruct(varargin)
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If in compact form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,5)
        
        % Gamma blocks
        if ~isempty(varargin{1})
          Propertylist = {'gblock', varargin{1}};
        else
          Propertylist = {};
        end
        
        
        % W blocks
        if nargin > 1
          if nargin < 3
            error('MCS:cmstruct:MissingParameter','Missing parameter Eigv.')
          end
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('wblock',varargin(2:end))];
        end
        
        % Jordan blocks with zero eigenvalue
        if nargin > 3
          Propertylist = [Propertylist, {'zjblock', varargin{4}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});
      
      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'cmstruct')
          % If input is a congruence matrix structure object, make a copy.
          obj = varargin{1}.copy;
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to a cmstruct object.', class(varargin{1}))
        end
      end
    end % End constructor
    %-------------------------------
    
    function cod = codim(obj,strata)
      %CODIM Compute the codimension of a matrix under congruence.
      %   CODIM(StructObj) determines the codimension of the congruence orbit of
      %   the matrix structure object StructObj. The codimension is determined
      %   with respect to the represented canonical structure not the tangent
      %   space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %      'orbit'   Determines the codimension of the orbit.
      %                Assumes that all eigenvalues are specified.
      %                (default)
      %      'bundle'  Not implemented!
      %
      %   See also CMCODIM.
      
      narginchk(1,2);
      
      blocksv = obj.get('gblock','zjblock','StructInt','segre');
      gblk  = blocksv{1};
      zjblk  = blocksv{2};
      wblkcai = obj.get('wblock','segre');
      ng = length(gblk);
      nz = length(zjblk);
      nw = length(wblkcai);
      wblkca = cell(nw,1);
      eigv = zeros(1,nw);
      
      if nargin == 1
        % Strata is orbit, do nothing.
      elseif strcmpi(strata,'bundle')
        error('MCS:cmstruct:codim:Bundle',...
          'Codimension for bundle is not implemented/undefined!')
      elseif ~strcmpi(strata,'orbit') 
        error('Unknown second argument.');
      end
      
      cJ = 0;
      cGH = 0;
      rJ=0;
      
      if ~isempty(wblkcai{1})
        for i = 1:nw
          wblkca{i} = 2*wblkcai{i}{1};
          eigv(i) = wblkcai{i}{2}(1);
        end
        for i = 1:nw
          wblkv = wblkca{i};
          if ~isempty(wblkv)
            m = 1:2:length(wblkv)*2;
            cJ = cJ + sum(wblkv.*m)/2;
            if eigv(i)^2 == 1
              n = 0:2:length(wblkv)*2-1;
              cJ = cJ + sum(wblkv.*n)/2;
            end
            for j= 1:length(wblkv)
              if eigv(i)==(-1)^(wblkv(j)/2)
                cJ= cJ + 2*(ceil(wblkv(j)/4));
              end
            end
            for j = i+1:length(wblkca)
              if eigv(i)*eigv(j)==1
                n = 0:2:length(wblkv)*2-1;
                cJ = cJ + sum(wblkv.*n)/2;
              end
            end
          end
        end
        
        if ~isempty(eigv)
          for i = 1:length(wblkca)
            wblkv = wblkca{i};
            if ~isempty(wblkv)
              rJ = rJ + sum(wblkv);
            end
            for k = 1 : ng
              if eigv(i)==(-1)^(gblk(k)+1)
                for j=1:length(wblkv)
                  cGH=cGH + 2*min(gblk(k),wblkv(j)/2);
                end
              end
            end
          end
        end
      end
      
      cJ0 = 0;
      Jodd=0;
      cJ0 = cJ0 + sum(round(zjblk./2));
      for j = 1:nz-1
        for i = j+1:nz
          if mod(zjblk(i),2)==0
            cJ0= cJ0 + zjblk(i);
          else
            if zjblk(i)==zjblk(j)
              cJ0= cJ0 + zjblk(j)+1;
            else
              cJ0= cJ0 + zjblk(j);
            end
          end
        end
      end
      for j = 1:nz
        if mod(zjblk(j),2)==1
          Jodd=Jodd+1;
        end
      end
      
      cR = 0;
      cR = cR + sum(floor(gblk./2));
      for i = 1:ng-1
        for j=i+1:ng
          if mod(gblk(i)-gblk(j),2)==0
            cR = cR + gblk(j);
          end
        end
      end
      
      cJHR = (sum(gblk) + rJ)*Jodd;
      cod = cJ+cJ0+cR+cGH+cJHR;
    end % End codim
    %-------------------------------
    
  end % End public methods
end % End classdef