%MCSSTRUCT Abstract class for generic canonical structures.
%   MCSSTRUCT provides templates and generic methods for canonical
%   structure objects.
%
%   StructObj =
%   MCSSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...,'Notation',Notation)
%   creates a new structure object with the specified blocks.
%
%   MCSSTRUCT Methods:
%     CODIM   - Compute the codimension of a canonical structure object.
%     SIZE    - Total size of the represented structure.
%     NUMBLK  - Number of canonical block objects.
%     ISEMPTY - True for an empty canonical structure object.
%     CHAR    - Convert a structure object to a string.
%     EXIST   - Check if a canonical block of a specified type 
%               already exist in the canonical structure object.
%     COPY    - Return a copy of the canonical structure object.
%     COMPARE - Compare two canonical structure objects.
%     SET     - Set the canonical structure information (not implemented).
%     GET     - Get the canonical structure information.
%     GETVALIDBLOCKS   - Return a list of valid canonical blocks.
%     GETSGCONSTRAINTS - Return a list of available StratiGraph constraints.
%
%   MCSSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also MCSBLOCK.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef (HandleCompatible) mcsstruct
  properties (Abstract, Access = {?mcsstruct, ?canonicalforms})
    cstructure % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Abstract, Constant, Access = {?mcsstruct})
    cblocks % A string cell-array with all possible canonical blocks for
            % the canonical structure
    SGObjectName   % StratiGraph class name of the canonical structure
    SGSetupName    % StratiGraph setup name
    SGExpanderName % StratiGraph expander class name. 
                   % Use an empty string for the default expander. 
  end
  
  methods
    % Constructor
    %   This constructor can only be called from a subclass to MCSSTRUCT.
    function obj = mcsstruct(varargin)

      % If no arguments return.
      % Conversion between objects are handled by the subclass.
      if nargin == 0 
        return
      end
      
      % Set the correct notation
      if strcmpi('notation',varargin{end-1})
        % If present, last two arguments are for notation
        not = lower(varargin{varargin{end}});
        if ~strcmp(not,{'sizes','segre','weyr'})
          error('%s is not a valid notation.', not)
        end
        notsize = 2;
      else
        not = 'sizes';
        notsize = 0;
      end
      
      kstr = 0;
      % Creates the block objects
      for k = 1:2:length(varargin)-notsize
        blockname = lower(varargin{k});
        value = varargin{k+1};
        
        if isempty(value)
          % If an empty block continue to next entry
          continue
        elseif ~any(strcmp(blockname,obj.cblocks))
          % Return an error if the block is not valid for this structure
          error('Property %s is not a valid block class.',blockname);
        end
        
        % If the Jordan blocks has an infinit eigenvalue
        % change to an ijblock object
        if strcmp(blockname,'fjblock') 
          if iscell(value)
            if isscalar(value{2}) && isinf(value{2})
              blockname = 'ijblock';
              value = value{1};
            end
          else
            value = {value NaN};
          end
        end
        
        if ~iscell(value)
          % The block has no parameter...
          [b,idx] = obj.find(blockname);
          if b 
            % Extend the block if the same type already exist.
            obj.extendblock(idx,value,not);
          else
            % If not an existing block, create a new
            kstr = kstr+1;
            obj.cstructure{kstr} = feval(blockname,value,not);
          end
        elseif ~isempty(value{1}) 
          % If the block has a parameter...
          [b,idx] = obj.find(blockname,value{2});
          if b
            % Extend the block if the same type already exist
            % Blocks with unspecified parameters are not extended
            obj.extendblock(idx,value{1},not);
          else
            % If not an existing block, create a new
            kstr = kstr+1;
            obj.cstructure{kstr} = feval(blockname,value{:},not);
          end
        end
      end
    end 
    %----------------------

    function b = ne(obj1,obj2)
      %NE (~=) Not equal relation between two structure objects.
      %   StructObj1 ~= StructObj2 checks if the two canonical structure
      %   objects are not equal.
      %
      %   See also EQ, COMPARE.
      
      b = ~eq(obj1,obj2);
    end
    
    function b = eq(obj1,obj2)
      %EQ (==) Equal relation between two structure objects.
      %   StructObj1 == StructObj2 checks if the two canonical structure
      %   objects are equal.
      %
      %   See also NE, COMPARE.

      if ~strcmp(class(obj1),class(obj2))
        b = false; 
        return
      end
      
      objt = obj2.copy;
      b = length(obj1.cstructure) == length(objt.cstructure);
      k = 0;
      while b && k < length(obj1.cstructure)
        k = k+1;
        l = 1;
        b = (obj1.cstructure{k} == objt.cstructure{l});
        % The block objects can come in different order
        while ~b && l < length(objt.cstructure)
          l = l+1;
          b = (obj1.cstructure{k} == objt.cstructure{l});
        end
        if b
          objt.cstructure(l) = [];
        end
      end
    end
    %----------------------
    
    function varargout = size(obj,dim)
      %SIZE Total size of the represented structure.
      %   D = SIZE(StructObj) returns a two-element row vector D = [M, N]
      %   containing the number of rows M and columns N of the structure
      %   object in matrix or matrix pencil form, i.e., the sum of the
      %   sizes of all the canonical blocks. 
      %
      %   [M,N] = SIZE(StructObj) returns the number of rows and columns in
      %   separate output variables.
      %
      %   SIZE(StructObj,Dim) returns the length of the dimension specified
      %   by the scalar Dim. For example, SIZE(StructObj,1) returns the
      %   number of rows.
      
      nargoutchk(0,2)
      
      sizes = [0 0]; % [m n]
      for k = 1:length(obj.cstructure)
        sizes = sizes + obj.cstructure{k}.size;
      end
      
      if nargin == 1
        nargoutchk(0,2);
        
        if nargout < 2
          varargout{1} = sizes;
        else
          varargout{1} = sizes(1);
          varargout{2} = sizes(2);
        end
      else
        nargoutchk(0,1);
        
        switch dim
          case 1
            varargout{1} = sizes(1);
          case 2
            varargout{1} = sizes(2);
          otherwise
            varargout{1} = 1;
        end
      end
    end
    %----------------------

    function n = numblk(obj)
      %NUMBLK Number of canonical block objects.
      %   N = NUMBLK(StructObj) returns the number of canonical block
      %   objects of different types in the structure object StructObj.
      %   Canonical blocks with equal parameters (eigenvalues) are of the
      %   same type, while blocks with different parameters are not.
      %
      %   See also SIZE, MCSBLOCK/NUMBLK.
      
      n = length(obj.cstructure);
    end
    %----------------------
    
    function b = isempty(obj)
      %ISEMPTY True for empty canonical structure.
      %   ISEMPTY(StructureObj) returns 1 if StructureObj is an empty
      %   canonical structure object and 0 otherwise. An empty structure
      %   object has no canonical blocks or all are empty.
      
      if ~isempty(obj.cstructure)
        b = 1; k = 0;
        while b && k < length(obj.cstructure)
          k = k+1;
          b = isempty(obj.cstructure{k});
        end
      else
        b = 1;
      end
    end
    %-------------------------------
    
    function [c, d] = compare(obj1,obj2,varargin)
      %COMPARE Compare two canonical structure objects.
      %   C = COMPARE(S1,S2) compares the two canonical structure objects S1 and
      %   S2 of the same class and returns a vector C with the same length as
      %   number of existing canonical blocks. The two canonical structure
      %   objects must have the same number of canonical blocks.
      %
      %   The comparison is done by comparing the sizes of the canonical blocks
      %   for each pair of blocks (B1,B2) of the same class from S1 and S2,
      %   respectively. For each type of canonical block; first the largest block
      %   from each block object are compared then the second largest until one
      %   is larger than the other or no more blocks exist. The corresponding
      %   element in C is 0 if the blocks B1 and B2 are equal, 1 if B1 > B2, and
      %   -1 if B1 < B2. Consequently, if all elements in C are 0 then S1 and S2
      %   have the same block structure. Any parameters are not compared, but
      %   have an impact on which block objects is compared to which block
      %   object. A block in S1 which does not exist in S2 always returns 1.
      %
      %   C = COMPARE(S1,S2,'weyr') uses the Weyr characteristics resulting in
      %   that the comparison is done by comparing the number of blocks. First
      %   the number of all blocks are compared then the number of second
      %   smallest and larger blocks until one quantity is larger than the other.
      %
      %   [C2, D] = COMPARE(S1,S2,Tol) or 
      %   [C2, D] = COMPARE(S1,S2,'weyr',Tol) returns the 2-column matrix C2 =
      %   [C,P], where C is the result from above and the P(k) is the result from
      %   comparing the corresponding parameter (eigenvalue) of the block C(k).
      %   It uses the tolerance Tol for comparing the parameters. P(k) = 0 if
      %   abs(p1-p2) <= Tol for all parameters p1 and p2 from the same block
      %   class in S1 and S2, respectively, otherwise P returns 1. For blocks
      %   with no parameter, Tol is ignored and the corresponding element in P is
      %   0. Consequently, if both S and P are 0 then the two structures are
      %   equal with respect to Tol. The optional D returns the difference
      %   abs(p1-p2) between the two parameters, or NaN if the blocks have no
      %   parameter.
      %
      %   C = COMPARE(S1,S2,'size') only compares the total sizes (m*n) of
      %   structure objects. Returns 0 if equal, 1 if S1 > S2, and -1 if S1 < S2.
      %
      %   See also EQ.
      
      narginchk(2,4);
      
      if ~strcmp(class(obj1),class(obj2))
        error('MCS:mcsstruct:compare:Uncomparable',...
          'The two structure objects must be of the same class.');
      end
      
      if length(obj1.cstructure) ~= length(obj2.cstructure)
        error('MCS:mcsstruct:compare:Uncomparable',...
          'The two structure objects must have the same number of canonical blocks.');
      end

      usetol = false;
      
      if nargin == 2
        cmpfnc = @(x,y) compare(x,y);
      else
        if nargin == 4
          cmpfnc = @(x,y) compare(x,y,varargin{1},varargin{2});
          usetol = true;
        elseif isnumeric(varargin{1})
          cmpfnc = @(x,y) compare(x,y,varargin{1});
          usetol = true;
        else
          if nargout > 1
            error('To many output arguments.')
          end
          switch varargin{1}
            case 'weyr'
              cmpfnc = @(x,y) compare(x,y,'weyr');
            case 'size'
              [m1,n1] = obj1.size();
              [m2,n2] = obj2.size();
              if m1*n1 > m2*n2
                c = 1;
              elseif m1*n1 < m2*n2
                c = -1;
              else
                c = 0;
              end
              return
            otherwise
              if ischar(varargin{1})
                error('Additional argument must be ''weyr'' or ''size''.');
              end
          end
        end
      end
      
      if usetol
        c = zeros(length(obj1.cstructure),2);
        d = zeros(length(obj1.cstructure),1);
      else
        c = zeros(length(obj1.cstructure),1);
      end

      str1blocks = 1:length(obj1.cstructure);
      str2blocks = 1:length(obj2.cstructure);
      L = zeros(length(obj1.cstructure),2);
      while any(str1blocks)
        bcb = 1;
        bm = [inf 0]; % Best match
        k = str1blocks(find(str1blocks,1));
        str2temp = str2blocks(str2blocks > 0);
        for i = 1:length(str2temp)
          j = str2temp(i);
          if strcmp(class(obj1.cstructure{k}),class(obj2.cstructure{j}))
            [cb,db] = cmpfnc(obj1.cstructure{k},obj2.cstructure{j});
            if ~any(cb) && isnan(db)
              % Found an exact match
              str2blocks(j) = 0;
              bm = [db j];
              bcb = cb;
              break;
            elseif ~any(db) && db <= 2*eps
              % Found an exact match
              str2blocks(j) = 0;
              bm = [db j];
              bcb = cb;
              break;
            end
            if bm(1) > db
              bm = [db j];
              bcb = cb;
            end
          end
        end
 
        j = bm(2);
        if j > 0
          if L(j,1) == 0
            L(j,:) = [bm(1) k];
          elseif L(j,1) > bm(1)
            str1blocks(L(j,2)) = L(j,2);
            L(j,:) = [bm(1) k];
          end
        end
        str1blocks(k) = 0;
        c(k,:) = bcb;
        if usetol, d(k) = bm(1); end
      end
      
    end
    %-------------------------------
    
    function s = char(obj)
      %CHAR Convert a structure object to a string.
      %   S = CHAR(StructObj) returns the canonical structure information
      %   of the structure object StructObj as a string in canonical block
      %   notation.
      %
      %   See also MCSBLOCK/CHAR.

      % If empty structure object
      if isempty(obj)
        s = sprintf('Empty %s object',class(obj));
        return
      end
      
      n = length(obj.cstructure);
      out = cell(n,1);
      % The length of out will be the sum of all blockv
      uenr = 1;
      for k = 1:n
        out{k} = char(obj.cstructure{k});
        if contains(out{k},'NaN')
          out{k} = strrep(out{k},'NaN',['e',num2str(uenr)]);
          uenr = uenr + 1;
        end
      end
      
      if n > 1
        s = [out{1} sprintf(' + %s',out{2:n})];
      else
        s = out{1};
      end
    end
    %-------------------------------

    function disp(obj)
      %DISP Display a canonical structure object.
      %   DISP(StructObj) is called for the structure object StructObj
      %   when the semicolon is not used to terminate a statement. Returns
      %   the canonical form represented as a string.
      
      cmp = isequal(get(0,'FormatSpacing'),'compact');
      
      % If empty structure object
      if isempty(obj)
        fprintf('     Empty %s object\n',class(obj))
        if ~cmp, disp(' '); end
        return
      end
      
      % Get the default format ('segre', 'segrec', 'weyr', 'sizes', or 'block')
      not = mcsformat;
      
      switch not
        case {'segre','segrec','weyr','sizes'}
          out = cell(length(obj.cstructure),3);
          uenr = 1;
          for k = 1:length(obj.cstructure)
            out(k,:) = obj.cstructure{k}.block2cell(not);
            if contains(out(k,1),'NaN')
              out(k,1) = strrep(out(k,1),'NaN',['e',num2str(uenr)]);
              uenr = uenr + 1;
            end
          end
          
          max1 = 0;
          max2 = 0;
          max3 = 0;
          for i=1:size(out,1)
            max1 = max(max1,length(out{i,1}));
            max2 = max(max2,length(out{i,2}));
            max3 = max(max3,length(out{i,3}));
          end
          
          fmt = ['     %',num2str(max1),'s %-',num2str(max2+2),...
            's %-',num2str(max3),'s\n'];
          
          for i=1:size(out,1)
            fprintf(fmt,out{i,1},out{i,2},out{i,3});
          end
        case 'block'
          % Get the maximum length (may be smaller)
          maxl = sum(cellfun(@numblk,obj.cstructure));
          out = cell(maxl,1);
          n = 1;
          uenr = 1;
          for k = 1:length(obj.cstructure)
            blk = obj.cstructure{k}.block2cell(not);
            if contains(blk,'NaN')
              blk = strrep(blk,'NaN',['e',num2str(uenr)]);
              uenr = uenr + 1;
            end
            nblk = length(blk);
            out(n:n+nblk-1,1) = blk;
            n = n+nblk;
          end
          out = out(1:n-1,1);
          
          rc = 6;
          fprintf(1,'     ');
          if ~isempty(out)
            fprintf(1,'%s',out{1});
          end
          for i = 2:length(out)
            if rc + length(out{i}) > 70
              rc = 5;
              fprintf(1,'\n     ');
            end
            rc = rc + length(out)+3;
            fprintf(1,' + %s',out{i});
          end
          fprintf(1,'\n');
      end
      
      if ~cmp, disp(' '); end
    end
    %----------------------

    function b = exist(obj,bclass,param)
      %EXIST Check if a canonical block of a specified type already exists.
      %   B = exist(StructObj,BlockClass) returns 1 (true) if a canonical
      %   block object of type BlockClass already exist in the structure
      %   object StructObj and 0 (false) otherwise. BlockClass can either
      %   be as a string or a canonical block object.
      %   
      %   B = exist(StrObj,BlockClass,Param) is used when the canonical
      %   block object also has a parameter, e.g., an eigenvalue. 
      
      if nargin == 3 && isempty(param)
        error('MCS:mcsstruct:exist',...
          'Parameter must be a float or NaN.')
      end
      
      if ischar(bclass)
        bc = bclass;
      elseif isa(bclass,'mcsblock')
        bc = class(bclass);
      else
        error('MCS:mcsstruct:exist',...
          '''BlockClass'' must be a string or a canonical block object.')
      end
      
      [blogic,~] = find(obj,bc,param);
      b = double(blogic);
    end
    %----------------------
    
    function newobj = copy(obj)
      %COPY Return a copy of the canonical structure object.
      %   StructObjNew = COPY(StructObj) returns a copy of the canonical
      %   structure object StructObj.
      
      newobj = eval(class(obj));
      
      for k = 1:length(obj.cstructure)
        newobj.cstructure{k} = obj.cstructure{k}.copy;
      end
    end
    %----------------------

    function obj = set(obj,varargin)
      %SET Set the canonical structure information (not implemented).
      %   SET is not implemented for canonical structure objects.
      %   Instead use the following procedure to change any canonical
      %   structure information.
      %
      %   1. Obtain the canonical block object(s) which is/are going to be
      %      modified. Example (where pstr is a PSTRUCT object):
      %      >> blkobj = pstr.get('lsblock','object');
      %
      %   2. Modify the block object by using set(). Example:
      %      >> blkobj{1}.set('StructInt',[3 1]);
      %  
      %   3. Done!
      %
      %   See also GET, MCSBLOCK/SET
      
      warning('MCS:mcsstruct:set','SET is not implemented for canonical structure objects.')
      
    end
    %----------------------
    
    function varargout = get(obj,varargin)
      %GET Get the canonical structure information.
      %   V = GET(StructObj,'PropertyName') returns the value of the
      %   specified property for the canonical structure object StructObj. 
      %   V = GET(StructObj,'PropertyName1','PropertyName2',...) may be
      %   used to refine which property to return.
      %   
      %   The property name can be in any order (described below):
      %    1) Any valid canonical block name.
      %    2) 'StructInt', 'Parameter', or 'Eigenvalue'.
      %    3) Any valid notation.
      %    4) 'Unroll'
      %    5) 'Object' (properties of type 2, 3, and 4 are ignored).
      %
      %   1) 
      %   By specifying any valid block name GET returns the structure
      %   integer partition and any parameter associated with the canonical
      %   blocks of type PropertyName. Multiple block names can be
      %   specified. If no block names are given, properties for all
      %   existing blocks are returned. PropertyName is a string and
      %   can, e.g., be one of:
      %      'sblock'   - Singular blocks (both right and left)
      %      'rsblock'  - Right singular blocks
      %      'lsblock'  - Left singular blocks
      %      'jblock'   - Jordan blocks (both finite and infinite
      %                   eigenvalues)
      %      'fjblock'  - Jordan blocks associated with finite eigenvalues
      %
      %   For a complete list of all valid canonical blocks for specific
      %   structure use the method getvalidblocks, see also
      %   mcsgetvalidblocks. Calling mcsgetvalidblocks without any
      %   arguments will return all implemented canonical block classes.
      %
      %   The returned V is a cell-array, where each element in V is a
      %   vector with the sizes of corresponding canonical block. For
      %   blocks associated with a parameter (e.g. an eigenvalue) the size
      %   vector and the parameter are combined in a cell-array tuple, one
      %   for each parameter. If more than one block name is given, then
      %   all structure integer partitions corresponding to the same block
      %   class are gathered in a cell-array. V will then have the same
      %   length as number of specified blocks.
      %
      %     Example 1:
      %       >> pstr = pstruct([3 1],[2 0],{[2 1] [3]},[3 0]);
      %       >> V = pstr.get('lsblock','fjblock');
      %
      %     Then V = {[2 0], {{[2 1], 3}, {[3], 0}}}, i.e., the matrix
      %     pencil object pstr has two left singular blocks of sizes 3x2
      %     and 1x0, two Jordan blocks of sizes 2x2 and 1x1 both associated
      %     with the eigenvalue 3, and one Jordan block of size 3x3
      %     associated with the zero eigenvalue.
      %
      %   [V,Param] = GET(StructObj,'Blockname') returns the structure
      %   integer partitions in the cell-array V and the associated
      %   parameters in the vector Param, where V{k} has the parameter
      %   Param(k).
      %
      %   [V,Param] = GET(StructObj,'Blockname1','Blockname2',...) returns
      %   the structure integer partitions and the parameters in two
      %   separate cell-arrays. If the canonical block(s) in V{k} has/have
      %   an associated parameter, then the associated parameter of the
      %   blocks V{k}{j} is Param{k}(j). If no parameter exist for block k,
      %   Param{k} is empty.
      %
      %   If StructObj is a vector of length M of structure objects, then V
      %   (and Param) will be M-by-X cell-array of values (some may be
      %   empty), where X is the number of requested canonical blocks in
      %   the structure.
      %
      %   2) 
      %   It is also possible to specify which information to return.
      %   'StructInt' only returns the structure integer partition, and
      %   'Parameter' or 'Eigenvalue' the associated parameter (e.g.
      %   eigenvalue).
      %
      %   3)
      %   The notation of the structure integer partition can be specified
      %   by setting any property name to one of:
      %      'sizes'   Sizes may be unordered. (Default)
      %      'segre'   Sizes are ordered in a non-increasing order.
      %      'weyr'    Weyr characteristics.
      %
      %   4)
      %   By passing 'Unroll' as a property, multiple blocks of same class
      %   are not encapsulated in a struct. Instead they are appended to
      %   the cell-array V as separate vectors. This option is only
      %   available if one of 'StructInt' or 'Parameter'/'Eigenvalue' is
      %   specified, not both.
      %
      %     Example 2 (compare with Example 1):
      %       >> pstr = pstruct([3 1],[2 0],{[2 1] [3]},[3 0]);
      %       >> V = pstr.get('lsblock','fjblock','StructInt','Unroll');
      %
      %     Returns the cell-array V = {[3 1], [2 1], [3]}.
      %
      %   5) 
      %   By passing 'Object' as a property, the associated canonical block
      %   objects will be returned in the cell-array V. In this case, any
      %   parameter specifying notation and/or which information to return
      %   is neglected.
      %
      %   See also SET, MCSBLOCK, GETVALIDBLOCKS, SUBSREF.

      nargoutchk(0,2);
      
      % Call builtin get if OBJ isn't a structure object.
      if ~isa(obj, 'mcsstruct')
        try
          varargout = builtin('get', obj, varargin{:});
        catch aException
          rethrow(aException);
        end
        return;
      end
      
      % Split property names into block names and options
      optInd = cellfun('isempty',regexp(varargin,'[a-z]{1,3}blocks?$','match'));
      options = varargin(optInd);
      getBlocks = varargin(~optInd);
      
      % If no block names are given, assume all possible
      if isempty(getBlocks)
        getBlocks = obj.cblocks;
      end
      
      % Return objects...
      if any(strcmpi('Object',options))
        varargout{1} = cell(length(obj),0);
        for i = 1:length(obj)
          cstr = cellfun(@class,obj(i).cstructure,'UniformOutput',false);
          k = 1;
          for j = 1:length(getBlocks)
            tf = strcmp(getBlocks{j},cstr);
            if any(tf)
              varargout{1}(i,k:k+sum(tf)-1) = obj(i).cstructure(tf);
              k = k + sum(tf);
            else
              varargout{1}{i,k} = eval(getBlocks{j});
              k = k+1;
            end
          end
        end
      else % Return canonical information
        
        not = 'sizes'; % Default notation
        structint = false; param = false; unroll = false;
        for k = 1:length(options)
          switch lower(options{k})
            case 'segre'
              not = 'segre';
            case 'weyr'
              not = 'weyr';
            case 'sizes'
              not = 'sizes';
            case 'structint'
              structint = true;
            case {'parameter','eigenvalue'}
              param = true;
            case 'unroll'
              unroll = true;
            otherwise
              error('MCS:mcsstruct:get:InvalidProperty',...
                ['Property name unknown: ', options{k}])
          end
        end
        if ~structint && ~param
          structint = true; param = true;
        end
        if unroll && structint && param
          unroll = false;
        end
        
        varargout{1} = {};
        if nargout == 2, varargout{2} = {}; end
        % Loop through all objects and BlockNames
        % i - object index
        % j - block index
        for i = 1:length(obj)
          jblk = 1;
          for j = 1:length(getBlocks)
            bidx = find(cellfun(@(x)isa(x,getBlocks{j}),obj(i).cstructure));
            if length(bidx) > 1 
              v1 = cell(1,length(bidx));
              v2 = length(bidx);
            else
              v1 = []; v2 = [];
            end
            if ~isempty(bidx)
              isPBlock = isa(obj(i).cstructure{bidx(1)},'pblock');
            end
            
            % For all matching blocks
            for bk = 1:length(bidx)
              if structint
                s = obj(i).cstructure{bidx(bk)}.get('StructInt',not);
              else
                s = [];
              end
              if param && isPBlock
                p = obj(i).cstructure{bidx(bk)}.get('Parameter');
              else
                p = [];
              end
              
              % If one single block
              if length(bidx) == 1
                if nargout == 2
                  if isPBlock && length(getBlocks) > 1
                    v1 = {s};
                  else
                    v1 = s;
                  end
                  v2 = p;
                elseif structint && param
                  if isPBlock
                    v1 = {s p};
                  else
                    v1 = s;
                  end
                elseif structint
                  v1 = s;
                else
                  v1 = p;
                end
              else % If multiple blocks of same type
                if nargout == 2
                  v1{bk} = s;
                  v2(bk) = p;
                elseif structint && param
                  if isPBlock
                    v1{bk} = {s p};
                  else
                    v1{bk} = s;
                  end
                elseif structint
                  v1{bk} = s;
                else
                  v1{bk} = p;
                end
              end
            end % End for bk
            
            % If only one type of blocks is asked for
            if length(getBlocks) == 1 && length(obj) == 1
              if length(bidx) > 1
                varargout{1}(1,:) = v1;
              else
                varargout{1} = {v1};
              end
              if nargout == 2
                varargout{2} = v2;
              end
            else
              if nargout == 2
                varargout{2}{i,jblk} = v2;
              end
              if unroll && iscell(v1)
                varargout{1}(i,jblk:jblk+length(bidx)-1) = v1;
                jblk = jblk+length(bidx);
              else
                varargout{1}{i,jblk} = v1;
                jblk = jblk+1;
              end
            end
          end % End for j (getBlocks)
        end % End for i (obj)
      end
    end 
    %------------------------------
    
    %------------------------------
    % The following methods are for overloading subscripting
    %------------------------------
    
    function ind = end(obj,~,~)
      %END Number of canonical block objects.
      %   N = END(StructObj) returns the number of canonical block
      %   objects of different types in the structure object StructObj.
      %   Returns the same as NUMBLK.
      %
      %   See also NUMBLK.
      
      ind = length(obj.cstructure);
    end
    %------------------------------
    
    function n = numArgumentsFromSubscript(obj,~,indexingContext)
      % For internal usage. For help, see overloaded function
      % numArgumentsFromSubscript.
      
      switch indexingContext
        case matlab.mixin.util.IndexingContext.Statement
          n = numel(obj); % nargout for indexed reference used as statement
        case matlab.mixin.util.IndexingContext.Expression
          n = 1; % nargout for indexed reference used as function argument
        case matlab.mixin.util.IndexingContext.Assignment
          n = 0; % nargin for indexed assignment
      end
    end
    %------------------------------
    
    function n = numel(obj,varargin)
      % For internal usage. For help, see overloaded function NUMEL.
      
      if isempty(varargin)
        n = 0;
        %n = builtin('numel',obj,varargin);
      else
        if ischar(varargin{1}) % colon as subscript
          n = length(obj.cstructure);
        else
          n = length(varargin{1});
        end
      end
    end
    %------------------------------

    function varargout = subsref(obj,S)
      %SUBSREF Subscripted reference for canonical structure objects.
      %   The canonical blocks in a structure object StructObj can be
      %   accessed by the subscripts 'StructObj(k)' and 'StructObj{k}',
      %   which have different meanings. The subscript '()' returns a new
      %   canonical structure object with the same canonical structure as
      %   the specified block(s). Consequently, 'StructObj(:)' returns a
      %   copy of the structure object StructObj. The subscript '{}'
      %   returns the specified block objects in separate outputs. Note
      %   that for '{}' the returned block objects refers to the same
      %   objects as in the original canonical structure (for an example
      %   see below).
      %
      %   Examples (exemplified with matrix pencils):
      %      >> pstr = pstruct([0 1], 0, 1, 1e-4i)
      %      >> pstr([1 3])
      %      ans =
      %                      R = (1  0)   (1x3)
      %           J(0+0.0001i) = (1)      (1x1)
      %
      %      Class methods may be applied directly to the subobject (by
      %      chaining call):
      %
      %      >> [G,H] = pstr([1 3]).kcf
      %      G =
      %          0+0i  0+0i  1+0i  0+0i
      %          0+0i  0+0i  0+0i  0+0.0001i
      %      H =
      %          0     1     0     0
      %          0     0     0     1
      %
      %      >> [b1,b2] = pstr{1:2}
      %      b1 =
      %          R = (1  0) (1x3)
      %      b2 =
      %          L = (0) (1x0)
      %
      %      Changing the block object b1 or b2 will also change the
      %      canonical structure of pstr:
      %
      %      >> b1.set('StructInt',[2 1]);
      %      >> pstr
      %      pstr =
      %                       R = (2  1)   (3x5)
      %                       L = (0)      (1x0)
      %            J(0+0.0001i) = (1)      (1x1)
      %
      %      Chaining call with '{}' is only possible if one single
      %      subscript is given, for example:
      %
      %      >> [G,H] = pstr{1}.kcf
      %      G =
      %         0    0    1
      %      H =
      %         0    1    0
      %
      %   See also GET.
      
      if strcmp(S(1).type,'.')
        % Call class method
        if nargout < 2
          try
            varargout{1} = builtin('subsref',obj, S);
          catch ME
            if strcmp(ME.identifier,'MATLAB:maxlhs')
              builtin('subsref',obj, S);
            else
              MCSME = MCSException(ME);
              throwAsCalled(MCSME)
            end
          end
        else
          try
            [varargout{1:nargout}] = builtin('subsref',obj, S);
          catch ME
            MCSME = MCSException(ME);
            throwAsCalled(MCSME)
          end
        end
        return
      end
      
      if length(S(1).subs) > 1
        error('MCS:mcsstruct:subsref',...
          'The object do not support multidimensional subscripting.')
      end
      % If not colon as subscript, check range of subscript
      if ~ischar(S(1).subs{1}) 
        assert(max(S(1).subs{1}) <= length(obj.cstructure),'Index exceeds number of canonical blocks.')
        assert(min(S(1).subs{1}) > 0,'Subscript indices must be real positive integers.')
      end
        
      % We overload '()' and '{}'
      switch S(1).type
        case '{}'
          % Returns the subscipted objects
          if length(S(1).subs{1}) < 2 
            if ischar(S(1).subs{1}) % colon as subscript
              vout = cell(length(obj.cstructure),1);
              for k = 1:length(obj.cstructure)
                vout{k} = obj.cstructure{k};
              end
            else
              vout{1} = builtin('subsref',obj.cstructure,S(1));
            end
          else
            vout = cell(length(S(1).subs{1}),1);
            ind = 0;
            for k = S(1).subs{1}
              ind = ind+1;
              vout{ind} = obj.cstructure{k};
            end
          end
          if numel(S) > 1
            % Chaining call
            if length(vout) > 1
              error('MCS:mcsstruct:subsref',...
                'Cannot do chaining call when more than one subscript is given with ''{}''.')
            end
            [varargout{1:nargout}] = subsref(vout{1}, S(2:end)); % Recursive call with remaining subscripts
          else
            varargout = vout;
          end
          % Chaining call with {}. NOT FULLY WORKING!
          %  if numel(S) > 1
          %    % Chaining call
          %    if length(vout) < nargout
          %      error('Insufficient number of outputs from right hand side of equal sign to satisfy assignment.')
          %    end
          %      for k = 1:length(vout)
          %        [v2out{1:nargout}] = subsref(vout{k}, S(2:end)); % Recursive call with remaining subscripts
          %        varargout{k} = v2out;
          %      end
          %    end
        case '()'
          % Make a copy of the structure object containing only the
          % selected canonical blocks.
          vout{1} = eval(class(obj));
          if ischar(S(1).subs{1}) % colon as subscript
            subs = 1:length(obj.cstructure);
          else
            subs = S(1).subs{1};
          end
          ind = 0;
          for k = subs
            ind = ind+1;
            vout{1}.cstructure{ind} = obj.cstructure{k}.copy;
          end
          if numel(S) > 1
            % Chaining call
            [varargout{1:nargout}] = subsref(vout{1}, S(2:end)); % Recursive call with remaining subscripts
          else
            varargout = vout;
          end
        otherwise
          error('MCS:mcsstruct:subsref',...
            'The object do not support the subscript.')
      end
      
    end
    %------------------------------
    
    function obj = subsagn(obj,~,varargin)
      % For internal usage. For help, see overloaded function NUMEL.
      
      error('Cannot do indexed assignment on a canonical structure object.')
    end
    %------------------------------
    
    function blocklist = getvalidblocks(obj)
      %GETVALIDBLOCKS Return a list of valid canonical blocks.
      %   GETVALIDBLOCKS(StructObj) returns a list of all valid canonical
      %   block objects for StructObj.
      %
      %   See also MCSGETVALIDBLOCKS.
      
      blocklist = obj.cblocks;
    end 
    %------------------------------

    function constr = getsgconstraints(obj)
      %GETSGCONSTRAINTS Return a list of available StratiGraph constraints.
      %   GETSGCONSTRAINTS returns a list of all available constraints for the
      %   corresponding setup in StratiGraph. The returned list is a cell-array
      %   of strings.
      %
      %   See also SGGETCONSTRAINTS.
      
      constr = sggetconstraints(obj);
    end
    
  end % End public methods

%% Static methods
  methods (Static, Access = protected)
    function Propertylist = parseFJBlocks(blkname,vargs)
      %PARSEFJBLOCKS Parse finite Jordan blocks.
      %   PropertyList = PARSEFJBLOCKS(Vargs) parse the varargin of a
      %   struct class's constructor to obtain the PropertyList with Jordan
      %   blocks associated with finite or unspecified eigenvalues. Vargs
      %   must be varargin{k:end}, where k is the index of varargin with
      %   the sizes of the Jordan blocks. Only the first two arguments in
      %   Vargs will be parsed, the remaining (if present) are ignored.
      
      if iscell(vargs{1})
        if length(vargs) > 1
          if length(vargs{1}) ~= length(vargs{2})
            error('MCS:mcsstruct:NumberEigenvalueMissmatch', ['The number of eigenvalues (%d) does ' ...
              'not match the number of Jordan block structures (%d).'],length(vargs{2}),length(vargs{1}));
          end
          eigv = vargs{2};
        else
          eigv = nan(1,length(vargs{1}));
        end
        
        Propertylist = cell(1,2*length(vargs{1}));
        for k = 1:length(vargs{1})
          Propertylist{k*2-1} = blkname;
          Propertylist{k*2} = {vargs{1}{k} eigv(k)};
        end
      else
        if length(vargs) > 1
          if length(vargs{2}) > 1
            error('MCS:mcsstruct:NumberEigenvalueMissmatch', ['The number of eigenvalues (%d) does ' ...
              'not match the number of Jordan block structures (1).'],length(vargs{2}));
          end
          Propertylist = {blkname, {vargs{1} vargs{2}}};
        else
          Propertylist = {blkname, {vargs{1} NaN}};
        end
      end
    end
  end

%% Abstract public methods
  methods (Abstract)
    c = codim(obj,strata) % Compute the codimension of a canonical structure object.
  end
  %------------------------------

%% Protected methods
  methods (Access = protected)
    function obj = block2struct(obj,varargin)
      %BLOCK2STRUCT Assign a set of canonical block objects to a structure.
      %    BLOCK2STRUCT validates that the canonical block objects are valid
      %    for the canonical structure object. The block objects can be provided
      %    as a list of objects or as a cell array of objects.
      %   
      %    See also COPY.
      
      if isobject(varargin{1})
        cstr = varargin;
      elseif iscell(varargin{1})
        cstr = varargin{1};
      end
      
      for k = 1:length(cstr)
        if any(strcmp(class(cstr{k}),obj.cblocks))
          obj.cstructure{k} = cstr{k}.copy;
        else
          if isobject(cstr{k})
            % Return an error if the block is not valid for this structure
            error('%s is not a valid block object.',class(cstr{k}))
          else
            error(['If first property is a canonical block object,',...
              ' the remaining properties must also be canonical block objects.'])
          end
        end
      end
    end
    %------------------------------
    
    
    function extendblock(obj,k,newblk,not)
      %EXTENDBLOCK Add new blocks to an existing canonical block object. 
      %   EXTENDBLOCK(StructObj,Idx,Blockv,Notation) add the canonical
      %   blocks represented by the structure integer partition Blockv, in
      %   notation Notation, to the existing structure integer partition of
      %   the same type. Idx must be an index of an existing canonical
      %   structure in the cell-array cstructure. Use FIND to obtain the
      %   index Idx.
      
      blockobj = obj.cstructure{k};
      oldblk = blockobj.get('StructInt');

      % First convert blk to Sizes since we extend an existing block
      if strcmp(not,'weyr')
        if isregular(blockobj)
          newblk = weyr2segre(newblk,'regular');
        else
          newblk = weyr2segre(newblk,'singular');
        end
      end
      
      blockobj.set('StructInt',[oldblk newblk]); % Set the new integer partition
    end
    %------------------------------
  
    function [b,k] = find(obj,bclass,param)
      %FIND Check if a canonical block already exist.
      %   [B,Idx] = FIND(StructObj,BlockClass) returns B = True and the
      %   index Idx of the cell-array cstructure if a canonical block
      %   object of type BlockClass already exist in the structure object
      %   StructObj, otherwise B = False. BlockClass should be the class
      %   name as a string.
      %   
      %   [B,Idx] = FIND(StrObj,BlockClass,Param) is used when the
      %   canonical block object also has a parameter, e.g., an eigenvalue.
      %   If Param is NaN or empty FIND always returns False.
      %
      %   The diffence with EXIST is that FIND also returns the index Idx.
      %   FIND is a protected method and only intended to use internally in
      %   MCS toolbox.
      
      k = 0; b = false;
      if nargin == 3 && (isempty(param) || isnan(param))
        return
      end

      while ~b && k < length(obj.cstructure)
        k = k+1;
        if isa(obj.cstructure{k},bclass)
          if nargin == 3 
            if (isnan(param) && isnan(get(obj.cstructure{k},'Parameter'))) ...
                || get(obj.cstructure{k},'Parameter') == param
              b = true;
            end
          else
            b = true;
          end
        end
      end
    end
    %------------------------------
    
    function setupName = getSGSetupName(obj)
      setupName = obj.SGSetupName;
    end
    %------------------------------
    
    function expanderName = getSGExpanderName(obj,expanderName)
      %GETSGEXPANDERNAME Validate the expander (constraint) name.
      %   ExpanderName = GETSGEXPANDERNAME(ExpanderName) check if the name
      %   ExpanderName is a valid expander (constraint). The validated name is
      %   returned. Override this method to validate expanders and set the
      %   property SGExpanderName.
      
      if ~isempty(expanderName) && ~strcmpi(obj.SGExpanderName,expanderName)
        error('Unknown constraint ''%s'' for setup %s.', ...
          expanderName(1:end-8), obj.SGSetupName)
      end
      expanderName = obj.SGExpanderName;
    end
    %------------------------------
    
    function objectName = getSGObjectName(obj)
      objectName = obj.SGObjectName;
    end
    %------------------------------
    
    function jsetupSize = getSGSetupSize(obj,~)
      %GETSGSETUPSIZE Return the size parameters for the structure object.
      %   JSetupSize = GETSGSETUPSIZE(StructObj, ExpanderName) returns the size
      %   parameters JSetupSize of the structure object StructObj using the
      %   StratiGraph expander expanderName.
      %
      %   See also GETSGSETUPNAME, TOJAVA.
      
      [m,n] = obj.size();

      % Create a java size parameter and set it
      jsetupSize = se.umu.stratigraph.core.structure.SizeParameters();
      spm = se.umu.stratigraph.core.structure.SizeParameter('m');
      spn = se.umu.stratigraph.core.structure.SizeParameter('n');
      spm.setConstraint(se.umu.stratigraph.core.structure.SizeParameter.GREATER, 0);
      spn.setConstraint(se.umu.stratigraph.core.structure.SizeParameter.GREATER, 0);
      spm.setValue(m);
      spn.setValue(n);
      jsetupSize.addParameter(spm);
      jsetupSize.addParameter(spn);
    end
    
    function jblockTypes = getSGBlockTypes(obj,~)
      %GETSGBLOCKTYPES Return the existing StratiGraph block types.
      %   JBlockTypes = GETSGBLOCKTYPES(StructObj) returns the set of existing
      %   block types for the canonical structure object.
      
      try
        jblockTypes = se.umu.stratigraph.core.structure.BlockType.UNDEFINED;
      catch err
        rethrow(err)
      end
    end
  end % End protected methods

%% Protected StratiGraph methods  
  methods (Access = {?mcsstruct, ?sgcom})
    function [jsetupName,jexpanderName,jsetupSize,jblockParameters] = tojava(obj, varargin)
      %TOJAVA Convert a structure object to a Java representation.
      %   [JSetupName, JExpanderName, JSetupSize, JBlockParameters] =
      %   TOJAVA(StructObj) returns the StratiGraph object representations of the
      %   setup in form of the setup name JSetupName, the expander name
      %   JExpanderName, the SizeParameter object JSetupSize, and the
      %   BlockParameters object JBlockParameters.
      %
      %   [...] = TOJAVA(StructObj, ExpanderName) also specifies the name of the
      %   expander to be used. If not specified, the default expander (setup
      %   constraint) is used.

      narginchk(1,2);
      nargoutchk(0,4);

      if obj.isempty()
        error('The structure object cannot be empty.')
      end
      
      if nargin == 1
        expander = '';
      else
        expander = varargin{1};
      end
      
      %jobjectName = obj.getSGObjectName();
      jsetupName = obj.getSGSetupName();
      jexpanderName = obj.getSGExpanderName(expander);
      jsetupSize = obj.getSGSetupSize(jexpanderName);
      try
        jblockParameters = se.umu.stratigraph.core.structure.BlockParameters(...
          obj.getSGBlockTypes(jexpanderName));
        for k = 1:length(obj.cstructure)
          jblockParameters.addBlock(obj.cstructure{k}.tojava());
        end
      catch err
        rethrow(err)
      end
    end
    %------------------------------

  end
end % End classdef