%MPSTRUCT Create a matrix polynomial structure object.
%   MPSTRUCT creates a canonical structure object representing a matrix
%   polynomial P(s) = P_d s^d + ... + P_1 s + P_0, where P_d ~= 0. The
%   invariants are represented as Kronecker canonical blocks of a matrix
%   pencil with the same invariants as P(s).
%
%   An MPSTRUCT object can consist of the following canonical blocks:
%     rsblock - Right singular block defined as:
%                      |0 1    0|     |1 0    0|
%               L_n := |   . .  |  - s|   . .  |        n-by-(n+1)
%                      |0    0 1|     |0    1 0|
%
%     lsblock - Left singular block defined as:
%                      |0     0|     |1     0|
%             L_n^T := |1  .   |  - s|0  .   |          (n+1)-by-n
%                      |   .  0|     |   .  1|
%                      |0     1|     |0     0|
%
%     fjblock - Jordan block associated with a finite eigenvalue mu,
%               defined as:
%                               |mu  1    0|
%            J_n(mu) - s*I_n := |   mu .   | - s*I_n      n-by-n
%                               |      .  1|
%                               |0       mu|
%
%     zjblock - Jordan block associated with the zero eigenvalue (mu=0).
%
%     ijblock - Jordan block associated with the infinite eigenvalue,
%               defined as:
%                      |1 0    0|     |0 1    0|
%               N_n := |  1 .   |  - s|  0 .   |          n-by-n
%                      |    .  0|     |    .  1|
%                      |0      1|     |0      0|
%
%
%   StructObj = MPSTRUCT() returns an empty canonical structure object
%   StructObj representing a matrix polynomial
%       P(s) = P_d s^d + ... + P_1 s + P_0,  where P_d ~= 0,
%   of degree d. To follow the declarations of the other canonical
%   structure objects, the invariants of P(s) are represented in the form
%   of canonical blocks associated with the Kronecker canonical form of a
%   matrix pencil with the same invariants as P(s). The degree d ensures
%   that the matrix polynomial is unique. The canonical structure
%   information can be specified using one of the three forms 'Size vector
%   form', Property-value form', or 'Object form', which are explained
%   below.
%
%   Size vector form
%   ----------------
%   StructObj = MPSTRUCT(d,RSBlocks,LSBlocks,FJBlocksv) returns a new
%   canonical structure object StructObj representing a matrix polynomial.
%   The parameters RSBlocks and LSBlocks define the right and left singular
%   blocks (associated with the column and row minimal indices),
%   respectively, of the structure and are given as row-vectors with the
%   sizes of the blocks. The parameter FJBlocksv defines the Jordan blocks
%   (associated with the finite elementary divisors) of the structure and
%   must be either a row-vector with the sizes of Jordan blocks, all
%   associated with the same eigenvalue, or a cell-array of row-vectors
%   where each vector contains the sizes of blocks associated with the same
%   eigenvalue. By default the associated eigenvalues are set to
%   unspecified (NaN).
%
%   StructObj = MPSTRUCT(d,RSBlocks,LSBlocks,FJBlocksv,Eigv) also sets the
%   eigenvalues, where Eigv is a row-vector with the values of each
%   associated eigenvalue corresponding to the row-vectors in FJBlocksv.
%   Jordan blocks with an unspecified associated eigenvalue are defined by
%   setting the corresponding eigenvalue in Eigv to NaN.
%
%   StructObj = MPSTRUCT(d,RSBlocks,LSBlocks,FJBlocksv,Eigv,IJBlocks) also
%   sets the sizes of the Jordan blocks with an associated infinite
%   eigenvalue (associated with the infinite elementary divisors).
%
%   Property-value form
%   -------------------
%   StructObj =
%   MPSTRUCT(d,'BlockName1',StructInt1,'BlockName2',StructInt2,...)
%   specifies the block types BlockName and the sizes StructInt in
%   property-value form. BlockName is a string specifying the canonical
%   block to be created, and can be one of:
%      'rsblock'  - Right singular blocks
%      'lsblock'  - Left singular blocks
%      'fjblock'  - Jordan blocks associated with finite eigenvalues
%      'zjblock'  - Jordan blocks associated with the zero eigenvalue
%      'ijblock'  - Jordan blocks associated with the infinite
%                   eigenvalue
%   Jordan blocks associated with a specified finite eigenvalue are given
%   as a cell-array tuple in the form {StructInt Eigenvalue}. 
%
%   Example:
%      >> mpstr = mpstruct(3,'fjblock',{[3 2],0},'fjblock',{[1], -3})
%      creates a matrix polymomial structure object of degree 3 with one
%      Jordan block of size 3x3 and one of size 2x2 both with eigenvalue 0,
%      and one Jordan block of size 1x1 with eigenvalue -3.
%
%   MPSTRUCT(d,'BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------------
%   StructObj = MPSTRUCT(d,BlockObj1,BlockObj2,...) creates a structure
%   object from the listed canonical block objects. The block objects must
%   be valid blocks for the structure.
%
%   Examples:
%      >> mpstr = mpstruct([],[2],[3 1],4,[1])
%      returns a matrix polynomial structure with one left singular block
%      of size 3x2, one Jordan block of size 3x3 and one of size 1x1 both
%      with eigenvalue 4, and one Jordan block of size 1x1 with an infinite
%      eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> mpstr = mpstruct('lsblock',2,'ijblock',1,'fjblock',{[3 1],4})
%
%      >> mpstr = mpstruct([], [], {[2] [1]}, [1 3], [2])
%      returns a matrix polynomial of the following form:
%                      | J2(1)    0     0   |
%                      |   0    J1(3)   0   |
%                      |   0      0    N2   |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class MPSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   MPSTRUCT Methods:
%     CODIM       - Compute the codimension of a matrix polynomial.
%     SIZE        - Total size of the represented structure.
%     NUMBLK      - Number of canonical block objects.
%     RANK        - Compute the normal rank.
%     HASFULLRANK - True for canonical structure with full normal rank.
%     ISEMPTY     - True for empty canonical structure.
%     CHAR        - Convert a structure object to a string.
%     EXIST       - Check if a canonical block of a specified type 
%                   already exist.
%     COPY        - Return a copy of the structure object.
%     COMPARE     - Compare two canonical structure objects.
%     SET         - Set the canonical structure information (not implemented).
%     GET         - Get the canonical structure information.
%     GETVALIDBLOCKS   - Return a list of valid canonical blocks.
%     GETSGCONSTRAINTS - Return a list of available StratiGraph constraints.
%
%   MPSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also RSBLOCK, LSBLOCK, FJBLOCK, ZJBLOCK, IJBLOCK, PSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef mpstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {'rsblock','lsblock','fjblock','ijblock','zjblock'}; 

    % StratiGraph class name of the canonical structure
    SGObjectName = 'polymat.MatrixPolyStructure';
    
    % StratiGraph setup name
    SGSetupName = 'Matrix Polynomial';

    % StratiGraph expander class name
    SGExpanderName = ''; % Is set in getSGExpanderName()
  end
  
  properties (Access = private)
    Degree 
  end
    
  methods
    % Constructor
    function obj = mpstruct(d,varargin)
      nargoutchk(0,1);
      
      if nargin > 0
        if isnumeric(d)
          if ~isreal(d) || rem(d,1) ~= 0 || min(d) < 1
            error('MCS:mpstruct:InvalidDegree','The degree must be integer valued and greater than zero.')
          end
        elseif ~isobject(d)
          error('First argument must be the degree or another canonical structure object.');
        end
      end   
      
      % In which form is the constructor called.
      % If in compact form, convert to Property-Value form...
      if nargin > 1 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(2,6)
        
        % Right singular blocks
        if ~isempty(varargin{1})
          Propertylist = {'rsblock', varargin{1}};
        else
          Propertylist = {};
        end
          
        % Left singular blocks
        if nargin > 2
          Propertylist = [Propertylist, {'lsblock', varargin{2}}];
        end
        
        % Finite Jordan blocks
        if nargin > 3 
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('fjblock',varargin(3:end))];
        end
        
        % Jordan blocks with infinite eigenvalue
        if nargin > 5
          Propertylist = [Propertylist, {'ijblock', varargin{5}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});
      
      if nargin == 0
        % New empty object
        return
      elseif isnumeric(d)
        [b,dmax] = obj.validateMP(d);
        if ~b
          error('MCS:mpstruct:InvalidMatrixPolynomial',...
            ['The canonical structure information is not valid for a matrix'...
            ' polynomial of degree %d.\nLargest valid degree is %d'],d,dmax)
        end
        
        % Set degree
        obj.Degree = d;
      elseif nargin > 0 && isempty(obj)
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(d,'mpstruct')
          % If input is a matrix polynomial structure object, make a copy.
          obj = d.copy;
        elseif isobject(d)
          error('Cannot convert an object of type %s to an mpstruct object.', class(d))
        end
      end
      
    end % End constructor
    %-------------------------------

    function c = codim(obj,strata)
      %CODIM Compute the codimension of a matrix polynomial.
      %   CODIM(StructObj) determines the codimension of the orbit of the matrix
      %   polynomial structure object StructObj. The codimension is computed from
      %   the invariants of a Fiedler linearization and is determined with
      %   respect to the represented canonical structure not the tangent space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %     'orbit'      Determines the codimension of the orbit.
      %                  Assumes that all eigenvalues are specified. (default)
      %     'bundle'     Determines the codimension of the bundle:
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct eigenvalues)
      %                  Assumes that all (finite and infinite) eigenvalues are
      %                  unspecified.
      %     'semibundle' Determines the codimension of the specified
      %                  structure, i.e. the codimension is computed as
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct unspecified eigenvalues)

      narginchk(1,2);
      
      blocksv = obj.get('rsblock','lsblock','fjblock','ijblock','StructInt','segre');
      rblk  = blocksv{1} + obj.Degree - 1 ;
      lblk  = blocksv{2};
      jblkv = [blocksv{3} blocksv(4)]; % Concat fjblock and ijblock
      nr = length(rblk);
      nl = length(lblk);
      
      if nargin < 2 || strcmpi(strata,'orbit')
        offset = 0;
      elseif strcmpi(strata,'bundle')
        % -1 for each finite eigenvalue
        eigv = obj.get('fjblock','Eigenvalue');
        offset = length(eigv); 
      elseif strcmpi(strata,'semibundle')
        % -1 for each unspecified eig
        eigv = obj.get('fjblock','Eigenvalue');
        offset = sum(isnan(cell2mat(eigv))); 
      else
        error('Unknown second argument.');
      end
      
      cJ = 0;
      for i = 1:length(jblkv)
        if ~isempty(jblkv{i})
          m = 1:2:length(jblkv{i})*2;
          cJ = cJ + sum(jblkv{i}.*m);
        end
      end
      cJ = cJ - offset;
      
      cR = 0;
      for i = 1:nr-1
        for j=i+1:nr
          if rblk(i) > rblk(j)
            cR = cR + rblk(i)-rblk(j)-1;
          end
        end
      end
      
      cL = 0;
      for i = 1:nl-1
        for j=i+1:nl
          if lblk(i) > lblk(j)
            cL = cL + lblk(i)-lblk(j)-1;
          end
        end
      end
      
      cJS = (nr + nl) * sum([jblkv{:}]);
      cS = sum(rblk)*nl + sum(lblk)*nr + 2*nl*nr;
      
      c = cJ+cL+cR+cS+cJS;
    end % End codim
    %-------------------------------
  
    function b = hasfullrank(obj)
      %HASFULLRANK True for canonical structure with full normal rank.
      %   HASFULLRANK(StructObj) returns logical 1 (true) if StrcutObj has the
      %   normal rank R = min(m,n) of the matrix polynomial structure object
      %   StructObj, otherwise logical 0 (false).
      
      [m,n,~] = obj.size();
      b = obj.rank == min(m,n);
    end
    %-------------------------------

    function r = rank(obj)
      %RANK Compute the normal rank.
      %   R = RANK(StructObj) determines the normal rank R of the matrix
      %   polynomial structure object StructObj from its canonical structure
      %   information. The normal rank is equal to the rank of P(s) at any
      %   complex value s which is not a zero of P(s).
      
      blocksv = obj.get('StructInt','unroll');
      r = sum(cell2mat(blocksv))/obj.Degree;
    end
    %-------------------------------

    function varargout = size(obj,varargin)
      %SIZE Total size of the represented structure.
      %   S = SIZE(StructObj) returns a three-element row vector S=[M N D]
      %   containing the size M-by-N of the coefficient matrices and degree D of
      %   the matrix polynomial.
      %
      %   [M,N,D] = SIZE(StructObj) returns the number of rows, columns, and
      %   degree in separate output variables.
      %
      %   SIZE(StructObj,Dim) returns the length of the dimension specified by
      %   the scalar Dim, where Dim maps as 1 -> M, 2 -> N, and 3 -> D.

      narginchk(1,2)
      nargoutchk(0,3)
      
      % Return M, N, and D...
      blocksv = obj.get('StructInt','unroll');
      r = sum(cell2mat(blocksv))/obj.Degree;
      m = length(blocksv{2}) + r;
      n = length(blocksv{1}) + r;
      
      if nargin == 2
        switch varargin{1}
          case 1
            varargout{1} = m;
          case 2
            varargout{1} = n;
          case 3
            varargout{1} = obj.Degree;
        end
        return
      end
      
      if nargout < 2
        varargout{1} = [m n obj.Degree];
      else
        varargout = {m n obj.Degree};
      end
    end
    %-------------------------------

  end % End public methods

  methods (Access = protected)
    
    function expanderName = getSGExpanderName(obj, expanderName)
      if isempty(expanderName) || strcmpi(expanderName,'MatrixPolyExpander')
        return
      end
      
      if strcmpi(expanderName,'MatrixPolyFullRankExpander')
        [m,n,~] = obj.size();
        if obj.rank() ~= min(m,n)
          fprintf(2,'The matrix polynomial is not of full normal rank.\nThe setup with no constraint is used instead.\n')
          expanderName = '';
        end
      else
        error('Unknown constraint ''%s'' for setup %s.',...
          expanderName(1:end-8), obj.SGSetupName)
      end
    end
    %-------------------------------
    
    function jsetupSize = getSGSetupSize(obj, expanderName)
      % JSETUPSIZE = GETSGSETUPSIZE(StructObj)
      
      [m,n,d] = obj.size();
      
      % Create a java size parameter and set it
      jsetupSize = se.umu.stratigraph.core.structure.SizeParameters();
      spm = se.umu.stratigraph.core.structure.SizeParameter('m');
      spn = se.umu.stratigraph.core.structure.SizeParameter('n');
      spd = se.umu.stratigraph.core.structure.SizeParameter('d');
      spm.setValue(m);
      spn.setValue(n);
      spd.setValue(d);
      jsetupSize.addParameter(spm);
      jsetupSize.addParameter(spn);
      jsetupSize.addParameter(spd);
      
      spis = se.umu.stratigraph.core.structure.SizeParameter('isigma');
      spcs = se.umu.stratigraph.core.structure.SizeParameter('csigma');
      
      if strcmpi(expanderName,'MatrixPolyFullRankExpander')
        [m,~,~] = obj.size();
        if obj.rank() == m
          spis.setValue(0);
          spcs.setValue(d-1);
        elseif obj.rank() == n
          spis.setValue(d-1);
          spcs.setValue(0);
        else 
          fprintf(2,'The matrix polynomial is not of full normal rank.\nThe setup with no constraint is used instead.\n')
          spis.setValue(0);
          spcs.setValue(0);
        end
      else
        spis.setValue(0);
        spcs.setValue(0);
      end
      jsetupSize.addParameter(spis);
      jsetupSize.addParameter(spcs);
    end
    
    function jblockTypes = getSGBlockTypes(obj,expanderName)
      %GETSGBLOCKTYPES Return the existing StratiGraph block types.
      %   JBlockTypes = GETSGBLOCKTYPES(StructObj, ExpanderName) returns the set
      %   of existing block types for the canonical structure object with the
      %   expander ExpanderName.
      
      if strcmpi(expanderName,'MatrixPolyFullRankExpander')
        [m,n,~] = obj.size();
        if m == n && obj.rank() == m
          jblockTypes = [se.umu.stratigraph.core.structure.BlockType.J 
            se.umu.stratigraph.core.structure.BlockType.N];
          return
        elseif obj.rank() == m
          jblockTypes = [se.umu.stratigraph.core.structure.BlockType.R 
            se.umu.stratigraph.core.structure.BlockType.J 
            se.umu.stratigraph.core.structure.BlockType.N];
          return
        elseif obj.rank() == n
          jblockTypes = [se.umu.stratigraph.core.structure.BlockType.L 
            se.umu.stratigraph.core.structure.BlockType.J 
            se.umu.stratigraph.core.structure.BlockType.N];
          return
        else
          fprintf(2,'The matrix polynomial is not of full normal rank.\nThe setup with no constraint is used instead.\n')
        end
      end
      jblockTypes = [se.umu.stratigraph.core.structure.BlockType.R 
        se.umu.stratigraph.core.structure.BlockType.L 
        se.umu.stratigraph.core.structure.BlockType.J 
        se.umu.stratigraph.core.structure.BlockType.N];
    end
    
  end % End protected methods
  
  methods (Access = private)
    function [b,d] = validateMP(obj,d)
      %VALIDATEMP Validate the canonical structure information.
      %   VALIDATEMP(StructObj) validates that the canonical structure
      %   information represents a matrix polynomial of degree d.
      
      blk = obj.get('rsblock','lsblock','ijblock','fjblock','StructInt','unroll');
      
      % Normal rank
      r = sum(cell2mat(blk))/d;

      % Must satisfy the 'index sum identity'
      b = (rem(r,1) == 0);
      if b, b = length(blk{3}) < r; end
      if b, b = all(cellfun(@length,blk(4:end)) <= r); end
      
      if ~b
        % Compute maximal valid degree
        rmin = max([length(blk{3})+1 cellfun(@length,blk(4:end))]);
        s = sum(cell2mat(blk));
        for d = floor(s/rmin):-1:1
          if rem(s/d,1) == 0
            return
          end
        end
      end      
    end
  end % End private methods
    

end % End classdef