%MSTRUCT Create a matrix structure object.
%   MSTRUCT creates a canonical structure object representing a matrix
%   A in its canonical form.
%
%   An MSTRUCT object can only consist of the canonical blocks:
%     fjblock - Jordan block associated with a finite eigenvalue mu,
%               defined as:
%                           |mu  1    0|
%                J_n(mu) := |   mu .   |       n-by-n
%                           |      .  1|
%                           |0       mu|
%
%     zjblock - Jordan block associated with the zero eigenvalue.
%
%
%   StructObj = MSTRUCT() returns an empty canonical structure object
%   StructObj representing a matrix A. The canonical structure information
%   can be specified using one of the three forms 'Size vector form',
%   Property-value form', or 'Object form', which are explained below.
%
%   Size vector form
%   ----------------
%   StructObj = MSTRUCT(JBlocksv) returns a new canonical structure object
%   StructObj representing a matrix. The parameter JBlocksv defines the
%   Jordan blocks of the structure and must be either a row-vector with the
%   sizes of Jordan blocks, all with the same associated eigenvalue, or a
%   cell-array of row-vectors where each vector contains the sizes of
%   blocks with the same associated eigenvalue. By default the associated
%   eigenvalues are set to unspecified (NaN).
%
%   StructObj = MSTRUCT(JBlocksv, Eigv) also sets the eigenvalues, where
%   Eigv is a row-vector with the values of each associated eigenvalue
%   corresponding to the row-vectors in JBlocksv. Jordan blocks with an
%   unspecified associated eigenvalue are defined by setting the
%   corresponding eigenvalue in Eigv to NaN.
%
%   Property-value form
%   -------------------
%   StructObj = MSTRUCT('fjblock',StructInt1,'fjblock',StructInt2,...)
%   specifies the Jordan blocks in property-value form, where each
%   StructInt is the sizes of the blocks associates with the same
%   eigenvalue. Jordan blocks associated with a specified finite eigenvalue
%   are given as a cell-array tuple in the form {StructInt Eigenvalue}. The
%   block 'zjblock' for a Jordan block with zero eigenvalue is also
%   allowed, but then StructInt can only be a vector with sizes. Note that
%   an 'fjblock' with a zero eigenvalue is not
%
%   StructObj = MSTRUCT('fjblock',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------------
%   StructObj = MSTRUCT(BlockObj1,BlockObj2,...) creates a structure object
%   from the listed canonical block objects. The block objects must be
%   valid blocks for the structure.
%
%   Examples:
%      >> mstr = mstruct([3 1])
%      returns a matrix structure with two Jordan blocks of sizes 3x3 and
%      1x1, both with the same but unspecified eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> mstr = mstruct('fjblock',[3 1])
%
%      >> mstr = mstruct({[3 2] [1]},[0 -3])
%      A matrix structure with one Jordan block of size 3x3 and one of size
%      2x2 both with the eigenvalue 0, and one Jordan block of size 1x1
%      with the eigenvalue -3.
%
%      Alternatively
%      >> mstr = mstruct('fjblock',{[3 2],0},'fjblock',{[1], -3})
%
%      In the property-value form, Jordan blocks associated with the same
%      finite eigenvalue (not equal to NaN) are joined together, i.e.,
%      >> mstruct('fjblock',{3,4},'fjblock',{[1 2],4})
%      returns
%         J(4) = (3 2 1)  (6x6)
%
%      However, observe that an 'fjblock' associated with a zero eigenvalue
%      is not the same as a 'zjblock', e.i.,
%      >> mstruct('fjblock',{3,0},'zjblock',[1 2])
%      returns
%         J(0) = (3)    (3x3)   <- an fjblock
%         J(0) = (2 1)  (3x3)   <- two zjblocks
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class MSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   MSTRUCT Methods:
%     CODIM   - Compute the codimension of a matrix.
%     JCF,JNF - Return the matrix in the Jordan Canonical Form.
%     SIZE    - Total size of the represented structure.
%     NUMBLK  - Number of canonical block objects.
%     ISEMPTY - True for empty canonical structure.
%     CHAR    - Convert a structure object to a string.
%     EXIST   - Check if a canonical block of a specified type 
%               already exist.
%     COPY    - Return a copy of the structure object.
%     COMPARE - Compare two canonical structure objects.
%     SET     - Set the canonical structure information (not implemented).
%     GET     - Get the canonical structure information.
%     GETVALIDBLOCKS   - Return a list of valid canonical blocks.
%     GETSGCONSTRAINTS - Return a list of available StratiGraph constraints.
%
%   MSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also FJBLOCK, ZJBLOCK, PSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef mstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblocks
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure
    cblocks = {'fjblock'}; 

    % StratiGraph Object name of the canonical structure
    SGObjectName = 'builtin.MatrixStructure';
    
    % StratiGraph setup name
    SGSetupName = 'Matrix';

    % StratiGraph expander class name
    SGExpanderName = 'MatrixExpander';
  end
    
  methods
    % Constructor
    function obj = mstruct(varargin)

      if nargin == 0
        % Return an empty matrix object...
        return
      end
      nargoutchk(0,1);
      
      if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
        % Convert from a set of block objects to a structure object
        obj = block2struct(obj,varargin{1});
        return
      elseif isa(varargin{1},'mstruct')
        % If input is a matrix structure object, make a copy
        obj = varargin{1}.copy;
        return
      elseif isobject(varargin{1})
        error('Cannot convert an object of type %s to a mstruct object.', class(varargin{1}))
      end
      
      % In which form is the constructor called.
      % Property-Value form...
      if ischar(varargin{1})
        if mod(nargin,2)
          error('PropertyName and PropertyValue must come in pairs.')
        end
        
        % Set the correct notation
        if any(strcmpi('Notation',varargin))
          not = varargin{find(strcmpi('Notation',varargin),1)+1};
        else
          not = 'sizes';
        end
        
        kstr = 0;
        evVec = [];
        zkstr = 0;
        % For each property
        for k = 1:2:(nargin-1)
          blockname = lower(varargin{k});
          value = varargin{k+1};
          switch blockname
            case 'fjblock'
              if iscell(value) % {[sizes] ev}
                if ~isscalar(value{2}) || isinf(value{2})
                  error('MCS:mstruct:Eigenvalue','The eigenvalues must be finite scalars or NaN.')
                end
                % Check if an eigenvalue already exist.
                % Possible to instead call: 
                %   obj.find('jblock',value{2})
                % but the following is faster in the matrix case.
                evChk = value{2} ~= evVec;
                if isempty(evChk) | evChk %#ok<OR2> % If not a duplicated eigenvalue
                  evVec(end+1) = value{2};
                  kstr = kstr+1;
                  obj.cstructure{kstr} = fjblock(value{1},value{2},not);
                else
                  obj.extendblock(find(evChk == 0),value{1},not); %#ok<FNDSB>
                end
              elseif isnumeric(value) && (min(size(value)) < 2) % [sizes]
                kstr = kstr+1;
                obj.cstructure{kstr} = fjblock(value,NaN,not);
              else
                error('StructInt must be a vector or a cell-array for ''fjblock''.')
              end
            case 'zjblock'
              if isnumeric(value) && (min(size(value)) < 2) % [sizes]
                if zkstr
                  obj.extendblock(zkstr,value,not);
                else
                  kstr = kstr+1;
                  zkstr = kstr;
                  obj.cstructure{kstr} = zjblock(value,not);
                end
              else
                error('StructInt must be a vector for ''zjblock''.')
              end
            case 'notation'
              % Do nothing...
            otherwise
              error('Property %s is not a valid block class.',blockname);
          end
        end
      else % If in compact form...
        narginchk(1,2)
        
        % Create and set the Jordan blocks
        if iscell(varargin{1})
          if nargin == 2
            if length(varargin{1}) ~= length(varargin{2})
              error('MCS:mstruct:NumberEigenvalueMissmatch', ...
                ['The number of eigenvalues does not match the number' ...
                ' of Jordan block structures.']);
            end
            eigv = varargin{2};
          else
            eigv = nan(1,length(varargin{1}));
          end
          kstr = 0;
          for k = 1:length(varargin{1})
            % Eigenvalue is checked in fjblock/set.ev
            if ~isempty(varargin{1}{k})
              kstr = kstr+1;
              obj.cstructure{kstr} = fjblock(varargin{1}{k},eigv(k));
            end
          end
        elseif isnumeric(varargin{1}) && (min(size(varargin{1})) < 2)
          if ~isempty(varargin{1})
            if nargin == 2
              % Eigenvalue is checked in fjblock/set.ev
              obj.cstructure{1} = fjblock(varargin{1},varargin{2});
            else
              obj.cstructure{1} = fjblock(varargin{1},NaN);
            end
          end
        else
          error('JBlocksv must be a vector or a cell-array.')
        end
      end
    end % Constructor mstruct
    %------------------------------
    
    function c = codim(obj,strata)
      %CODIM Compute the codimension of a matrix.
      %   CODIM(StructObj) determines the codimension of the orbit of the matrix
      %   structure object StructObj. The codimension is determined with respect
      %   to the represented canonical structure.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %     'orbit'      Determines the codimension of the orbit.
      %                  Assumes that all eigenvalues are specified. (default)
      %     'bundle'     Determines the codimension of the bundle:
      %                     codim(bundle) = codim(orbit) 
      %                        - (number of distinct eigenvalues)
      %                  Assumes that all eigenvalues are unspecified.
      %     'semibundle' Determines the codimension of the specified
      %                  structure, i.e. the codimension is computed as
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct unspecified eigenvalues)
      %
      %   See also MCODIM.
      
      if nargin < 2 || strcmpi(strata,'orbit')
        jblkv = obj.get('fjblock','StructInt','segre'); % The Jordan blocks
        offset = 0;  % No adjustment for orbits.
      elseif strcmpi(strata,'bundle')
        jblkv = obj.get('fjblock','StructInt','segre'); % The Jordan blocks
        offset = length(jblkv); % -1 for each eigenvalue for bundles
      elseif strcmpi(strata,'semibundle')
        [jblkv,eigv] = obj.get('fjblock','segre');
        offset = sum(isnan(eigv)); % -1 for each unspecified eigenvalue
      else
        error('Unknown second argument');
      end
      
      
      c = 0; % Codimension
      for i = 1:length(jblkv)
        % Getting the sizes of the Jordan blocks associated to
        % eigenvalue i in Segre characteristics, i.e., sorted in
        % decreasing order.
        if ~isempty(jblkv{i})
          % sum(1*j1+3*j2+5*j3+...+2*n-1*jn)
          m     = 1:2:length(jblkv{i})*2;
          c     = c + sum(jblkv{i}.*m);
        end
      end
      c = c - offset;
    end
    %-------------------------------
    
    function e = eig(obj)
      %EIG Return the eigenvalues of the canonical structure.
      
      n = size(obj,1);
      e = zeros(n,1);
      k = 1;
      j = 1;
      while k <= n
        neigs = size(obj.cstructure{j},1);
        e(k:k+neigs-1) = get(obj.cstructure{j},'Eigenvalue');
        k = k + neigs;
        j = j + 1;
      end
      
    end
    %-------------------------------

  end % methods
  
  methods (Access = protected)
    function jblockTypes = getSGBlockTypes(obj,~)
      %GETSGBLOCKTYPES Return the existing StratiGraph block types.
      %   JBlockTypes = GETSGBLOCKTYPES(StructObj) returns the set of existing
      %   block types for the canonical structure object.
      
      jblockTypes = se.umu.stratigraph.core.structure.BlockType.J;
    end
  end
end % classdef
