%PSTRUCT Create a matrix pencil canonical structure object.
%   PSTRUCT creates a canonical structure object representing a matrix
%   pencil G-sH in its canonical form.
%
%   A PSTRUCT object can consist of the following canonical blocks:
%     rsblock - Right singular block defined as:
%                      |0 1    0|     |1 0    0|
%               L_n := |   . .  |  - s|   . .  |        n-by-(n+1)
%                      |0    0 1|     |0    1 0|
%
%     lsblock - Left singular block defined as:
%                      |0     0|     |1     0|
%             L_n^T := |1  .   |  - s|0  .   |          (n+1)-by-n
%                      |   .  0|     |   .  1|
%                      |0     1|     |0     0|
%
%     fjblock - Jordan block associated with a finite eigenvalue mu,
%               defined as:
%                               |mu  1    0|
%            J_n(mu) - s*I_n := |   mu .   | - s*I_n      n-by-n
%                               |      .  1|
%                               |0       mu|
%
%     zjblock - Jordan block associated with the zero eigenvalue (mu=0).
%
%     ijblock - Jordan block associated with the infinite eigenvalue,
%               defined as:
%                      |1 0    0|     |0 1    0|
%               N_n := |  1 .   |  - s|  0 .   |          n-by-n
%                      |    .  0|     |    .  1|
%                      |0      1|     |0      0|
%
%
%   StructObj = PSTRUCT returns an empty canonical structure object
%   StructObj representing a matrix pencil G-sH. The canonical structure
%   information can be specified using one of the three forms 'Size vector
%   form', Property-value form', or 'Object form', which are explained
%   below.
%
%   Size vector form
%   ----------------
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv) returns a new
%   canonical structure object StructObj representing a matrix pencil. The
%   parameters RSBlocks and LSBlocks define the right and left singular
%   blocks (associated with the column and row minimal indices),
%   respectively, of the structure and are given as row-vectors with the
%   sizes of the blocks. The parameter FJBlocksv defines the Jordan blocks
%   (associated with the finite elementary divisors) of the structure and
%   must be either a row-vector with the sizes of Jordan blocks, all
%   associated with the same eigenvalue, or a cell-array of row-vectors
%   where each vector contains the sizes of blocks associated with the same
%   eigenvalue. By default the associated eigenvalues are set to
%   unspecified (NaN).
%
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv) also sets the
%   eigenvalues, where Eigv is a row-vector with the values of each
%   associated eigenvalue corresponding to the row-vectors in FJBlocksv.
%   Jordan blocks with an unspecified associated eigenvalue are defined by
%   setting the corresponding eigenvalue in Eigv to NaN.
%
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv,IJBlocks) also
%   sets the sizes of the Jordan blocks with an associated infinite
%   eigenvalue (associated with the infinite elementary divisors).
%
%   Property-value form
%   -------------------
%   StructObj =
%   PSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...) specifies
%   the block types BlockName and the sizes StructInt in property-value
%   form. BlockName is a string specifying the canonical block to be
%   created, and can be one of:
%      'rsblock'  - Right singular blocks
%      'lsblock'  - Left singular blocks
%      'fjblock'  - Jordan blocks associated with finite eigenvalues
%      'zjblock'  - Jordan blocks associated with the zero eigenvalue
%      'ijblock'  - Jordan blocks associated with the infinite
%                   eigenvalue
%   Jordan blocks associated with a specified finite eigenvalue are given
%   as a cell-array tuple in the form {StructInt Eigenvalue}. 
%
%   Example:
%      >> pstr = pstruct('fjblock',{[3 2],0},'fjblock',{[1], -3})
%      creates a pencil structure object with one Jordan block of
%      size 3x3 and one of size 2x2 both with eigenvalue 0, and one
%      Jordan block of size 1x1 with eigenvalue -3.
%
%   PSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also specifies
%   the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------
%   StructObj = PSTRUCT(BlockObj1,BlockObj2,...) creates a structure object
%   from the listed canonical block objects. The block objects must be
%   valid blocks for the structure.
%
%   StructObj = PSTRUCT(Xstr) converts the structure object Xstr to a
%   matrix pencil structure object. Xstr can be a canonical structure
%   object of the type: (skew-)symmetric matrix pencil, matrix, or
%   state-space system.
%
%   Examples:
%      >> pstr = pstruct([],[2],[3 1],4,[1])
%      returns a matrix pencil structure with one left singular block of
%      size 3x2, one Jordan block of size 3x3 and one of size 1x1 both with
%      eigenvalue 4, and one Jordan block of size 1x1 with an infinite
%      eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> pstr = pstruct('lsblock',2,'ijblock',1,'fjblock',{[3 1],4})
%
%      >> pstr = pstruct([], [], {[2] [1]}, [1 3], [2])
%      returns a matrix pencil of the following form:
%                      | J2(1)    0     0   |
%                      |   0    J1(3)   0   |
%                      |   0      0    N2   |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class PSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   PSTRUCT Methods:
%     CODIM   - Compute the codimension of a matrix pencil.
%     KCF     - Return the matrix pencil in the Kronecker Canonical Form.
%     SIZE    - Total size of the represented structure.
%     NUMBLK  - Number of canonical block objects.
%     ISEMPTY - True for empty canonical structure.
%     CHAR    - Convert a structure object to a string.
%     EXIST   - Check if a canonical block of a specified type 
%               already exist.
%     COPY    - Return a copy of the structure object.
%     COMPARE - Compare two canonical structure objects.
%     SET     - Set the canonical structure information (not implemented).
%     GET     - Get the canonical structure information.
%     GETVALIDBLOCKS   - Return a list of valid canonical blocks.
%     GETSGCONSTRAINTS - Return a list of available StratiGraph constraints.
%
%   PSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also RSBLOCK, LSBLOCK, FJBLOCK, ZJBLOCK, IJBLOCK, SPSTRUCT,
%   SSPSTRUCT, MSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef pstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {'rsblock','lsblock','fjblock','ijblock','zjblock'}; 

    % StratiGraph Object name of the canonical structure
    SGObjectName = 'builtin.PencilStructure';
    
    % StratiGraph setup name
    SGSetupName = 'Matrix Pencil';

    % StratiGraph expander class name
    SGExpanderName = 'MatrixPencilExpander';
  end
    
  methods
    % Constructor
    function obj = pstruct(varargin)
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If in compact form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,5)
        
        % Right singular blocks
        if ~isempty(varargin{1})
          Propertylist = {'rsblock', varargin{1}};
        else
          Propertylist = {};
        end
          
        % Left singular blocks
        if nargin > 1
          Propertylist = [Propertylist, {'lsblock', varargin{2}}];
        end
        
        % Finite Jordan blocks
        if nargin > 2 
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('fjblock',varargin(3:end))];
        end
        
        % Jordan blocks with infinite eigenvalue
        if nargin > 4
          Propertylist = [Propertylist, {'ijblock', varargin{5}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});

      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'pstruct')
          % If input is a matrix pencil structure object, make a copy
          obj = varargin{1}.copy;
        elseif isa(varargin{1},'mstruct') || isa(varargin{1},'ssstruct')
          % If input is a matrix or a system pencil structure object, copy
          % the canonical blocks to the new matrix pencil object.
          blkobj = varargin{1}.get('object');
          for k = 1:length(blkobj)
            obj.cstructure{k} = copy(blkobj{k});
          end
        elseif isa(varargin{1},'sspstruct')
          % If input is a skew-symmetric matrix pencil structure object, 
          % transform the canonical blocks to the new matrix pencil object.
          % Skew-symmetric -> Standard: M_n -> L_n and R_n, SH_n(a) -> 2J_n(a), SK_n ->2J_n(\infty)

          blk = varargin{1}.get('smblock','shblock','skblock');
          k = 0;
          if ~isempty(blk{1})
            % Singular blocks
            obj.cstructure{1} = rsblock(blk{1});
            obj.cstructure{2} = lsblock(blk{1});
            k = 2;
          end
          if ~isempty(blk{2})
            if iscell(blk{2}{1})
              % More than one eigenvalue...
              for b = 1:length(blk{2})
                obj.cstructure{k+b} = fjblock([blk{2}{b}{1}, blk{2}{b}{1}], blk{2}{b}{2});
              end
              k = k+b;
            else
              % One eigenvalue...
              k = k+1;
              obj.cstructure{k} = fjblock([blk{2}{1}, blk{2}{1}], blk{2}{2});
            end
          end
          if ~isempty(blk{3})
            % Infinite eigenvalue
            obj.cstructure{k+1} = ijblock([blk{3}{1}, blk{3}{1}]);
          end
        elseif isa(varargin{1},'spstruct') 
          % If input is a symmetric matrix pencil structure object,
          % transform the canonical blocks to the new matrix pencil object.
          % Symmetric -> Standard: M_n -> L_n and R_n, H_n(a) -> J_n(a), K_n -> J_n(\infty)
          
          blk = varargin{1}.get('mblock','hblock','kblock');
          k = 0;
          if ~isempty(blk{1})
            % Singular blocks
            obj.cstructure{1} = rsblock(blk{1});
            obj.cstructure{2} = lsblock(blk{1});
            k = 2;
          end
          if ~isempty(blk{2})
            if iscell(blk{2}{1})
              % More than one eigenvalue...
              for b = 1:length(blk{2})
                obj.cstructure{k+b} = fjblock(blk{2}{b}{1},blk{2}{b}{2});
              end
              k = k+b;
            else
              % One eigenvalue...
              k = k+1;
              obj.cstructure{k} = fjblock(blk{2}{1},blk{2}{2});
            end
          end
          if ~isempty(blk{3})
            % Infinite eigenvalue
            obj.cstructure{k+1} = ijblock(blk{3}{1});
          end
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to a pstruct object.', class(varargin{1}))
        end
      end      
    end % End constructor
    %-------------------------------

    function c = codim(obj,strata)
      %CODIM Compute the codimension of a matrix pencil.
      %   CODIM(StructObj) determines the codimension of the orbit of the matrix
      %   pencil structure object StructObj. The codimension is determined with
      %   respect to the represented canonical structure not the tangent space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %     'orbit'      Determines the codimension of the orbit.
      %                  Assumes that all eigenvalues are specified. (default)
      %     'bundle'     Determines the codimension of the bundle:
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct eigenvalues)
      %                  Assumes that all (finite and infinite) eigenvalues
      %                  are unspecified.
      %     'semibundle' Determines the codimension of the specified
      %                  structure, i.e. the codimension is computed as
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct unspecified eigenvalues)
      %
      %   See also PCODIM.

      narginchk(1,2);
      
      blocksv = obj.get('rsblock','lsblock','fjblock','ijblock','StructInt','segre');
      rblk  = blocksv{1};
      lblk  = blocksv{2};
      jblkv = [blocksv{3} blocksv(4)]; % Concat fjblock and ijblock
      nr = length(rblk);
      nl = length(lblk);
      
      if nargin < 2 || strcmpi(strata,'orbit')
        offset = 0;
      elseif strcmpi(strata,'bundle')
        % -1 for each finite eigenvalue
        eigv = obj.get('fjblock','Eigenvalue');
        offset = length(eigv); 
      elseif strcmpi(strata,'semibundle')
        % -1 for each unspecified finite eig
        eigv = obj.get('fjblock','Eigenvalue');
        offset = sum(isnan(cell2mat(eigv))); 
      else
        error('Unknown second argument.');
      end
      
      cJ = 0;
      for i = 1:length(jblkv)
        if ~isempty(jblkv{i})
          m = 1:2:length(jblkv{i})*2;
          cJ = cJ + sum(jblkv{i}.*m);
        end
      end
      cJ = cJ - offset;
      
      cR = 0;
      for i = 1:nr-1
        for j=i+1:nr
          if rblk(i) > rblk(j)
            cR = cR + rblk(i)-rblk(j)-1;
          end
        end
      end
      
      cL = 0;
      for i = 1:nl-1
        for j=i+1:nl
          if lblk(i) > lblk(j)
            cL = cL + lblk(i)-lblk(j)-1;
          end
        end
      end
      
      cJS = (nr + nl) * sum([jblkv{:}]);
      cS = sum(rblk)*nl + sum(lblk)*nr + 2*nl*nr;
      
      c = cJ+cL+cR+cS+cJS;
    end % End codim
    %-------------------------------
  
  end % End public methods

  methods (Access = protected)
    function jblockTypes = getSGBlockTypes(obj,~)
      %GETSGBLOCKTYPES Return the existing StratiGraph block types.
      %   JBlockTypes = GETSGBLOCKTYPES(StructObj) returns the set of existing
      %   block types for the canonical structure object.
      
      jblockTypes = [se.umu.stratigraph.core.structure.BlockType.R 
        se.umu.stratigraph.core.structure.BlockType.L 
        se.umu.stratigraph.core.structure.BlockType.J 
        se.umu.stratigraph.core.structure.BlockType.N];
    end
  end
  
end % End classdef