%SCMSTRUCT Create a *congruence matrix structure object.
%   SCMSTRUCT creates a canonical structure object representing a matrix
%   A in its canonical form under *congruence.
%
%   A SCMSTRUCT object can consist of the following canonical blocks:
%     sgblock - *Gamma block associated with a complex parameter mu,
%               defined as:
%                                  |0         ...|
%                                  |        ...  |
%                *Gamma_n(mu) := mu|      1  1   |        n-by-n
%                                  |  -1 -1      |
%                                  |1  1        0|
%               where |mu| = 1.
%
%     swblock - *W block associated with a specified and admissible
%               eigenvalue mu, defined as:
%                                |  0    I_m|
%                    *W_n(mu) := |          |            2n-by-2n
%                                |J_m(mu) 0 |
%               where |mu| > 1 and J_m(mu) is an m-by-m Jordan
%               block associated with the eigenvalue mu.
%
%     zjblock - Jordan block associated with the zero eigenvalue,
%               defined as:
%                                |0 1   0|
%                      J_n(0) := |  0 .  |                n-by-n
%                                |    . 1|
%                                |0     0|
%
%
%   StructObj = SCMSTRUCT() returns an empty canonical structure object
%   StructObj representing a matrix A under *congruence. The canonical
%   structure information can be specified using one of the three forms
%   'Size vector form', Property-value form', or 'Object form', which are
%   explained below.
%
%   Size vector form
%   ----------------
%   StructObj = SCMSTRUCT(SGBlocksv,Muv,SWBlocksv,Eigv) returns a new
%   canonical structure object StructObj representing a matrix under
%   *congruence. The parameter SGBlocksv defines the *Gamma blocks of the
%   structure and is given as a cell-array of row-vectors where each vector
%   contains the sizes of blocks associated with the same parameter mu. Muv
%   is a row-vector with the values of each associated parameters mu
%   corresponding to the row-vectors in SGBlocksv. The parameter SWBlocksv
%   defines the *W blocks associated with Jordan blocks of the structure
%   and must be a cell-array of row-vectors where each vector contains the
%   sizes of the blocks associated with the same eigenvalue. Eigv is a
%   row-vector with the values of each associated eigenvalue corresponding
%   to the row-vectors in SWBlocksv.
%
%   StructObj = SCMSTRUCT(SGBlocks,Muv,SWBlocksv,Eigv,ZJBlocks) also sets
%   the sizes of the Jordan blocks with an associated zero eigenvalue.
%
%   Property-value form
%   -------------------
%   StructObj =
%   SCMSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...)
%   specifies the block types BlockName and the sizes StructInt in
%   property-value form. BlockName is a string specifying the canonical
%   block to be created, and can be one of:
%      'sgblock' - *Gamma blocks
%      'swblock  - *W blocks associated with Jordan blocks
%                  (associated with finite eigenvalues)
%      'zjblock' - Jordan blocks associated with the zero eigenvalue.
%
%   *Gamma and *W blocks associated with a specified parameter are
%   given as a cell-array tuple in the form {StructInt Parameter}.
%
%   Example:
%      >> scmstr = scmstruct('swblock',{[3 2],2},'sgblock',{[1], 1i})
%      creates a structure object with one *W block of size 6x6 and
%      one of size 4x4 both with the associated eigenvalue 2, and one
%      *Gamma block of size 2x2 with the associated complex parameter
%      1i.
%
%   SCMSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------------
%   StructObj = SCMSTRUCT(BlockObj1,BlockObj2,...) creates a structure
%   object from the listed canonical block objects. The block objects must
%   be valid blocks for the structure.
%
%   Examples:
%      >> scmstr = scmstruct([],[],[3 1],4,[1])
%      returns a matrix structure with one *W block of size 6x6 and one of
%      size 2x2 both with eigenvalue 4, and one Jordan block of size 1x1
%      with the zero eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> scmstr = scmstruct('zjblock',1,'swblock',{[3 1],4})
%
%      >> scmstr = scmstruct([], [], {[2] [1]}, [5 3], [2])
%      returns a matrix of the following form:
%                      | SW2(5)    0     0   |
%                      |   0    SW1(3)   0   |
%                      |   0      0    J2(0) |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class SCMSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   SCMSTRUCT Methods:
%     CODIM     - Compute the codimension of a matrix under *congruence.
%     CCF       - Return the matrix in the congruence canonical form.
%     SIZE      - Total size of the represented structure.
%     NUMBLK    - Number of canonical block objects.
%     ISEMPTY   - True for empty canonical structure.
%     CHAR      - Convert a structure object to a string.
%     EXIST     - Check if a canonical block of a specified type
%                 already exist.
%     COPY      - Return a copy of the structure object.
%     COMPARE   - Compare two canonical structure objects.
%     SET       - Set the canonical structure information (not implemented).
%     GET       - Get the canonical structure information.
%     GETVALIDBLOCKS - Return a list of available canonical blocks.
%
%   PSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also SGBLOCK, SWBLOCK, ZJBLOCK, CMSTRUCT, MSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef scmstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {'sgblock','swblock','zjblock'};
    
    % StratiGraph Object name of the canonical structure
    SGObjectName = '';
    
    % StratiGraph setup name
    SGSetupName = '';

    % StratiGraph expander class name
    SGExpanderName = '';
  end
  
  methods
    % Constructor
    function obj = scmstruct(varargin)
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If in compact form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,5)
        
        % *Gamma blocks
        if ~isempty(varargin{1})
          if nargin < 2
            error('MCS:scmstruct:MissingParameter','Missing parameter Muv.')
          end
          % Parameter values are checked in sgblock
          Propertylist = mcsstruct.parseFJBlocks('sgblock',varargin(1:end));
        else
          Propertylist = {};
        end
        
        
        % *W blocks
        if nargin > 2
          if nargin < 4
            error('MCS:scmstruct:MissingParameter','Missing parameter Eigv.')
          end
          % Parameter values are checked in swblock
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('swblock',varargin(3:end))];
        end
        
        % Jordan blocks with zero eigenvalue
        if nargin > 4
          Propertylist = [Propertylist, {'zjblock', varargin{5}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});
      
      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'scmstruct')
          % If input is a *congruence matrix structure object, make a copy.
          obj = varargin{1}.copy;
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to a scmstruct object.', class(varargin{1}))
        end
      end
    end % End constructor
    %-------------------------------
    
    function cod = codim(obj,strata)
      %CODIM Compute the codimension of a matrix under *congruence.
      %   CODIM(StructObj) determines the codimension of the *congruence orbit of
      %   the matrix structure object StructObj. The codimension is determined
      %   with respect to the represented canonical structure not the tangent
      %   space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %      'orbit'   Determines the codimension of the orbit.
      %                Assumes that all eigenvalues are specified. (default)
      %      'bundle'  Not implemented!
      %
      %   See also SCMCODIM.
      
      narginchk(1,2);

      if nargin == 1
        % Strata is orbit, do nothing.
      elseif strcmp(strata,'bundle')
        error('MCS:scmstruct:codim:Bundle',...
          'Codimension for bundle is not implemented/undefined!')
      elseif ~strcmpi(strata,'orbit') 
        error('Unknown second argument');
      end
      
      blocksv = obj.get('zjblock','StructInt','segre');
      zjblk  = blocksv{1};
      nz = length(zjblk);
      wblkcai = obj.get('swblock','segre');
      nw = length(wblkcai);
      wblkca = cell(nw,1);
      eigv = zeros(1,nw);
      gblkcai = obj.get('sgblock','segre');
      ng = length(gblkcai);
      gblkca = cell(ng,1);
      geigv = zeros(1,nw);
      
      cJ = 0;
      cJ0 = 0;
      Jodd=0;
      cR = 0;
      rJ=0;
      
      if ~isempty(wblkcai{1})
        for i = 1:nw
          wblkca{i} = 2*wblkcai{i}{1};
          eigv(i) = wblkcai{i}{2}(1);
        end
        for i = 1:nw
          wblkv = wblkca{i};
          if ~isempty(wblkv)
            m = 1:2:length(wblkv)*2;
            cJ = cJ + sum(wblkv.*m);
            rJ = rJ + sum(wblkv);
          end
        end
      end
      
      
      cJ0 = cJ0 + sum(round(zjblk./2));
      for j = 1:nz-1
        for i = j+1:nz
          if mod(zjblk(i),2)==0
            cJ0= cJ0 + zjblk(i);
          else
            if zjblk(i)==zjblk(j)
              cJ0= cJ0 + zjblk(j)+1;
            else
              cJ0= cJ0 + zjblk(j);
            end
          end
        end
      end
      cJ0 = 2*cJ0;
      for j = 1:nz
        if mod(zjblk(j),2)==1
          Jodd=Jodd+1;
        end
      end
      
      if ~isempty(gblkcai{1})
        for i = 1:ng
          gblkca{i} = gblkcai{i}{1};
          geigv(i) = gblkcai{i}{2}(1);
        end
        for i = 1:ng
          gblkv = gblkca{i};
          if ~isempty(gblkv)
            n = length(gblkv);
            cR = cR + sum(gblkv);
            rJ = rJ + sum(gblkv);
            for i1 = 1:n-1
              for j=i1+1:n
                if mod(gblkv(i1)-gblkv(j),2)==0
                  cR = cR + 2*gblkv(j);
                end
              end
            end
          end
          for j1 = i+1:ng
            if geigv(i)== -geigv(j1)
              gblkv1 = gblkca{j1};
              n1 = length(gblkv1);
              for l = 1:n
                for k=1:n1
                  if mod(gblkv(l)-gblkv1(k),2)==0
                    cR = cR + 2*min(gblkv(l),gblkv1(k));
                  end
                end
              end
            end
            if geigv(i)*geigv(i)== -geigv(j1)*geigv(j1)
              gblkv1 = gblkca{j1};
              n1 = length(gblkv1);
              for l = 1:n
                for k=1:n1
                  if mod(gblkv(l)-gblkv1(k),2)==1
                    cR = cR + 2*min(gblkv(l),gblkv1(k));
                  end
                end
              end
            end
          end
        end
      end
      
      cJHR = 2*rJ*Jodd;
      cod = cJ+cJ0+cR+cJHR;
    end % End codim
    %-------------------------------
    
  end % End public methods
end % End classdef