%SSPSTRUCT Create a skew-symmetric matrix pencil structure object.
%   SSPSTRUCT creates a canonical structure object representing a
%   skew-symmetric matrix pencil G-sH in its canonical form.
%
%   A SSPSTRUCT object can consist of the following canonical blocks:
%     smblock - Singular SM block defined as:
%                      |  0     G_n|     |   0    F_n|
%              SM_n := |           |  - s|           |     (2n+1)-by-(2n+1)
%                      |-G_n^T   0 |     |-F_n^T   0 |
%              where
%                      |0 1   0|            |1 0   0|
%               G_n := |  . .  | and F_n := |  . .  |
%                      |0   0 1|            |0   1 0|
%
%     shblock - SH block associated with a finite or unspecified
%               eigenvalues mu, defined as:
%                      |  0        J_n(mu)|     |  0    I_n|
%              SH_n := |                  |  - s|          |   2n-by-2n
%                      |-J_n(mu)^T    0   |     |-I_n    0 |
%               where J_n(mu) is an n-by-n Jordan block associated with the
%               eigenvalue mu.
%
%     skblock - SK block associated with the infinite eigenvalue,
%               defined as:
%                      |  0    I_n|     |  0        J_n(0)|
%              SK_n := |          |  - s|                 |    2n-by-2n
%                      |-I_n    0 |     |-J_n(0)^T    0   |
%               where J_n(0) is an n-by-n Jordan block associated with the
%               zero eigenvalue.
%
%
%   StructObj = SSPSTRUCT() returns an empty canonical structure object
%   StructObj representing a skew-symmetric matrix pencil G-sH. The
%   canonical structure information can be specified using one of the three
%   forms 'Size vector form', Property-value form', or 'Object form', which
%   are explained below.
%
%   Size vector form
%   ----------------
%   StructObj = SSPSTRUCT(SMBlocks,SHBlocksv) returns a new canonical
%   structure object StructObj representing a skew-symmetric matrix pencil.
%   The parameter SMBlocks defines the singular SM blocks of the structure
%   and is given as a row-vector with the sizes of the blocks. The
%   parameter SHBlocksv defines the SH blocks associated with the finite or
%   unspecified eigenvalues and must be either a row-vector with the
%   indices of blocks, all associated with the same eigenvalue, or a
%   cell-array of row-vectors where each vector contains the sizes of
%   blocks associated with the same eigenvalue. By default the associated
%   eigenvalues are set to unspecified (NaN).
%
%   StructObj = SSPSTRUCT(SMBlocks,SHBlocksv,Eigv) sets the eigenvalues,
%   where Eigv is a row-vector with the values of each associated
%   eigenvalue corresponding to the row-vectors in SHBlocksv. The SH blocks
%   with an unspecified associated eigenvalue are defined by setting the
%   corresponding eigenvalue in Eigv to NaN.
%
%   StructObj = SSPSTRUCT(SMBlocks,SHBlocksv,Eigv,SKBlocks) also sets the
%   sizes of the SK blocks associated with the infinite eigenvalue.
%
%   Property-value form
%   -------------------
%   StructObj =
%   SSPSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...)
%   specifies the block types BlockName and the sizes StructInt in
%   property-value form. BlockName is a string specifying the canonical
%   block to be created, and can be one of:
%      'smblock'  - Singular SM blocks.
%      'shblock'  - SH blocks associated with finite eigenvalues.
%      'skblock'  - SK blocks associated with the infinite eigenvalue.
%   SH blocks associated with a specified finite eigenvalue are given as a
%   cell-array tuple in the form {StructInt Eigenvalue}. 
%
%   Example:
%      >> sspstr = sspstruct('shblock',{[10 3],[7]},'smblock',[3,2])
%      creates a pencil structure object with two SH blocks of the sizes
%      20x20 and 6x6 both with the eigenvalue 7, and two singular blocks of
%      the sizes 7x7 and 5x5.
%
%   SSPSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Indices are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Indices may be unordered. (default)
%
%   Object form
%   -----------
%   StructObj = SSPSTRUCT(BlockObj1,BlockObj2,...) creates a structure
%   object from the listed canonical block objects. The block objects must
%   be valid blocks for the structure.
%
%   Example:
%      >> sspstr = sspstruct([2], [3 1], 4, [1])
%      returns a skew-symmetric matrix pencil structure with one SM
%      block of size 5, one SH block of size 6 and one of size 2,
%      both associated with eigenvalue 4, and one SK block of size 2
%      associated with the infinite eigenvalue.
%
%      Alternatively, the same structure can be created with
%      >> sspstr = sspstruct('smblock',2,'skblock',1,'shblock',{[3 1],4})
%
%      >> sspstr = sspstruct([4], {[2] [1]}, [1 3], [2])
%      returns a skew-symmetric matrix pencil of the following form:
%
%                      |SM4   0      0     0   |
%                      | 0  SH2(1)   0     0   |
%                      | 0    0    SH1(3)  0   |
%                      | 0    0      0    SK2  |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class SSPSTRUCT also provides methods for extracting information
%   and modifying the canonical structure object.
%
%   SSPSTRUCT Methods:
%     CODIM     - Compute the codimension of a skew-symmetric matrix 
%                 pencil.
%     KCF       - Return the skew-symmetric matrix pencil in a 
%                 Kronecker-like canonical form (described above).
%     SIZE      - Total size of the represented structure.
%     NUMBLK    - Number of canonical block objects.
%     ISEMPTY   - True for empty canonical structure.
%     CHAR      - Convert a structure object to a string.
%     EXIST     - Check if a canonical block of a specified type
%                 already exist.
%     COPY      - Return a copy of the structure object.
%     COMPARE   - Compare two canonical structure objects.
%     SET       - Set the canonical structure information (not implemented).
%     GET       - Get the canonical structure information.
%     GETVALIDBLOCKS - Return a list of available canonical blocks.
%
%   SSPSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also SMBLOCK, SHBLOCK, SKBLOCK, SPSTRCUT, PSTRUCT.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef sspstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblocks
  end
  
  properties (Constant, Access = {?mcsstruct})
    cblocks = {'smblock','shblock','skblock'};
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    
    % StratiGraph Object name of the canonical structure
    SGObjectName = '';
    
    % StratiGraph setup name
    SGSetupName = '';

    % StratiGraph expander class name
    SGExpanderName = '';
  end
  
  methods
    
    % Constructor
    function obj = sspstruct(varargin)
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If old form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,4)
        
        % Singular blocks (Mblock)
        if ~isempty(varargin{1})
          Propertylist = {'smblock', varargin{1}};
        else
          Propertylist = {};
        end
        
        
        % Blocks associated with finite Jordan blocks (SHblock)
        if nargin > 1
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('shblock',varargin(2:end))];
        end
        
        % Blocks associated with Jordan blocks with infinite eigenvalue (SKblock)
        if nargin > 3
          Propertylist = [Propertylist, {'skblock', varargin{4}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});
      
      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'sspstruct')
          % If input is a skew-symmetric matrix pencil structure object, make a copy.
          obj = varargin{1}.copy;
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to a sspstruct object.', class(varargin{1}))
        end
      end
    end % End constructor
    %-------------------------------
    
    function c = codim(obj,strata)
      %CODIM Compute the codimension of a skew-symmetric matrix pencil.
      %   CODIM(StructObj) determines the codimension of the orbit of the
      %   skew-symmetric matrix pencil structure object StructObj. The
      %   codimension is determined with respect to the represented structure not
      %   the tangent space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %     'orbit'      Determines the codimension of the orbit.
      %                  Assumes that all eigenvalues are specified. (default)
      %     'bundle'     Determines the codimension of the bundle.
      %                  Assumes that all (finite and infinite) eigenvalues are
      %                  unspecified.
      %     'semibundle' Determines the codimension of the specified
      %                  structure, i.e., the codimension is computed as
      %                  codim(orbit) - (number of distinct unspecified
      %                  eigenvalues).
      %
      %   Remark:
      %     codim(bundle) = codim(orbit) - (number of distinct eigenvalues)
      %
      %   See also SSPCODIM.
      
      narginchk(1,2);
      
      blocksv = obj.get('smblock','shblock','skblock','StructInt','segre');
      mblk  = blocksv{1};
      jblkv = [blocksv{2} blocksv(3)]; % Concat shblock and skblock
      nm = length(mblk);
      
      if nargin < 2 || strcmpi(strata,'orbit')
        offset = 0;
      elseif strcmpi(strata,'bundle')
        % -1 for each finite eigenvalue
        eigv = obj.get('shblock','Eigenvalue');
        offset = length(eigv); 
      elseif strcmpi(strata,'structure')
        eigv = obj.get('shblock','Eigenvalue');
        offset = sum(isnan(cell2mat(eigv))); % -1 for each unspecified eig
      else
        error('Unknown second argument');
      end
      
      cJ = 0;
      for i = 1:length(jblkv)
        if ~isempty(jblkv{i})
          m = 1:1:length(jblkv{i});
          cJ = cJ + sum(jblkv{i}.*(4*m-3));
        end
      end
      cJ = cJ - offset;
      
      cL = 0;
      n = length(mblk);
      for i = 1:n-1
        for j=i+1:n
          if eq(mblk(i),mblk(j))
            cL = cL + 2*mblk(i)+2;
          else
            cL = cL + 2*mblk(i)+1;
          end
        end
      end
      cJS = 2*nm*sum([jblkv{:}]);
      c = cJ+cL+cJS;
    end % End codim
    
    
  end % End public methods
end % End classdef

