%SSSTRUCT Create a state-space system structure object.
%   SSSTRUCT creates a canonical structure object representing the system
%   pencil S-sT = [A B; C D] - s[I 0; 0 0] in its canonical form. The
%   system pencil S-sT is associated with the continuous-time state-space
%   model
%       dx/dt = Ax(t) + Bu(t),
%           y = Cx(t) + Du(t).
%
%   An SSSTRUCT object can consist of the following canonical blocks:
%     rsblock - Right singular block defined as:
%                      |0 1    0|     |1 0    0|
%               L_n := |   . .  |  - s|   . .  |         n-by-(n+1)
%                      |0    0 1|     |0    1 0|
%
%     lsblock - Left singular block defined as:
%                      |0     0|     |1     0|
%             L_n^T := |1  .   |  - s|0  .   |           (n+1)-by-n
%                      |   .  0|     |   .  1|
%                      |0     1|     |0     0|
%
%     fjblock - Jordan block associated with a finite eigenvalue mu,
%               defined as:
%                               |mu  1    0|
%            J_n(mu) - s*I_n := |   mu .   | - s*I_n       n-by-n
%                               |      .  1|
%                               |0       mu|
%
%     zjblock - Jordan block associated with the zero eigenvalue (mu=0).
%
%     ijblock - Jordan block associated with the infinite eigenvalue,
%               defined as::
%                      |1 0    0|     |0 1    0|
%               N_n := |  1 .   |  - s|  0 .   |           n-by-n
%                      |    .  0|     |    .  1|
%                      |0      1|     |0      0|
%
%
%   StructObj = SSSTRUCT() returns an empty canonical structure
%   object StructObj representing the system pencil
%        S-sT = [A B; C D] - s[I 0; 0 0],
%   associated with the continuous-time state-space model
%       dx/dt = Ax(t) + Bu(t),
%        y(t) = Cx(t) + Du(t).
%   The canonical structure information can be specified using one of the
%   three forms 'Size vector form', Property-value form', or 'Object form',
%   which are explained below.
%
%   Size vector form
%   ----------------
%   StructObj = SSSTRUCT(RSBlocks,LSBlocks,FJBlocksv) returns a new
%   canonical structure object StructObj representing a system pencil. The
%   parameter RSBlocks and LSBlocks define the right and left singular
%   blocks (associated with the column and row minimal indices),
%   respectively, of the structure and is given as row-vectors with the
%   sizes of the blocks. The parameter FJBlocksv defines the Jordan blocks
%   (associated with the finite elementary divisors) of the structure and
%   must be either a row-vector with the sizes of Jordan blocks, all
%   associated with the same eigenvalue, or a cell-array of row-vectors,
%   each containing the sizes of blocks associated with the same
%   eigenvalue. By default the associated eigenvalues are set to
%   unspecified (NaN).
%
%   StructObj = SSSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv) also sets the
%   eigenvalues, where Eigv is a row-vector with the values of each
%   associated eigenvalue corresponding to the row-vectors in FJBlocksv.
%   Jordan blocks with an unspecified associated eigenvalue are defined by
%   setting the corresponding eigenvalue in Eigv to NaN.
%
%   StructObj = SSSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv,IJBlocks) also
%   sets the sizes of the Jordan blocks with an associated infinite
%   eigenvalue (associated with the infinite elementary divisors).
%
%   Property-value form
%   -------------------
%   StructObj =
%   SSSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...) specifies
%   the block types BlockName and the sizes StructInt in property-value
%   form. BlockName is a string specifying the canonical block to be
%   created, and can be one of:
%      'rsblock'  - Right singular blocks
%      'lsblock'  - Left singular blocks
%      'fjblock'  - Jordan blocks associated with finite eigenvalues
%      'zjblock'  - Jordan blocks associated with the zero eigenvalue
%      'ijblock'  - Jordan blocks associated with the infinite
%                   eigenvalue
%   Jordan blocks associated with a specified finite eigenvalue are given
%   as a cell-array tuple in the form {StructInt Eigenvalue}.
%
%   Example:
%       >> ssstr = ssstruct('fjblock',{[3 2],0},'fjblock',{[1], -3})
%       creates a state-space structure object with one Jordan block of
%       size 3x3 and one of size 2x2 both with eigenvalue 0, and one Jordan
%       block of size 1x1 with eigenvalue -3.
%
%   SSSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------
%   StructObj = SSSTRUCT(BlockObj1,BlockObj2,...) creates a structure
%   object from the listed canonical block objects. The block objects must
%   be valid blocks for the structure.
%
%   StructObj = SSSTRUCT(Pstr) converts the matrix pencil structure object
%   Pstr to a system pencil structure object with the same canonical
%   structure information.
%
%   Examples:
%      >> ssstr = ssstruct([2],[0 1],[2],4)
%      returns a state-space structure with one right singular block of
%      size 2x3, two left singular blocks of sizes 1x0 and 2x1, and one
%      Jordan block of size 2x2 with eigenvalue 4.
%
%      Alternatively, the same structure can be created with
%      >> ssstr = ...
%           ssstruct('rsblock',2,'lsblock',[0 1],'fjblock',{2,4})
%
%      >> ssstr = ssstruct('rsblock',3,'fjblock',{3,-1})
%      returns a controllability pair structure with one right singular
%      block of size 3x4 and one Jordan block of size 3x3 with eigenvalue
%      -1.
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class SSSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   SSSTRUCT Methods:
%     CODIM     - Compute the codimension of a state-space model.
%     OFTYPE    - State-space model type.
%     CTRBPAIR  - Controllability pair.
%     OBSVPAIR  - Observability pair.
%     BCF       - Return the system pencil in the Brunovsky Canonical Form.
%     KCF       - Return the matrix pencil in the Kronecker Canonical Form.
%     SIZE      - Total size of the represented structure.
%     NUMBLK    - Number of canonical block objects.
%     ISEMPTY   - True for empty canonical structure.
%     CHAR      - Convert a structure object to a string.
%     EXIST     - Check if a canonical block of a specified type 
%                 already exist.
%     COPY      - Return a copy of the structure object.
%     COMPARE   - Compare two canonical structure objects.
%     SET       - Set the canonical structure information (not implemented).
%     GET       - Get the canonical structure information.
%     GETVALIDBLOCKS   - Return a list of valid canonical blocks.
%     GETSGCONSTRAINTS - Return a list of available StratiGraph constraints.
%
%   SSSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also RSBLOCK, LSBLOCK, FJBLOCK, ZJBLOCK, IJBLOCK.

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

classdef ssstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblock
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {'rsblock','lsblock','fjblock','ijblock','zjblock'}; 
    
    % StratiGraph class name of the canonical structure
    SGObjectName = 'statespace.StateSpaceStructure';
    
    % StratiGraph setup name
    SGSetupName = 'State-Space System';

    % StratiGraph expander class name
    SGExpanderName = ''; % Is set in getSGExpanderName()
  end
    
  methods
    % Constructor
    function obj = ssstruct(varargin)
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If old form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,5)
        
        % Right singular blocks
        if ~isempty(varargin{1})
          Propertylist = {'rsblock', varargin{1}};
        else
          Propertylist = {};
        end
        
        % Left singular blocks
        if nargin > 1 
          Propertylist = [Propertylist, {'lsblock', varargin{2}}];
        end
        
        % Finite Jordan blocks
        if nargin > 2
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks('fjblock',varargin(3:end))];
        end
        
        % Jordan blocks with infinite eigenvalue
        if nargin > 4
          Propertylist = [Propertylist, {'ijblock', varargin{5}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});

      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'ssstruct')
          % If input is a state-space structure object, make a copy.
          obj = varargin{1}.copy;
        elseif isa(varargin{1},'pstruct')
          blkobj = varargin{1}.get('object');
          blk = 1;
          for k = 1:length(blkobj)
            if ~blkobj{k}.isempty
              obj.cstructure{blk} = copy(blkobj{k});
              blk = blk+1;
            end
          end
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to an ssstruct object.', class(varargin{1}))
        end
      end      
    end % End constructor
    %-------------------------------

    function varargout = size(obj,varargin)
      %SIZE Total size of the represented structure.
      %   D = SIZE(StructObj) returns a two-element row vector D = [M, N]
      %   containing the number of rows M and columns N of the system pencil
      %   structure, i.e., the sum of the sizes of all the canonical blocks.
      %
      %   [M,N] = SIZE(StructObj) returns the number of rows and columns in
      %   separate output variables.
      %
      %   SIZE(StructObj,Dim) returns the length of the dimension specified by
      %   the scalar Dim. For example, SIZE(StructObj,1) returns the number of
      %   rows.
      %
      %   [N,M,P] = SIZE(StructObj) returns the sizes of the matrices in the
      %   state-space model. The returned sizes are the sizes of the N-by-N
      %   matrix A (number of states N), the N-by-M matrix B (number of inputs
      %   M), and the P-by-N matrix C (number of outputs P).

      narginchk(1,2)
      nargoutchk(0,3)
      
      if nargout < 3
        % Call size in mcsstruct
        [varargout{1:nargout}] = size@mcsstruct(obj,varargin{:});
        return
      end

      % Return N, M, and P...
      
      if nargin == 2
        warning('MCS:ssstruct:SizeDimension','Specified dimension ignored.')
      end
      % Call size in mcsstruct
      sizes = size@mcsstruct(obj);
      
      blocksv = obj.get('rsblock','lsblock','ijblock','StructInt');
      r0 = length(blocksv{1});
      l0 = length(blocksv{2});
      ginf = length(blocksv{3});

      m = r0 + ginf;    % Number of inputs
      p = l0 + ginf;    % Number of outputs
      n = sizes(1) - p; % Number of states
      
      varargout = {n m p};
    end
    %-------------------------------

    function type = oftype(obj)
      %OFTYPE State-space model type.
      %   Type = OFTYPE(StructObj) returns a string Type indicating the
      %   non-empty matrices (A, B, C, and D) in the state-space model
      %       dx/dt = Ax(t) + Bu(t),      (1)
      %        y(t) = Cx(t) + Du(t).
      %   The type is determined from the canonical structure information.
      %
      %   Type can be:
      %     'a'    - StructObj represents a state-space model with only a state
      %              matrix A. The state-space structure consists only of Jordan
      %              blocks with finite eigenvalues (associated with the finite
      %              elementary divisors).
      %
      %     'ab'   - StructObj represents a controllability pair (A,B) associated
      %              with the particular system
      %                 dx/dt = Ax(t) + Bu(t),
      %              of (1). The state-space structure consists only of right
      %              singular blocks (associated with the column minimal indices)
      %              and Jordan blocks with finite eigenvalues (associated with
      %              the finite elementary divisors).
      %              
      %     'ac'   - StructObj represents an observability pair (A,C) associated
      %              with the particular system
      %                 dx/dt = Ax(t),
      %                  y(t) = Cx(t),
      %              of (1). The state-space structure consists only of left
      %              singular blocks (associated with the row minimal indices)
      %              and Jordan blocks with finite eigenvalues (associated with
      %              the finite elementary divisors).
      %
      %     'abc'  - StructObj represents a state-space model without any
      %              feed-forward, i.e., D is a zero matrix. The state-space
      %              structure do not have any Jordan blocks assosiated with the
      %              infinite eigenvalue of size 1-by-1 (no infinite elementary
      %              divisors of order 1).
      %
      %     'abcd' - StructObj represents the full state-space model in (1).
      %
      %     '' (empty string) - StructObj is an empty object.
      
      if isempty(obj)
        type = '';
        return
      end
      
      blocksv = obj.get('rsblock','lsblock','ijblock','StructInt');
      r0 = length(blocksv{1});
      l0 = length(blocksv{2});
      ginf = length(blocksv{3});
      ij1 = sum(blocksv{3}<2); % Number of N_1 blocks

      m = r0 + ginf; % Number of inputs
      p = l0 + ginf; % Number of outputs
      
      if ~p  % p == 0
        if m
          type = 'ab';
        else
          type = 'a';
        end
      elseif ~m  % m == 0
        type = 'ac';
      elseif ij1 % if N_1 blocks exist
        type = 'abcd';
      else
        type = 'abc';
      end
    end
    %-------------------------------

    function varargout = ctrbpair(obj)
      %CTRBPAIR Controllability pair.
      %   PairObj = CTRBPAIR(StructObj) returns a new state-space system
      %   structure object PairObj representing the controllability pair (A,B),
      %   i.e., the matrices C and D in the state-space model are assumed to be
      %   empty matrices.
      %
      %   [A,B] = CTRBPAIR(StructObj) returns the controllability pair (A,B)
      %   associated with the particular system
      %       dx/dt = Ax(t) + Bu(t),
      %   of the state-space model represented by the structure object StructObj.
      %   The matrix pair (A,B) is returned in Brunovsky canonical form (BCF).
      %
      %   NOTE! The new canonical form is determined exactly from the BCF, i.e.,
      %   the pair (A,B) is assumed to be in BCF and no computational routines
      %   are called.
      %
      %   See also OBSVPAIR.
      
      narginchk(1,1)
      nargoutchk(0,2)

      % Compute the canonical form for (A,B)
      [blocksv,param] = obj.get('rsblock','lsblock','fjblock','ijblock');
      
      if isempty(blocksv{1}) && isempty(blocksv{4}) 
        warning('MCS:ssstruct:ctrbpair',...
          'Matrix B is empty. An empty state-space system object is returned.')
        s2str = ssstruct;
      else
        % N_k -> L_{k-1}
        if ~isempty(blocksv{4})
          rblk = [blocksv{1} blocksv{4}{1}-1];
        else
          rblk = blocksv{1};
        end
        % L_k^T -> J_k(0), k > 0
        lblk = blocksv{2}(blocksv{2}>0); % LSBLOCK > 0
        if ~isempty(lblk)
          fjblk = [blocksv{3} lblk];
          eigv = [param{3} 0];
        else
          fjblk = blocksv{3};
          eigv = param{3};
        end
        
        s2str = ssstruct(rblk,[],fjblk,eigv);
      end
      
      if nargout == 2
        [At,Bt,~,~] = s2str.bcf;
        varargout{1} = At;
        varargout{2} = Bt;
      else
        varargout{1} = s2str;
      end
    end
    %-------------------------------

    function varargout = obsvpair(obj)
      %OBSVPAIR Observability pair.
      %   PairObj = OBSVPAIR(StructObj) returns a new state-space system
      %   structure object PairObj representing the observability pair (A,C),
      %   i.e., the matrices B and D in the state-space model are assumed to be
      %   empty matrices.
      %
      %   [A,C] = OBSVPAIR(StructObj) returns the observability pair (A,C)
      %   associated with the particular system
      %       dx/dt = Ax(t), 
      %        y(t) = Cx(t), 
      %   of the state-space model represented by the structure object StructObj.
      %   The matrix pair (A,C) is returned in Brunovsky canonical form (BCF).
      %
      %   NOTE! The new canonical form is determined exactly from the BCF, i.e.,
      %   the pair (A,C) is assumed to be in BCF and no computational routines
      %   are called.
      %
      %   See also CTRBPAIR.

      narginchk(1,1)
      nargoutchk(0,2)

      % Compute the canonical form for (A,C)
      [blocksv,param] = obj.get('rsblock','lsblock','fjblock','ijblock');

      if isempty(blocksv{2}) && isempty(blocksv{4})
        warning('MCS:ssstruct:obsvpair',...
          'Matrix C is empty. An empty state-space system object is returned.')
        s2str = ssstruct;
      else
        % N_k -> L_{k-1}^T
        if ~isempty(blocksv{4})
          lblk = [blocksv{2} blocksv{4}{1}-1];
        else
          lblk = blocksv{2};
        end
        % L_k -> J_k(0), k > 0
        rblk = blocksv{1}(blocksv{1}>0); % RSBLOCK > 0
        if ~isempty(rblk)
          fjblk = [blocksv{3} rblk];
          eigv = [param{3} 0];
        else
          fjblk = blocksv{3};
          eigv = param{3};
        end
        
        s2str = ssstruct([],lblk,fjblk,eigv);
      end
      
      if nargout == 2
        [At,~,Ct,~] = s2str.bcf;
        varargout{1} = At;
        varargout{2} = Ct;
      else
        varargout{1} = s2str;
      end
    end
    %-------------------------------

    function c = codim(obj,strata)
      %CODIM Compute the codimension of a state-space system pencil.
      %   CODIM(StructObj) determines the codimension of the orbit of the
      %   state-space structure object StructObj. The codimension is determined
      %   with respect to the represented canonical structure not the tangent
      %   space.
      %
      %   CODIM(StructObj,Strata) where the optional argument Strata can be
      %     'orbit'      Determines the codimension of the orbit.
      %                  Assumes that all eigenvalues are specified. (default)
      %     'bundle'     Determines the codimension of the bundle:
      %                     codim(bundle) = codim(orbit) 
      %                      - (number of distinct finite eigenvalues)
      %                  Assumes that all finite eigenvalues are unspecified.
      %                  Infinite eigenvalues are still treated as specified.
      %     'semibundle' Determines the codimension of the specified
      %                  structure, i.e. the codimension is computed as
      %                     codim(bundle) = codim(orbit)
      %                      - (number of distinct unspecified eigenvalues)
      %
      %   See also S2CODIM.

      narginchk(1,2);
      
      blocksv = obj.get('rsblock','lsblock','fjblock','ijblock','StructInt','segre');
      rblk  = blocksv{1}; 
      lblk  = blocksv{2};
      ijblk = blocksv{4};
      jblkv = [blocksv{3} blocksv(4)]; % Concat fjblock and ijblock
      nr = length(rblk);
      nl = length(lblk);
      
      if nargin < 2 || strcmpi(strata,'orbit')
        offset = 0;
      elseif strcmpi(strata,'bundle')
        % -1 for each finite eigenvalue
        eigv = obj.get('fjblock','Eigenvalue');
        offset = length(eigv); 
      elseif strcmpi(strata,'semibundle')
        % -1 for each unspecified eig
        eigv = obj.get('fjblock','Eigenvalue');
        offset = sum(isnan(cell2mat(eigv))); 
      else
        error('Unknown second argument.');
      end
      
      % Regular blocks
      cJ = 0;
      for i = 1:length(jblkv)
        if ~isempty(jblkv{i})
          m = 1:2:length(jblkv{i})*2;
          cJ = cJ + sum(jblkv{i}.*m);
        end
      end
      cJ = cJ - offset;
      
      % Right singular blocks
      cR = 0;
      for i = 1:nr-1
        for j=i+1:nr
          if rblk(i) > rblk(j)
            cR = cR + rblk(i)-rblk(j)-1;
          end
        end
      end
      
      % Left singular blocks
      cL = 0;
      for i = 1:nl-1
        for j=i+1:nl
          if lblk(i) > lblk(j)
            cL = cL + lblk(i)-lblk(j)-1;
          end
        end
      end
      
      % Interaction between singular and regular blocks
      cJS = (nr + nl) * sum([jblkv{:}]);
      % Singular blocks
      cS = sum(rblk)*nl + sum(lblk)*nr + 2*nl*nr;
      
      m = nr + length(ijblk);    % Number of inputs
      p = nl + length(ijblk);    % Number of outputs

      c = cJ+cL+cR+cS+cJS - m*p;
      
    end % End codim
  end % End public methods
  
  methods (Access = protected)
    function expanderName = getSGExpanderName(obj, expanderName)
      if isempty(expanderName) || strcmpi(expanderName,'StateSpaceExpander')
        return
      end
      
      type = obj.oftype();
      switch expanderName
        case 'ControllabilityPairExpander'
          if strcmp(type,'ab') || strcmp(type,'a')
            return
          end
          fprintf(2,'The state-space system is not a controllability pair (A,B).\nThe setup with no constraint is used instead.\n')
          expanderName = '';
        case 'ObservabilityPairExpander'
          if strcmp(type,'ac') || strcmp(type,'a')
            return
          end
          fprintf(2,'The state-space system is not an observability pair (A,C).\nThe setup with no constraint is used instead.\n')
          expanderName = '';
        case 'ABCExpander'
          if strcmp(type,'abcd')
            fprintf(2,'The state-space system has direct feedforward (D~=0).\nThe setup with no constraint is used instead.\n')
            expanderName = '';
          end
        otherwise
        error('Unknown constraint ''%s'' for setup %s.', ...
          expanderName(1:end-8), obj.SGSetupName)
      end
    end
    %-------------------------------
    
    function jsetupSize = getSGSetupSize(obj, ~)
      [n,m,p] = obj.size();

      % Create a java size parameter and set it
      jsetupSize = se.umu.stratigraph.core.structure.SizeParameters();
      spn = se.umu.stratigraph.core.structure.SizeParameter('n');
      spm = se.umu.stratigraph.core.structure.SizeParameter('m');
      spp = se.umu.stratigraph.core.structure.SizeParameter('p');
      spn.setConstraint(se.umu.stratigraph.core.structure.SizeParameter.GREATER, 0);
      spm.setConstraint(se.umu.stratigraph.core.structure.SizeParameter.GREATER, 0);
      spp.setConstraint(se.umu.stratigraph.core.structure.SizeParameter.GREATER, 0);
      spn.setValue(n);
      spm.setValue(m);
      spp.setValue(p);
      jsetupSize.addParameter(spn);
      jsetupSize.addParameter(spm);
      jsetupSize.addParameter(spp);
    end
    
    function jblockTypes = getSGBlockTypes(obj,expanderName)
      %GETSGBLOCKTYPES Return the existing StratiGraph block types.
      %   JBlockTypes = GETSGBLOCKTYPES(StructObj, ExpanderName) returns the set
      %   of existing block types for the canonical structure object with the
      %   expander ExpanderName.
      
      switch expanderName
        case 'ControllabilityPairExpander'
          type = obj.oftype();
          if strcmp(type,'ab') || strcmp(type,'a')
            jblockTypes = [se.umu.stratigraph.core.structure.BlockType.R 
              se.umu.stratigraph.core.structure.BlockType.J];
            return
          end
          fprintf(2,'The state-space system is not a controllability pair (A,B).\nThe setup with no constraint is used instead.\n')
        case 'ObservabilityPairExpander'
          type = obj.oftype();
          if strcmp(type,'ac') || strcmp(type,'a')
            jblockTypes = [se.umu.stratigraph.core.structure.BlockType.L 
              se.umu.stratigraph.core.structure.BlockType.J];
            return
          end
          fprintf(2,'The state-space system is not an observability pair (A,C).\nThe setup with no constraint is used instead.\n')
        otherwise
          % Do nothing
      end
      jblockTypes = [se.umu.stratigraph.core.structure.BlockType.R 
        se.umu.stratigraph.core.structure.BlockType.L 
        se.umu.stratigraph.core.structure.BlockType.J 
        se.umu.stratigraph.core.structure.BlockType.N];
    end
 end
end % End classdef