% *** DELETE THIS ***
% Template class for a canonical structure class.
% + Change class name XSTRUCT, where X is an appropriate abreviation for the
% structure type.
% + Add available canonical forms under listed methods below.
% *** END DELETE THIS ***

% This file is part of the Matrix Canonical Structure (MCS) Toolbox for Matlab.
% MCS Toolbox is a freely-available software package licensed under the modified
% BSD license.

%XSTRUCT Create a X structure object.
%   XSTRUCT creates a canonical structure object representing a/an XXX in
%   its canonical form.
%
%   A XSTRUCT object can consist of the following N canonical blocks:
%   *** CHANGE ***
%     rsblock - Right singular blocks.
%     lsblock - Left singular blocks.
%     jblock  - Jordan blocks.
%     ijblock - Jordan blocks associated with the infinite eigenvalue.
%   *** CHANGE ***
%
%   *** EXAMPLE OF A CONSTRUCTOR FOR MATRIX PENCILS ***
%   *** CHANGE ***
%
%   StructObj = PSTRUCT() returns an empty canonical structure object
%   StructObj representing a matrix pencil G-sH. The three forms the
%   canonical structure information can be specified are explained below.
%
%   Size vector form
%   ----------------
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv) returns a new
%   canonical structure object StructObj representing a matrix pencil. The
%   parameters RSBlocks and LSBlocks define the right and left singular
%   blocks (associated with the column and row minimal indices),
%   respectively, of the structure and are given as row-vectors with the
%   sizes of the blocks. The parameter FJBlocksv defines the Jordan blocks
%   (associated with the finite elementary divisors) of the structure and
%   must be either a row-vector with the sizes of Jordan blocks, all
%   associated with the same eigenvalue, or a cell-array of row-vectors
%   where each vector contains the sizes of blocks associated with the same
%   eigenvalue. By default the associated eigenvalues are set to
%   unspecified (NaN).
%
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv) also sets the
%   eigenvalues, where Eigv is a row-vector with the values of each
%   associated eigenvalue corresponding to the row-vectors in FJBlocksv.
%   Jordan blocks with an unspecified associated eigenvalue are defined by
%   setting the corresponding eigenvalue in Eigv to NaN.
%
%   StructObj = PSTRUCT(RSBlocks,LSBlocks,FJBlocksv,Eigv,IJBlocks) also
%   sets the sizes of the Jordan blocks with an associated infinite
%   eigenvalue (associated with the infinite elementary divisors).
%
%   Property-value form
%   -------------------
%   StructObj =
%   PSTRUCT('BlockName1',StructInt1,'BlockName2',StructInt2,...) specifies
%   the block types BlockName and the sizes StructInt in property-value
%   form. BlockName is a string specifying the canonical block to be
%   created, and can be one of:
%      'rsblock'  - Right singular blocks
%      'lsblock'  - Left singular blocks
%      'fjblock'  - Jordan blocks associated with finite eigenvalues
%      'zjblock'  - Jordan blocks associated with the zero eigenvalue
%      'ijblock'  - Jordan blocks associated with the infinite
%                   eigenvalue
%   Jordan blocks associated with a specified finite eigenvalue are given
%   as a cell-array tuple in the form {StructInt Eigenvalue}. 
%
%   Example:
%      >> pstr = pstruct('fjblock',{[3 2],0},'fjblock',{[1], -3})
%      creates a pencil structure object with one Jordan block of
%      size 3x3 and one of size 2x2 both with eigenvalue 0, and one
%      Jordan block of size 1x1 with eigenvalue -3.
%
%   PSTRUCT('BlockName1',StructInt1,...,'Notation',Notation) also
%   specifies the notation used for StructInt. Valid notations are:
%      'segre'   Sizes are ordered in a non-increasing order.
%      'weyr'    Weyr characteristics.
%      'sizes'   Sizes may be unordered. (default)
%
%   Object form
%   -----------
%   StructObj = PSTRUCT(BlockObj1,BlockObj2,...) creates a structure object
%   from the listed canonical block objects. The block objects must be
%   valid blocks for the structure.
%
%   StructObj = PSTRUCT(Xstr) converts the structure object Xstr to a
%   matrix pencil structure object. Xstr can be a canonical structure
%   object of the type: (skew-)symmetric matrix pencil, matrix, or
%   state-space system.
%
%   Examples:
%      >> pstr = pstruct([],[2],[3 1],4,[1]);
%      returns a matrix pencil structure with one left singular block of
%      size 3x2, one Jordan block of size 3x3 and one of size 1x1 both with
%      eigenvalue 4, and one Jordan block of size 1x1 with an infinite
%      eigenvalue.
%
%      Alternativly, the same structure can be created with
%      >> pstr = pstruct('lsblock',2,'ijblock',1,'fjblock',{[3 1],4})
%
%      >> pstr = pstruct([], [], {[2] [1]}, [1 3], [2]);
%      returns a matrix pencil of the following form:
%                      | J2(1)    0     0   |
%                      |   0    J1(3)   0   |
%                      |   0      0    N2   |
%
%   Subscripting
%   ------------
%   To access the canonical blocks in the structure object it is possible
%   to use subscipts. See SUBSREF for detail and examples. If more control
%   is needed of what should be retrieved, use the method GET instead.
%
%
%   The class XSTRUCT provides the following methods for extracting
%   information and modifying the canonical structure object.
%
%   XSTRUCT Methods:
%     CODIM   - Compute the codimension of a XXX.
%     KCF     - Return the matrix pencil in Kronecker Canonical Form.
%     SIZE    - Total size of the represented structure.
%     NUMBLK  - Number of canonical block objects.
%     ISEMPTY - True for empty canonical structure.
%     EXIST   - Check if a canonical block of a specified type 
%               already exist.
%     COPY    - Return a copy of the structure object.
%     COMPARE - Compare two canonical structure objects.
%     SET     - Set the canonical structure information (not implemented).
%     GET     - Get the canonical structure information.
%     GETVALIDBLOCKS - Return a list of valid canonical blocks.
%
%   XSTRUCT Operators:
%     ==, ~= (EQ, NE)  - Check if two structure objects are equal.
%     (), {} (SUBSREF) - Index reference for canonical structure objects.
%
%   See also RSBLOCK, LSBLOCK, FJBLOCK, ZJBLOCK, IJBLOCK, SPSTRUCT,
%   SSPSTRUCT, MSTRUCT.

classdef xstruct < mcsstruct & canonicalforms
  
  properties (Access = {?mcsstruct, ?canonicalforms})
    cstructure = {}; % Canonical structure: Cell-array of object of type mcsblocks
  end
  
  properties (Constant, Access = {?mcsstruct})
    % A string cell-array with all possible canonical blocks objects for
    % the canonical structure.
    cblocks = {}; 

    % StratiGraph class name of the canonical structure
    SGObjectName = 'builtin.XStructure';
    
    % StratiGraph setup name
    SGSetupName = 'Setup Name';

    % StratiGraph expander class name
    SGExpanderName = '';
  end
    
  methods
    % Constructor
    function obj = xstruct(varargin)
      
      nargoutchk(0,1);
      
      % In which form is the constructor called.
      % If old form, convert to Property-Value form...
      if nargin > 0 && ~ischar(varargin{1}) && ~isobject(varargin{1})
        narginchk(1,5)
        
        % Right singular blocks
        if ~isempty(varargin{1})
          Propertylist = {'rsblock', varargin{1}};
        else
          Propertylist = {};
        end
        
        % Left singular blocks
        if nargin > 1
          Propertylist = [Propertylist, {'lsblock', varargin{2}}];
        end
        
        % Finite Jordan blocks
        if nargin > 2
          Propertylist = [Propertylist, mcsstruct.parseFJBlocks(varargin(3:end))];
        end
        
        % Jordan blocks with infinite eigenvalue
        if nargin > 4
          Propertylist = [Propertylist, {'ijblock', varargin{5}}];
        end
      else
        Propertylist = varargin;
      end
      
      % Call constructor of mcsstruct
      obj = obj@mcsstruct(Propertylist{:});

      if nargin > 0 && isempty(obj)
        % Convert to another structure object...
        if isa(varargin{1},'mcsblock') || (iscell(varargin{1}) && isa(varargin{1}{1},'mcsblock'))
          % Convert from a set of block objects to a structure object
          obj = block2struct(varargin{1});
        elseif isa(varargin{1},'pstruct')
          % If input is a matrix pencil structure object, make a copy.
          obj = varargin{1}.copy;
        elseif isa(varargin{1},'mstruct') || isa(varargin{1},'ssstruct')
          % If input is a matrix or a pair structure object, copy the canonical
          % blocks to the new matrix pencil object.
          blkobj = varargin{1}.get('object');
          for k = 1:length(blkobj)
            obj.cstructure{k} = copy(blkobj{k});
          end
        elseif isobject(varargin{1})
          error('Cannot convert an object of type %s to a pstruct object.', class(varargin{1}))
        end;
      end      
    end % End constructor
    %-------------------------------

    function c = codim(obj,strata)
      %CODIM Compute the codimension of a matrix pencil.
      %   CODIM(StructObj) determines the codimension of the orbit of the matrix
      %   pencil structure object StructObj. The codimension is determined with
      %   respect to the represented structure not the tangent space.
      %
      %   CODIM(StructObj,Strata) optional argument Strata can be
      %      'orbit'   determines the codimension of the pencil orbit.
      %      'bundle'  determines the codimension of the pencil bundle.
      %
      %   See also PCODIM.

      narginchk(1,2);

      % Add code here...
      
      c = cJ+cL+cR+cS+cJS;
    end % End codim
    
  end % End public methods
end % End classdef